unit gossdat;
//## ==========================================================================================================================================================================================================================
//##
//## MIT License
//##
//## Copyright 2025 Blaiz Enterprises ( http://www.blaizenterprises.com )
//##
//## Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
//## files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
//## modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
//## is furnished to do so, subject to the following conditions:
//##
//## The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//##
//## THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
//## OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
//## LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
//## CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//##
//## ==========================================================================================================================================================================================================================
//## Library.................. static data (gossdat.pas)
//## Version.................. -
//## Items.................... -
//## Last Updated ............ 13apr2025
//## Lines of Code............ -
//##
//## main.pas ................ app code
//## gossroot.pas ............ console/gui app startup and control
//## gossio.pas .............. file io
//## gossimg.pas ............. image/graphics
//## gossnet.pas ............. network
//## gosswin.pas ............. 32bit windows api's
//## gosssnd.pas ............. sound/audio/midi/chimes
//## gossgui.pas ............. gui management/controls
//## gossdat.pas ............. static data/icons/splash/help documents
//##
//## ==========================================================================================================================================================================================================================
//## Important Format(s) Note:
//##
//## Many of the images and documents used below require a specific file format and storage in a binary pascal array in order to work.  A free gui tool
//## is available at http://www.blaizenterprises.com/blaiztools.html  The tool provides one button actions for pasting and converting images into
//## TEA/JPEG/PNG image formats, and packing them into a pascal array, as well as support for loading, compressing and packing text documents into
//## pascal arrays.
//## ==========================================================================================================================================================================================================================
//## Performance Note:
//##
//## The runtime compiler options "Range Checking" and "Overflow Checking", when enabled under Delphi 3
//## (Project > Options > Complier > Runtime Errors) slow down graphics calculations by about 50%,
//## causing ~2x more CPU to be consumed.  For optimal performance, these options should be disabled
//## when compiling.
//## ==========================================================================================================================================================================================================================

interface

uses
   gossroot;

const
//program icons ----------------------------------------------------------------
//app icon (required) - 32w x 32h size in TEA format only, accessed via tepIcon32
//Note: This icon is used by Gossamer internally for display on the app's Window Header (Title bar) and must be in the TEA (text picture) format.
//      A separate MS icon (.ico) is required for display on Windows Desktop/Explorer/Taskbar etc, currently stored within the resource file
//      "app-icon-16-256px.res".

program_icon32h
:array[0..4806] of byte=(
84,69,65,51,35,1,0,0,0,0,0,0,0,0,0,0,0,0,0,32,0,0,0,32,0,0,0,0,0,0,3,0,94,0,0,1,54,87,0,0,1,160,76,0,0,1,216,45,0,0,1,238,26,0,0,1,238,12,0,0,1,238,5,0,0,2,238,21,0,0,1,238,76,0,0,1,238,78,0,0,1,238,44,0,0,1,238,58,0,0,1,238,57,0,0,1,238,60,0,0,1,238,85,0,0,1,238,90,0,0,2,238,96,0,0,3,238,95,0,0,1,238,91,0,0,1,238,76,0,0,1,216,74,0,0,1,160,76,0,0,1,54,0,0,0,4,0,97,7,7,1,2,96,0,0,1,143,94,0,0,1,255,77,0,0,1,255,55,0,0,1,255,40,0,0,1,255,34,0,0,1,255,24,0,0,1,255,17,0,0,1,255,8,0,0,1,255,11,0,0,1,255,33,0,0,2,255,20,0,0,1,255,9,0,0,1,255,2,1,1,1,255,1,1,1,1,255,51,0,0,1,255,92,0,0,1,255,97,0,0,1,255,96,0,0,1,255,95,0,0,1,255,93,0,0,1,255,90,0,0,1,255,89,0,0,1,255,87,0,0,2,255,85,0,0,1,255,68,0,0,1,143,46,7,7,1,2,0,0,0,2,0,91,0,0,1,143,90,0,0,1,255,87,0,0,1,255,67,0,0,1,255,51,0,0,1,255,46,0,0,1,255,38,0,0,1,255,30,0,0,1,255,21,0,0,1,255,13,0,0,1,255,4,0,0,1,255,2,1,1,1,255,13,0,0,1,255,10,0,0,1,255,1,1,1,3,255,48,0,0,1,255,94,0,0,1,255,95,0,0,1,255,91,0,0,1,255,86,0,0,1,255,83,
0,0,1,255,81,0,0,1,255,84,0,0,1,255,87,0,0,2,255,90,0,0,1,255,82,0,0,1,255,71,0,0,1,143,0,0,0,1,0,77,0,0,1,54,81,0,0,1,255,84,0,0,1,255,86,0,0,1,255,64,0,0,1,255,47,0,0,1,255,45,0,0,1,255,39,0,0,1,255,37,0,0,1,255,18,0,0,1,255,3,1,1,1,255,1,1,1,2,255,11,0,0,1,255,7,0,0,1,255,6,0,0,1,255,1,1,1,2,255,63,0,0,1,255,94,0,0,1,255,86,0,0,1,255,73,0,0,1,255,69,0,0,1,255,70,0,0,1,255,69,0,0,1,255,74,0,0,1,255,78,0,0,1,255,85,0,0,1,255,94,0,0,1,255,90,0,0,1,255,86,0,0,1,255,86,0,0,1,54,72,0,0,1,160,77,0,0,1,255,88,0,0,1,255,85,0,0,1,255,73,0,0,1,255,52,0,0,1,255,48,0,0,1,255,49,0,0,1,255,34,0,0,1,255,6,0,0,1,255,1,1,1,2,255,7,0,0,1,255,19,0,0,1,255,22,0,0,1,255,23,0,0,1,255,2,1,1,1,255,29,1,1,1,255,88,0,0,1,255,91,0,0,1,255,72,0,0,1,255,66,0,0,1,255,65,0,0,1,255,62,0,0,1,255,63,0,0,1,255,69,0,0,1,255,72,0,0,1,255,84,0,0,1,255,91,0,0,1,255,94,0,0,1,255,88,0,0,1,255,86,0,0,1,160,78,0,0,1,215,82,0,0,1,255,78,0,0,1,255,75,0,0,1,255,64,0,0,1,255,53,0,0,1,255,52,0,0,1,255,53,0,0,1,255,23,0,0,
1,255,1,1,1,2,255,2,1,1,1,255,33,0,0,1,255,37,0,0,1,255,32,0,0,1,255,15,0,0,1,255,18,1,1,1,255,77,0,0,1,255,96,0,0,1,255,83,0,0,1,255,73,0,0,1,255,69,0,0,1,255,61,0,0,1,255,59,0,0,2,255,66,0,0,1,255,75,0,0,1,255,89,0,0,1,255,91,0,0,1,255,89,0,0,1,255,90,0,0,1,255,91,0,0,1,215,80,0,0,1,237,82,0,0,1,255,79,0,0,1,255,80,0,0,1,255,66,0,0,1,255,53,0,0,1,255,59,0,0,1,255,49,0,0,1,255,16,0,0,1,255,1,1,1,1,255,4,1,1,1,255,21,14,14,1,255,40,7,7,1,255,33,0,0,1,255,18,8,8,1,255,36,70,86,1,255,65,82,100,1,255,84,87,96,1,255,83,76,81,1,255,77,27,27,1,255,79,0,0,1,255,72,0,0,1,255,58,54,64,1,255,59,46,52,1,255,59,0,0,1,255,67,0,0,1,255,77,0,0,1,255,87,0,0,1,255,90,0,0,1,255,92,0,0,1,255,93,0,0,1,255,92,0,0,1,237,84,0,0,1,237,84,0,0,1,255,79,0,0,1,255,83,0,0,1,255,67,0,0,1,255,62,0,0,1,255,68,0,0,1,255,40,0,0,1,255,7,0,0,1,255,48,102,102,1,255,144,198,199,1,255,218,238,238,1,255,195,220,230,1,255,60,166,177,1,255,113,213,222,1,255,55,187,244,1,255,57,183,231,1,255,106,229,244,1,255,78,192,211,
1,255,32,120,162,1,255,43,91,121,1,255,51,72,98,1,255,66,126,158,1,255,65,89,98,1,255,64,0,0,1,255,57,0,0,1,255,55,0,0,1,255,58,0,0,1,255,70,0,0,1,255,86,0,0,1,255,94,0,0,1,255,97,0,0,1,237,86,0,0,1,237,62,0,0,1,255,34,0,0,1,255,36,0,0,1,255,45,0,0,1,255,65,0,0,1,255,70,0,0,1,255,39,0,0,1,255,41,103,106,1,255,89,175,180,1,255,173,220,220,1,255,239,253,253,1,255,215,253,254,1,255,116,252,255,1,255,209,255,255,1,255,89,223,255,1,255,43,187,249,1,255,127,247,254,1,255,144,240,244,1,255,69,198,218,1,255,73,166,193,1,255,49,159,187,1,255,97,181,203,1,255,72,99,104,1,255,50,0,0,1,255,41,0,0,1,255,34,0,0,1,255,33,0,0,1,255,44,0,0,1,255,61,0,0,1,255,79,0,0,1,255,92,0,0,1,237,67,0,0,1,237,33,0,0,1,255,18,0,0,1,255,20,0,0,1,255,23,0,0,1,255,38,0,0,1,255,52,0,0,1,255,61,0,0,1,255,65,5,5,1,255,61,0,0,1,255,73,125,125,1,255,188,252,252,1,255,179,252,255,1,255,139,254,255,1,255,198,254,255,1,255,90,236,245,1,255,72,92,109,1,255,75,75,78,1,255,83,112,112,1,255,65,187,190,1,255,138,240,243,1,
255,116,238,243,1,255,104,231,235,1,255,86,110,111,1,255,63,0,0,1,255,57,0,0,1,255,52,0,0,1,255,50,0,0,1,255,53,0,0,1,255,58,0,0,1,255,63,0,0,1,255,76,0,0,1,237,59,0,0,1,237,31,0,0,1,255,16,0,0,1,255,14,0,0,2,255,23,0,0,1,255,64,0,0,1,255,89,0,0,1,255,87,0,0,1,255,77,0,0,1,255,77,132,132,1,255,192,251,253,1,255,96,249,255,1,255,114,255,255,1,255,123,248,255,1,255,63,226,232,1,255,81,2,2,1,255,87,0,0,1,255,89,0,0,1,255,81,14,14,1,255,94,221,227,1,255,155,246,247,1,255,141,253,255,1,255,89,132,132,1,255,81,0,0,1,255,76,0,0,1,255,72,0,0,1,255,68,0,0,1,255,69,0,0,1,255,70,0,0,1,255,67,0,0,1,255,69,0,0,1,237,37,0,0,1,237,32,0,0,1,255,29,0,0,1,255,32,0,0,1,255,30,0,0,1,255,53,0,0,1,255,84,0,0,1,255,93,0,0,1,255,94,0,0,1,255,87,0,0,1,255,71,123,129,1,255,66,224,253,1,255,92,254,255,1,255,95,232,255,1,255,43,241,255,1,255,98,208,211,1,255,81,0,0,1,255,87,0,0,1,255,95,0,0,1,255,91,5,5,1,255,119,228,231,1,255,165,251,255,1,255,164,252,255,1,255,108,141,141,1,255,92,0,0,1,255,88,0,0,1,255,
84,0,0,1,255,82,0,0,1,255,84,0,0,2,255,75,0,0,1,255,71,0,0,1,237,21,0,0,1,237,27,0,0,1,255,39,0,0,1,255,47,0,0,1,255,41,0,0,1,255,68,0,0,1,255,94,0,0,1,255,98,0,0,1,255,99,0,0,1,255,98,0,0,1,255,74,125,137,1,255,40,226,254,1,255,49,222,255,1,255,41,226,255,1,255,54,239,255,1,255,64,209,217,1,255,82,1,1,1,255,87,32,32,1,255,88,51,52,1,255,88,13,13,1,255,53,198,226,1,255,108,224,239,1,255,99,215,229,1,255,78,102,106,1,255,88,0,0,1,255,89,0,0,1,255,94,0,0,2,255,89,0,0,1,255,76,0,0,1,255,77,0,0,1,255,73,0,0,1,237,5,0,0,1,237,15,0,0,1,255,37,0,0,2,255,47,0,0,1,255,79,0,0,1,255,98,0,0,1,255,99,0,0,3,255,78,128,129,1,255,26,228,248,1,255,34,237,245,1,255,8,201,241,1,255,18,195,222,1,255,30,168,191,1,255,85,1,1,1,255,75,75,76,1,255,34,160,186,1,255,44,94,133,1,255,73,48,48,1,255,72,48,48,1,255,76,39,39,1,255,79,0,0,1,255,78,0,0,1,255,80,0,0,1,255,87,0,0,2,255,82,0,0,1,255,76,0,0,1,255,84,0,0,1,255,92,0,0,1,237,1,1,1,1,237,6,1,1,1,255,24,0,0,1,255,32,0,0,1,255,61,0,0,1,255,93,0,0,1,255,
99,0,0,4,255,69,111,127,1,255,3,162,202,1,255,2,150,190,1,255,7,145,176,1,255,42,117,136,1,255,27,90,130,1,255,64,61,80,1,255,60,48,80,1,255,8,75,127,1,255,18,97,135,1,255,81,14,14,1,255,85,0,0,1,255,82,0,0,1,255,74,0,0,1,255,72,0,0,1,255,74,0,0,1,255,78,0,0,1,255,87,0,0,1,255,84,0,0,2,255,90,0,0,1,255,94,0,0,1,237,2,1,1,1,237,1,1,1,1,255,12,0,0,1,255,48,0,0,1,255,84,0,0,1,255,97,0,0,1,255,98,0,0,2,255,99,0,0,1,255,88,41,49,1,255,30,76,131,1,255,5,92,143,1,255,2,91,135,1,255,2,74,126,1,255,0,69,122,1,255,1,67,119,1,255,0,61,115,1,255,0,53,113,1,255,2,56,112,1,255,41,54,93,1,255,87,0,0,1,255,85,0,0,1,255,88,0,0,1,255,83,0,0,1,255,78,0,0,1,255,73,0,0,1,255,72,0,0,1,255,91,0,0,1,255,94,0,0,1,255,89,0,0,1,255,93,0,0,1,255,94,0,0,1,237,19,0,0,1,237,11,1,1,1,255,42,0,0,1,255,57,0,0,1,255,79,0,0,1,255,94,0,0,1,255,98,0,0,1,255,99,0,0,2,255,80,61,61,1,255,23,108,137,1,255,11,98,134,1,255,0,51,113,1,255,0,49,112,1,255,0,54,112,1,255,0,56,112,1,255,0,55,112,1,255,0,53,111,1,255,4,62,116,
1,255,17,71,113,1,255,87,13,13,1,255,88,0,0,1,255,86,0,0,1,255,85,0,0,1,255,79,0,0,1,255,76,0,0,1,255,74,0,0,1,255,90,0,0,1,255,99,0,0,1,255,96,0,0,1,255,98,0,0,1,255,97,0,0,1,237,43,0,0,1,237,38,0,0,1,255,42,0,0,1,255,52,0,0,1,255,74,0,0,1,255,93,0,0,1,255,97,0,0,1,255,98,0,0,1,255,95,0,0,1,255,83,0,0,1,255,49,77,90,1,255,3,56,114,1,255,11,64,118,1,255,6,69,122,1,255,2,76,126,1,255,22,84,122,1,255,70,42,44,1,255,71,57,61,1,255,18,97,146,1,255,54,71,92,1,255,84,0,0,2,255,78,0,0,1,255,84,0,0,1,255,79,0,0,1,255,72,0,0,1,255,77,0,0,1,255,91,0,0,1,255,99,0,0,2,255,98,0,0,1,255,97,0,0,1,237,63,0,0,1,237,58,0,0,1,255,53,0,0,1,255,66,0,0,1,255,87,0,0,1,255,97,0,0,1,255,98,0,0,1,255,95,0,0,1,255,85,0,0,1,255,64,0,0,1,255,35,78,92,1,255,33,102,142,1,255,65,117,153,1,255,31,132,165,1,255,29,147,176,1,255,57,141,163,1,255,58,1,1,1,255,48,52,61,1,255,51,40,57,1,255,59,0,0,1,255,77,0,0,1,255,78,0,0,1,255,85,0,0,1,255,84,0,0,1,255,78,0,0,1,255,74,0,0,1,255,82,0,0,1,255,94,0,0,1,255,98,0,0,
1,255,94,0,0,1,255,89,0,0,1,255,88,0,0,1,237,37,0,0,1,237,63,0,0,1,255,71,0,0,1,255,80,0,0,1,255,93,0,0,1,255,95,0,0,1,255,90,0,0,1,255,79,0,0,1,255,68,0,0,1,255,50,0,0,1,255,27,106,114,1,255,33,166,205,1,255,107,206,221,1,255,65,223,231,1,255,55,234,240,1,255,63,210,215,1,255,50,2,2,1,255,47,0,0,1,255,45,0,0,1,255,46,3,3,1,255,87,156,156,1,255,85,141,163,1,255,61,101,165,1,255,76,43,65,1,255,78,0,0,1,255,72,0,0,1,255,84,0,0,1,255,97,0,0,1,255,95,0,0,1,255,89,0,0,1,255,82,0,0,1,255,77,0,0,1,237,23,0,0,1,237,38,0,0,1,255,79,0,0,1,255,89,0,0,1,255,88,0,0,2,255,69,0,0,1,255,60,0,0,1,255,50,0,0,1,255,44,0,0,1,255,39,130,131,1,255,161,240,250,1,255,209,248,254,1,255,122,250,255,1,255,130,248,255,1,255,142,215,215,1,255,62,0,0,1,255,51,0,0,1,255,49,0,0,1,255,45,5,5,1,255,134,233,234,1,255,165,249,255,1,255,125,233,253,1,255,81,102,114,1,255,83,0,0,1,255,78,0,0,1,255,90,0,0,1,255,95,0,0,1,255,87,0,0,1,255,78,0,0,1,255,68,0,0,1,255,62,0,0,1,237,15,0,0,1,237,47,0,0,1,255,79,0,0,1,255,
81,0,0,1,255,88,0,0,1,255,75,0,0,1,255,54,0,0,1,255,47,0,0,2,255,43,0,0,1,255,63,128,128,1,255,230,252,252,1,255,221,255,255,1,255,162,255,255,1,255,170,255,255,1,255,162,215,215,1,255,88,0,0,1,255,63,0,0,1,255,42,0,0,1,255,38,9,9,1,255,184,237,237,1,255,193,254,255,1,255,172,254,255,1,255,83,107,107,1,255,87,0,0,1,255,84,0,0,1,255,89,0,0,1,255,82,0,0,1,255,58,0,0,1,255,36,0,0,1,255,25,0,0,1,255,28,0,0,1,237,11,0,0,1,237,33,0,0,1,255,51,0,0,1,255,58,0,0,1,255,70,0,0,1,255,46,0,0,1,255,28,0,0,1,255,16,0,0,1,255,17,0,0,1,255,49,0,0,1,255,67,112,112,1,255,237,252,252,1,255,220,255,255,1,255,191,255,255,1,255,224,255,255,1,255,216,225,225,1,255,53,1,1,1,255,26,0,0,1,255,10,0,0,1,255,13,24,24,1,255,174,238,238,1,255,163,229,229,1,255,182,244,244,1,255,97,113,113,1,255,89,0,0,1,255,84,0,0,1,255,77,0,0,1,255,56,0,0,1,255,33,0,0,1,255,19,0,0,1,255,14,0,0,1,255,20,0,0,1,237,1,1,1,1,237,7,0,0,1,255,18,0,0,1,255,35,0,0,1,255,33,0,0,1,255,8,1,1,1,255,2,1,1,1,255,1,1,1,1,255,1,3,3,1,255,24,
9,9,1,255,85,108,108,1,255,168,237,252,1,255,175,231,255,1,255,158,237,255,1,255,226,254,254,1,255,219,244,245,1,255,81,137,141,1,255,29,76,91,1,255,88,119,125,1,255,58,167,175,1,255,120,213,224,1,255,154,230,234,1,255,137,176,177,1,255,77,7,7,1,255,87,0,0,1,255,77,0,0,1,255,66,0,0,1,255,56,0,0,1,255,46,0,0,1,255,36,0,0,1,255,28,0,0,1,255,21,0,0,1,237,1,1,1,1,237,1,1,1,1,255,7,0,0,1,255,19,0,0,1,255,3,1,1,1,255,1,1,1,3,255,13,76,76,1,255,112,187,196,1,255,137,203,217,1,255,49,185,254,1,255,28,151,251,1,255,45,158,244,1,255,115,211,229,1,255,68,182,209,1,255,110,173,194,1,255,52,135,176,1,255,81,145,173,1,255,45,142,175,1,255,42,145,176,1,255,59,112,120,1,255,37,2,2,1,255,72,0,0,1,255,81,0,0,1,255,71,0,0,1,255,55,0,0,1,255,38,0,0,1,255,37,0,0,1,255,35,0,0,1,255,37,0,0,1,255,32,0,0,1,237,1,1,1,1,237,8,0,0,1,255,21,0,0,1,255,22,0,0,1,255,5,0,0,1,255,2,1,1,1,255,1,1,1,3,255,4,39,39,1,255,74,141,148,1,255,56,146,192,1,255,70,109,164,1,255,64,104,136,1,255,20,111,164,1,255,4,92,144,
1,255,12,105,136,1,255,5,54,118,1,255,14,72,113,1,255,11,71,99,1,255,13,46,54,1,255,23,0,0,1,255,33,0,0,1,255,54,0,0,1,255,58,0,0,1,255,55,0,0,1,255,28,0,0,1,255,7,0,0,1,255,29,0,0,1,255,45,0,0,1,255,30,0,0,1,255,16,0,0,1,237,4,0,0,1,215,10,0,0,1,255,6,0,0,1,255,9,0,0,1,255,2,1,1,1,255,1,1,1,4,255,22,1,1,1,255,74,0,0,1,255,97,0,0,1,255,98,0,0,2,255,86,3,3,1,255,44,38,38,1,255,5,52,57,1,255,4,48,52,1,255,1,7,7,1,255,1,1,1,3,255,8,0,0,1,255,39,0,0,1,255,47,0,0,1,255,31,0,0,1,255,3,1,1,1,255,1,1,1,1,255,15,0,0,1,255,66,0,0,1,255,80,0,0,1,255,67,0,0,1,215,3,1,1,1,160,17,0,0,1,255,18,0,0,1,255,14,0,0,1,255,1,1,1,4,255,15,1,1,1,255,72,0,0,1,255,90,0,0,1,255,97,0,0,1,255,98,0,0,3,255,88,0,0,1,255,59,0,0,1,255,32,0,0,1,255,16,1,1,1,255,10,1,1,1,255,20,1,1,1,255,29,0,0,1,255,14,0,0,2,255,24,0,0,1,255,15,0,0,1,255,1,1,1,1,255,2,1,1,1,255,1,1,1,1,255,41,0,0,1,255,86,0,0,1,255,88,0,0,1,160,1,1,1,1,54,31,0,0,1,255,53,0,0,1,255,18,0,0,1,255,1,1,1,1,255,2,1,1,1,255,7,1,1,1,255,30,0,0,1,255,
67,0,0,1,255,87,0,0,1,255,84,0,0,1,255,89,0,0,1,255,93,0,0,1,255,91,0,0,1,255,94,0,0,2,255,93,0,0,1,255,89,0,0,1,255,87,0,0,1,255,86,0,0,1,255,88,0,0,1,255,62,0,0,1,255,2,1,1,1,255,1,1,1,2,255,2,1,1,1,255,15,0,0,1,255,25,0,0,1,255,14,0,0,1,255,23,0,0,1,255,81,0,0,1,255,97,0,0,1,54,0,0,0,1,0,67,0,0,1,144,46,0,0,1,255,1,1,1,1,255,28,0,0,1,255,73,0,0,1,255,84,0,0,1,255,89,0,0,1,255,90,0,0,1,255,88,0,0,1,255,89,0,0,1,255,79,0,0,1,255,72,0,0,1,255,75,0,0,1,255,84,0,0,1,255,91,0,0,1,255,93,0,0,1,255,91,0,0,1,255,98,0,0,1,255,99,0,0,2,255,90,0,0,1,255,48,0,0,1,255,20,1,1,1,255,18,1,1,1,255,31,0,0,1,255,52,0,0,1,255,50,0,0,1,255,36,0,0,1,255,27,0,0,1,255,60,0,0,1,143,0,0,0,2,0,76,7,7,1,2,23,1,1,1,143,4,1,1,1,255,43,0,0,1,255,90,0,0,1,255,98,0,0,1,255,95,0,0,1,255,94,0,0,1,255,97,0,0,1,255,87,0,0,1,255,66,0,0,1,255,53,0,0,1,255,60,0,0,1,255,73,0,0,1,255,83,0,0,1,255,82,0,0,1,255,80,0,0,1,255,98,0,0,1,255,99,0,0,3,255,94,0,0,1,255,81,0,0,1,255,70,0,0,1,255,66,0,0,1,255,64,0,0,1,255,61,
0,0,1,255,59,0,0,1,255,50,0,0,1,143,42,7,7,1,2,0,0,0,4,0,20,0,0,1,54,38,0,0,1,160,78,0,0,1,215,95,0,0,1,236,93,0,0,1,236,96,0,0,2,236,83,0,0,1,236,56,0,0,1,236,47,0,0,1,236,61,0,0,1,236,75,0,0,1,236,76,0,0,1,236,50,0,0,1,236,75,0,0,1,236,98,0,0,3,236,99,0,0,2,236,97,0,0,1,236,89,0,0,1,236,82,0,0,1,236,73,0,0,1,215,61,0,0,1,160,65,0,0,1,54,0,0,0,3,0);


//app small icon (optional) - 20w x 20h size in TEA format only, accessed via tepIcon20
program_icon20h
:array[0..1981] of byte=(
84,69,65,51,35,1,0,0,0,0,0,0,0,0,0,0,0,0,0,20,0,0,0,20,0,0,0,0,0,0,1,0,97,0,0,1,31,89,0,0,1,163,66,0,0,1,236,33,0,0,1,254,14,0,0,1,254,7,0,0,1,254,46,0,0,1,254,63,0,0,1,254,44,0,0,1,254,46,0,0,1,254,73,0,0,1,254,91,0,0,1,254,96,0,0,2,254,92,0,0,1,254,82,0,0,1,236,82,0,0,1,163,69,0,0,1,31,0,0,0,1,0,90,0,0,1,31,90,0,0,1,232,80,0,0,1,255,51,0,0,1,255,40,0,0,1,255,28,0,0,1,255,13,0,0,1,255,4,1,1,1,255,14,0,0,1,255,4,1,1,1,255,1,1,1,1,255,49,0,0,1,255,95,0,0,1,255,91,0,0,1,255,85,0,0,1,255,83,0,0,1,255,86,0,0,1,255,90,0,0,1,255,80,0,0,1,232,75,0,0,1,31,75,0,0,1,163,85,0,0,1,255,79,0,0,1,255,50,0,0,1,255,45,0,0,1,255,31,0,0,1,255,3,1,1,1,255,1,1,1,1,255,13,0,0,1,255,16,0,0,1,255,2,1,1,1,255,71,0,0,1,255,87,0,0,1,255,69,0,0,1,255,66,0,0,1,255,67,0,0,1,255,75,0,0,1,255,90,0,0,1,255,92,0,0,1,255,87,0,0,1,163,79,0,0,1,236,79,0,0,1,255,71,0,0,1,255,53,0,0,1,255,54,0,0,1,255,17,0,0,1,255,1,1,1,1,255,22,0,0,1,255,36,0,0,1,255,20,0,0,1,255,50,0,0,1,255,94,0,0,1,255,78,0,0,1,255,68,0,0,1,255,
59,0,0,1,255,61,0,0,1,255,75,0,0,1,255,91,0,0,1,255,90,0,0,1,255,91,0,0,1,236,83,0,0,1,254,81,0,0,1,255,76,0,0,1,255,61,0,0,1,255,53,0,0,1,255,10,22,22,1,255,111,155,155,1,255,185,193,195,1,255,60,143,152,1,255,82,169,201,1,255,84,178,204,1,255,79,169,185,1,255,53,80,114,1,255,61,69,90,1,255,62,95,114,1,255,62,0,0,1,255,64,0,0,1,255,73,0,0,1,255,89,0,0,1,255,95,0,0,1,254,74,0,0,1,254,32,0,0,1,255,35,0,0,1,255,59,0,0,1,255,53,0,0,1,255,57,108,112,1,255,152,190,190,1,255,231,254,254,1,255,147,253,255,1,255,156,239,252,1,255,71,182,214,1,255,111,207,212,1,255,86,205,219,1,255,82,191,208,1,255,86,145,154,1,255,51,0,0,1,255,40,0,0,1,255,43,0,0,1,255,61,0,0,1,255,84,0,0,1,254,51,0,0,1,254,18,0,0,1,255,15,0,0,1,255,33,0,0,1,255,80,0,0,1,255,82,0,0,1,255,88,134,134,1,255,154,248,255,1,255,134,253,255,1,255,87,235,241,1,255,83,1,1,1,255,88,0,0,1,255,95,171,173,1,255,139,247,248,1,255,114,188,189,1,255,78,0,0,1,255,70,0,0,1,255,66,0,0,1,255,68,0,0,1,255,69,0,0,1,254,30,0,0,1,254,36,0,0,
1,255,39,0,0,1,255,67,0,0,1,255,94,0,0,1,255,96,0,0,1,255,76,125,139,1,255,47,234,255,1,255,76,235,255,1,255,81,222,230,1,255,83,0,0,1,255,93,1,1,1,255,88,152,167,1,255,130,237,250,1,255,120,181,185,1,255,90,0,0,2,255,89,0,0,1,255,82,0,0,1,255,71,0,0,1,254,7,0,0,1,254,32,0,0,1,255,41,0,0,1,255,83,0,0,1,255,99,0,0,2,255,79,131,134,1,255,27,218,241,1,255,8,198,232,1,255,39,166,187,1,255,85,2,2,1,255,46,123,149,1,255,62,79,104,1,255,72,78,89,1,255,76,36,39,1,255,78,0,0,1,255,85,0,0,2,255,79,0,0,1,255,90,0,0,1,254,1,1,1,1,254,15,0,0,1,255,62,0,0,1,255,96,0,0,1,255,98,0,0,1,255,98,1,1,1,255,56,80,123,1,255,2,109,155,1,255,8,110,146,1,255,13,80,123,1,255,23,67,112,1,255,10,71,119,1,255,67,61,82,1,255,86,0,0,1,255,82,0,0,1,255,75,0,0,1,255,74,0,0,1,255,91,0,0,1,255,88,0,0,1,255,93,0,0,1,254,25,0,0,1,254,41,0,0,1,255,65,0,0,1,255,94,0,0,1,255,98,0,0,1,255,94,13,13,1,255,48,90,111,1,255,2,58,114,1,255,1,54,114,1,255,4,63,115,1,255,43,53,93,1,255,21,76,120,1,255,66,54,78,1,255,86,0,0,1,
255,83,0,0,1,255,78,0,0,1,255,75,0,0,1,255,96,0,0,1,255,98,0,0,1,255,98,0,0,1,254,58,0,0,1,254,52,0,0,1,255,75,0,0,1,255,96,0,0,2,255,81,0,0,1,255,42,84,96,1,255,50,109,147,1,255,29,133,164,1,255,48,146,168,1,255,61,7,7,1,255,46,59,85,1,255,70,3,3,1,255,80,0,0,1,255,84,0,0,1,255,76,0,0,1,255,81,0,0,1,255,97,0,0,1,255,94,0,0,1,255,89,0,0,1,254,36,0,0,1,254,73,0,0,1,255,88,0,0,1,255,91,0,0,1,255,73,0,0,1,255,55,0,0,1,255,38,121,127,1,255,139,219,237,1,255,104,239,246,1,255,107,228,231,1,255,52,0,0,1,255,47,0,0,1,255,77,148,149,1,255,134,209,223,1,255,75,121,155,1,255,78,0,0,1,255,86,0,0,1,255,94,0,0,1,255,84,0,0,1,255,73,0,0,1,254,23,0,0,1,254,72,0,0,1,255,82,0,0,1,255,70,0,0,1,255,46,0,0,1,255,43,0,0,1,255,87,133,133,1,255,241,254,254,1,255,159,255,255,1,255,173,232,232,1,255,77,0,0,1,255,41,0,0,1,255,140,171,171,1,255,172,251,252,1,255,133,181,181,1,255,86,0,0,2,255,68,0,0,1,255,37,0,0,1,255,28,0,0,1,254,6,0,0,1,254,28,0,0,1,255,47,0,0,1,255,24,0,0,1,255,5,1,1,1,255,11,3,3,1,
255,102,125,125,1,255,195,241,254,1,255,201,245,255,1,255,226,244,244,1,255,51,96,99,1,255,60,92,99,1,255,90,186,194,1,255,168,229,231,1,255,114,134,134,1,255,86,0,0,1,255,69,0,0,1,255,49,0,0,1,255,30,0,0,1,255,21,0,0,1,254,1,1,1,1,254,13,0,0,2,255,1,1,1,2,255,51,105,108,1,255,111,188,204,1,255,40,170,246,1,255,70,154,221,1,255,66,166,197,1,255,76,143,172,1,255,58,122,156,1,255,27,123,154,1,255,47,83,91,1,255,58,0,0,1,255,73,0,0,1,255,51,0,0,1,255,33,0,0,1,255,35,0,0,1,255,27,0,0,1,254,7,0,0,1,236,11,0,0,1,255,7,1,1,1,255,1,1,1,2,255,4,1,1,1,255,67,1,1,1,255,93,10,10,1,255,93,22,22,1,255,57,57,75,1,255,5,50,74,1,255,2,34,50,1,255,1,4,4,1,255,6,1,1,1,255,34,0,0,1,255,45,0,0,1,255,12,1,1,1,255,7,1,1,1,255,64,0,0,1,255,65,0,0,1,236,8,1,1,1,164,37,0,0,1,255,7,1,1,1,255,1,1,1,1,255,6,1,1,1,255,60,0,0,1,255,87,0,0,1,255,95,0,0,1,255,96,0,0,1,255,94,0,0,1,255,77,0,0,1,255,62,0,0,1,255,61,0,0,1,255,45,0,0,1,255,6,1,1,1,255,11,1,1,1,255,6,1,1,1,255,10,0,0,1,255,45,0,0,1,255,91,0,0,1,164,
37,1,1,1,31,49,0,0,1,231,14,1,1,1,255,73,0,0,1,255,87,0,0,1,255,90,0,0,1,255,88,0,0,1,255,74,0,0,1,255,75,0,0,1,255,89,0,0,1,255,91,0,0,1,255,97,0,0,1,255,99,0,0,1,255,86,0,0,1,255,44,0,0,1,255,35,0,0,1,255,51,0,0,1,255,46,0,0,1,255,36,0,0,1,231,83,0,0,1,31,0,0,0,1,0,2,1,1,1,31,28,0,0,1,164,84,0,0,1,236,95,0,0,1,255,96,0,0,1,255,85,0,0,1,255,52,0,0,1,255,61,0,0,1,255,77,0,0,1,255,64,0,0,1,255,95,0,0,1,255,98,0,0,1,255,99,0,0,1,255,95,0,0,1,255,82,0,0,1,255,71,0,0,1,236,63,0,0,1,164,60,0,0,1,31,0,0,0,1,0);
//:array[0..0] of byte=(0);


//app splash/about screen image (optional) - recommended dimensions of 208w x 208h for best scaling, supports image formats of TEA and PNG
program_splash_logo_208px
:array[0..37635] of byte=(
137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,208,0,0,0,208,8,6,0,0,0,118,164,239,116,0,0,32,0,73,68,65,84,120,218,220,189,89,172,37,89,118,158,247,173,29,17,39,206,112,167,188,115,230,205,169,178,178,230,161,171,154,236,170,174,110,182,56,88,32,219,146,73,147,2,12,209,0,37,18,148,253,102,62,232,69,15,6,228,7,194,240,43,65,251,65,246,131,108,64,176,0,90,48,44,11,164,68,201,38,169,102,147,236,102,119,245,84,93,213,213,53,100,101,85,86,142,119,30,207,24,17,123,249,33,226,100,69,70,198,142,136,91,93,213,45,249,0,7,247,230,205,51,196,176,255,189,214,250,215,191,214,18,254,3,120,252,125,223,55,36,73,199,170,46,39,112,62,130,179,17,172,141,193,159,0,99,96,2,196,217,51,41,121,218,138,191,89,64,115,223,39,217,211,228,126,151,146,227,154,190,198,0,62,208,2,58,64,23,152,201,158,29,32,4,2,192,203,125,142,205,142,53,202,142,125,148,61,167,231,146,100,175,13,178,247,183,179,159,126,225,115,154,60,164,112,172,197,115,35,119,76,154,253,180,142,107,100,178,239,247,115,199,50,253,220,143,
251,208,220,245,151,220,79,205,253,127,241,181,249,99,77,32,142,96,51,130,187,19,184,21,139,236,168,231,13,255,125,28,219,159,244,218,245,127,18,95,250,95,250,190,145,56,94,50,240,25,224,11,26,199,159,83,120,34,129,213,24,58,49,248,17,200,116,1,198,133,103,25,104,172,227,103,82,184,129,249,69,103,115,11,67,28,139,50,255,26,83,248,254,56,119,163,181,176,32,242,139,116,122,30,81,110,35,152,126,110,241,117,154,3,23,37,11,172,248,183,60,88,188,220,130,215,2,136,138,139,181,108,67,169,218,84,52,247,239,226,123,181,226,56,93,199,173,21,32,43,94,211,236,239,170,233,37,26,170,234,150,141,227,183,127,14,94,245,224,107,62,188,230,249,254,238,191,249,9,0,74,126,92,95,244,235,65,96,108,20,157,5,126,65,224,63,55,240,121,3,171,25,136,165,184,99,231,45,207,164,0,164,50,176,216,18,64,197,133,157,76,11,139,67,28,139,162,108,87,159,90,160,54,208,203,89,160,94,102,133,130,233,137,228,64,16,103,231,48,204,61,167,0,154,2,50,200,62,55,40,177,62,90,177,240,242,11,186,104,53,138,214,67,10,139,84,11,
32,47,90,136,60,16,189,79,113,145,148,157,83,17,72,182,196,154,71,217,223,4,212,131,216,131,45,15,254,218,192,191,2,254,76,130,224,238,31,69,145,253,255,5,128,126,67,36,84,213,87,18,248,251,22,190,172,176,106,192,147,130,187,51,93,112,117,224,113,1,71,29,214,199,150,88,33,215,14,171,13,0,20,22,0,52,117,227,90,37,0,138,50,183,173,8,160,36,251,14,175,196,93,202,91,12,117,88,183,178,5,239,103,32,156,2,178,149,115,43,203,64,105,75,62,151,220,249,26,135,27,248,73,1,167,184,97,217,146,227,163,176,49,70,185,103,82,2,118,3,137,192,22,240,111,21,254,153,21,249,250,31,171,142,255,163,4,208,111,137,116,84,245,63,181,240,59,22,62,111,161,109,115,23,38,255,197,137,195,250,68,37,224,201,251,235,73,97,33,148,89,162,164,0,50,151,229,177,0,146,59,42,213,7,0,228,229,226,149,110,6,162,217,66,28,228,229,22,255,244,124,242,22,104,148,187,249,154,91,0,166,6,60,214,241,51,239,194,5,57,11,217,206,197,85,65,69,92,165,53,241,212,167,1,30,91,1,38,45,121,77,254,190,70,185,235,106,11,128,47,177,182,35,11,
127,157,192,255,148,136,252,241,255,171,58,252,143,2,64,191,109,76,128,181,191,96,225,191,37,5,78,171,24,180,22,47,86,82,0,207,196,1,158,50,139,83,230,47,39,14,0,21,23,9,133,207,120,0,64,25,136,76,193,69,42,115,227,242,22,40,127,78,211,141,96,74,30,140,11,0,42,46,216,178,64,218,58,130,254,252,194,155,30,91,59,71,114,76,159,237,220,177,121,133,32,222,181,16,76,197,194,208,194,70,244,113,1,164,21,113,80,217,223,243,113,103,92,216,16,189,130,197,228,193,123,59,137,225,175,99,248,31,34,99,254,236,207,173,141,254,131,4,208,127,181,176,32,246,224,224,9,224,191,3,126,85,161,163,142,5,80,4,64,209,250,228,193,147,56,222,87,182,67,107,193,149,75,10,68,2,14,183,109,250,250,42,11,84,180,66,157,12,68,189,220,34,245,114,22,53,41,108,8,227,220,57,89,199,78,76,137,239,111,11,139,231,190,213,45,128,221,83,189,111,29,167,192,158,205,254,221,169,0,145,148,176,99,77,45,200,105,153,57,173,1,73,153,133,162,176,41,37,37,172,161,20,126,22,173,87,182,65,15,39,240,127,199,240,187,118,97,225,237,191,60,
56,208,79,98,221,123,159,136,213,17,105,235,104,244,95,3,255,20,120,37,91,103,149,212,100,153,111,27,23,118,24,235,184,224,166,100,17,72,5,37,45,37,166,222,84,80,215,226,136,3,76,201,255,81,160,173,39,57,218,122,156,139,229,162,18,2,164,140,244,136,11,190,254,253,88,80,132,88,4,43,242,16,120,0,172,8,145,200,125,38,47,127,190,94,201,185,136,99,241,73,197,181,209,6,187,175,45,16,28,167,1,79,21,147,87,70,0,153,138,251,106,30,166,246,3,224,89,133,191,147,140,70,131,179,34,111,220,254,136,207,249,201,1,232,183,225,34,240,79,128,127,8,44,0,82,22,244,218,130,41,182,5,147,28,87,184,92,218,0,44,82,113,243,197,225,43,75,201,98,144,138,64,218,5,210,164,4,60,69,34,36,46,161,213,203,172,101,17,68,113,6,142,166,143,68,132,196,65,80,184,54,5,106,242,97,85,185,167,211,208,212,52,160,188,235,0,84,181,33,154,138,92,88,246,187,0,179,10,191,104,225,169,117,248,198,29,56,252,137,0,232,31,204,204,200,139,147,201,207,0,127,0,252,172,102,159,165,37,187,145,107,161,36,142,24,71,27,248,156,101,129,110,
153,27,82,204,147,148,89,31,173,72,70,26,7,80,245,97,23,225,65,139,81,96,140,18,135,171,233,114,211,238,3,71,78,239,101,91,145,251,196,66,144,35,18,124,199,166,112,26,119,76,26,196,72,77,226,159,211,130,167,46,103,85,181,225,149,188,215,83,120,90,225,151,214,224,251,107,189,222,205,123,81,244,227,3,208,111,251,190,199,104,244,235,192,255,166,112,89,115,199,169,142,103,49,97,216,4,60,82,178,243,80,97,61,234,110,186,41,201,139,84,185,111,226,88,88,197,243,42,186,93,101,57,171,170,107,242,80,204,246,49,129,67,193,205,109,229,168,247,32,71,107,27,135,5,111,226,178,213,29,149,214,208,214,234,32,17,154,110,154,226,112,229,132,122,85,137,60,252,167,21,11,191,98,163,232,214,138,239,191,185,101,173,126,234,0,250,45,207,243,53,73,254,27,224,247,20,206,184,24,148,226,133,42,179,62,137,3,60,82,97,77,234,226,157,42,106,214,37,109,193,225,91,187,100,39,69,235,147,119,187,92,27,130,43,151,83,100,152,84,62,25,94,199,240,160,68,168,149,35,58,60,199,57,54,181,36,77,192,163,14,144,232,199,252,108,
105,112,255,77,205,154,208,242,115,237,42,252,146,90,123,178,226,121,223,218,82,213,79,13,64,191,229,121,62,73,242,143,128,223,5,122,90,3,158,50,70,41,46,97,148,236,41,119,187,211,238,154,101,150,71,75,114,82,101,110,155,75,98,226,210,147,57,100,40,165,185,14,251,41,129,103,186,152,58,185,103,152,115,229,202,72,144,170,164,114,83,118,13,71,30,199,181,70,92,196,194,199,189,10,210,0,212,14,171,214,82,248,89,84,237,138,231,125,237,52,32,106,12,160,223,244,125,143,212,242,252,46,25,69,77,205,34,169,115,225,146,18,26,186,9,136,154,6,176,85,138,131,178,140,190,84,176,71,150,135,19,184,182,4,52,226,200,147,184,68,157,247,175,131,124,178,41,57,147,203,9,117,114,84,123,80,17,7,158,198,242,212,221,115,106,210,12,90,179,241,186,44,139,58,200,165,50,111,69,79,65,68,25,8,20,190,128,234,241,138,239,127,107,179,161,59,215,8,64,191,217,235,137,142,199,191,14,252,94,209,242,84,249,181,80,157,224,44,75,172,154,146,147,175,218,113,92,49,151,212,0,168,184,232,171,118,214,135,192,50,141,81,114,207,
166,76,97,153,68,37,145,79,94,16,226,2,144,207,233,85,6,218,192,170,86,197,119,101,155,164,43,6,114,129,72,106,172,92,157,87,34,13,65,4,252,140,88,251,254,122,175,247,198,221,6,196,66,35,53,182,237,247,127,6,248,125,77,243,115,78,127,183,42,233,86,101,178,181,230,130,73,205,14,196,41,192,70,13,81,81,166,168,118,38,91,31,248,2,73,245,194,13,220,156,226,34,251,52,30,117,214,85,62,198,231,212,129,134,6,32,209,138,69,93,182,126,170,132,181,182,176,49,25,199,57,154,220,103,169,131,197,205,89,229,25,129,223,143,251,253,15,129,191,252,145,45,208,111,164,121,158,63,0,46,53,181,56,46,18,193,214,144,7,52,176,6,90,195,246,105,201,69,162,0,10,45,113,185,40,137,117,146,130,229,172,99,198,196,145,147,160,228,24,62,77,235,147,183,64,61,30,174,89,106,90,223,83,5,140,98,12,88,231,154,157,134,137,171,218,36,181,130,18,175,210,241,73,3,118,47,119,207,186,192,231,207,193,31,221,170,201,19,85,2,232,55,68,58,192,63,81,248,89,114,9,210,170,133,236,114,129,138,177,79,92,177,3,75,67,26,212,229,22,184,
226,153,38,55,216,165,65,107,66,43,187,18,122,101,236,159,253,148,98,159,252,177,116,115,0,106,59,104,236,38,224,113,229,243,92,250,188,38,36,74,241,94,107,3,2,163,42,238,118,229,168,202,24,93,117,88,169,2,248,150,5,206,159,23,249,195,155,21,138,5,39,128,254,222,220,156,232,120,252,15,128,127,168,105,226,169,145,235,85,185,219,82,174,81,211,138,19,164,194,245,81,135,91,80,180,62,101,239,171,218,85,139,207,166,57,153,38,138,135,34,9,193,167,8,160,169,38,174,91,96,225,164,129,21,175,34,129,92,94,132,173,217,88,181,1,107,170,53,155,178,43,68,104,146,138,112,185,243,14,121,144,8,60,97,224,238,35,243,243,223,126,127,60,62,93,12,20,31,29,61,33,240,143,53,101,39,106,221,54,193,93,69,89,231,110,229,165,233,148,220,96,26,90,189,178,68,171,109,192,216,41,142,26,163,79,112,113,187,24,168,79,235,145,215,193,121,84,171,7,154,16,40,73,141,203,86,116,133,170,54,196,186,32,159,154,13,241,180,172,97,81,109,97,29,174,123,9,177,224,123,240,143,147,195,195,175,0,111,53,182,64,127,215,152,64,85,127,
79,225,11,117,59,66,157,159,89,181,179,219,18,95,86,78,17,251,208,32,231,35,184,203,31,92,226,214,143,107,25,92,242,31,151,149,211,79,209,2,181,129,185,156,5,42,186,112,117,27,138,226,150,25,37,37,64,170,139,97,170,54,144,166,247,182,204,181,116,89,159,58,113,49,13,98,162,76,185,62,107,96,241,170,49,127,248,174,170,109,100,129,18,107,127,1,248,53,106,24,47,169,160,129,165,65,142,165,204,20,219,154,0,183,234,228,203,118,169,164,193,238,247,73,128,167,110,33,40,205,180,126,159,212,35,95,149,234,211,76,93,65,141,203,86,22,235,20,173,133,173,137,57,168,185,127,174,117,101,169,46,193,167,33,88,76,5,29,94,124,93,110,83,252,53,107,237,63,3,254,93,173,5,250,53,145,174,194,255,162,112,213,149,252,162,38,72,107,154,79,169,98,225,228,20,166,186,170,84,161,202,242,232,143,16,239,212,129,184,46,129,139,35,248,205,31,255,199,117,33,77,102,125,166,22,168,83,200,1,85,37,81,109,77,220,218,196,13,173,107,16,98,27,18,12,80,157,236,149,194,253,23,154,107,253,234,136,133,66,76,20,24,184,248,164,200,
191,248,97,170,216,114,91,160,68,245,151,72,107,122,42,111,144,235,64,202,226,15,227,112,175,154,230,35,196,193,225,151,213,8,73,3,54,207,197,180,125,18,49,79,241,59,138,64,206,31,151,231,184,134,185,123,65,146,75,210,54,182,62,170,15,104,224,242,133,116,166,34,152,46,198,129,49,15,119,55,178,53,177,138,86,108,126,90,115,127,197,225,178,185,226,21,165,190,53,89,85,146,213,148,124,158,226,44,56,124,69,82,108,252,75,23,22,248,207,68,218,22,126,39,129,160,140,101,169,11,186,92,20,162,43,63,98,28,65,167,56,76,169,75,137,235,57,62,183,142,192,104,90,66,241,163,2,169,140,236,40,214,235,4,60,216,24,100,250,187,119,58,109,35,194,131,234,131,162,11,215,212,149,109,170,89,164,129,27,104,29,241,166,173,240,110,154,228,251,154,184,112,174,30,18,174,141,93,220,70,35,48,240,59,255,133,72,219,9,32,171,250,114,2,175,184,202,176,93,193,178,161,188,191,154,171,250,177,174,248,169,233,14,229,170,7,1,119,98,207,58,24,55,62,97,198,13,71,126,42,15,120,191,192,148,121,5,246,108,10,46,57,5,136,90,170,
206,228,169,56,232,253,58,154,186,41,107,168,21,57,55,91,113,63,78,67,64,216,83,184,145,101,64,162,36,239,83,149,248,46,172,195,87,80,125,185,20,64,127,43,8,140,194,111,106,174,123,78,93,178,202,245,59,14,75,101,74,22,140,171,82,82,42,220,67,151,101,171,99,189,202,148,211,159,116,50,211,117,220,197,155,228,149,88,79,175,228,239,30,205,139,221,166,185,159,50,0,85,145,7,101,229,229,101,49,99,147,29,223,150,144,66,77,228,63,52,136,201,234,226,241,170,248,139,26,230,206,213,43,80,31,38,55,127,243,215,130,192,60,20,3,217,40,58,171,240,229,58,31,181,184,8,202,178,236,56,220,150,226,123,167,23,198,171,185,73,101,126,105,89,2,44,255,253,85,29,109,202,154,141,252,200,192,81,45,237,61,0,238,242,243,170,218,150,186,36,117,217,163,171,202,108,14,64,46,226,160,204,10,36,84,151,102,52,217,40,172,131,93,173,42,41,80,71,252,97,43,98,217,58,117,189,139,120,16,220,69,154,90,114,60,101,199,174,240,101,77,27,132,222,126,192,2,41,252,60,176,86,199,46,85,53,221,171,106,55,69,197,110,108,26,36,250,
164,198,244,54,97,252,74,53,120,159,132,245,201,192,227,21,220,47,207,225,166,85,89,224,34,8,155,38,92,131,12,60,211,78,60,97,73,222,199,69,26,212,169,11,234,210,9,197,215,185,92,179,38,101,12,90,195,218,53,145,142,53,81,86,104,5,43,231,98,16,179,199,26,41,86,62,114,225,190,236,251,30,240,171,69,111,161,170,124,214,69,21,219,138,19,118,185,117,85,149,132,46,138,187,10,200,117,77,22,127,212,124,79,89,46,192,119,144,0,249,159,174,6,31,197,230,31,249,6,141,77,20,17,83,218,122,182,224,190,249,37,68,77,93,158,167,206,242,148,109,150,117,32,105,74,210,184,74,33,160,94,35,215,4,92,80,223,218,184,46,150,202,46,229,175,254,114,138,153,212,133,211,56,94,4,62,239,66,100,157,68,163,238,64,170,178,208,82,193,247,187,192,84,198,204,217,146,155,232,234,157,253,73,130,199,168,62,0,18,143,102,29,110,202,154,163,228,207,53,41,252,172,250,156,158,234,125,0,149,169,14,92,109,125,109,197,198,215,36,1,90,87,84,233,250,44,211,96,177,22,221,77,115,138,252,99,153,219,86,70,137,83,98,229,235,42,3,178,
77,239,243,26,199,75,192,150,159,189,240,249,188,251,38,37,185,21,151,60,197,197,138,72,5,69,218,20,249,218,112,225,21,253,104,173,113,221,62,73,198,109,218,173,180,85,66,23,55,41,12,116,5,174,15,148,81,84,60,218,170,204,231,172,79,187,97,236,211,228,126,24,154,11,58,155,184,73,85,49,157,197,221,41,137,6,100,82,93,151,160,58,161,114,89,142,169,44,157,146,197,236,107,192,115,192,159,250,217,127,126,65,115,132,66,147,0,183,170,119,179,203,71,174,154,217,211,4,60,85,199,150,80,175,120,248,164,243,61,158,234,67,173,163,188,138,32,182,78,197,145,39,64,30,104,109,85,65,89,207,243,160,226,32,196,173,56,160,38,31,83,150,4,173,106,194,111,27,228,188,234,98,167,178,192,93,106,64,228,82,93,187,66,129,38,189,51,138,127,55,142,181,108,62,218,55,191,0,252,169,255,203,158,231,105,146,124,78,75,76,21,148,55,217,168,242,37,93,200,182,13,114,11,82,177,115,11,238,158,112,85,199,244,73,74,117,138,199,83,156,134,224,83,222,140,164,137,75,148,183,58,249,54,89,174,247,248,5,240,84,185,110,85,215,70,
29,59,111,211,6,29,85,224,41,234,215,140,195,242,24,220,101,215,77,24,225,211,168,92,78,163,130,175,106,163,38,240,210,175,123,158,231,139,181,29,224,9,169,96,222,92,108,91,83,129,100,93,38,186,140,182,54,142,56,169,236,98,217,134,204,219,167,97,125,138,163,68,164,36,73,88,117,67,139,199,154,239,112,26,59,192,238,101,49,207,156,195,117,171,3,80,85,112,95,229,10,229,227,77,109,0,176,38,64,148,18,16,213,185,125,242,49,146,160,77,99,46,106,0,60,253,63,15,30,87,107,59,190,81,93,178,37,244,181,56,24,28,26,6,157,226,96,86,92,201,176,42,53,110,85,161,84,149,124,228,33,201,253,39,72,28,180,40,87,60,231,45,110,89,45,190,11,60,113,14,60,99,210,94,216,46,240,204,86,176,110,222,41,92,238,38,192,1,247,116,58,106,220,83,87,188,84,124,111,21,120,154,36,61,155,164,65,212,65,229,55,177,56,14,162,98,77,84,151,124,96,67,210,230,114,78,51,233,186,233,101,0,170,107,42,94,199,242,148,5,112,46,218,218,213,180,162,204,226,125,82,15,41,137,123,242,187,190,214,128,166,120,29,147,2,120,166,79,173,0,79,
209,117,11,43,92,183,186,123,80,37,192,109,210,155,224,227,180,195,114,141,136,108,18,187,84,148,96,215,186,99,218,224,111,208,76,236,44,233,165,223,240,129,179,148,148,53,84,201,52,234,90,241,158,214,231,172,11,58,93,166,218,54,72,160,157,246,6,87,31,168,62,192,186,5,37,46,83,19,215,166,12,60,249,241,46,101,121,159,64,149,25,112,186,110,85,37,228,117,180,114,89,134,190,46,35,15,167,175,172,181,14,107,160,212,23,185,65,117,187,176,166,49,141,22,60,3,42,210,41,53,110,158,7,156,245,5,214,167,189,173,203,118,115,169,201,229,212,53,137,168,50,195,174,93,146,6,62,175,86,208,179,90,179,211,125,92,203,227,231,128,83,100,221,76,3,224,40,229,115,63,243,207,34,120,4,8,51,240,204,124,12,240,84,245,205,171,218,109,133,102,125,255,56,5,120,212,193,190,81,67,24,73,131,191,157,218,13,119,128,168,169,43,155,253,186,238,75,58,175,84,234,198,74,156,182,156,91,104,214,204,174,74,80,40,14,16,81,195,4,58,115,76,170,31,175,84,59,151,44,13,28,150,167,110,151,46,118,54,173,27,99,34,153,203,54,109,142,
152,159,201,90,212,186,85,185,110,85,0,56,237,40,147,186,188,92,209,21,43,211,194,217,6,187,62,13,114,64,63,10,123,170,21,172,96,19,23,84,114,183,200,111,66,247,85,145,4,117,149,126,52,4,16,21,150,163,202,106,85,101,211,63,9,183,205,100,58,183,38,224,169,155,76,81,214,31,60,95,111,51,5,235,116,52,201,180,175,245,20,52,61,30,30,219,88,86,74,210,180,7,117,221,130,212,10,154,217,149,211,41,91,172,85,223,219,196,210,84,29,235,105,242,78,77,188,145,211,148,161,3,248,101,166,172,14,189,77,101,42,197,58,249,211,140,152,175,3,80,93,206,71,43,216,151,166,241,78,94,24,154,7,142,139,38,46,179,128,117,227,26,31,104,105,156,125,231,116,14,107,39,7,154,98,127,183,60,136,75,211,11,34,72,54,166,178,74,143,168,13,60,7,106,54,191,233,191,35,207,195,122,222,71,179,91,163,232,161,90,166,170,196,40,212,206,245,121,232,24,235,146,238,122,74,114,224,180,64,242,93,139,190,105,80,87,101,121,138,59,88,211,3,87,170,235,125,202,168,116,219,32,62,59,173,213,41,170,170,93,150,199,214,88,68,87,78,42,63,36,
88,0,79,132,192,24,90,158,71,24,4,132,198,208,74,18,90,73,146,2,197,90,84,21,171,122,191,220,26,17,18,207,35,9,2,172,239,147,180,90,36,237,54,86,132,246,238,46,173,225,16,99,45,168,54,202,133,73,9,219,85,188,135,197,215,90,17,198,157,14,147,11,23,176,27,27,224,121,232,209,17,222,7,31,224,239,237,225,71,81,105,106,194,252,8,196,64,222,34,186,8,1,173,32,73,154,88,42,109,112,28,190,43,168,42,179,18,117,5,110,56,216,149,228,99,248,219,174,242,9,168,158,244,246,163,184,115,82,2,28,227,176,58,46,169,188,109,240,251,3,86,71,4,9,2,252,153,25,130,249,121,90,11,11,180,102,102,8,102,103,9,186,93,164,223,39,182,10,106,177,253,62,19,213,212,205,203,26,159,107,16,96,187,93,180,215,67,187,93,116,102,22,122,51,216,241,136,241,219,111,97,118,119,49,199,199,72,20,193,120,140,76,38,120,131,1,65,28,151,86,186,218,83,16,65,144,38,123,71,139,139,68,143,62,10,47,188,72,119,238,44,106,2,134,246,132,120,229,117,248,218,215,8,246,246,74,181,101,69,33,173,75,230,3,213,37,15,210,48,148,144,138,216,186,206,
130,185,194,23,223,53,117,89,168,207,10,215,185,101,182,112,83,164,129,213,41,18,6,85,89,117,215,46,255,113,172,209,84,85,61,5,143,17,193,24,131,233,245,144,217,89,196,152,135,53,125,81,132,68,17,50,28,34,147,9,36,201,253,157,190,138,212,16,17,196,243,240,103,102,240,150,150,240,87,86,104,157,219,160,115,254,18,139,107,23,153,159,63,195,124,188,192,91,199,109,198,254,132,200,19,90,26,177,178,218,167,237,89,246,246,19,76,52,38,81,136,125,159,164,23,98,218,109,198,113,192,138,39,116,124,97,251,224,128,214,133,171,12,251,71,208,63,38,137,35,180,127,2,123,187,152,187,119,241,182,182,240,15,15,9,250,253,7,192,36,212,215,231,76,31,177,231,49,58,123,150,232,217,103,137,174,62,71,60,232,49,103,23,89,156,155,195,140,15,121,111,117,27,49,230,33,235,85,183,110,92,186,56,203,195,202,251,58,178,33,95,74,175,167,92,227,218,192,34,250,182,65,224,86,102,89,76,73,158,198,101,6,77,195,100,102,49,73,150,175,141,57,141,242,192,213,188,241,62,3,151,197,6,82,248,30,3,24,207,195,235,245,240,230,231,
241,22,23,241,207,157,163,119,229,10,139,115,115,105,76,33,130,167,202,196,90,142,15,15,153,244,251,36,91,91,216,157,29,116,123,27,246,246,144,193,128,233,140,166,7,110,178,8,210,110,227,45,44,208,90,91,195,191,112,129,224,242,99,120,231,206,19,172,156,229,177,133,71,9,204,58,203,23,123,116,239,5,188,125,211,112,171,43,68,99,240,226,132,39,94,178,156,63,171,152,131,24,63,74,240,163,132,183,239,42,222,89,97,46,128,237,237,9,75,222,132,32,26,179,57,223,103,110,233,44,201,209,49,51,131,33,59,39,125,90,131,62,157,213,1,119,47,245,241,135,123,152,173,187,120,119,239,224,237,238,226,29,29,17,156,156,224,79,38,120,214,62,180,112,139,214,54,246,60,70,231,207,19,127,238,115,152,243,79,112,124,205,167,125,118,145,97,187,139,245,3,22,147,22,6,173,140,105,92,150,194,58,92,121,235,96,252,154,148,254,195,195,83,28,202,92,52,115,138,148,204,125,0,53,237,163,165,13,0,86,197,181,75,141,245,209,18,240,20,27,133,184,220,166,170,233,15,121,98,160,116,194,179,8,38,8,48,115,115,248,107,107,248,23,
47,18,62,250,40,87,47,92,224,197,75,151,248,210,165,75,92,94,94,78,23,78,150,135,25,71,17,253,225,144,205,163,35,254,240,195,45,254,228,205,155,176,243,46,201,59,111,98,223,123,15,217,220,196,36,73,122,94,198,64,175,135,183,188,140,119,254,60,193,163,143,98,30,121,28,59,243,56,94,235,18,18,238,16,173,197,140,47,95,224,232,198,34,183,238,194,227,109,152,24,161,31,131,47,48,241,3,62,220,132,149,71,148,115,103,45,179,177,101,127,160,44,142,45,59,137,197,244,98,104,129,138,178,96,44,173,78,7,223,51,180,194,54,254,112,194,92,111,196,189,147,33,220,61,226,221,235,71,132,227,22,115,107,203,244,206,61,65,168,125,188,253,109,228,238,93,188,205,77,252,253,125,188,227,99,252,201,36,141,159,10,215,62,246,60,70,27,27,36,47,189,132,44,95,161,127,183,197,164,183,196,234,92,139,118,224,113,71,224,64,192,230,202,220,169,73,121,104,77,2,83,11,214,167,105,153,188,41,88,159,226,38,221,36,206,145,38,49,16,53,201,181,50,159,180,76,9,125,90,57,134,203,117,43,246,93,112,21,65,85,5,233,197,160,249,161,
166,38,34,152,118,59,93,216,231,206,17,92,186,132,247,200,19,44,92,125,134,223,120,238,10,159,61,187,198,220,220,28,75,157,14,243,34,248,170,76,50,0,77,31,73,146,240,153,103,199,188,127,188,203,55,229,38,115,107,111,224,175,125,149,228,181,215,208,123,247,192,243,48,203,203,4,151,47,19,62,249,36,75,79,61,205,198,83,79,114,102,227,60,155,195,21,238,220,232,176,127,243,43,216,227,19,238,78,18,122,35,97,103,8,115,79,24,122,143,64,119,91,17,27,97,183,182,120,253,213,35,158,125,242,10,102,185,197,72,133,51,51,202,163,151,133,232,186,161,165,105,54,111,232,43,145,40,118,162,24,81,70,154,166,191,140,181,236,199,9,215,195,25,162,249,128,147,221,46,251,247,6,4,123,3,54,122,33,231,23,230,25,172,62,66,148,12,48,123,91,112,231,14,102,107,11,111,111,15,239,228,4,47,3,147,53,134,209,198,6,241,75,47,225,173,93,37,60,12,105,45,47,49,235,181,152,233,6,36,173,128,176,229,147,36,130,136,160,190,239,220,217,155,166,76,154,164,71,192,221,165,169,184,97,86,89,155,38,225,70,254,119,191,110,216,108,
217,14,81,22,219,20,3,186,50,225,162,107,172,4,133,192,186,24,184,147,35,35,168,176,64,15,149,108,23,220,204,251,4,129,8,94,175,135,191,188,76,112,225,2,225,227,143,115,245,51,159,225,194,197,171,36,243,23,249,213,231,206,113,105,110,134,150,49,12,128,161,42,103,72,69,158,237,44,79,115,255,38,24,195,250,156,207,223,249,27,93,254,236,213,117,116,120,158,141,149,21,146,165,85,216,221,6,17,130,115,231,88,123,225,5,22,175,60,198,217,71,46,178,188,120,134,51,97,200,211,9,12,122,240,127,232,23,153,91,80,230,199,33,119,196,176,122,25,238,46,24,6,219,67,78,238,237,160,187,183,225,240,38,81,16,114,243,245,37,158,253,133,85,38,106,232,26,165,21,10,171,107,202,230,29,240,76,192,73,75,185,97,148,48,81,226,177,18,2,145,133,150,133,13,171,108,206,64,60,15,94,0,50,227,49,158,180,249,144,8,97,130,127,52,100,52,244,137,131,25,102,158,190,68,235,153,17,254,254,22,108,109,98,118,118,48,195,33,4,1,246,153,103,97,245,42,178,215,102,118,105,137,78,187,69,212,110,17,133,62,251,173,128,94,24,16,218,
144,193,236,60,218,237,166,22,62,23,99,85,229,170,154,116,45,173,42,243,47,198,206,249,159,54,183,6,202,154,116,186,40,252,170,120,201,247,26,36,211,92,83,219,138,253,206,92,189,18,170,202,9,138,165,226,211,127,251,60,220,88,195,69,36,84,53,129,44,2,210,107,181,8,86,87,105,61,250,40,157,103,158,225,226,243,207,243,242,19,79,112,229,202,21,230,150,150,232,134,33,231,69,24,138,16,1,3,96,81,228,190,91,217,41,170,33,178,133,113,229,138,229,151,227,144,201,204,69,78,14,102,25,111,92,192,38,49,210,22,102,207,116,184,120,245,2,189,249,121,58,129,207,130,17,150,69,24,250,194,147,151,224,43,199,93,124,17,244,8,86,206,195,192,131,227,123,202,205,219,62,58,187,6,102,22,241,23,153,187,144,240,189,227,9,243,71,35,206,247,218,24,17,124,81,86,151,149,227,19,24,239,66,184,166,180,213,34,177,101,103,146,30,219,209,0,218,70,88,106,43,126,7,146,85,88,24,9,71,45,131,61,54,196,137,199,126,226,49,27,182,24,180,103,56,58,142,217,218,26,17,218,128,217,153,30,221,199,47,19,62,21,17,138,37,177,134,120,
18,48,119,212,197,172,44,50,9,91,116,194,0,27,6,72,24,224,181,125,194,176,69,215,244,48,126,208,168,47,122,217,58,177,52,211,56,86,121,46,101,141,94,252,28,144,180,132,92,80,154,105,228,30,0,16,21,254,103,209,242,184,170,74,139,39,129,3,68,137,227,228,161,188,211,40,5,223,215,21,251,20,153,47,41,186,109,34,248,243,243,180,46,95,166,253,236,179,44,190,242,10,47,189,240,34,107,151,47,113,101,113,17,47,8,240,69,82,149,179,8,93,82,97,231,116,80,85,76,170,122,126,88,117,43,140,129,139,109,225,236,57,229,109,17,152,95,102,110,102,145,177,7,237,53,56,35,48,219,22,186,42,132,34,36,146,50,113,32,188,229,195,133,11,240,193,117,88,232,192,45,133,185,9,92,65,184,131,71,236,3,139,243,244,22,230,153,233,252,5,45,179,199,155,119,159,100,253,202,99,116,91,66,223,166,201,213,245,243,134,229,30,220,235,40,71,177,37,246,149,185,142,226,41,76,66,8,124,225,56,2,109,131,157,129,203,11,194,245,216,96,2,143,193,222,4,223,243,24,74,204,73,144,16,4,1,68,33,137,157,101,123,146,32,71,17,18,199,60,210,21,
226,72,216,74,124,158,90,158,165,215,241,137,131,128,97,6,158,94,59,96,220,246,153,105,7,248,198,199,148,180,37,118,181,212,170,34,129,170,0,85,150,250,112,1,103,186,209,22,129,84,85,83,36,53,57,161,251,0,210,6,166,213,58,36,40,197,221,222,101,137,92,109,146,92,109,122,93,253,198,170,64,148,191,168,247,205,183,231,17,172,174,18,62,245,20,103,190,248,69,54,94,126,153,229,103,158,229,167,215,215,25,7,62,45,17,2,160,45,130,159,125,134,151,211,155,117,179,255,51,185,188,201,88,149,192,24,68,83,55,137,72,216,52,130,215,82,218,19,24,25,161,119,22,86,186,48,107,161,39,208,83,193,55,16,138,208,22,216,179,41,184,174,116,133,67,15,38,6,250,91,208,142,32,14,148,133,101,195,201,246,9,163,195,24,239,204,28,39,223,250,1,51,151,44,195,139,143,114,253,174,194,37,97,214,8,151,141,112,222,40,215,151,132,155,135,176,220,81,246,80,22,134,202,108,32,196,2,161,10,59,125,1,1,47,132,93,21,58,129,33,241,61,58,234,97,143,35,142,136,105,133,9,189,182,229,66,98,177,214,178,99,149,147,49,236,71,112,35,16,
230,66,161,227,25,198,198,163,215,242,105,183,124,252,40,96,208,9,8,58,1,203,97,64,216,246,153,101,66,104,132,81,6,162,252,189,177,52,235,12,212,4,68,46,43,148,7,79,156,83,147,228,63,179,8,34,175,64,54,20,147,200,101,36,154,95,197,112,185,36,40,121,1,100,130,187,3,105,213,160,165,162,56,212,213,39,206,214,168,28,202,228,59,15,48,121,237,54,193,198,6,225,103,62,195,236,23,127,142,159,250,210,23,233,62,250,40,115,51,61,98,129,142,42,179,170,248,34,60,170,138,21,193,207,172,138,37,85,63,183,178,207,206,51,46,219,214,114,62,163,197,125,96,100,225,80,83,32,121,17,152,37,97,174,151,214,238,88,47,189,86,2,4,2,3,11,19,129,39,140,112,79,96,107,4,251,2,253,33,248,35,8,3,24,248,48,27,64,226,37,140,246,223,194,147,71,240,158,254,117,118,253,132,253,27,29,118,63,20,94,58,7,107,161,97,162,138,245,132,48,84,54,12,104,2,51,237,212,50,25,17,218,137,208,177,130,31,27,162,65,204,132,54,155,145,225,133,5,195,219,99,143,243,139,30,183,213,167,109,98,174,4,9,231,173,69,147,132,15,18,165,31,41,203,
45,101,61,134,235,99,97,219,23,230,187,134,119,141,199,134,239,243,92,232,115,43,246,217,25,4,188,188,30,48,27,250,60,26,250,96,3,222,110,133,244,219,109,226,140,250,215,10,240,148,53,177,183,13,93,186,162,21,242,74,128,226,202,29,122,133,184,200,181,118,197,145,198,241,93,201,76,165,188,102,101,250,156,228,254,94,164,22,171,154,189,151,209,139,94,141,160,176,233,24,192,7,98,49,17,188,185,185,52,214,249,169,159,162,243,194,207,193,249,159,230,204,19,23,56,105,249,188,62,86,22,125,120,70,160,43,41,253,107,211,245,143,74,202,32,33,66,71,132,36,35,15,238,231,64,68,88,54,15,26,251,160,35,108,180,225,56,81,136,160,29,64,71,224,200,194,172,128,145,116,7,220,16,225,201,172,173,75,63,115,11,175,157,129,203,51,176,119,11,78,38,217,49,36,130,181,138,213,54,24,195,222,206,62,11,222,57,46,156,155,231,40,132,201,42,252,241,182,240,247,54,82,82,164,143,208,245,148,112,78,232,15,132,153,22,220,24,246,89,236,182,8,141,208,25,11,209,200,131,120,27,125,255,136,209,179,235,248,158,97,117,198,35,
176,30,47,181,99,142,71,9,135,163,132,87,227,132,160,109,25,171,34,137,101,16,65,207,40,28,11,24,3,45,65,186,30,119,124,143,57,207,103,50,19,176,104,3,158,238,250,172,182,60,146,192,227,166,116,209,78,7,245,253,210,110,177,101,131,167,155,142,139,172,26,55,10,15,79,193,168,42,165,247,11,223,235,149,252,44,211,114,78,255,230,187,92,163,178,98,175,124,209,215,20,72,197,214,75,174,222,207,101,13,230,167,38,86,74,244,86,85,18,124,23,255,127,63,241,42,130,57,115,134,240,249,231,153,253,210,151,88,126,233,139,28,78,30,103,233,210,18,135,73,204,104,24,179,134,176,36,134,5,35,244,179,28,171,145,244,247,249,169,38,77,4,223,24,76,174,204,160,159,185,123,97,161,44,162,47,240,152,192,77,35,244,3,136,21,14,72,235,68,150,76,234,170,253,141,204,162,109,0,195,204,69,60,15,140,5,134,6,54,54,96,229,2,252,213,53,144,247,148,120,164,140,253,0,102,175,34,210,226,56,128,87,46,10,139,143,192,155,6,222,142,133,247,39,240,133,16,44,202,190,66,183,45,168,53,44,139,207,7,163,33,63,189,52,195,161,24,54,
60,67,215,243,24,126,97,131,119,125,195,210,196,199,59,227,177,30,248,140,162,136,40,142,249,126,63,102,223,196,72,104,217,104,39,44,138,18,91,139,106,10,246,201,145,160,109,225,216,51,172,244,12,18,120,236,27,159,174,4,156,87,159,153,142,207,245,192,160,190,33,17,75,203,152,251,234,115,83,161,95,116,17,64,69,37,65,93,50,222,53,165,174,44,30,199,145,40,206,63,243,32,42,235,141,248,128,87,82,22,235,228,1,83,124,70,60,44,197,167,130,70,116,129,199,53,197,206,84,200,213,139,86,204,47,72,61,180,215,163,245,220,115,172,253,202,175,176,254,185,87,24,203,44,123,223,125,155,96,116,204,206,145,153,82,251,0,0,32,0,73,68,65,84,219,202,140,8,227,36,225,15,124,159,199,55,54,184,186,186,202,202,194,2,139,237,54,129,8,51,34,169,104,211,243,240,179,220,135,136,48,204,190,63,44,185,121,71,192,181,236,122,236,196,96,77,234,66,181,73,43,22,95,150,148,209,187,152,41,171,135,217,231,180,129,51,64,224,165,239,153,49,176,126,21,110,188,49,68,180,197,76,207,35,150,57,58,51,134,51,29,195,181,190,242,
183,67,248,124,40,252,235,99,248,95,135,208,245,225,21,47,5,233,77,133,133,142,112,25,161,51,63,199,82,43,224,172,49,244,124,195,66,224,179,210,242,153,252,242,21,142,142,34,22,70,99,230,147,128,31,28,79,248,250,65,196,153,11,49,163,123,49,27,51,9,161,103,153,179,150,142,85,78,80,118,16,54,206,193,214,88,152,95,240,136,66,131,223,241,8,3,31,79,124,110,227,177,211,241,24,123,134,158,39,200,224,144,81,28,97,85,157,242,170,178,184,21,220,37,34,208,172,60,194,56,164,96,101,178,160,58,207,38,15,34,83,176,70,211,181,87,89,41,57,109,114,49,118,128,39,41,9,176,242,193,87,82,1,32,23,181,237,85,100,166,139,113,150,95,184,24,214,243,224,236,89,206,252,252,207,243,220,231,63,207,240,100,204,173,215,190,131,124,255,27,220,189,125,11,177,22,162,136,56,73,176,75,75,124,184,182,198,159,172,175,19,108,108,112,254,236,89,158,57,119,142,159,186,120,145,43,189,30,151,91,45,48,6,47,115,231,146,44,16,45,99,100,182,179,227,56,201,98,166,85,129,99,133,21,224,170,129,99,210,186,158,24,184,199,71,109,
177,70,164,211,107,207,73,10,172,77,133,67,129,227,133,54,195,125,97,161,11,222,57,159,193,17,196,93,56,51,15,63,184,11,95,186,12,231,90,176,235,195,191,136,83,144,94,145,52,230,58,1,14,129,245,249,121,142,173,50,2,98,129,190,49,60,229,25,6,113,194,196,247,120,189,239,243,84,226,243,162,239,243,84,24,113,61,138,120,190,157,160,36,104,98,89,77,44,198,90,222,207,206,41,154,131,117,12,71,24,230,91,134,189,192,227,196,120,108,77,124,30,235,25,122,29,195,5,35,244,15,247,249,198,251,239,176,253,198,107,176,187,139,205,145,47,117,222,132,139,40,114,21,118,86,77,194,168,107,58,162,13,193,148,7,145,87,112,3,125,91,18,239,76,193,51,118,128,39,42,248,173,101,242,111,45,148,51,72,9,187,225,234,97,108,169,238,153,157,7,80,80,8,230,38,237,54,230,210,37,90,27,27,124,240,131,55,57,252,254,107,196,223,127,29,239,189,107,180,182,183,241,162,40,117,81,61,143,168,221,38,238,245,136,231,230,24,45,46,242,238,226,34,239,158,59,199,191,122,225,5,46,190,242,10,255,227,19,79,96,51,107,116,162,202,162,
8,237,156,82,59,127,33,175,103,215,167,15,156,24,88,81,120,22,184,108,210,235,52,34,93,220,38,115,221,186,217,185,30,146,146,15,103,37,181,74,61,96,121,2,254,186,48,51,6,111,41,77,130,38,2,231,86,32,182,194,86,12,175,238,193,147,103,224,171,35,216,13,224,15,198,240,143,90,233,247,220,32,109,62,114,41,187,62,219,70,248,46,134,61,17,214,141,112,209,24,118,49,200,140,199,150,245,136,240,121,50,136,104,217,152,32,138,57,76,44,131,56,97,54,177,236,171,114,86,149,33,112,32,194,196,8,93,99,192,51,116,61,131,111,61,206,246,12,157,142,97,213,164,249,166,120,120,204,187,223,255,14,242,234,171,120,155,155,72,70,32,148,209,195,30,229,29,80,139,121,70,109,8,158,186,174,186,101,214,168,140,29,46,179,68,249,13,254,62,11,151,56,8,131,73,137,11,87,116,219,108,131,90,141,50,229,130,169,209,181,249,60,60,62,69,74,130,75,147,139,161,238,127,78,24,34,189,30,195,191,250,43,134,239,189,135,119,227,6,225,206,14,193,112,136,159,211,194,137,181,196,81,68,124,114,194,100,107,139,137,239,19,135,33,209,242,
50,201,112,200,202,103,63,203,93,224,53,85,186,153,82,123,142,180,15,245,146,8,87,114,231,187,47,194,173,108,241,111,1,158,166,86,96,205,75,255,102,166,59,120,6,152,249,236,220,14,73,45,19,146,94,215,145,64,168,96,44,204,206,193,242,35,112,56,129,237,3,101,217,196,116,227,152,227,65,66,187,19,243,222,205,152,217,129,50,111,1,99,185,37,202,109,47,33,86,101,132,18,7,1,79,244,122,72,171,197,162,49,76,140,97,168,202,150,129,69,95,152,237,11,126,203,210,247,60,194,150,143,157,196,120,81,76,18,37,204,199,9,179,113,194,170,181,248,86,57,204,70,70,154,12,68,11,70,56,49,134,123,198,48,39,194,18,134,118,0,43,217,197,149,192,67,198,99,204,241,241,3,194,212,170,186,27,26,42,19,104,160,195,116,77,77,172,155,222,93,55,240,139,18,16,249,69,58,49,42,121,22,199,155,215,141,189,168,107,246,93,213,7,33,79,47,22,221,191,60,193,161,60,60,84,86,69,72,146,4,249,238,119,49,195,33,193,209,17,173,201,4,147,1,167,200,234,25,210,190,3,45,85,252,201,132,120,50,97,96,12,145,8,179,173,86,10,208,76,16,57,45,
163,62,200,142,47,15,160,215,129,93,77,217,54,11,44,1,79,152,20,40,219,5,11,185,154,89,159,233,194,184,81,208,214,121,9,188,218,135,245,85,240,70,150,206,181,59,116,110,111,178,230,237,19,222,56,34,58,233,19,152,49,179,39,125,110,70,3,218,201,24,47,142,208,56,230,127,142,83,235,170,158,135,204,207,243,47,207,158,101,105,99,131,11,87,174,112,180,186,74,59,236,112,56,49,180,124,203,19,243,134,111,69,48,231,195,29,79,56,239,11,158,182,88,143,18,162,36,193,198,22,147,88,30,177,150,145,194,77,210,118,94,29,17,198,34,24,73,243,79,158,192,117,36,165,207,129,121,3,94,183,195,218,133,11,220,124,242,73,236,181,107,180,55,55,241,172,117,110,182,224,158,240,208,164,31,7,21,229,12,46,139,228,82,92,75,201,218,142,93,37,48,128,31,59,88,183,184,0,28,91,82,135,161,53,18,245,170,11,150,20,126,22,173,144,87,17,0,106,1,64,247,127,170,210,57,58,66,142,142,32,211,172,217,26,41,208,67,23,223,243,144,86,139,165,86,139,123,89,229,231,185,156,75,59,201,84,10,228,142,255,85,77,63,107,144,29,71,47,130,181,
32,37,6,166,253,13,166,177,228,212,170,79,99,192,175,243,81,217,118,2,220,29,195,98,15,2,207,226,221,184,78,240,198,87,56,123,235,135,112,184,201,104,103,7,127,119,7,61,57,161,53,26,97,38,19,2,107,9,84,209,169,155,36,130,182,90,36,51,51,76,150,151,185,125,254,60,31,94,190,140,94,185,130,247,228,147,204,108,92,34,56,179,200,90,219,227,188,24,14,19,229,42,150,191,220,188,203,103,207,157,99,185,221,225,196,42,247,146,132,36,73,73,132,89,171,44,170,98,73,197,180,91,34,76,50,75,122,34,2,54,102,211,243,216,77,98,174,13,250,108,92,188,200,47,124,233,231,248,231,135,135,232,237,219,4,185,13,172,137,229,17,78,55,41,162,110,29,26,220,19,70,138,94,83,158,13,182,133,251,92,246,189,247,93,184,184,0,154,216,225,174,85,141,169,168,170,43,42,59,97,75,125,163,16,83,161,18,47,211,61,153,92,190,166,170,133,112,105,147,189,108,241,73,171,69,95,132,45,17,206,229,128,209,207,216,179,124,27,215,27,34,188,169,41,8,78,34,8,39,48,227,167,177,79,72,54,198,44,123,95,222,149,75,128,77,224,158,166,127,31,
8,244,20,62,56,134,133,37,152,156,156,144,124,240,77,204,55,254,144,214,235,175,227,237,237,225,15,6,24,107,211,186,164,194,162,180,133,243,136,140,97,114,227,6,227,119,222,97,178,188,76,188,177,65,114,249,50,7,143,60,194,193,227,143,179,244,248,227,180,207,109,112,34,109,46,181,124,94,88,95,67,130,0,13,124,86,51,138,126,98,149,3,107,217,180,202,186,42,35,85,66,133,8,101,0,204,147,210,254,203,131,136,59,215,222,230,191,191,246,46,222,218,26,95,126,234,179,108,204,207,227,143,198,200,96,240,16,128,180,66,20,172,21,113,114,21,177,228,178,78,166,102,131,151,10,16,213,49,118,102,10,160,38,195,101,171,202,103,155,42,87,169,240,57,113,248,191,101,181,243,117,131,169,160,190,180,219,82,222,224,207,182,219,104,167,195,150,231,113,78,149,174,8,187,34,92,200,88,56,5,22,114,199,251,173,169,117,74,224,104,15,218,109,184,218,78,115,142,199,153,203,215,206,222,183,145,185,112,83,235,243,42,112,60,134,51,173,148,37,187,61,130,216,75,115,149,29,85,118,199,199,152,155,55,233,221,188,73,144,36,
149,29,96,243,129,173,168,210,74,18,186,73,66,60,30,51,218,223,103,116,243,38,227,183,222,34,90,91,35,186,124,153,157,199,30,35,126,238,57,226,103,158,103,103,227,28,27,189,46,43,198,128,17,14,141,161,107,12,39,83,214,208,42,19,85,46,168,210,207,206,63,86,56,204,128,228,141,133,85,129,214,51,207,240,220,194,34,75,51,33,103,246,4,15,197,75,18,90,14,240,20,159,166,0,36,87,172,92,69,103,67,179,49,40,85,161,70,89,252,238,218,232,253,164,144,21,110,82,6,45,53,7,93,87,14,209,4,64,182,68,201,144,119,235,138,211,224,60,71,172,228,234,201,45,133,157,100,42,12,213,32,64,219,109,146,48,196,102,228,192,92,6,132,177,8,103,73,199,138,0,236,136,240,215,89,32,57,176,112,210,129,185,46,204,153,116,225,141,51,74,57,202,64,52,101,225,6,217,255,125,61,74,193,178,98,96,98,225,189,109,232,45,131,149,244,156,6,54,5,18,153,123,86,214,70,185,170,23,192,52,198,11,226,152,238,201,9,227,193,128,209,246,54,195,155,55,25,95,191,142,188,255,62,246,230,77,254,253,203,47,179,255,216,99,124,121,105,137,53,17,
118,166,223,33,194,161,194,146,17,142,5,94,179,16,74,202,24,78,84,137,128,121,85,236,210,18,193,210,18,7,214,114,125,108,249,69,44,63,20,115,159,189,242,29,27,164,75,92,106,248,248,227,26,171,152,185,166,222,145,75,149,83,180,62,146,39,17,170,154,189,151,201,125,154,182,193,114,129,72,75,152,181,226,69,244,74,20,11,83,234,218,213,33,180,104,118,93,141,237,147,130,127,123,31,168,198,160,65,128,231,121,32,169,24,114,57,163,175,19,85,30,201,41,16,254,66,83,203,209,246,82,70,45,14,33,180,160,126,74,39,191,159,251,190,126,6,160,40,251,247,251,10,131,33,172,118,211,227,220,62,134,15,71,240,82,43,125,109,52,26,163,147,17,100,193,183,109,232,182,184,238,141,7,116,172,165,61,30,211,222,222,166,127,120,200,96,123,27,221,219,99,124,114,194,247,146,132,246,139,47,242,119,103,102,184,100,12,239,169,210,3,238,198,208,159,164,64,143,85,216,109,193,172,15,143,138,176,13,236,0,199,34,204,91,203,145,8,95,108,25,238,168,240,175,196,160,34,15,21,69,86,205,23,130,102,115,81,181,1,128,170,152,57,168,
110,225,235,170,74,40,246,101,144,188,5,2,119,23,209,42,0,212,185,104,85,253,216,212,17,19,153,18,125,92,30,56,33,245,189,169,203,116,85,182,144,73,78,114,223,145,7,145,138,164,45,166,68,88,17,185,223,220,112,13,56,151,89,159,61,132,175,100,249,29,85,24,36,160,22,186,49,236,180,82,43,115,143,143,154,191,207,146,38,85,71,217,113,255,105,146,130,105,232,193,137,133,215,247,96,99,49,205,33,37,164,229,208,50,25,192,112,120,191,230,200,54,92,28,117,177,94,71,149,214,120,76,235,238,93,36,142,17,99,152,172,172,112,237,153,103,248,11,85,126,46,139,177,14,38,16,140,82,177,171,237,10,107,61,101,228,165,86,121,35,219,16,68,132,142,194,196,24,158,85,229,69,148,127,154,88,238,1,43,25,128,92,141,31,77,133,85,49,13,44,144,109,232,41,73,141,151,84,71,138,81,65,109,251,46,57,68,85,11,94,173,217,13,235,80,92,167,101,210,18,169,78,144,45,196,233,115,106,125,76,137,245,209,18,69,132,45,252,45,41,0,73,10,121,39,36,173,221,9,50,90,186,5,60,158,59,190,175,198,105,57,130,248,48,153,192,126,38,83,152,
239,165,196,65,12,236,103,159,57,3,60,153,125,206,49,240,142,194,221,17,156,153,73,203,28,222,29,131,134,208,234,125,84,131,116,56,158,164,170,137,92,115,68,28,249,12,83,97,233,165,66,237,49,155,36,176,179,67,114,247,46,54,73,104,155,22,223,26,65,236,41,139,129,178,159,40,158,15,203,139,112,207,164,106,140,80,193,34,124,40,176,158,185,163,29,1,131,240,171,192,119,5,110,91,195,35,198,99,228,251,208,110,63,148,251,43,19,47,87,81,217,85,83,56,154,116,92,170,99,135,155,150,145,151,177,184,126,241,34,55,105,0,111,27,196,69,218,224,0,113,40,12,242,89,234,169,213,153,234,202,166,214,199,231,193,170,85,28,2,193,98,188,224,58,246,132,143,10,239,212,243,232,4,1,131,41,163,37,194,5,82,213,243,9,176,53,132,255,235,24,102,122,176,59,128,177,133,96,33,181,38,146,49,111,107,89,252,179,156,237,212,139,192,221,140,194,254,139,19,216,37,173,37,217,138,225,173,81,154,56,149,48,253,255,174,194,241,209,49,51,91,155,200,120,252,64,23,33,26,4,197,197,220,137,107,254,142,144,78,125,48,158,151,182,239,
74,124,142,15,148,119,186,176,18,88,124,137,105,141,71,132,19,248,82,167,195,66,171,197,97,198,24,222,181,194,110,166,247,155,33,157,251,190,33,194,191,19,120,33,128,205,185,25,38,235,235,16,134,247,75,186,93,113,140,43,68,168,234,235,87,71,50,84,197,63,56,54,117,151,97,168,10,89,124,26,80,188,101,98,211,38,128,56,77,61,123,145,122,244,115,238,90,59,247,204,79,131,115,13,62,166,160,236,182,14,218,189,120,65,61,210,30,7,126,167,195,197,32,96,195,24,66,17,102,50,193,231,205,44,103,243,207,21,172,166,18,155,225,17,28,174,195,145,192,172,166,117,61,97,102,105,4,216,203,190,119,8,220,1,134,17,252,96,156,126,224,68,225,218,16,194,89,152,145,143,64,108,173,50,140,99,102,162,8,111,52,114,82,177,117,93,148,164,198,75,184,111,229,125,31,140,199,112,8,51,29,229,32,76,221,205,157,247,222,227,219,175,189,134,191,186,202,239,127,238,115,60,29,4,156,100,121,176,111,152,20,56,63,200,206,53,2,254,24,238,171,219,15,195,144,32,235,9,103,106,220,53,28,185,58,105,176,65,187,44,148,156,210,250,84,
253,14,238,214,106,166,170,183,64,211,209,245,197,130,187,184,68,201,16,57,18,180,197,90,30,191,16,235,148,129,39,31,247,184,146,100,46,105,123,85,231,83,3,152,32,192,235,116,48,190,79,59,59,230,16,184,149,49,113,95,85,248,78,0,102,17,246,143,97,176,10,113,43,181,66,161,164,11,175,155,115,9,215,178,243,220,202,174,211,87,99,72,102,224,142,192,251,253,244,239,131,24,108,146,126,255,188,194,120,98,105,111,237,32,251,251,136,181,15,157,167,203,165,112,2,4,119,135,154,196,243,178,206,57,105,233,130,245,210,107,60,171,208,190,117,155,209,247,94,195,19,97,62,152,94,15,229,48,99,38,131,236,190,244,45,124,67,211,188,86,76,58,177,237,121,129,57,125,184,65,140,171,112,82,74,188,9,83,113,255,12,238,238,181,230,99,62,155,28,227,67,105,148,184,4,64,249,66,185,170,192,170,42,183,82,229,155,150,53,17,201,79,64,8,11,46,91,88,96,220,170,230,97,150,185,163,90,226,30,20,233,94,32,45,91,8,2,146,225,144,215,15,14,56,215,237,242,168,8,247,178,255,223,142,148,255,243,68,8,73,45,198,7,139,208,109,193,
142,5,245,210,69,53,189,134,11,217,113,47,231,98,161,239,78,224,13,31,198,35,248,240,0,162,86,218,29,103,195,79,193,215,206,94,55,33,97,150,40,77,66,14,135,206,29,212,82,222,247,89,104,214,72,93,179,115,78,58,29,18,171,244,101,68,59,105,241,83,190,112,73,96,249,229,151,185,242,252,139,204,206,118,240,91,33,147,236,211,34,224,153,108,83,89,4,110,69,41,145,210,111,167,199,244,74,230,170,190,238,25,250,97,120,191,25,165,105,192,174,149,185,105,182,194,130,90,202,39,125,52,73,234,107,67,151,206,21,23,41,224,71,148,215,166,23,213,214,69,4,230,131,243,166,100,65,89,241,91,62,25,90,180,60,69,182,173,110,14,38,21,128,41,115,25,139,128,178,198,96,163,136,100,115,147,27,119,239,178,180,190,206,200,24,54,73,25,182,63,217,134,40,2,185,9,111,92,132,214,92,90,223,179,108,96,83,82,160,140,178,221,185,77,26,100,47,101,199,255,225,8,94,157,64,210,73,173,149,127,230,163,70,23,177,66,215,64,71,225,182,194,56,137,225,228,136,214,96,128,20,196,152,212,100,227,229,20,0,146,76,185,33,195,33,108,222,
99,178,243,33,235,151,31,103,221,116,216,17,129,217,89,194,46,92,245,224,186,17,78,68,56,155,197,132,97,78,146,180,6,220,20,238,207,110,237,35,172,4,1,103,231,230,216,241,253,82,66,170,9,77,109,29,108,93,221,134,217,84,90,38,14,61,92,213,224,236,135,0,52,161,188,61,110,177,84,193,148,28,168,229,225,58,241,166,5,79,121,224,20,1,212,106,0,156,186,78,251,90,163,141,130,135,7,45,133,113,76,235,238,93,236,157,59,60,214,110,115,104,12,111,1,243,42,252,63,119,224,157,67,56,215,7,51,11,178,158,86,146,126,197,194,47,25,248,219,192,155,25,97,48,204,244,115,211,153,62,119,35,248,254,173,180,214,103,176,5,131,117,72,230,96,189,147,78,104,154,205,136,3,155,192,174,133,245,241,136,201,201,73,170,104,206,41,16,154,230,220,78,163,8,105,197,49,225,237,219,68,63,124,139,149,87,126,134,71,102,91,236,11,105,201,130,8,109,35,44,11,188,11,188,171,194,177,8,79,101,239,237,3,155,9,180,12,204,7,169,202,98,54,115,117,7,158,199,113,214,14,204,56,214,79,29,144,164,66,49,210,244,188,165,193,107,169,136,195,
180,38,109,224,79,28,186,177,178,164,106,177,180,181,74,98,65,141,79,238,149,128,40,40,36,73,203,220,181,38,163,212,203,218,5,23,45,102,177,76,216,100,137,198,193,100,66,178,176,128,204,207,243,67,85,182,38,202,135,187,202,59,123,66,215,10,7,49,140,46,193,74,144,38,17,49,240,13,224,2,240,55,179,239,123,39,211,185,157,1,246,18,248,218,7,240,245,187,48,20,144,0,76,7,180,5,126,8,207,248,89,89,119,2,253,4,46,160,92,29,13,121,127,119,151,254,193,1,98,109,173,69,111,146,138,112,61,102,147,132,201,238,46,214,51,172,204,205,49,107,12,129,49,116,69,56,50,134,11,217,181,137,18,97,223,66,127,148,182,201,90,201,146,197,139,99,56,110,165,247,42,4,30,37,85,165,223,205,41,54,164,98,115,53,53,74,3,131,123,186,133,214,16,40,208,108,148,101,25,123,43,21,250,187,7,186,242,68,13,178,186,166,176,24,181,134,59,175,82,61,151,245,238,42,2,200,53,64,234,52,19,158,77,201,141,178,133,159,69,186,212,144,118,241,241,102,103,185,109,61,110,221,134,55,143,160,55,11,179,6,70,158,178,55,35,92,234,102,61,2,50,
198,109,156,129,230,60,233,196,230,21,224,3,96,115,8,239,189,13,119,118,33,58,159,73,130,2,48,189,244,228,118,19,120,63,73,123,26,180,129,123,2,145,42,201,222,46,131,237,237,116,234,67,54,237,1,78,63,239,166,177,98,196,243,32,142,241,109,196,108,214,149,104,218,88,101,17,184,174,112,237,36,179,58,54,85,92,124,55,130,197,8,54,60,232,123,233,121,63,146,109,36,227,236,115,71,153,5,106,226,74,106,141,86,206,52,136,153,56,37,75,137,35,158,151,26,81,64,62,38,43,85,34,64,245,28,83,45,89,184,198,145,124,117,245,168,118,129,168,108,144,149,212,208,139,74,125,83,252,98,62,164,168,252,85,210,108,186,157,153,129,32,224,253,27,49,251,129,178,186,166,132,2,99,15,14,20,158,92,72,123,16,236,102,199,24,146,86,152,190,7,124,54,115,225,78,20,238,237,192,245,27,32,125,216,10,211,11,61,16,232,46,66,232,167,57,163,73,148,126,200,7,192,243,217,226,187,57,25,113,235,195,27,76,62,248,32,45,205,40,169,230,116,53,249,255,56,211,171,79,140,225,120,110,142,168,219,101,43,50,76,68,216,19,232,137,220,63,191,
145,194,74,152,126,241,172,192,130,15,67,11,59,199,240,189,93,104,95,129,95,241,62,114,223,166,21,204,227,154,251,165,53,247,175,174,43,143,203,42,156,134,76,41,19,141,214,141,33,205,191,222,79,26,104,137,154,12,131,45,235,238,88,164,185,139,77,15,235,158,101,101,221,82,115,156,174,198,248,101,59,90,209,45,136,125,159,100,110,142,73,43,228,204,170,207,156,164,164,193,224,36,13,146,189,16,38,237,20,8,139,2,71,217,204,226,89,224,106,102,133,238,36,240,141,123,105,60,211,25,195,221,22,44,164,249,68,180,11,94,47,5,224,78,12,143,249,105,213,234,8,104,11,236,10,204,217,132,96,127,31,255,246,109,90,71,71,141,88,204,211,14,47,123,96,49,136,48,153,155,99,244,196,19,220,246,46,241,216,16,86,103,82,229,121,55,139,231,32,109,183,213,107,195,217,32,5,85,31,248,96,148,50,8,45,47,83,54,100,242,165,99,85,222,2,54,179,194,187,186,28,99,83,229,64,211,250,160,186,53,171,53,57,159,186,247,231,67,29,63,169,1,69,221,110,86,214,143,56,239,26,37,5,48,213,241,248,165,53,62,184,203,113,235,124,254,178,
170,70,165,188,127,131,167,138,36,9,182,29,178,216,109,19,78,82,177,232,245,249,244,5,243,34,120,11,233,226,217,214,148,129,123,1,184,166,169,154,64,128,199,2,152,9,96,71,225,120,62,45,79,16,63,213,202,205,4,105,179,145,8,184,26,164,49,210,124,150,251,185,153,49,120,211,248,15,210,177,40,101,147,221,160,124,80,84,83,214,233,161,247,5,1,218,233,48,240,91,252,155,111,192,223,252,60,204,207,192,103,178,62,13,158,129,243,115,105,83,200,11,153,194,162,107,224,234,42,172,251,169,144,182,63,173,195,82,229,134,85,190,27,39,105,51,17,145,210,242,127,253,184,128,111,248,25,77,170,162,235,58,157,150,181,210,42,114,5,190,109,192,98,52,173,61,247,10,140,75,147,97,196,82,145,208,242,40,239,141,208,68,206,226,162,29,169,208,86,133,214,226,169,226,117,58,180,35,31,25,192,112,25,14,207,164,34,209,147,17,108,121,96,198,240,82,27,14,35,248,240,16,238,14,32,138,161,21,194,252,25,56,183,4,209,38,12,218,105,155,42,241,82,234,122,224,167,139,111,81,210,44,190,106,10,168,123,154,90,179,89,129,158,17,
102,150,151,137,159,126,154,193,157,59,4,31,126,136,159,41,178,165,196,2,85,53,95,169,203,123,220,191,6,198,32,97,155,65,43,224,96,15,254,247,127,173,188,240,188,242,185,167,83,119,110,143,143,44,237,50,41,96,22,21,54,60,101,197,42,55,21,238,170,178,167,202,45,171,220,182,22,162,24,107,245,1,9,207,105,165,93,46,203,91,71,233,215,253,127,177,183,187,43,52,145,26,47,198,150,89,32,161,92,212,217,20,72,77,122,123,85,89,15,23,152,170,190,67,106,130,68,173,225,253,243,73,59,241,60,196,243,185,117,232,211,237,194,40,20,194,49,236,30,65,111,14,206,90,88,108,193,104,12,223,251,38,152,24,230,231,83,128,220,156,133,221,56,237,164,179,24,66,255,4,38,49,140,3,8,23,96,189,149,246,110,179,154,198,83,231,77,74,3,191,118,4,235,51,112,21,133,48,36,122,234,41,90,215,175,51,254,193,15,136,238,220,193,159,76,42,165,60,167,45,167,127,160,180,64,4,105,181,82,34,33,80,104,43,209,56,109,234,248,129,90,190,99,97,62,19,142,174,103,138,138,29,133,5,85,94,80,216,82,101,95,149,15,18,229,79,110,89,186,243,
150,39,66,139,76,98,130,220,96,227,42,201,76,25,43,39,21,139,185,137,213,177,14,130,160,216,74,88,107,136,40,42,164,107,15,149,51,136,195,133,83,199,66,175,75,74,157,102,70,105,157,91,39,167,164,39,235,192,84,246,222,169,46,204,132,109,70,61,131,44,8,71,125,8,199,240,242,60,92,187,14,195,43,176,213,131,139,65,10,156,209,1,108,222,132,96,6,102,219,176,52,73,221,182,55,230,97,118,54,85,103,207,5,176,228,195,178,164,250,183,221,49,180,2,56,152,164,42,134,126,8,183,37,45,107,88,84,176,11,103,240,47,94,100,184,188,76,220,110,195,100,82,235,178,202,41,172,80,254,26,36,198,96,179,226,65,127,146,48,145,4,124,101,105,78,185,23,9,183,98,216,181,202,205,3,216,86,101,161,167,208,130,207,136,242,245,88,249,243,4,222,137,21,59,80,14,14,18,110,140,44,163,121,101,126,239,144,86,127,112,159,69,180,21,128,206,91,70,219,192,107,105,122,126,90,146,28,205,171,109,108,3,144,150,37,221,243,161,141,31,87,88,32,91,226,142,21,27,194,107,197,69,105,162,72,40,238,172,6,119,115,188,166,147,195,248,24,55,
64,201,154,50,134,33,98,132,245,21,97,171,15,12,224,220,42,236,239,10,131,81,218,183,90,108,86,101,186,6,71,215,161,127,7,146,243,208,27,129,31,64,127,12,103,13,68,157,84,161,208,238,195,197,89,232,38,96,251,16,79,224,36,134,235,119,97,230,114,234,226,121,30,180,142,132,239,180,97,201,120,44,183,90,104,175,135,13,2,167,43,93,58,216,57,11,218,5,30,234,155,80,118,206,177,49,36,190,143,85,75,167,61,97,48,7,28,8,7,34,252,249,190,16,15,97,101,1,222,136,148,63,189,167,204,89,229,233,37,229,236,156,242,71,35,232,136,114,14,229,120,168,156,137,45,186,171,180,143,135,140,174,95,71,175,191,71,231,240,48,237,242,90,227,114,169,35,97,175,13,238,97,21,75,92,54,36,161,76,139,89,54,222,164,172,181,176,87,0,188,147,68,48,133,96,203,224,110,9,84,182,3,84,205,16,106,66,2,72,3,240,52,5,77,211,199,253,221,216,51,12,135,9,91,55,2,214,23,160,127,4,239,92,131,229,115,169,246,109,56,2,219,74,155,128,44,62,13,155,243,32,203,224,205,195,222,33,44,46,192,108,7,142,77,74,99,111,255,0,222,92,3,239,124,90,
129,26,134,112,114,11,250,199,192,205,180,15,92,28,192,65,172,244,215,132,150,142,241,183,119,104,103,253,236,234,8,20,72,27,222,247,23,23,137,22,23,211,97,198,227,49,193,201,9,173,225,144,96,50,33,136,227,135,188,134,251,139,44,142,177,81,68,215,27,112,56,147,176,209,73,123,65,92,187,147,42,36,62,80,97,247,182,98,135,202,193,129,242,181,1,124,189,163,72,27,104,43,65,172,68,35,69,3,165,215,63,100,225,214,123,204,189,249,109,204,247,95,163,181,189,253,64,46,139,10,111,160,56,141,163,44,102,51,84,87,143,186,138,238,170,250,126,8,229,125,63,138,249,167,178,137,13,78,18,193,245,134,38,224,41,246,35,176,21,110,96,149,194,246,227,230,54,62,206,67,167,0,2,226,225,144,241,96,159,245,115,107,152,216,240,206,15,33,57,129,193,49,28,143,96,62,132,104,2,43,51,176,127,6,214,23,96,123,23,22,102,97,121,9,218,9,188,121,7,14,110,128,222,4,179,13,43,79,101,215,115,9,100,17,162,179,41,69,190,187,14,173,54,216,3,69,141,101,174,5,225,238,128,214,237,219,4,155,155,4,227,113,101,204,9,48,50,134,147,
165,37,198,47,190,136,190,244,50,218,233,96,55,183,240,183,238,33,187,187,248,155,155,180,118,119,105,29,30,210,30,14,241,147,228,35,47,34,73,210,209,141,253,62,97,52,230,121,63,198,183,150,237,125,56,220,130,248,12,28,220,17,70,91,112,117,94,9,215,224,157,182,210,243,193,67,104,77,148,51,170,108,143,39,116,182,182,152,187,123,131,222,247,191,77,235,155,223,164,253,222,123,132,131,65,163,66,202,50,102,180,172,111,133,61,133,5,42,150,241,91,7,243,38,212,247,85,192,225,133,249,174,44,183,41,100,232,203,92,182,178,164,105,89,215,125,173,33,13,234,74,15,248,148,193,115,63,55,149,36,152,131,3,204,219,111,211,121,225,5,130,231,214,73,18,197,222,78,19,28,227,29,104,91,8,6,112,166,3,223,85,72,124,240,231,224,137,217,116,66,195,59,199,192,14,232,135,224,221,75,57,223,158,159,74,122,246,222,6,59,11,193,25,8,61,165,125,30,6,170,36,6,204,162,50,107,45,93,85,238,37,49,179,199,199,120,199,199,4,89,57,131,75,162,50,20,225,224,204,25,198,207,61,135,254,210,223,34,254,204,75,244,219,243,204,189,
191,71,183,179,207,254,222,46,225,157,15,25,223,190,133,127,227,6,131,219,183,9,118,118,232,236,237,209,138,34,68,21,226,24,57,56,96,230,205,31,48,51,84,134,49,76,34,229,252,161,98,230,21,227,41,129,53,180,119,133,149,16,188,76,49,17,160,196,6,194,86,192,133,131,67,228,206,135,180,222,120,141,224,141,55,232,222,188,73,103,56,44,45,98,108,18,199,84,133,19,117,214,167,88,248,89,69,89,75,77,88,81,54,131,53,255,187,147,133,43,227,194,165,130,209,130,135,123,15,88,154,149,201,74,13,105,240,105,2,39,127,92,221,40,162,251,225,135,68,219,219,36,221,46,99,147,22,200,197,151,33,184,4,254,187,112,112,2,137,73,229,45,254,10,180,77,90,89,170,22,182,198,74,175,5,241,113,218,222,42,60,3,59,71,9,227,71,12,171,171,169,107,39,30,140,71,202,222,15,21,127,75,233,190,168,116,19,101,12,28,91,229,56,177,4,199,9,146,164,85,156,197,225,188,197,107,49,241,60,6,27,27,140,95,124,145,248,241,171,12,47,158,229,100,40,44,63,214,102,143,53,6,43,67,182,214,31,165,119,101,151,238,83,119,104,239,220,197,187,126,
141,209,91,111,209,222,221,37,10,2,116,50,161,247,237,111,227,93,187,198,100,56,68,226,152,197,56,65,53,45,41,87,145,180,52,219,90,142,84,9,141,161,173,138,38,9,161,49,104,24,98,250,125,252,59,119,8,111,223,166,115,120,72,43,19,193,186,74,6,164,130,29,148,18,197,133,52,244,124,202,64,98,27,18,72,167,169,31,123,32,6,114,49,104,166,68,174,67,5,79,78,5,120,76,3,151,205,5,34,126,76,32,106,91,75,144,36,248,11,11,105,0,47,233,92,211,185,182,226,169,226,47,195,209,62,12,151,32,88,130,85,148,118,2,239,31,43,135,10,218,1,219,131,246,227,176,245,29,37,25,41,230,42,120,87,147,180,38,104,21,52,178,140,71,22,185,170,68,106,73,222,84,22,47,194,112,70,241,39,150,209,129,18,236,78,192,15,176,115,115,76,124,159,78,28,87,130,200,12,6,200,187,239,98,103,102,144,86,192,236,185,139,108,69,9,251,251,9,18,40,210,153,37,106,117,24,117,230,177,115,235,116,206,156,99,118,227,50,195,189,125,196,131,222,251,215,240,255,234,175,210,6,35,73,146,90,165,124,9,182,49,216,108,160,152,111,109,218,171,33,83,49,
168,231,145,24,131,88,75,107,50,161,149,36,149,22,162,41,57,80,182,113,22,73,173,226,186,171,42,207,174,170,107,51,13,54,112,215,255,221,7,144,113,72,22,160,188,33,132,75,52,154,7,16,14,157,156,161,122,40,172,249,49,91,159,251,231,59,221,105,91,45,252,36,33,72,44,99,96,118,86,208,35,197,155,1,219,129,238,146,50,159,40,237,56,237,157,221,242,192,90,97,168,48,136,148,3,95,105,93,84,70,119,20,239,44,196,6,116,87,73,218,22,59,180,48,81,212,90,8,45,73,152,112,243,155,74,111,201,226,175,43,222,80,137,35,195,96,126,131,238,226,34,54,8,176,25,1,80,118,45,58,113,204,252,141,27,248,253,62,114,124,140,29,14,137,126,230,103,24,159,123,132,253,87,79,136,67,139,77,44,146,88,140,167,168,55,139,241,124,186,126,136,223,29,176,161,119,88,61,56,96,174,223,199,43,52,111,44,235,127,97,42,146,140,80,221,235,239,180,10,130,178,181,210,52,6,130,250,217,82,90,177,129,215,17,92,247,99,160,98,75,41,41,113,203,234,168,208,42,240,52,77,154,150,149,253,254,56,193,3,48,54,134,113,175,71,44,66,55,177,132,214,
98,98,229,218,174,161,183,32,236,244,148,245,64,89,67,209,113,122,214,251,22,246,78,32,178,10,190,146,180,148,100,4,227,99,101,118,94,97,69,57,62,177,72,55,65,199,22,34,11,86,49,81,132,28,79,224,48,129,137,101,240,93,159,238,186,193,11,19,236,112,66,156,120,216,133,5,226,118,59,85,101,59,174,125,11,88,136,34,130,205,77,152,76,176,81,4,34,68,191,56,75,175,27,115,28,79,72,182,44,122,79,177,145,5,177,36,162,28,218,57,90,19,33,9,133,184,219,101,18,134,116,6,131,135,118,243,226,198,105,113,247,150,208,138,77,184,169,234,160,204,186,20,139,56,113,120,76,77,192,83,70,149,75,137,226,165,14,64,211,207,241,201,166,40,107,174,243,139,75,97,237,10,224,18,202,103,91,154,186,164,37,110,17,233,143,219,125,3,136,68,24,135,33,209,234,42,199,51,179,236,14,35,146,137,97,220,51,136,39,44,139,178,96,21,19,167,253,3,238,100,45,111,195,129,210,199,98,39,10,81,218,8,192,30,43,131,53,75,114,96,17,77,176,195,4,98,11,137,197,59,24,178,114,231,26,179,247,54,161,175,12,91,62,131,249,69,102,47,158,99,239,
205,30,253,175,30,32,243,199,105,73,247,104,228,212,247,229,25,170,25,107,145,253,125,120,247,93,244,236,89,146,227,19,24,70,36,199,81,42,157,70,211,44,238,161,77,107,18,140,37,110,69,28,47,116,88,184,116,153,241,219,111,19,102,0,170,83,61,91,220,77,218,93,224,105,154,76,23,7,96,235,122,12,54,1,180,171,244,189,42,108,168,35,179,62,234,11,55,5,210,253,171,165,15,93,40,139,187,236,182,216,177,167,172,94,197,101,129,234,20,216,63,174,135,81,69,194,16,89,90,98,193,248,68,131,132,107,137,37,9,64,251,208,54,202,177,66,31,37,178,41,24,76,100,25,220,75,96,65,83,235,50,2,142,21,51,82,236,178,69,37,5,16,163,4,198,9,50,154,176,246,214,187,156,127,255,59,132,223,251,14,76,38,140,206,93,96,103,237,44,147,229,47,97,15,215,82,13,208,252,4,25,143,241,75,242,55,101,59,182,144,22,4,158,12,210,236,127,50,182,28,221,61,70,251,49,236,107,218,192,109,148,129,40,177,144,36,88,223,114,120,108,89,191,120,134,100,102,6,155,229,108,168,144,174,148,121,31,117,150,160,73,255,182,50,202,186,40,252,172,210,
195,73,13,200,138,121,29,169,137,205,171,24,58,74,1,244,208,55,166,128,74,114,162,43,136,99,0,0,32,0,73,68,65,84,44,144,87,146,152,146,18,240,80,97,181,202,98,31,191,196,250,152,31,179,245,153,158,179,4,1,106,12,219,163,132,155,106,25,239,131,38,138,118,149,205,177,226,133,74,219,87,198,177,50,24,89,226,189,4,211,79,80,107,17,201,92,187,19,139,57,81,226,57,11,147,52,150,138,147,132,36,142,48,253,17,115,123,247,232,124,231,91,204,126,251,219,200,104,196,193,149,109,122,95,248,79,216,254,250,54,253,125,131,105,141,64,211,78,29,249,5,93,181,211,63,176,176,90,45,198,55,15,137,183,78,82,176,12,73,139,115,146,148,28,192,104,26,253,154,132,241,196,18,73,72,188,180,68,116,243,38,97,54,193,207,149,247,171,235,126,234,154,129,218,180,169,166,113,176,116,197,164,62,84,215,126,185,88,99,42,54,244,58,221,103,89,120,227,55,89,84,54,99,100,172,35,206,113,113,235,234,80,56,84,129,199,252,132,192,163,64,236,121,104,214,19,123,111,28,225,199,138,57,176,216,68,25,238,41,93,79,9,206,88,142,198,202,
104,160,105,29,246,61,139,37,193,111,91,20,75,114,100,209,3,139,132,9,28,88,176,150,94,98,25,36,150,36,137,48,131,17,50,142,144,36,193,139,34,90,81,68,251,240,144,96,60,129,104,0,225,49,36,19,84,163,148,225,202,170,69,139,128,73,178,182,185,82,232,92,42,97,136,70,1,251,175,126,8,154,181,201,81,157,118,16,73,199,231,25,5,77,96,100,137,84,57,62,130,238,234,42,73,16,64,20,213,46,30,109,224,178,85,165,50,170,232,235,178,53,148,87,40,20,39,185,75,133,21,42,139,165,154,8,2,234,128,211,204,2,229,79,52,7,162,188,245,113,5,154,46,177,105,217,88,146,186,10,212,31,231,35,246,60,226,153,25,146,176,205,201,206,136,163,221,4,221,84,72,20,245,44,163,37,75,156,88,38,199,22,35,22,59,206,198,50,140,45,214,90,36,176,232,32,1,155,96,176,105,191,95,171,12,98,203,204,98,135,94,59,196,111,123,12,111,181,24,47,44,48,92,89,33,238,247,73,254,63,226,222,36,200,142,44,75,207,251,206,189,238,254,134,120,49,35,2,99,2,200,9,168,28,43,59,43,171,178,38,86,171,154,180,110,54,37,138,108,201,184,147,140,45,173,
196,149,204,100,146,105,171,13,87,164,204,180,209,130,38,51,201,140,70,45,68,45,40,107,54,201,98,53,217,67,205,99,86,206,5,100,38,18,51,2,49,143,111,240,225,222,171,133,223,23,112,56,220,253,61,32,179,171,158,89,88,0,241,38,31,238,127,207,57,255,249,207,57,231,206,161,102,102,96,48,130,112,128,211,25,105,234,112,179,179,100,237,54,54,73,176,64,58,30,67,57,59,75,214,235,161,179,44,111,188,152,36,104,239,234,185,231,158,35,75,2,76,54,128,72,10,171,216,131,39,240,110,92,156,223,65,231,12,38,51,199,55,176,110,182,78,213,198,88,215,138,202,77,72,90,214,185,161,229,216,167,202,139,41,51,195,82,225,254,217,6,16,209,64,103,215,229,173,154,88,191,224,113,118,232,178,171,86,55,116,75,26,192,51,238,125,80,6,143,254,45,90,159,227,93,83,4,23,70,28,141,34,14,214,142,112,123,22,250,46,239,59,21,89,236,158,197,142,12,193,200,47,186,161,133,61,135,139,28,102,215,230,186,142,52,119,219,164,103,17,242,188,82,203,24,90,221,126,158,75,58,119,142,189,235,11,204,95,188,140,158,159,71,28,164,139,39,
88,31,244,24,17,66,58,194,97,25,244,83,210,115,23,24,62,247,28,233,246,54,54,8,200,22,23,49,43,43,216,19,43,36,237,57,196,57,130,44,70,199,3,84,28,163,172,37,158,91,102,119,77,147,133,32,89,74,224,114,217,17,214,209,251,226,42,209,202,28,123,223,191,142,141,51,112,22,101,82,162,208,162,6,125,130,44,155,56,30,196,77,144,225,76,35,254,164,193,5,43,3,106,146,149,114,21,138,133,166,214,102,143,203,4,78,114,87,131,199,93,100,229,25,165,52,248,156,85,177,78,177,125,85,93,151,209,207,27,24,83,127,102,16,96,231,207,112,176,229,144,228,16,183,231,96,224,242,201,191,153,197,57,75,214,55,184,196,66,234,199,49,244,243,231,80,22,213,118,184,212,226,142,44,22,139,114,150,118,154,145,26,195,238,245,235,68,203,61,58,50,203,81,170,185,223,58,205,206,194,34,105,63,38,219,181,236,5,189,124,22,167,73,192,89,118,210,22,173,131,46,171,111,126,11,73,98,156,40,18,21,49,138,133,193,190,229,224,150,194,165,16,170,144,40,154,65,103,49,34,142,116,219,178,27,245,200,156,32,163,20,131,228,204,160,115,4,29,
203,226,155,79,49,247,198,51,220,252,199,127,138,27,25,2,19,211,142,64,239,238,18,24,83,155,204,172,27,85,51,137,40,106,2,9,53,11,124,146,138,95,53,72,123,38,245,203,254,188,214,139,251,44,22,168,172,154,173,42,255,174,114,217,138,125,223,234,134,99,61,142,245,113,159,211,69,57,14,78,87,87,81,157,25,220,110,159,48,107,19,223,243,223,176,96,115,144,12,44,174,147,15,223,181,195,156,133,99,224,127,99,177,125,31,164,139,165,125,228,200,140,193,154,252,245,163,224,20,131,187,150,189,27,31,195,140,99,61,225,193,149,16,223,108,219,164,249,103,57,71,230,224,222,97,139,237,93,235,37,14,96,156,96,117,136,73,21,118,156,240,207,114,151,76,92,59,119,207,90,185,203,205,56,41,58,214,238,43,199,225,91,247,208,58,101,230,205,223,1,103,145,52,99,182,147,208,6,130,253,253,92,101,64,125,97,25,52,183,146,154,212,248,176,142,157,109,34,35,170,44,83,85,142,178,174,122,212,254,53,109,194,238,113,98,32,42,98,159,73,163,243,138,46,219,180,224,145,207,17,52,211,184,16,197,29,106,48,51,147,143,184,215,1,
233,189,33,241,232,40,111,124,240,148,228,245,6,58,143,117,24,250,90,127,28,18,91,92,223,230,150,72,231,11,159,196,33,97,254,252,24,60,217,120,174,105,226,112,3,15,150,204,34,129,35,255,40,127,187,157,103,202,36,167,196,141,19,6,210,246,7,58,14,254,93,62,206,110,100,65,131,168,66,201,244,120,103,27,185,252,2,11,56,124,131,106,231,48,251,25,219,223,189,195,238,143,55,113,253,132,48,25,113,226,11,109,90,107,31,211,218,219,67,87,116,0,130,233,38,178,215,229,254,234,226,164,186,69,58,77,5,107,241,84,117,13,73,80,53,229,123,82,207,194,166,169,36,159,137,68,40,18,9,82,145,33,46,142,96,172,234,60,26,86,252,4,53,204,219,147,130,228,73,193,99,129,163,217,89,14,47,93,226,232,181,55,216,136,231,24,108,42,72,135,249,155,15,114,247,12,241,0,138,93,94,250,156,185,28,60,248,197,44,57,227,70,146,19,92,253,200,161,201,37,59,206,57,172,5,151,250,0,94,242,91,233,172,243,163,75,60,144,12,249,115,206,230,110,151,45,128,102,12,34,151,199,51,164,121,85,159,211,146,91,158,226,248,243,241,56,240,177,
206,164,248,25,169,195,110,12,80,38,227,196,114,198,124,27,162,27,55,233,28,30,86,182,48,163,198,77,106,186,182,138,230,73,219,210,0,8,166,160,188,155,62,83,77,32,51,38,245,51,172,219,200,235,20,24,143,21,3,85,89,159,170,174,49,77,224,169,35,13,62,247,148,206,20,86,204,0,135,11,11,28,189,240,2,253,223,249,50,183,147,37,214,146,57,76,98,192,248,24,231,200,66,232,232,93,94,162,247,244,18,14,199,198,95,125,140,75,210,220,173,27,91,38,235,114,112,37,46,119,183,58,249,73,138,7,137,205,252,230,99,93,110,33,2,119,188,106,68,242,49,245,185,245,241,22,201,146,131,68,23,110,245,24,4,26,136,92,14,92,165,30,236,90,198,61,188,114,141,67,252,205,16,231,114,203,105,28,100,176,212,25,114,254,82,151,238,251,111,209,185,119,247,88,61,77,69,138,162,74,253,80,149,43,41,91,42,77,243,24,71,166,160,198,31,199,27,169,82,205,88,166,111,200,248,56,249,71,121,18,18,97,220,162,8,159,127,80,21,113,144,46,197,60,77,224,249,60,165,58,143,27,55,25,145,28,60,47,189,196,225,107,95,225,214,126,151,117,55,75,182,
15,12,227,60,22,9,188,52,167,109,9,210,29,90,129,165,245,252,101,182,254,242,10,38,141,243,197,104,114,170,90,66,133,158,239,96,15,19,236,230,40,191,121,65,30,162,152,164,144,139,49,249,45,84,22,156,202,1,229,164,0,14,103,143,203,177,173,245,49,140,118,5,95,196,151,105,71,14,151,230,175,63,190,2,99,224,249,51,13,84,14,78,107,197,91,57,15,84,28,209,140,162,181,183,73,239,234,85,102,125,255,185,38,74,154,82,38,95,77,136,79,202,170,130,170,216,185,169,252,127,218,105,10,77,174,24,212,43,104,92,131,229,156,148,251,41,62,31,60,233,130,117,34,199,179,59,165,130,44,104,114,215,62,175,34,57,121,194,29,203,136,112,176,180,196,209,203,47,115,244,202,151,184,190,213,102,83,102,49,67,96,63,201,87,125,102,142,23,59,187,142,225,149,123,180,23,206,96,244,2,246,96,0,46,243,153,125,3,206,209,62,191,196,201,255,244,53,134,119,142,88,255,103,63,1,149,15,27,54,206,183,225,17,127,4,214,247,79,203,188,219,118,60,159,210,121,79,171,224,110,141,9,0,11,162,157,199,96,193,93,139,220,131,169,197,206,33,
30,67,57,41,1,78,64,20,190,187,169,119,25,165,96,165,200,167,146,15,91,45,162,56,206,165,76,84,183,0,171,171,80,158,148,103,41,198,70,150,234,30,125,48,221,88,202,73,196,67,109,185,71,5,161,80,55,14,165,174,210,160,150,184,213,227,214,67,242,248,75,89,77,25,231,52,209,212,238,9,193,240,164,224,201,60,120,250,95,252,34,135,47,191,206,205,251,154,13,219,197,58,155,183,203,57,190,219,38,7,145,115,144,10,49,61,54,254,242,16,103,127,158,151,34,168,7,177,12,214,145,221,223,33,89,95,7,186,185,85,26,185,188,116,19,124,66,211,187,79,199,222,152,7,147,183,68,34,121,108,227,198,46,29,60,84,214,155,147,4,238,216,149,115,85,173,108,92,225,76,173,247,219,53,40,17,50,196,7,170,249,119,14,247,13,71,207,157,128,175,126,149,224,218,53,150,126,253,107,90,62,15,84,101,85,154,84,201,110,194,162,158,182,241,76,211,132,248,186,114,154,105,45,146,173,1,210,52,0,146,6,171,21,116,252,61,74,156,195,60,6,136,148,115,143,176,107,173,39,96,216,170,20,190,159,55,120,198,143,84,132,253,217,89,250,47,191,204,
224,213,47,115,243,150,97,221,205,96,141,131,195,97,174,25,139,60,175,227,28,100,177,223,177,53,244,193,198,121,223,52,252,130,23,237,176,228,192,72,239,100,172,255,243,247,243,21,107,50,31,176,75,222,206,211,60,136,73,142,173,203,241,157,245,110,215,216,162,91,247,112,36,108,10,32,137,60,171,70,225,125,249,88,113,239,193,185,210,197,118,15,109,140,162,192,57,1,229,216,203,58,220,120,187,207,169,87,158,98,229,84,159,244,211,79,105,23,132,171,85,241,208,164,133,59,109,239,131,58,50,167,206,138,88,38,55,211,175,139,137,202,10,133,178,53,108,138,129,202,108,94,213,134,17,180,120,48,226,61,113,142,116,10,16,137,115,199,131,127,139,147,179,199,64,42,207,244,41,107,163,202,162,65,87,147,91,250,60,147,169,6,216,139,34,14,158,126,154,244,181,215,72,79,44,177,116,186,67,116,254,2,113,26,112,255,127,251,94,14,249,212,20,18,94,99,119,199,228,191,125,66,50,207,181,60,200,175,48,202,9,4,51,24,62,218,198,200,120,55,11,30,88,32,113,143,12,191,113,206,7,252,222,221,115,15,173,0,7,86,30,220,168,
210,93,28,135,80,57,170,199,241,148,111,220,141,96,197,199,64,8,162,241,177,144,98,55,235,210,190,51,100,113,101,133,180,215,195,245,251,143,232,205,234,2,109,121,12,224,76,3,164,166,190,230,210,16,31,53,205,163,42,127,95,25,68,117,12,94,85,229,117,157,36,73,207,195,155,2,127,248,80,190,167,1,68,226,65,210,241,63,237,18,128,234,122,89,23,131,180,73,243,62,39,89,154,39,1,87,6,236,247,122,152,110,23,117,239,30,225,149,15,145,173,251,16,56,244,107,95,32,181,150,228,218,118,78,69,91,79,37,235,49,173,236,232,126,105,149,249,111,61,77,247,133,85,192,146,110,28,230,241,72,236,1,228,30,4,249,216,124,183,31,79,160,21,225,1,136,196,21,92,49,30,210,251,143,47,187,200,241,218,247,214,75,42,78,92,188,229,241,122,55,45,133,55,230,207,171,72,80,90,97,69,208,74,16,165,16,17,255,113,249,23,74,154,178,112,174,75,107,99,141,246,238,46,65,33,174,173,43,63,169,155,160,48,201,99,144,6,102,143,138,164,124,213,100,132,137,51,110,31,115,19,110,2,207,20,107,238,59,129,174,202,222,58,87,9,34,33,31,154,
212,41,0,168,104,117,138,46,91,209,251,168,219,193,202,147,180,171,204,164,124,14,214,103,124,97,22,15,15,113,87,175,230,238,156,82,236,172,172,144,70,17,230,169,167,88,248,214,243,28,253,240,6,174,159,86,208,57,142,246,138,162,247,116,68,184,188,140,238,70,12,62,184,143,216,188,234,224,33,234,25,7,74,114,114,204,7,241,199,185,155,113,78,166,232,130,249,51,148,2,93,37,170,224,58,73,97,233,20,0,154,31,219,248,111,5,16,229,44,73,254,221,129,96,37,255,16,81,227,29,88,114,121,143,100,96,45,35,19,50,52,33,233,202,10,233,205,155,68,113,252,72,67,193,186,123,225,30,195,250,84,53,41,156,150,76,122,28,11,231,166,248,254,186,239,124,156,242,136,135,44,16,222,2,61,34,165,40,116,185,212,206,29,207,194,25,255,20,45,79,80,2,79,21,101,56,77,167,81,55,129,81,121,82,43,164,128,208,57,66,231,136,156,35,176,22,147,166,36,65,64,214,235,193,229,103,80,39,102,25,188,123,215,187,64,246,1,73,128,195,236,223,67,226,109,130,213,147,36,219,49,195,247,238,162,85,238,221,29,47,226,241,145,141,93,52,69,73,
207,239,10,172,155,20,222,3,162,36,143,79,156,224,84,193,226,100,227,24,199,77,55,203,94,252,248,7,37,72,91,16,165,9,3,149,127,190,86,100,74,161,181,128,202,173,149,19,197,76,11,102,163,132,232,238,93,90,113,92,235,186,185,199,32,2,166,113,223,38,197,53,182,129,96,40,38,239,171,166,116,63,238,218,105,170,1,106,120,124,39,40,190,121,156,248,10,143,55,85,247,16,219,22,149,64,83,36,11,202,58,36,91,202,224,142,215,130,42,124,143,162,186,193,251,180,23,248,113,119,163,114,62,163,155,36,28,221,187,71,124,229,10,230,220,57,230,223,252,6,195,107,235,244,127,112,243,65,214,223,127,123,178,217,97,251,123,25,219,63,250,65,158,247,81,227,235,83,136,59,170,146,10,70,30,92,92,124,28,163,189,241,40,220,53,231,10,224,195,85,84,148,201,163,223,81,4,166,123,64,77,163,4,9,20,58,80,56,37,104,63,113,206,142,69,11,227,152,74,44,74,165,132,93,149,15,0,42,73,121,234,202,186,153,64,69,203,148,86,160,73,193,93,215,164,228,73,166,80,212,129,87,152,206,211,169,2,253,67,74,132,162,169,14,10,96,130,106,77,
91,88,145,12,117,165,128,189,92,91,225,215,77,237,48,217,58,34,97,210,200,189,105,31,85,245,249,145,115,244,246,246,72,62,249,4,187,178,130,91,92,228,244,127,249,6,155,179,109,246,191,123,53,207,7,141,23,173,19,47,183,201,242,47,14,5,155,229,159,40,74,249,28,11,133,32,166,244,111,253,0,103,162,10,192,83,242,104,226,69,23,226,41,13,40,133,62,63,135,59,26,97,247,134,249,78,52,254,10,117,172,95,61,206,253,160,115,107,99,181,66,5,57,90,51,17,130,177,135,103,45,206,40,84,96,153,117,9,179,93,75,176,118,143,200,119,17,45,47,220,73,189,167,221,132,100,230,164,197,222,244,185,211,230,106,158,148,192,248,172,159,17,148,219,251,192,163,243,103,52,143,118,206,145,10,154,179,74,58,33,13,110,93,209,90,77,42,229,149,207,96,125,154,90,53,117,211,148,225,218,26,233,187,239,66,28,147,189,182,206,202,183,223,32,92,157,101,235,255,250,121,133,114,214,83,198,134,227,85,236,36,143,123,30,176,97,249,80,45,71,129,102,246,39,168,252,150,101,143,3,127,15,34,87,32,25,84,78,100,104,147,177,252,218,10,51,
207,44,161,180,35,104,181,232,255,232,10,102,103,128,238,181,144,72,19,244,90,168,40,64,116,128,139,51,70,65,151,254,202,10,217,200,48,248,224,46,65,168,81,126,238,105,48,174,175,115,150,142,115,168,44,101,117,37,162,125,239,35,90,247,238,17,100,89,227,208,41,104,238,254,73,41,184,103,66,250,194,53,108,112,211,228,126,62,47,16,77,235,226,85,89,161,64,243,232,232,6,104,110,38,87,204,241,149,23,228,36,115,238,158,192,146,52,125,214,227,212,232,151,155,141,143,173,93,111,48,192,126,244,17,163,173,45,134,219,219,216,153,25,210,245,160,148,179,241,113,67,80,186,219,74,121,166,14,198,179,16,4,135,132,130,82,130,53,185,104,1,201,19,166,58,20,204,216,151,21,159,39,210,32,206,19,15,42,207,154,106,151,114,178,59,226,196,222,117,228,59,127,129,4,1,186,213,98,206,1,233,16,181,235,16,107,17,145,92,233,29,4,184,118,155,225,211,207,208,63,127,134,187,31,90,2,29,18,68,26,173,132,64,229,51,79,83,160,227,44,218,26,230,162,17,115,225,144,240,211,107,180,247,246,42,55,192,186,251,81,238,119,64,131,
74,129,9,32,172,115,21,93,3,59,247,89,201,165,207,162,193,123,40,145,26,82,61,199,180,169,63,129,45,45,96,85,195,189,79,179,251,52,205,252,121,146,139,52,13,120,76,233,156,149,115,244,134,67,88,95,103,180,186,10,65,196,224,234,218,67,7,41,161,28,91,138,60,151,226,1,51,166,142,139,175,13,20,42,4,241,113,7,153,128,113,168,142,96,149,127,159,206,221,64,2,121,0,80,11,90,4,227,132,5,233,115,230,233,22,221,15,62,68,127,244,17,50,28,162,172,69,251,41,122,99,45,98,142,195,220,250,141,90,45,236,246,14,187,7,61,6,155,179,4,174,133,94,84,160,132,214,113,12,228,8,157,161,27,15,89,90,118,180,222,121,143,246,237,219,143,88,159,105,93,175,114,39,38,253,132,247,172,201,37,84,19,172,207,103,153,218,241,89,107,207,130,168,184,144,74,174,87,213,40,120,87,34,7,234,230,7,213,245,18,158,134,143,151,39,56,49,153,224,110,148,103,196,148,199,92,28,207,142,241,93,121,80,133,140,128,136,119,183,20,42,200,105,102,45,158,18,182,96,141,32,33,104,149,167,43,181,18,92,32,57,174,198,238,89,34,72,144,119,58,66,
147,79,21,86,15,0,52,62,239,16,197,200,57,36,140,208,201,1,209,181,107,204,110,111,87,250,255,229,209,151,86,132,240,228,105,134,247,98,92,247,4,233,145,198,132,154,176,37,132,222,125,83,54,99,46,233,51,183,154,17,93,125,159,232,163,143,232,28,30,78,172,149,169,202,197,212,145,3,159,103,87,217,73,61,218,62,143,28,225,103,217,164,131,182,95,60,85,67,135,38,53,145,176,53,49,211,164,4,215,180,224,145,207,176,67,216,9,224,201,74,191,45,121,103,82,163,53,22,75,231,133,211,164,87,119,142,41,103,23,40,116,164,114,230,113,108,52,148,224,16,18,7,90,231,93,114,148,159,76,61,148,220,210,40,132,32,130,164,47,24,237,223,220,145,227,248,71,105,149,247,116,19,193,248,164,109,46,234,12,115,193,174,239,67,93,149,14,112,133,127,91,17,70,43,43,48,63,143,202,90,160,58,160,53,38,22,218,203,138,22,96,227,33,109,211,167,59,59,32,124,231,23,180,222,127,159,206,214,214,113,15,192,38,143,64,106,92,170,186,248,231,73,239,155,48,89,243,38,127,77,192,121,146,92,83,208,226,65,127,171,172,180,43,79,162,38,
167,109,211,43,53,92,125,83,39,252,207,114,65,170,198,90,148,193,147,149,206,55,243,0,74,131,32,215,143,110,15,189,229,1,186,26,213,214,68,34,222,235,202,65,162,69,80,2,25,194,64,114,87,42,240,127,11,148,144,146,43,0,44,66,208,134,204,10,18,229,207,35,121,110,70,139,208,86,194,12,66,38,142,67,64,156,165,215,138,144,189,3,108,150,145,149,174,185,41,103,235,69,24,44,44,48,120,254,121,142,122,39,233,247,151,64,117,65,43,134,123,138,197,51,142,208,237,17,180,14,137,14,239,163,222,250,21,193,149,43,180,118,118,242,38,140,212,79,101,87,53,139,187,174,189,239,231,209,219,66,77,17,251,76,154,29,245,121,183,5,168,115,47,131,176,176,192,116,5,136,170,244,107,84,0,163,204,212,85,53,74,108,146,249,124,222,224,177,165,152,199,212,128,232,33,43,100,45,206,24,8,52,173,115,139,12,195,91,208,18,232,106,194,64,19,40,241,22,72,80,74,29,131,201,138,144,250,127,103,34,180,148,16,139,16,169,252,255,206,255,29,43,68,97,254,247,196,3,48,84,66,232,223,211,17,97,128,195,56,135,82,67,92,146,119,39,53,13,
84,177,19,97,52,55,199,232,197,23,137,95,253,50,155,71,43,244,87,79,67,28,32,169,35,204,70,112,103,159,153,115,59,184,91,31,33,239,188,67,112,235,22,81,191,143,117,46,47,55,226,209,121,184,77,108,102,25,44,238,49,201,131,105,23,110,211,188,168,166,105,236,159,53,71,248,88,36,130,46,197,52,186,98,119,174,242,117,235,198,219,87,1,168,220,92,164,10,92,159,135,136,180,10,60,197,77,160,12,152,242,70,225,172,205,155,179,15,135,4,39,150,160,23,32,45,65,71,154,72,105,66,37,136,95,236,226,1,18,73,254,239,208,235,205,54,68,176,74,104,139,144,42,1,165,48,254,57,92,254,92,18,200,177,181,138,198,234,0,129,174,8,115,56,6,206,161,77,136,245,214,33,171,217,253,172,8,195,217,89,178,151,95,38,251,242,55,88,79,207,98,150,151,136,14,51,116,114,72,175,51,162,215,233,51,19,222,199,252,244,61,244,199,31,231,83,239,178,172,114,66,181,171,73,33,212,45,172,39,221,236,220,148,86,168,110,168,117,211,16,172,39,37,154,160,185,90,181,22,64,182,134,174,86,5,229,128,162,122,218,92,221,72,122,213,96,153,198,9,
89,245,57,2,167,106,26,115,121,78,171,41,128,168,137,80,112,73,2,163,17,210,17,164,23,18,4,66,160,53,145,86,68,74,229,128,81,66,87,9,74,20,218,91,142,200,91,157,25,165,114,144,120,218,91,105,69,168,114,43,117,36,138,20,33,8,242,215,47,106,69,168,20,3,173,152,247,157,70,59,2,214,100,232,88,97,211,148,204,179,109,229,155,155,1,195,110,151,244,249,231,81,111,126,149,65,251,4,157,209,144,208,222,100,49,219,39,112,7,4,201,14,114,103,13,238,221,35,88,91,35,24,141,16,111,117,202,201,242,114,189,140,52,44,44,249,12,113,78,83,159,183,58,128,76,211,110,247,113,143,165,46,105,251,56,163,87,0,130,180,100,17,130,18,0,138,32,178,21,9,82,169,1,79,249,185,160,194,58,125,30,243,79,39,229,121,170,64,100,106,172,148,243,125,3,242,12,191,130,118,132,104,65,124,70,63,84,57,24,34,17,58,42,7,79,203,3,102,81,9,27,74,177,160,132,190,82,40,17,18,45,104,165,80,90,17,40,69,71,9,3,39,216,32,167,241,218,90,163,148,208,209,138,208,3,104,14,72,109,134,236,105,236,96,128,53,230,161,205,107,124,131,71,74,49,152,
159,71,180,70,253,228,199,68,198,228,99,80,226,24,25,14,209,195,33,122,48,64,199,49,218,15,205,42,83,206,82,113,95,109,77,242,155,9,187,126,83,173,207,231,197,190,209,160,74,152,198,34,186,41,242,80,117,21,178,181,0,138,201,165,57,227,71,113,113,23,173,79,217,221,169,219,33,154,100,240,159,119,3,249,73,83,34,154,104,107,83,241,186,49,136,242,24,95,227,58,17,169,18,58,161,166,171,53,161,86,40,37,204,41,69,224,131,127,237,45,83,95,43,122,90,145,73,14,184,88,41,250,74,48,74,17,106,197,140,202,65,55,182,56,51,65,254,122,180,34,20,197,188,60,184,62,41,25,110,24,29,247,192,174,90,204,206,185,124,2,246,230,38,98,237,113,57,182,242,22,75,251,223,80,221,167,90,213,108,34,85,180,117,29,128,244,4,176,8,211,13,214,170,43,35,151,9,36,195,227,196,205,211,130,167,46,23,86,151,163,12,226,194,155,194,2,136,170,92,177,172,34,3,61,105,40,112,211,12,212,207,106,117,154,232,106,55,37,96,170,158,23,192,133,10,186,249,68,93,27,104,58,65,64,75,43,90,222,242,88,37,180,149,98,214,107,206,14,181,102,65,
43,246,181,34,83,66,166,53,90,231,160,233,40,193,136,98,70,11,74,105,178,80,115,34,212,136,86,56,149,131,107,214,193,208,51,220,247,92,70,112,208,69,150,151,25,45,44,208,217,216,120,228,188,181,115,232,138,70,240,77,121,187,42,48,76,82,3,216,138,252,143,42,17,75,211,10,49,39,5,234,117,189,220,38,129,103,82,98,117,146,0,182,14,68,85,106,151,71,0,52,162,186,127,87,81,84,170,10,212,105,153,230,102,66,14,167,110,52,158,52,36,233,30,39,99,221,100,121,166,253,41,91,167,32,12,73,99,131,158,233,208,211,208,13,52,46,8,152,15,53,93,165,200,180,194,104,197,156,82,164,74,200,2,77,207,199,51,7,90,65,144,179,118,202,255,77,75,30,240,247,181,98,20,104,172,210,204,70,154,126,160,208,222,42,205,42,193,72,222,62,161,61,26,18,4,138,108,48,64,21,198,196,79,211,164,176,170,151,117,211,98,44,187,189,229,252,159,171,113,219,31,183,10,117,154,188,79,149,103,81,36,184,100,130,11,55,109,252,85,238,237,14,205,147,188,109,195,245,13,146,138,55,69,99,116,149,118,25,91,17,15,53,213,204,87,89,30,166,56,97,
153,144,219,113,13,86,199,77,25,251,84,185,118,6,242,241,38,97,8,81,136,238,117,233,105,208,161,226,72,7,180,92,64,216,85,216,64,115,50,208,156,80,138,145,86,172,233,60,14,178,90,17,106,77,18,106,230,3,141,29,43,162,61,101,109,253,223,102,3,205,250,152,108,208,138,231,2,77,219,147,13,41,112,238,192,114,231,104,15,54,54,80,163,209,99,181,169,157,38,9,46,21,215,173,92,94,210,180,107,151,91,84,213,117,90,114,13,52,177,155,194,179,168,154,80,7,147,71,47,78,98,104,235,214,83,157,101,162,193,141,13,210,9,59,91,80,35,207,169,147,255,72,69,188,243,56,131,131,93,205,46,72,197,14,81,181,8,234,8,4,59,69,124,100,1,171,84,222,64,7,69,54,55,195,129,134,40,20,90,42,228,218,182,166,219,213,44,119,52,189,48,128,64,211,9,20,203,34,164,62,182,233,123,144,4,90,49,171,20,247,180,162,27,232,99,87,175,23,104,142,68,56,148,156,168,72,80,36,145,226,64,41,14,149,112,10,199,200,36,88,39,168,36,65,172,125,162,70,233,110,2,128,84,233,26,20,153,86,77,189,216,210,142,165,77,165,210,111,120,188,170,210,105,
206,195,78,65,102,52,145,7,85,22,103,26,176,76,115,236,199,0,202,152,92,84,84,44,215,46,211,158,101,119,78,149,24,183,128,250,177,141,234,49,14,216,78,0,206,36,233,78,85,50,245,17,139,36,2,90,67,20,18,163,113,115,61,194,0,130,80,80,73,192,144,128,205,68,115,250,148,102,46,212,244,2,197,78,16,48,231,227,158,72,43,174,107,205,172,18,58,74,216,87,185,171,119,160,21,115,129,230,149,64,179,171,21,90,132,159,100,185,191,214,18,33,213,138,171,74,152,241,170,238,141,253,8,105,117,144,213,85,146,205,77,34,47,183,153,148,21,159,54,121,41,53,9,217,108,97,1,102,103,209,73,146,131,55,77,81,105,138,50,230,120,180,125,210,235,97,22,22,208,105,138,54,230,152,229,11,75,45,177,0,116,146,160,173,61,62,246,42,151,104,146,39,226,30,147,161,147,154,13,196,54,88,83,55,133,53,170,3,125,96,31,131,79,175,234,40,42,37,223,121,210,232,198,199,33,15,154,148,212,117,238,219,36,0,153,38,6,78,41,232,118,145,32,160,123,114,30,181,27,115,194,171,166,15,55,67,220,92,192,122,168,185,155,40,158,91,86,180,189,69,
113,90,49,19,104,174,5,154,57,165,88,21,136,69,56,161,132,150,214,28,106,197,125,173,121,90,9,75,42,207,35,153,76,24,134,66,164,133,67,37,68,8,125,28,23,172,229,136,5,244,23,222,128,173,187,184,219,183,145,173,173,70,151,168,169,99,77,213,130,117,37,89,144,17,193,156,56,1,191,243,59,132,151,47,99,141,129,195,195,252,103,56,66,6,125,24,14,17,99,176,171,171,200,115,207,147,17,192,168,143,138,71,168,195,67,148,53,15,175,159,209,8,125,251,54,193,198,6,51,187,187,199,163,42,31,103,90,194,227,20,237,73,67,146,180,174,68,99,218,94,220,77,113,90,80,254,34,67,115,99,59,93,33,225,40,198,54,186,194,10,233,39,72,118,149,119,8,83,225,150,185,26,240,84,37,81,155,104,236,227,27,229,123,179,153,157,29,122,95,16,186,182,67,75,67,162,3,54,54,66,100,33,32,182,138,143,71,194,107,90,161,230,20,115,158,52,232,120,160,44,169,188,167,193,140,228,42,132,89,37,124,34,138,158,8,111,11,188,238,25,188,149,80,184,129,32,45,69,215,143,240,233,224,216,114,142,197,103,2,66,187,192,78,212,122,68,98,51,137,68,
112,13,177,99,121,65,140,173,110,188,180,132,126,253,117,86,254,254,223,103,233,203,95,38,17,97,101,52,226,212,209,17,219,71,3,62,217,26,98,250,7,168,17,132,221,30,23,78,93,226,254,189,89,126,177,118,192,243,61,195,253,195,35,50,151,114,238,217,136,187,55,51,58,129,98,54,188,141,253,249,191,34,188,127,255,17,240,76,67,68,200,4,197,193,147,214,29,77,35,41,154,212,152,196,149,66,156,71,250,183,153,210,139,139,187,116,88,145,43,170,235,143,173,74,254,116,29,101,41,19,88,182,50,8,234,6,216,186,41,88,54,211,196,245,91,139,91,91,195,110,109,225,68,152,155,237,112,24,8,59,253,16,187,16,18,102,1,129,83,116,102,21,255,126,83,248,210,156,240,197,57,205,41,173,88,87,194,25,17,140,18,66,165,216,19,161,167,132,109,17,200,160,29,192,30,194,142,18,206,42,225,69,17,238,29,66,103,70,88,34,23,103,7,206,49,176,142,181,245,35,150,62,253,8,249,228,35,164,223,63,94,128,85,32,82,13,11,113,154,9,7,153,82,152,51,103,208,191,251,187,44,126,251,219,236,95,184,128,82,138,158,115,252,77,231,200,210,148,31,
236,91,222,221,53,116,183,115,201,82,216,154,161,171,5,53,236,243,218,27,51,252,201,47,28,183,111,239,162,85,143,155,24,150,194,152,174,185,79,112,116,68,48,28,214,74,127,170,202,32,152,64,74,61,14,105,48,137,81,155,54,182,108,234,229,29,148,93,171,186,65,177,198,179,115,197,124,81,57,190,81,53,172,27,53,129,108,211,252,24,71,115,237,142,125,66,21,66,213,224,219,241,177,7,89,134,245,174,74,20,105,146,168,197,129,210,236,239,68,16,6,172,172,6,156,234,41,150,181,226,110,172,248,88,9,39,3,161,215,17,50,200,173,142,167,172,207,41,197,166,8,45,129,150,134,190,22,150,85,222,87,225,142,87,108,159,108,67,168,32,145,188,81,203,25,231,8,156,197,141,14,217,255,245,135,168,15,63,36,218,222,62,142,53,235,220,10,161,185,231,90,211,220,207,76,107,108,183,75,214,106,177,222,235,209,82,185,172,104,73,41,46,1,187,65,192,215,59,96,187,66,124,94,208,41,216,125,104,103,112,250,176,75,58,171,24,181,193,5,61,110,220,215,216,40,36,72,98,116,118,128,218,217,57,30,27,233,106,220,173,186,49,140,66,181,
240,88,38,228,30,155,92,51,59,129,208,112,143,17,167,141,215,89,165,5,114,13,44,86,185,95,88,93,146,84,77,200,102,187,154,11,80,100,76,202,181,59,230,49,128,83,229,202,89,154,235,236,5,114,18,65,132,32,10,49,115,29,130,237,0,171,66,22,23,2,78,44,107,190,120,82,216,31,42,86,6,121,209,220,159,125,10,31,46,9,175,93,128,89,149,151,52,28,34,28,121,80,108,2,93,45,220,78,242,190,220,198,194,206,8,186,33,204,181,225,208,103,163,47,3,95,115,142,43,2,11,109,199,247,227,1,233,254,62,161,159,208,45,13,46,14,53,11,178,41,16,63,190,223,74,97,70,35,130,56,102,37,138,88,18,225,188,8,79,43,197,14,121,133,236,60,240,141,89,248,222,16,226,54,204,118,132,217,101,199,63,122,62,192,70,194,167,67,199,159,93,51,152,129,70,90,134,103,23,20,237,67,77,210,110,227,188,146,162,174,28,198,210,60,164,173,170,72,83,152,220,63,219,78,233,190,185,154,120,123,82,204,57,254,252,160,248,134,170,190,200,229,54,83,186,134,121,155,4,30,87,177,11,169,134,192,175,170,236,192,76,200,28,155,9,148,182,155,144,120,68,4,218,
109,116,175,71,208,155,65,218,29,246,51,205,169,19,33,97,75,243,194,83,138,97,146,91,152,182,192,57,13,255,245,138,176,213,129,57,35,156,243,101,220,59,192,13,224,93,96,0,204,0,23,2,184,63,204,187,73,141,50,216,114,249,46,126,42,128,55,59,240,60,249,92,44,241,205,63,122,90,179,59,51,131,40,5,198,28,223,7,51,165,126,108,210,132,234,241,223,163,56,198,237,239,35,71,71,200,96,128,157,95,224,190,18,82,17,94,22,225,69,224,62,48,23,192,210,8,222,221,5,49,208,62,43,180,86,224,25,5,203,111,10,170,223,229,223,255,34,37,140,28,103,150,2,78,117,78,243,214,137,19,100,237,14,237,163,195,70,154,186,42,206,169,3,79,21,128,166,149,233,52,1,170,78,133,64,77,72,49,254,9,202,185,28,85,227,71,151,21,217,117,230,84,77,96,65,202,121,8,85,17,160,21,193,147,54,88,31,26,212,5,117,23,107,98,18,55,12,209,189,30,103,186,29,142,92,200,133,83,26,43,1,175,44,43,230,67,197,161,131,94,8,255,227,42,188,28,9,29,201,193,146,0,179,228,253,233,119,253,247,220,3,118,93,126,236,237,20,100,4,221,25,48,45,120,165,5,167,
108,62,59,104,222,127,247,44,240,187,192,135,74,211,234,245,208,34,199,243,103,50,158,124,238,103,99,99,127,63,79,213,246,251,108,31,29,241,13,132,31,13,96,63,128,63,105,231,111,126,142,188,44,234,111,44,65,52,11,233,8,102,28,36,41,124,63,128,126,11,94,253,134,198,190,160,72,180,97,115,71,120,102,253,36,114,225,2,233,7,31,96,251,71,232,10,26,190,108,37,171,122,82,171,41,98,159,186,174,75,147,192,81,229,193,76,2,90,121,141,5,197,102,135,85,86,194,213,196,12,77,163,46,132,65,19,25,190,0,0,32,0,73,68,65,84,201,29,43,171,46,96,217,117,172,179,62,84,144,30,101,118,174,202,172,43,38,140,48,20,129,48,68,58,29,14,162,144,139,109,77,102,20,43,74,56,217,85,236,25,225,205,121,248,7,192,41,30,204,53,189,32,194,77,224,231,60,104,74,169,128,179,192,174,192,7,91,96,215,96,57,114,156,153,19,254,56,130,89,129,3,224,40,202,223,115,194,55,0,186,8,172,89,67,178,191,79,52,28,210,178,246,17,192,200,20,1,240,212,140,167,228,51,130,172,49,112,116,196,31,136,99,77,132,15,4,62,205,224,255,11,97,21,248,
61,111,69,239,42,184,21,128,30,226,251,65,64,47,128,219,17,60,117,81,72,156,102,107,86,17,207,157,99,245,249,231,217,94,88,128,59,119,80,5,0,53,169,187,167,33,16,104,112,223,44,147,167,211,53,185,253,211,190,199,149,1,52,166,168,169,176,10,117,44,155,166,122,82,89,147,50,183,137,191,119,53,224,201,26,120,252,98,177,220,36,37,109,213,230,240,208,77,80,10,21,69,184,44,195,38,9,210,238,240,108,8,47,7,194,37,96,7,248,186,228,22,163,248,230,192,57,22,69,56,9,140,128,107,192,182,7,211,115,192,253,46,92,191,98,57,209,17,254,214,25,248,102,43,127,223,62,240,9,112,213,159,231,41,255,183,115,14,218,190,245,112,171,80,114,45,53,50,156,39,213,166,9,16,88,139,234,247,113,251,251,252,141,48,100,81,132,223,109,9,239,146,91,161,132,188,191,228,123,146,91,200,8,56,1,220,235,194,114,0,39,129,13,3,111,116,242,10,144,159,90,88,16,184,189,23,179,237,9,153,170,53,240,56,189,16,38,149,45,184,9,41,13,87,193,220,150,221,126,83,19,43,187,134,184,155,177,11,231,42,110,144,170,48,165,85,149,166,194,116,
19,197,38,237,34,117,148,117,86,195,156,193,163,237,184,92,5,197,91,21,180,214,10,27,85,222,224,195,37,9,139,214,114,1,248,125,37,92,18,120,10,72,2,159,223,26,239,166,227,222,225,206,229,93,170,128,91,64,215,147,7,35,127,173,190,221,133,63,248,207,20,183,4,126,208,130,21,224,37,96,217,47,200,89,96,128,99,97,124,220,206,229,106,107,255,188,225,209,66,71,195,35,19,82,30,187,190,127,252,93,50,26,161,123,61,94,95,94,102,0,156,87,142,211,34,220,19,120,31,216,137,225,245,16,250,26,214,4,218,26,46,42,242,254,13,192,138,242,29,186,28,44,10,28,116,50,118,55,111,146,254,250,215,116,182,183,209,214,62,178,190,164,194,157,111,106,139,214,212,19,176,206,51,105,178,50,147,146,236,174,70,189,80,254,9,108,41,160,83,53,116,98,149,186,160,138,61,171,218,109,212,4,6,105,18,109,109,26,232,238,170,184,104,218,12,117,57,43,175,141,193,13,135,96,45,243,34,188,36,112,201,57,206,250,231,219,227,153,59,30,56,227,107,150,249,217,177,67,31,11,29,120,80,124,228,99,154,179,128,235,192,125,7,255,183,192,159,
2,151,188,149,218,242,192,219,247,241,222,60,176,139,123,164,16,177,204,118,102,21,215,104,82,159,105,234,220,215,40,66,207,204,208,81,10,69,238,178,157,20,184,226,160,43,48,8,224,95,108,193,255,188,146,55,135,108,75,14,236,143,253,119,207,9,92,119,112,134,220,250,172,15,6,12,238,223,134,219,183,137,246,246,106,41,116,71,117,243,146,166,77,120,146,214,175,138,61,174,178,50,134,234,14,77,85,169,142,38,93,97,80,181,136,203,139,191,88,138,221,212,222,183,92,174,59,109,195,9,199,228,158,109,117,89,246,73,21,147,77,126,114,89,130,228,124,115,117,29,69,172,132,33,71,206,241,161,183,56,167,11,113,98,177,17,71,230,65,16,23,110,192,16,88,4,254,30,176,14,44,121,75,178,34,240,101,224,135,192,77,255,221,125,207,212,157,240,64,74,201,167,72,140,227,49,53,238,60,90,145,46,24,79,86,201,74,12,93,83,158,163,252,72,149,194,246,122,24,165,184,151,36,124,195,57,34,17,22,129,11,146,159,219,149,0,46,119,114,114,100,211,31,235,120,147,216,4,122,192,188,7,213,188,203,219,115,101,195,62,178,187,139,78,
146,218,216,90,166,0,251,52,58,57,55,97,13,85,53,144,49,53,158,142,105,160,178,171,142,51,144,9,66,206,58,11,84,55,78,188,72,65,90,38,143,111,156,4,30,51,129,86,44,147,4,148,118,53,219,96,214,203,93,88,199,121,17,209,154,212,57,134,46,151,214,108,122,43,50,126,36,254,59,186,254,122,204,120,16,141,19,107,251,126,129,109,249,227,255,138,127,207,174,15,198,215,252,191,23,253,243,135,192,179,254,53,9,208,138,34,150,230,230,24,132,97,165,146,189,232,41,164,21,244,255,36,58,182,248,72,130,128,108,113,17,153,157,37,242,121,155,12,56,239,193,145,146,83,236,255,237,92,126,13,222,247,199,255,28,112,151,124,192,218,115,62,238,11,201,217,185,25,96,39,8,200,102,102,72,194,240,145,153,67,52,36,121,165,134,204,170,83,98,52,145,79,89,129,201,45,255,205,76,17,255,76,67,204,4,170,65,74,83,206,8,235,154,184,167,42,160,45,43,17,44,245,109,130,109,67,226,180,41,177,101,39,136,41,109,205,69,118,53,241,29,30,64,65,20,17,5,1,137,115,28,57,199,126,97,100,162,56,71,40,121,135,209,1,249,156,164,177,188,
105,207,131,102,224,41,236,37,224,111,121,87,237,146,143,137,66,224,191,0,126,224,207,175,239,111,240,162,183,98,137,119,227,2,175,165,115,34,199,12,214,164,236,123,217,10,77,211,170,215,57,135,203,50,152,155,163,51,51,195,158,115,156,18,225,21,15,148,101,224,239,2,23,252,125,217,243,22,104,252,153,107,6,158,211,249,185,246,201,251,232,119,59,29,78,95,188,200,213,103,159,101,120,235,22,237,181,181,135,88,56,38,36,124,167,153,169,235,38,128,39,45,253,148,129,100,26,18,239,117,68,84,85,14,42,80,13,254,101,121,145,21,137,131,58,90,175,110,56,171,170,0,86,241,2,100,13,39,233,38,88,159,58,117,65,21,216,168,176,60,99,203,138,82,152,40,194,137,176,121,116,196,204,220,28,93,173,121,182,116,49,35,191,144,214,201,201,133,168,112,206,191,244,96,24,187,86,161,255,61,244,175,59,4,94,240,175,187,231,65,115,166,104,125,156,99,48,26,177,183,191,159,143,155,244,153,252,226,53,211,21,11,71,151,226,160,58,90,255,145,56,212,90,208,26,221,106,177,21,4,92,19,161,239,55,134,125,224,117,224,69,255,158,
129,191,78,177,7,215,22,249,151,180,21,156,144,220,98,205,0,7,173,136,209,133,11,220,127,229,21,250,239,189,135,185,127,255,216,61,110,138,105,154,58,244,84,185,162,118,74,240,148,129,100,42,44,245,164,53,165,74,222,213,241,223,235,134,198,214,113,242,117,1,127,54,193,247,52,19,152,144,42,240,84,214,236,84,92,172,172,230,167,138,89,169,2,206,241,204,35,111,97,178,107,215,216,188,115,135,64,132,161,247,245,203,15,77,46,213,185,229,119,222,178,34,227,11,5,82,32,6,126,234,127,59,159,217,159,245,110,94,232,221,165,53,224,61,255,154,251,206,49,204,50,194,225,48,175,167,129,199,234,93,93,199,102,214,94,151,32,64,162,136,167,148,226,60,176,0,156,243,224,249,90,97,35,232,192,49,85,127,224,191,107,38,204,175,131,113,208,115,249,243,207,56,199,225,246,54,7,55,111,34,71,71,143,212,3,85,41,39,38,209,237,117,105,142,172,176,249,196,254,216,226,138,127,199,21,64,202,26,152,56,51,165,4,44,152,52,214,174,106,210,118,86,74,188,150,221,34,168,239,202,83,222,109,234,118,14,195,228,238,41,166,38,
51,175,104,46,53,174,34,71,2,0,99,112,215,174,161,206,158,69,31,30,162,69,56,227,221,184,153,130,136,214,250,221,56,243,100,192,56,152,63,227,99,153,131,49,105,0,220,241,187,242,182,119,135,94,242,150,235,85,191,32,207,0,167,61,168,70,228,154,184,251,74,177,184,180,196,237,83,167,8,55,55,233,249,190,8,197,230,144,105,197,53,43,251,253,117,9,232,226,181,201,180,198,69,17,136,112,193,57,206,251,243,84,222,253,156,43,124,254,39,254,24,18,127,126,202,187,118,75,249,224,114,182,200,25,186,12,216,27,141,224,222,61,90,27,27,143,168,16,170,192,161,120,180,31,93,93,114,190,105,51,173,115,217,178,9,185,158,166,184,81,26,254,29,52,141,195,171,2,142,170,97,124,234,92,168,114,140,81,38,25,76,233,228,147,9,186,55,71,189,194,26,38,55,184,47,79,220,27,91,160,0,104,27,131,234,247,177,113,76,63,73,8,129,47,137,48,91,222,229,157,227,99,239,90,29,122,5,66,175,224,94,141,73,130,101,224,182,87,23,180,61,216,186,30,88,145,7,83,224,23,225,138,143,45,86,128,197,197,69,190,124,238,28,215,103,103,73,194,
144,216,151,80,155,26,23,37,105,0,82,57,62,45,3,200,40,133,180,90,164,214,242,139,225,144,203,81,148,79,242,6,230,157,35,19,97,19,248,196,57,126,234,55,131,192,187,169,137,115,116,210,148,67,160,173,20,243,74,97,16,86,201,227,168,183,91,45,108,20,53,2,103,18,65,80,165,186,168,178,170,229,115,79,39,120,50,77,137,210,73,12,96,113,154,98,96,39,228,77,198,95,152,150,172,70,19,111,47,21,66,211,170,11,99,27,172,79,211,46,97,168,158,40,225,106,146,190,229,4,240,248,223,97,1,64,199,113,69,24,226,128,81,28,243,162,181,92,244,174,154,46,157,227,25,31,215,172,2,71,30,72,161,143,107,196,199,72,171,254,245,67,239,230,205,0,191,242,86,170,237,129,246,138,63,166,190,223,237,173,8,74,235,252,251,70,35,212,112,72,230,187,137,38,5,192,36,37,240,84,185,175,85,42,250,178,133,54,128,30,14,201,246,246,120,75,114,197,197,179,206,229,19,236,68,184,229,28,55,156,163,235,153,200,101,223,124,114,217,231,205,254,241,207,126,198,199,198,240,199,47,188,192,127,181,188,76,74,94,214,113,98,97,129,224,212,41,
204,236,44,110,119,247,33,55,78,152,110,78,42,13,210,173,38,240,100,13,121,68,55,65,189,208,4,154,170,54,95,65,93,103,254,226,20,128,180,132,254,128,250,18,237,42,235,83,39,80,181,53,59,106,25,20,118,66,178,53,171,136,113,202,238,89,80,186,81,101,90,254,120,240,156,214,216,48,68,133,33,87,69,56,5,124,85,228,17,23,241,153,220,20,229,147,174,189,219,118,29,120,217,75,114,222,244,86,166,239,173,202,109,255,253,125,239,174,93,242,159,19,249,115,30,31,191,242,236,91,187,221,38,153,153,225,40,8,136,146,228,33,63,191,232,211,39,13,204,82,147,250,252,120,99,75,83,92,191,15,97,72,166,20,111,59,199,45,17,254,192,57,62,117,142,219,254,24,53,48,231,28,89,28,243,246,206,14,202,24,250,90,211,126,239,61,86,78,159,102,112,204,194,9,215,1,187,180,196,226,153,51,236,246,122,88,95,233,59,141,196,106,82,223,106,219,16,171,84,109,26,182,129,120,154,148,235,41,3,151,10,114,44,176,53,106,216,42,157,154,225,225,6,35,85,109,124,171,100,233,186,34,70,161,96,217,234,152,146,58,166,175,169,86,232,88,85,
80,114,209,44,15,119,211,172,170,158,85,64,144,166,104,107,153,233,118,137,130,128,91,74,241,170,39,3,234,164,36,25,240,78,97,67,56,233,85,8,171,30,104,243,222,226,180,252,119,12,252,251,78,248,207,232,0,179,30,136,214,57,178,40,98,177,215,35,141,34,156,214,199,193,112,93,80,92,188,94,174,0,118,231,65,94,87,103,115,204,44,69,17,79,175,172,64,24,114,203,57,98,231,184,9,252,12,120,218,39,145,71,254,239,163,157,29,254,217,191,252,151,168,249,121,94,249,234,87,249,95,255,225,63,100,20,4,220,209,154,247,69,152,241,113,156,181,22,99,204,241,228,239,186,161,210,117,228,149,173,177,26,117,148,179,155,16,134,212,41,9,154,94,79,69,158,179,44,113,59,22,147,202,132,3,46,142,63,41,183,171,82,212,207,83,173,162,160,139,113,84,82,241,147,77,112,227,202,1,117,86,177,112,44,144,57,71,80,88,92,226,1,85,174,103,122,68,168,216,110,115,114,102,134,11,74,177,2,108,137,48,231,99,158,227,247,20,172,207,45,159,96,124,197,239,214,145,63,143,123,192,107,206,29,31,219,56,152,238,138,112,196,131,18,121,83,
148,70,249,209,41,58,8,72,22,23,233,47,44,192,209,17,137,115,15,1,40,241,18,162,137,153,251,194,107,198,109,139,31,82,137,132,33,173,103,158,33,58,121,146,53,132,61,231,248,186,115,172,123,192,187,193,128,239,29,30,50,18,225,153,110,151,133,126,159,197,43,87,232,124,229,43,180,131,128,247,163,136,45,165,216,4,150,68,120,1,232,14,135,172,127,242,9,59,159,124,66,120,112,112,28,191,149,11,232,234,132,198,147,202,15,234,72,17,85,58,55,219,144,83,170,154,117,212,52,167,85,149,114,139,234,88,72,92,138,105,138,7,168,74,32,170,155,216,61,73,100,90,55,58,221,20,220,146,226,239,108,130,140,167,172,194,118,82,77,132,58,201,27,21,218,66,14,162,72,93,87,106,236,180,70,133,33,59,89,70,108,45,54,8,184,233,137,132,147,21,223,97,189,123,118,217,91,131,216,127,110,236,9,131,84,242,153,66,174,98,65,255,165,207,242,191,48,118,7,143,115,11,66,230,28,163,253,125,226,131,3,82,231,72,188,5,74,200,27,129,60,81,227,118,239,74,29,171,67,148,66,175,174,210,57,121,18,167,131,227,254,220,123,192,41,231,248,
79,128,159,221,188,201,63,249,209,143,136,206,158,229,127,248,202,87,120,241,252,121,254,207,127,250,79,137,194,144,221,32,224,80,41,158,21,97,217,31,223,121,231,216,186,119,143,159,252,234,87,184,247,223,63,110,201,101,75,2,88,105,80,32,52,181,153,170,75,132,187,18,203,91,204,57,218,154,36,180,110,208,187,217,10,194,160,170,141,113,112,44,140,44,220,192,42,32,141,45,80,177,31,92,89,226,99,43,180,114,85,62,103,57,254,137,203,59,43,213,227,84,42,213,182,50,185,242,197,136,144,20,244,100,197,220,79,88,166,228,131,0,27,199,108,221,185,195,213,203,151,57,181,176,64,223,187,104,95,19,97,118,172,83,243,160,220,246,46,216,60,176,225,63,35,246,49,209,156,151,187,156,247,46,28,5,151,234,138,95,116,227,215,183,199,202,110,124,145,155,82,36,173,22,253,153,25,226,163,35,18,99,166,179,56,211,60,148,66,58,29,244,242,50,173,47,124,129,19,207,60,195,210,226,50,219,25,196,137,101,216,118,156,23,197,73,32,187,113,3,119,253,58,234,226,69,158,138,34,250,74,19,119,242,65,177,74,114,246,49,115,240,
12,46,167,182,215,215,249,238,79,126,194,181,159,252,132,240,214,45,194,56,126,136,158,182,53,114,157,73,34,225,73,214,103,154,150,95,52,108,236,85,233,16,55,129,80,32,207,29,202,155,192,31,50,229,205,105,170,145,104,98,79,138,244,99,145,73,42,251,247,73,201,175,111,162,29,173,76,223,154,222,121,197,180,52,176,115,14,136,131,128,108,110,142,103,222,120,131,111,191,242,10,139,90,163,37,47,113,190,143,112,146,124,204,35,94,51,118,203,231,113,18,207,178,69,254,92,196,63,55,231,207,99,169,192,66,189,35,194,156,167,177,231,10,150,48,40,198,112,214,178,233,28,31,14,135,12,246,247,201,250,253,207,14,28,17,212,204,12,193,233,211,180,95,124,145,222,87,191,198,249,111,126,147,139,95,249,58,191,140,46,176,113,239,46,7,247,63,229,163,56,230,239,204,244,152,115,142,147,103,206,49,248,218,215,25,92,188,200,217,168,197,71,222,85,91,245,238,113,234,28,145,151,59,117,118,119,248,217,79,127,202,255,243,239,254,29,193,79,127,202,204,218,26,161,79,4,75,131,158,15,170,139,220,220,132,88,200,53,36,
254,169,33,183,30,103,200,65,21,237,95,122,237,119,130,42,127,217,20,70,99,52,241,247,77,149,124,154,250,121,51,197,124,70,85,252,243,8,48,74,214,113,90,213,110,149,37,26,249,58,155,162,21,10,10,180,118,103,56,36,177,150,161,47,144,11,92,94,167,115,23,248,241,0,222,178,240,199,179,57,1,48,86,93,7,5,146,96,215,3,234,138,39,9,14,253,115,3,15,184,31,120,151,237,105,31,27,141,165,62,235,222,229,155,247,22,232,233,139,23,249,239,127,239,247,216,29,141,248,215,247,239,147,109,111,67,146,60,17,104,8,2,164,215,35,56,113,130,214,217,179,204,92,190,204,217,151,94,102,246,233,231,89,61,127,142,107,139,43,168,125,97,97,99,141,237,235,239,49,250,210,151,248,232,196,73,118,117,64,38,154,249,48,224,117,17,34,235,56,39,240,129,179,44,21,20,26,3,231,216,216,217,225,167,191,252,57,223,253,55,255,6,253,227,31,211,185,123,151,200,152,74,178,160,105,147,173,171,16,173,123,141,155,160,109,148,82,58,165,169,21,180,41,176,207,211,80,235,197,77,239,145,139,62,158,149,83,21,116,213,249,138,84,144,5,170,
1,60,229,156,70,173,111,239,1,229,74,133,108,79,242,200,60,136,194,10,0,41,242,62,105,214,24,182,7,3,48,134,69,173,193,57,14,29,188,27,195,225,1,108,180,225,239,104,136,84,110,65,34,30,200,251,87,189,229,137,125,204,212,6,246,44,252,47,41,232,8,254,145,228,113,194,49,17,225,133,163,1,112,215,57,218,190,152,206,33,4,167,207,240,181,75,151,249,243,23,94,192,30,29,97,118,119,177,253,62,110,52,130,44,203,117,108,21,215,9,173,161,213,66,117,187,168,249,121,130,229,101,162,211,167,89,120,246,57,86,47,92,228,204,197,103,57,183,188,196,247,227,1,59,71,251,180,102,186,60,219,237,240,183,95,185,204,187,47,93,226,164,10,248,105,226,88,82,41,248,57,174,51,90,49,82,138,87,252,108,216,235,254,92,183,157,163,179,185,193,71,191,252,5,255,225,187,223,69,255,232,135,204,92,191,78,203,131,189,73,163,88,167,107,171,3,72,221,107,160,126,130,248,241,230,89,218,244,155,154,210,24,154,251,106,60,28,3,77,112,123,76,105,247,175,74,148,154,26,31,182,220,145,212,214,88,158,108,90,119,76,62,143,97,230,144,
148,64,20,20,46,196,40,138,72,186,93,86,58,29,66,201,167,209,109,139,112,203,193,97,2,163,35,248,217,6,252,31,31,194,183,159,130,55,86,225,139,11,176,172,242,146,239,45,111,173,22,129,145,133,239,110,195,191,184,10,233,101,248,39,203,121,197,230,35,167,229,65,211,183,142,145,115,116,173,99,207,90,118,84,192,249,23,94,228,133,191,253,135,172,95,124,154,189,181,123,196,155,155,100,91,91,100,123,123,216,131,131,188,143,157,214,16,69,121,69,109,167,131,158,153,33,88,90,34,58,121,146,213,147,167,120,233,212,105,46,44,157,224,196,201,83,252,228,196,9,162,104,150,187,235,235,220,255,243,239,208,121,233,37,86,230,151,9,140,240,129,86,116,66,197,91,177,229,21,151,230,247,74,9,70,132,35,173,112,74,184,161,132,167,69,88,19,184,111,45,251,107,247,248,245,59,191,226,7,255,241,63,16,253,228,39,116,110,221,162,149,36,149,29,151,20,245,74,242,166,54,106,117,18,30,59,137,77,173,137,145,84,3,187,87,46,196,115,19,242,106,193,52,177,67,54,46,52,171,224,193,109,137,233,40,203,233,85,233,255,105,129,
56,24,83,177,127,237,15,173,81,139,139,184,36,193,245,251,56,99,142,75,11,138,63,226,201,20,89,92,228,143,46,93,98,49,8,8,69,72,128,79,118,5,185,10,170,5,201,8,250,155,240,39,159,194,159,12,225,210,235,224,44,100,1,44,47,193,66,0,97,10,87,238,193,245,109,208,22,254,167,47,194,5,149,43,150,111,137,228,154,51,231,232,59,152,115,22,101,29,167,141,229,35,107,121,214,58,54,141,229,207,157,229,227,217,5,158,254,210,155,252,209,197,231,25,237,238,176,191,191,199,39,59,219,124,176,181,205,254,238,54,54,206,35,46,105,247,8,180,166,213,158,161,213,237,242,210,210,28,207,207,47,114,126,118,150,165,94,15,221,238,176,171,3,54,219,194,51,163,148,152,144,211,39,222,100,61,61,65,255,208,208,215,35,222,67,241,124,160,120,69,11,118,160,216,87,57,125,223,86,194,166,111,117,245,61,37,252,129,128,201,18,222,189,117,131,159,190,251,54,235,63,252,1,209,47,127,73,112,247,46,58,77,31,81,133,64,125,41,134,173,113,165,234,210,41,85,9,84,106,98,149,166,184,188,73,181,95,55,144,173,138,104,8,166,245,163,
141,119,161,116,13,186,139,241,14,53,255,47,91,31,243,155,0,79,171,69,120,254,60,237,23,95,196,37,9,241,245,235,100,155,155,152,131,3,226,52,37,114,57,123,52,22,80,218,78,135,215,159,123,142,55,230,231,17,17,246,68,248,133,83,108,222,128,27,55,12,131,68,99,58,208,141,225,104,7,162,147,48,72,97,253,19,72,13,92,79,10,218,31,31,104,253,189,85,248,163,94,46,249,249,149,143,47,127,96,29,119,157,227,85,235,104,91,75,152,25,148,177,188,99,44,119,172,229,187,214,241,126,102,88,48,134,191,171,35,206,44,173,178,54,191,204,201,36,225,119,70,35,22,214,143,216,30,245,81,135,150,189,24,118,99,205,66,47,36,80,138,51,11,33,95,88,14,152,9,2,2,114,233,205,81,108,25,216,132,83,132,12,251,9,78,5,208,187,200,145,130,238,78,204,90,75,216,218,20,220,172,98,97,49,31,30,54,208,249,52,190,241,92,216,88,96,79,9,239,196,67,174,93,255,152,63,123,251,23,28,254,232,71,4,87,174,208,218,216,192,25,115,172,90,169,154,232,65,5,120,28,245,173,154,161,190,49,72,149,132,171,74,95,231,168,174,45,171,35,51,138,57,207,
38,253,28,133,188,226,212,15,91,200,35,148,37,58,82,163,21,42,250,160,69,0,253,70,192,211,237,18,61,253,52,75,223,252,38,111,126,243,91,124,120,228,136,175,127,192,222,167,31,51,186,121,19,179,185,201,104,111,47,31,231,225,28,78,41,228,220,57,122,139,139,220,23,193,160,184,121,40,252,197,134,240,214,186,160,7,96,35,71,230,132,40,131,246,146,165,163,21,12,161,51,7,233,141,210,22,53,159,203,14,174,223,115,88,11,3,113,28,58,71,223,57,222,205,28,98,45,31,26,195,122,102,89,50,134,77,99,193,88,126,102,44,113,98,216,59,50,124,85,27,102,49,236,27,203,200,58,58,198,128,211,40,233,177,58,223,97,24,56,90,169,67,13,28,86,57,2,129,40,114,12,98,131,25,102,28,58,199,39,222,93,21,7,23,142,70,252,220,192,209,142,227,48,22,22,87,225,240,80,88,52,194,112,87,184,123,32,124,160,133,103,3,69,172,21,78,231,131,147,35,37,28,226,200,14,247,248,183,119,174,243,209,59,191,164,255,195,31,160,174,93,35,220,223,199,56,71,92,218,233,165,66,253,49,77,71,157,38,234,184,202,229,114,19,152,189,170,234,102,69,181,
242,91,215,208,222,82,99,197,130,199,93,147,206,7,250,85,4,67,221,156,205,34,125,157,254,38,193,115,254,60,221,55,222,224,149,223,255,125,86,95,253,42,189,61,197,63,120,227,37,190,119,247,38,63,186,126,157,91,215,111,144,173,221,67,141,70,199,98,205,165,115,231,120,227,210,37,102,122,61,254,237,158,240,157,79,132,245,53,176,125,152,61,163,65,129,28,128,91,178,124,235,41,197,237,79,97,176,1,102,223,193,166,123,80,162,138,63,217,142,227,253,216,241,243,45,199,242,188,99,214,57,70,214,162,50,203,137,204,176,107,44,146,25,6,38,151,190,180,140,197,37,134,155,123,25,159,238,26,182,102,13,187,161,65,140,165,103,29,198,88,238,100,134,86,156,113,117,219,208,11,45,102,104,81,54,103,239,98,229,144,129,101,103,224,232,146,171,39,214,61,221,28,2,29,231,242,246,194,155,14,215,131,48,86,56,39,100,71,66,207,55,199,191,115,83,56,92,18,70,161,240,124,168,232,4,138,24,199,198,206,38,155,107,215,185,251,246,207,73,126,245,43,212,181,107,168,193,128,172,130,100,170,219,225,171,44,144,109,88,59,85,
32,155,68,113,75,131,235,55,105,108,35,13,212,120,85,19,148,224,73,215,231,152,96,152,182,65,124,246,27,4,143,19,65,69,17,193,137,19,68,103,207,114,125,113,145,48,140,248,239,46,118,249,86,123,150,248,226,57,58,151,95,229,195,219,27,252,77,119,64,96,50,14,189,196,230,98,24,114,242,242,23,248,243,96,129,79,110,40,158,234,8,163,142,112,115,31,50,13,113,4,167,34,136,51,197,82,11,238,206,192,222,190,163,255,241,0,134,17,100,2,39,32,234,64,218,113,184,212,162,148,227,159,191,231,248,111,190,108,153,179,150,253,212,112,62,205,72,51,67,156,25,140,177,140,140,101,193,90,246,19,195,219,183,82,14,92,134,141,13,63,140,13,11,93,203,57,107,9,140,101,152,25,246,82,67,103,100,216,222,50,28,88,75,71,25,14,158,10,25,45,135,232,249,121,230,63,216,96,102,148,176,7,196,42,31,49,9,121,249,193,166,115,28,166,48,28,194,204,12,152,35,63,36,217,40,230,21,12,12,28,109,9,187,123,16,45,9,63,142,20,169,78,200,14,54,184,182,113,139,254,91,63,35,123,247,93,228,222,61,84,146,28,19,67,76,16,173,78,234,196,100,
169,111,182,217,100,133,170,26,105,150,65,90,85,7,164,106,180,141,117,84,121,29,149,29,124,166,133,202,20,53,247,69,221,218,111,194,242,120,86,203,29,30,18,223,188,73,112,229,10,50,55,199,71,214,241,254,229,75,188,16,205,49,167,219,220,25,181,153,57,121,130,231,206,88,2,129,150,130,83,90,56,112,138,191,114,1,159,220,81,24,35,44,206,8,86,195,194,105,95,97,157,194,160,157,91,161,181,53,216,202,44,195,56,134,35,147,83,111,139,150,165,147,185,152,50,104,57,246,98,75,156,58,126,113,215,242,242,57,195,229,153,140,81,146,177,154,102,172,103,6,201,12,198,24,180,177,108,100,134,235,155,25,55,110,164,184,110,70,64,198,221,192,242,206,200,34,214,144,165,134,81,106,24,100,249,239,120,215,178,185,111,105,5,150,228,238,117,204,89,77,239,171,95,103,109,247,144,108,148,162,129,158,159,81,148,40,69,54,174,41,26,250,194,184,33,164,177,16,104,133,150,124,90,184,21,72,5,34,11,241,166,67,236,1,111,187,29,244,198,167,140,222,254,21,230,234,85,100,123,27,229,115,60,117,49,133,170,208,75,86,17,3,166,
38,182,152,6,56,85,139,188,202,229,178,13,108,90,81,27,40,83,196,100,143,12,25,254,76,11,149,250,161,194,84,28,248,111,244,97,12,102,125,157,193,91,111,97,14,15,73,215,214,248,223,111,188,194,119,95,121,133,103,151,47,178,127,212,225,212,9,197,167,161,226,253,79,225,242,41,1,37,12,173,176,149,40,182,143,20,235,135,194,198,65,174,131,187,116,17,238,246,97,123,31,54,175,195,146,113,188,124,209,241,139,67,135,164,64,87,67,104,209,198,210,9,45,17,22,218,150,172,111,217,139,45,108,25,254,213,15,13,255,249,235,41,75,42,67,210,140,131,212,160,141,161,157,26,178,81,198,199,123,41,87,54,50,204,90,134,44,100,56,157,33,45,203,173,161,97,46,205,56,240,128,115,153,97,144,89,236,129,99,176,99,73,2,71,18,27,140,109,193,210,46,233,198,62,218,90,180,151,4,25,149,255,182,146,79,2,79,18,208,129,48,218,7,29,230,122,61,148,162,175,132,158,130,237,68,200,146,20,29,239,35,178,133,185,125,133,248,253,247,112,55,111,162,10,243,138,170,54,74,83,16,28,143,189,142,170,146,150,186,216,164,73,249,210,84,202,
98,75,177,141,154,32,13,162,192,24,171,38,169,88,3,131,247,153,44,208,184,27,103,185,166,70,85,72,33,220,111,3,64,0,73,66,122,251,54,102,111,143,228,222,61,226,79,63,229,189,143,63,230,218,171,95,100,241,249,151,136,90,231,216,73,219,44,116,21,58,128,197,80,56,64,208,161,176,55,7,43,161,99,247,48,167,168,15,246,5,233,194,78,10,12,28,195,89,199,191,222,176,44,7,150,67,43,196,86,96,144,33,75,22,23,27,146,212,96,82,67,156,24,68,89,92,156,177,255,126,198,255,59,200,248,202,185,140,197,118,198,253,81,134,29,24,246,15,12,107,7,41,33,41,102,152,194,32,195,5,25,169,53,4,173,140,125,109,184,158,101,220,127,233,20,122,182,133,4,1,189,31,127,202,170,61,98,55,118,100,67,199,209,104,150,116,221,145,221,185,198,226,73,191,40,68,242,113,43,42,87,120,71,74,24,138,144,58,112,74,72,45,12,5,34,157,191,182,31,40,2,37,68,253,62,89,218,7,54,72,62,120,27,123,245,42,178,177,129,78,211,218,216,166,206,109,15,120,80,201,92,140,61,202,116,113,147,30,174,10,52,85,205,103,168,176,124,208,220,222,203,80,61,
204,120,82,18,85,62,139,5,82,94,7,21,148,0,84,213,113,231,183,253,16,99,112,123,123,36,253,62,102,99,131,244,222,61,146,27,55,24,189,244,9,7,47,191,204,193,229,23,121,113,238,20,231,103,218,249,168,250,56,95,96,11,93,199,186,117,132,153,16,57,97,255,8,118,182,32,216,112,156,92,118,12,149,229,146,179,12,86,29,63,251,190,133,150,133,200,48,179,100,152,209,25,131,65,198,40,205,24,237,90,28,25,28,165,144,100,140,62,201,248,254,142,65,207,100,100,135,25,36,25,4,25,202,165,44,7,41,65,146,33,131,12,137,50,108,150,225,16,134,105,204,102,150,160,126,253,14,193,114,151,153,23,94,102,120,127,7,61,200,208,6,54,182,29,214,230,173,183,220,8,226,163,124,68,74,36,130,19,33,241,201,80,39,249,248,201,64,198,11,78,72,44,88,13,45,17,2,109,57,50,3,172,238,35,7,247,72,222,125,27,123,253,58,122,127,63,111,1,76,253,180,132,186,156,74,209,26,81,162,139,77,141,251,69,141,197,41,214,63,149,43,113,203,133,146,85,137,124,153,146,225,155,212,210,247,137,88,56,241,192,169,27,32,92,165,58,40,43,24,126,155,128,
146,52,197,110,109,145,28,28,144,173,173,145,220,184,65,124,245,42,111,61,247,14,87,94,122,137,91,151,47,243,252,202,89,46,133,93,2,167,145,208,223,168,216,209,139,132,237,35,72,247,133,174,114,12,67,7,129,229,235,207,89,254,244,154,37,27,90,130,78,159,153,225,93,86,247,99,230,71,134,208,5,100,171,179,44,182,35,118,222,139,97,144,228,242,155,253,12,231,50,178,56,67,200,112,153,1,151,209,33,37,77,50,70,163,12,55,72,65,229,75,227,212,215,46,208,62,179,200,225,187,119,57,250,139,239,210,125,245,89,130,253,152,173,221,33,225,200,16,90,161,171,45,71,49,160,28,102,8,217,32,39,82,2,17,98,239,182,197,2,14,33,19,232,74,206,200,13,16,132,92,163,215,118,49,89,39,198,205,164,152,79,62,196,94,185,2,119,239,18,250,166,136,117,131,174,160,126,102,79,177,228,164,92,135,84,101,125,84,5,192,138,214,166,88,194,94,236,3,97,74,137,219,241,230,94,76,254,75,69,104,193,20,113,213,36,210,34,120,18,139,83,174,253,81,52,23,46,29,143,79,41,20,151,253,86,30,206,65,28,99,214,214,176,59,59,164,183,110,17,95,
185,194,232,253,247,249,171,103,158,225,231,207,61,199,242,51,207,241,236,202,121,78,46,46,49,31,116,144,174,98,111,219,209,138,133,104,198,177,155,58,244,130,227,242,140,101,109,219,178,117,55,165,219,223,224,84,120,149,213,224,151,68,215,118,209,65,128,206,122,152,120,129,236,245,87,9,90,171,108,189,149,97,119,134,121,0,178,147,34,81,134,115,38,207,190,234,148,192,101,140,18,131,73,50,200,50,92,63,7,208,224,189,15,136,122,47,66,43,32,113,103,72,223,25,114,248,246,143,104,103,142,8,225,96,224,24,198,128,205,111,171,137,29,201,48,159,185,170,200,167,133,35,99,201,148,243,253,23,28,8,4,206,98,108,138,178,9,225,217,136,108,180,65,246,238,175,113,215,175,19,236,236,160,124,83,120,104,30,116,213,68,5,23,45,9,165,132,102,217,245,114,165,124,98,214,160,155,44,2,200,242,232,228,196,242,102,173,11,57,30,93,3,154,186,24,168,73,29,30,76,3,158,162,94,76,215,236,66,110,2,149,121,220,249,179,80,226,251,144,63,250,27,4,150,56,7,163,17,246,254,125,226,237,109,210,155,55,25,125,240,1,217,217,
179,100,103,207,178,127,238,28,209,217,179,180,79,61,197,137,249,211,36,195,37,22,58,93,210,40,196,26,205,161,117,68,155,35,62,186,113,64,123,243,62,151,230,127,205,92,255,151,180,223,249,37,237,245,251,88,173,177,81,139,222,205,103,24,109,220,70,191,250,69,90,223,124,154,157,143,2,6,191,30,224,70,35,220,32,201,221,183,81,238,174,197,146,97,50,131,77,51,136,189,107,231,50,246,62,48,28,221,120,27,155,90,204,48,69,97,137,92,206,146,109,15,97,48,240,131,133,156,3,235,176,14,210,161,243,154,63,48,82,24,255,226,44,145,53,4,54,65,73,130,154,85,116,23,52,74,70,164,215,239,144,125,252,17,114,255,62,218,215,240,20,69,193,229,133,94,215,246,137,134,157,221,85,80,203,142,234,222,25,182,228,170,149,85,251,69,119,174,8,160,114,247,167,170,34,208,186,254,134,138,71,71,233,184,9,212,118,48,45,81,80,174,54,133,71,71,82,84,201,53,164,228,226,85,13,122,29,39,102,237,111,26,76,206,65,146,96,183,182,200,118,119,73,111,221,34,235,245,48,11,11,184,229,101,204,137,19,108,47,47,35,75,75,168,197,37,218,
157,25,194,246,28,139,86,99,14,98,88,223,228,212,209,29,130,27,31,209,185,246,9,51,247,239,211,245,139,47,12,2,130,253,61,250,247,239,51,186,113,3,253,252,243,180,46,125,129,209,31,156,227,240,142,226,240,106,138,221,79,32,75,112,195,148,17,41,152,60,78,34,206,192,229,183,213,57,75,186,215,207,175,140,228,0,177,126,206,106,54,174,110,48,214,175,10,135,75,29,153,114,56,237,227,14,1,107,51,140,137,209,146,162,230,21,44,133,100,42,195,28,237,146,94,189,135,93,91,195,237,236,32,195,33,170,80,61,170,107,104,233,178,124,70,106,116,111,85,165,2,101,82,128,10,171,230,74,49,79,82,3,160,178,8,121,44,23,163,16,175,105,30,46,231,15,42,64,100,106,220,197,34,176,234,132,174,141,0,82,21,86,198,208,92,99,174,104,238,108,170,106,114,72,191,109,194,65,103,25,81,150,209,30,12,232,110,109,209,185,113,131,86,20,161,219,109,84,167,3,173,22,180,219,121,19,66,165,80,73,66,112,116,68,116,112,64,235,240,144,153,56,206,167,42,248,115,92,200,50,218,219,219,28,30,30,114,180,190,206,224,250,117,70,87,175,
18,92,188,72,231,169,243,204,127,233,36,195,189,25,142,110,67,124,63,198,246,83,24,37,249,0,85,91,72,253,57,255,51,222,122,172,35,215,75,28,142,0,0,18,155,73,68,65,84,137,109,94,6,138,248,155,81,176,66,56,220,192,144,169,12,92,130,168,148,112,57,36,90,140,208,145,96,14,119,56,186,182,134,89,95,207,219,77,13,6,15,149,69,136,23,15,87,245,186,40,247,194,168,107,158,89,85,159,83,94,67,89,41,169,89,150,129,149,187,142,86,90,158,154,141,214,240,240,60,223,162,124,39,172,0,81,85,107,224,170,241,162,230,113,92,56,41,116,115,113,13,25,89,42,120,119,169,225,240,155,102,78,150,59,202,252,38,31,202,57,90,94,3,218,118,142,158,49,204,26,67,55,142,9,125,135,82,53,62,190,241,49,58,71,224,127,170,106,229,197,203,102,194,56,166,187,177,193,209,238,46,71,119,238,48,250,240,67,70,39,79,18,172,174,18,158,58,69,119,121,133,116,101,134,212,180,136,251,1,241,198,128,108,219,96,143,12,110,148,229,214,37,117,144,89,191,250,140,183,78,133,111,116,222,98,217,4,36,67,245,20,225,74,72,208,17,68,7,216,254,
46,217,167,107,140,54,54,114,208,140,70,143,214,18,21,239,139,175,7,43,174,3,85,227,137,40,170,39,120,184,154,120,169,188,147,151,9,167,170,154,177,170,174,77,77,229,47,99,149,204,152,70,47,55,221,12,121,184,53,91,185,86,173,170,187,107,93,153,69,80,163,196,168,164,20,235,154,127,171,138,247,148,15,200,52,0,136,223,34,177,48,110,43,53,67,94,17,218,241,64,26,95,228,136,188,42,85,151,54,149,169,45,27,208,117,142,86,146,208,219,222,102,184,187,75,255,246,109,250,221,46,195,94,143,209,220,28,178,176,0,115,115,200,220,28,97,111,22,115,177,131,213,17,89,170,177,153,6,227,242,152,201,88,68,105,68,135,199,57,158,220,113,79,16,177,164,27,71,232,182,224,178,62,118,111,155,248,218,54,118,111,15,119,120,152,147,23,246,241,179,113,99,237,227,177,27,86,176,78,85,238,93,80,17,212,235,130,43,228,166,76,150,86,117,93,205,152,190,118,204,122,75,58,169,237,89,149,117,172,234,37,94,83,33,155,5,94,107,248,144,132,104,188,80,30,103,152,173,84,128,174,216,195,217,252,22,45,76,19,120,186,206,49,75,222,
28,163,231,129,212,30,3,135,201,117,243,211,124,199,24,72,109,231,8,141,161,99,12,189,209,40,7,147,82,244,181,102,16,69,12,162,8,211,106,145,141,127,162,8,27,134,136,214,249,184,251,113,114,180,48,110,5,192,197,49,110,56,196,245,251,36,131,65,46,116,75,211,28,48,238,243,115,142,167,177,78,229,128,222,84,128,167,220,246,140,82,210,180,202,149,42,46,230,169,197,40,84,119,46,53,165,188,81,57,127,89,116,233,170,198,140,62,24,47,205,122,64,222,97,118,220,128,255,161,0,172,41,225,68,137,87,175,186,32,233,111,88,3,247,36,224,153,131,99,0,253,255,205,93,205,139,101,87,17,255,213,185,247,125,244,76,155,96,16,29,19,117,161,146,8,193,141,162,65,13,134,184,16,4,23,34,226,38,136,146,141,18,48,187,96,182,73,32,127,192,232,78,8,153,8,130,96,140,193,200,96,34,25,63,152,25,153,96,22,137,49,102,196,15,162,201,204,72,38,104,102,210,31,239,221,115,203,197,187,231,165,186,94,213,57,247,117,191,238,233,11,143,233,153,233,238,119,223,185,231,119,170,234,87,85,191,58,210,189,52,128,194,46,193,227,197,
135,3,153,30,136,17,195,24,49,156,76,230,74,166,85,55,21,98,86,122,67,249,42,142,78,33,104,14,170,131,204,10,8,49,72,221,68,231,185,60,154,22,103,35,113,154,5,207,146,66,50,83,161,129,161,95,149,120,46,186,114,33,137,136,90,19,62,186,123,110,0,92,168,9,120,141,153,55,48,155,210,55,63,57,200,40,133,176,0,37,223,208,154,186,208,30,66,240,160,139,79,174,3,230,0,58,42,220,183,33,22,27,194,246,250,41,18,147,35,203,224,221,210,145,180,33,19,35,118,72,215,80,130,184,237,92,187,92,156,75,153,56,185,164,58,187,91,15,38,73,154,121,122,134,58,94,35,227,239,82,144,68,124,190,13,0,175,213,3,224,50,3,151,184,83,175,213,126,173,151,92,146,172,73,196,226,56,197,195,232,178,165,107,44,192,115,157,3,158,0,187,48,113,175,86,15,206,239,203,229,30,182,174,117,2,186,231,1,17,28,230,182,85,148,181,21,75,231,230,26,197,61,30,196,13,102,18,2,186,58,188,86,85,11,86,40,162,135,10,8,111,224,18,136,46,215,163,16,54,155,24,207,183,192,205,236,228,110,172,15,168,25,148,86,153,206,195,122,13,153,113,61,102,
167,197,117,157,235,38,193,83,25,52,45,86,12,32,20,64,105,21,104,110,227,112,212,22,90,185,52,43,87,40,247,76,35,246,136,229,14,183,176,187,71,117,190,104,47,174,230,68,117,82,107,113,205,156,94,54,217,7,233,121,10,97,179,126,43,198,184,14,156,99,224,203,156,73,128,234,197,96,141,200,195,238,102,96,38,226,113,189,112,219,18,120,198,120,71,161,116,55,164,193,50,117,126,30,136,168,244,98,198,214,170,148,73,87,72,255,91,34,149,90,133,73,234,12,120,7,83,132,223,178,176,138,3,57,129,40,237,233,65,119,96,142,196,251,84,176,199,173,144,237,61,156,251,69,140,177,238,192,112,166,203,52,212,214,67,214,165,232,108,49,41,50,111,116,8,193,84,169,152,71,90,158,26,126,215,228,126,124,18,75,0,195,83,208,148,185,149,1,51,54,129,213,73,252,238,242,222,37,112,60,215,77,178,106,222,32,2,192,175,206,158,123,60,43,220,75,109,71,42,108,227,157,1,0,19,1,34,79,179,91,207,5,198,44,141,125,38,197,178,136,192,11,13,112,169,5,110,98,44,214,21,105,4,90,67,179,118,80,216,221,100,179,195,116,82,90,224,25,58,
185,128,156,213,88,133,21,178,40,110,11,64,146,61,74,167,229,24,192,118,183,9,166,221,166,224,3,90,67,93,15,105,137,202,48,22,91,23,172,138,6,175,66,193,18,77,92,229,165,149,161,18,197,61,20,96,151,247,173,73,145,206,74,93,98,224,133,57,128,218,170,122,51,182,237,89,0,95,75,230,84,214,193,145,193,227,235,169,219,178,218,53,118,15,121,114,8,64,20,58,208,188,43,3,30,96,81,148,34,44,1,162,189,88,42,202,184,115,114,146,222,28,60,216,41,196,63,233,38,55,164,116,1,175,120,237,106,33,251,85,193,215,179,246,212,60,45,139,170,171,24,114,86,104,63,168,247,52,237,66,39,105,7,176,123,147,12,162,231,44,87,213,155,136,113,6,160,73,140,145,136,158,4,240,213,244,44,27,34,4,161,145,173,123,41,8,59,27,234,228,148,131,212,255,113,173,45,17,1,56,218,37,74,53,85,61,48,252,117,121,56,180,240,75,148,60,16,173,138,157,211,86,40,249,235,99,248,133,149,19,204,148,119,22,134,52,203,249,64,61,221,179,100,241,134,138,210,183,122,117,116,210,209,211,40,240,102,72,233,152,243,32,218,255,173,74,7,157,96,245,
98,225,89,205,59,158,252,113,140,113,110,129,186,235,20,102,85,9,239,79,255,48,21,15,181,82,15,65,50,25,67,5,160,169,248,179,57,36,224,73,137,82,15,60,236,108,230,118,9,16,173,44,198,16,95,203,24,104,136,197,36,163,85,39,38,7,12,199,46,167,100,245,183,64,189,143,100,166,210,51,173,197,179,151,210,204,114,78,173,28,51,105,213,52,146,98,192,74,147,26,172,188,81,31,23,83,87,129,231,230,70,69,113,216,232,105,32,30,169,131,119,220,183,103,211,239,154,3,168,174,235,11,77,211,156,4,112,183,102,46,128,157,82,64,18,60,35,177,41,73,60,36,221,123,113,208,224,57,226,88,158,161,65,207,231,250,222,195,1,128,40,167,18,35,105,87,75,27,64,215,109,89,227,220,61,61,53,56,64,149,253,95,242,32,153,116,63,151,38,169,203,161,104,177,80,220,57,87,170,85,122,130,128,157,123,236,187,103,42,85,155,39,215,179,237,84,136,44,32,89,5,171,86,139,121,88,4,208,73,170,235,139,104,154,157,0,154,78,167,45,17,157,0,112,87,183,215,230,204,133,158,240,54,16,224,25,138,77,153,172,22,173,192,173,217,139,223,174,75,116,
100,125,155,55,252,75,79,192,211,214,104,25,16,113,15,10,27,40,207,85,34,195,157,179,166,19,88,122,209,203,140,115,247,38,155,107,240,200,196,185,28,120,188,236,156,38,235,179,234,66,85,8,118,215,74,145,80,231,106,106,106,220,138,191,218,212,107,38,42,233,165,69,181,10,71,157,196,234,54,1,39,30,105,154,118,193,2,205,82,57,116,142,153,207,0,184,83,63,120,237,23,15,149,235,70,202,39,142,200,183,64,236,199,85,51,207,139,66,19,112,18,120,70,6,105,144,147,146,181,154,5,219,93,156,144,125,227,36,46,0,79,215,211,89,122,103,37,197,78,171,19,216,179,2,242,115,38,215,108,91,197,16,147,21,211,204,220,105,176,183,133,170,247,74,49,130,214,48,97,86,30,4,11,32,73,75,31,29,203,205,54,65,116,6,68,231,100,205,225,14,0,181,109,187,69,68,199,1,220,222,97,3,36,172,205,154,178,58,114,67,90,211,231,154,3,180,58,99,230,57,96,142,8,183,205,202,245,244,217,180,94,241,172,39,254,231,129,196,155,150,214,231,30,60,11,230,137,168,151,154,20,57,99,233,0,123,64,116,20,49,176,174,48,216,175,58,199,57,241,161,
230,40,233,233,235,86,79,143,158,109,202,6,41,196,88,44,28,109,144,159,138,136,153,222,228,241,71,152,183,118,28,218,11,15,137,232,233,206,10,221,145,78,245,177,216,152,35,229,10,73,22,38,141,106,220,58,32,0,85,93,35,92,2,202,17,241,245,24,118,109,27,12,226,64,111,180,214,176,60,122,156,139,181,81,185,199,198,207,185,127,109,134,66,205,89,165,85,197,101,250,254,100,181,137,165,196,180,255,193,44,205,211,41,186,231,104,224,129,218,1,128,158,78,167,9,7,207,13,22,107,124,134,137,158,214,21,239,11,0,106,219,118,131,136,30,6,112,27,128,113,218,160,18,64,181,98,75,18,19,179,133,217,216,195,20,88,238,71,69,66,133,89,70,126,44,114,35,35,245,181,116,47,173,133,102,44,39,75,172,79,176,182,96,169,172,141,174,55,189,182,38,188,135,188,146,5,56,234,17,175,113,230,123,245,184,78,201,184,206,227,222,3,72,83,4,195,234,212,14,181,158,155,235,163,229,123,1,187,224,213,177,64,91,4,60,252,232,172,107,33,15,32,0,8,33,252,166,109,219,199,9,184,107,40,78,245,148,128,36,193,198,72,240,164,215,54,118,
87,130,111,157,212,41,23,37,107,151,52,72,36,88,172,177,141,132,126,74,147,192,162,200,184,150,90,178,54,88,11,95,169,38,231,30,17,118,95,249,144,83,198,9,5,112,245,125,22,26,64,41,23,181,38,8,132,171,251,24,231,38,235,163,159,173,213,77,42,15,243,4,4,42,228,150,114,34,142,237,78,111,227,113,14,225,148,213,209,107,2,40,198,56,33,162,135,2,243,157,67,224,198,161,114,135,18,219,150,154,230,182,21,120,74,167,18,137,197,241,244,230,52,51,36,91,172,7,198,73,228,105,214,145,99,45,74,39,56,59,192,139,25,159,219,106,26,211,128,177,36,113,105,73,0,121,236,29,161,127,197,247,50,57,169,180,254,177,3,208,142,252,9,51,54,247,201,10,201,18,166,161,65,175,235,26,181,180,225,147,148,176,4,18,148,251,102,185,207,150,5,2,240,58,3,15,157,104,219,169,119,143,230,181,190,190,126,190,185,114,229,193,26,248,193,0,168,229,102,141,216,57,113,78,150,151,52,133,19,169,82,153,238,145,227,110,233,218,41,75,131,59,39,112,65,25,118,198,218,36,148,113,219,52,51,167,187,40,45,5,127,43,143,64,70,158,103,55,237,
226,188,132,21,47,1,147,151,4,209,216,56,169,153,25,219,43,46,35,170,187,248,118,108,0,200,235,57,106,157,61,208,244,56,124,28,43,212,180,192,131,245,250,250,121,92,189,138,165,0,116,229,202,21,190,142,232,177,1,243,23,42,224,235,158,52,145,46,204,203,49,51,85,39,210,56,18,174,192,26,118,38,99,107,44,150,124,88,210,74,86,150,216,82,201,180,6,42,193,160,110,173,239,39,7,124,218,242,180,153,252,141,213,221,8,228,75,133,250,198,64,108,176,114,220,35,38,130,243,158,185,248,42,1,200,114,89,3,128,43,204,216,194,222,187,103,169,139,113,211,30,41,145,65,80,36,65,132,93,69,208,20,200,26,171,223,45,2,63,171,136,30,251,209,213,171,156,179,146,238,245,22,243,230,141,192,125,21,112,107,0,110,213,111,182,140,224,131,244,103,147,47,125,68,48,103,99,44,202,13,201,13,230,129,102,89,159,222,107,27,176,172,85,174,237,88,151,196,176,19,243,232,146,160,214,96,3,67,230,62,61,208,44,67,58,244,173,38,247,128,36,203,185,6,198,26,37,175,96,3,179,238,217,148,31,106,151,120,223,160,246,199,72,197,187,50,
223,232,197,161,193,57,72,245,171,53,14,54,96,65,147,225,165,10,184,239,9,230,205,146,155,153,189,254,2,188,122,59,240,29,2,158,32,224,61,22,237,151,128,148,99,221,72,5,163,35,101,137,116,71,40,57,44,22,80,206,232,123,155,64,51,48,33,19,167,228,70,17,198,30,174,155,206,67,132,37,65,212,246,136,97,218,93,90,46,44,1,54,46,196,83,218,189,75,41,140,169,144,148,98,216,195,6,180,119,81,59,108,159,69,8,161,71,222,203,18,126,212,154,116,186,223,167,219,207,111,76,129,111,63,3,188,218,39,78,43,94,131,181,181,211,213,230,230,189,0,126,8,224,168,57,186,177,7,253,92,169,69,146,47,171,100,158,50,180,44,59,193,190,117,154,146,195,90,121,99,57,180,43,151,27,127,110,77,47,35,227,97,67,129,199,202,61,245,1,145,151,48,12,123,96,244,250,88,110,15,68,65,49,116,219,80,210,187,61,88,202,224,228,121,44,61,118,47,142,9,40,151,65,69,3,64,250,144,142,192,219,83,224,187,245,218,218,25,108,110,246,74,171,20,175,127,54,13,110,169,170,151,137,249,45,6,238,156,2,245,118,151,243,217,232,254,204,49,111,164,136,
131,177,50,209,35,69,34,120,113,14,245,96,184,56,195,174,81,198,117,243,54,104,52,24,26,79,234,168,111,126,9,61,25,56,50,44,171,39,216,78,133,184,112,149,86,73,111,124,93,49,62,80,121,185,177,120,201,184,119,236,236,5,77,26,84,40,75,139,121,158,138,71,72,57,19,70,182,8,184,159,170,234,196,115,147,73,175,174,138,170,239,34,254,149,153,63,22,194,243,145,121,58,5,110,223,2,234,13,1,160,92,240,88,25,102,121,100,144,7,171,200,114,151,132,33,188,25,55,128,173,144,233,21,110,182,240,231,116,230,92,10,244,248,63,42,184,84,81,185,70,108,196,93,251,37,136,162,1,26,140,100,231,32,19,203,140,212,30,240,242,120,90,99,161,212,110,239,61,99,171,153,47,216,44,238,22,1,15,80,8,199,255,212,182,189,117,76,170,101,22,241,21,230,246,35,33,252,97,155,249,191,91,192,231,55,128,225,70,231,247,122,241,143,180,62,26,68,3,135,223,247,30,86,9,52,37,122,55,100,126,167,53,194,34,22,128,228,37,103,189,28,13,50,44,33,50,76,162,53,73,64,54,180,161,199,225,128,21,131,201,114,145,42,44,182,69,200,196,182,254,187,
149,199,171,51,108,107,233,94,188,216,42,39,148,223,125,255,219,4,220,79,33,28,127,185,109,151,170,64,171,150,93,188,191,49,183,55,85,213,31,55,153,255,190,1,220,177,9,28,153,20,172,143,117,194,88,139,169,153,171,62,109,17,222,184,20,50,8,4,20,44,15,247,0,78,169,234,121,25,75,131,158,224,209,150,199,42,187,119,78,213,149,3,167,4,36,111,38,143,22,52,244,132,70,66,15,75,202,206,243,244,126,214,74,135,136,159,127,3,192,61,168,170,71,255,188,132,229,217,53,128,0,224,223,204,124,195,218,218,75,87,155,230,204,6,112,91,75,244,94,151,128,48,128,50,48,2,69,157,32,235,243,240,91,135,102,134,195,168,5,228,135,65,89,224,233,11,166,101,99,158,92,37,129,103,121,116,234,64,15,168,170,236,13,178,47,214,7,153,216,43,100,54,109,159,57,171,228,188,79,174,130,36,100,222,63,243,187,95,2,240,141,106,109,237,228,11,61,99,158,149,0,8,0,46,55,13,54,128,87,153,232,41,0,55,1,184,69,135,47,105,186,157,6,138,54,221,53,242,37,56,125,192,211,22,204,123,31,53,208,82,188,227,205,138,209,247,146,35,60,188,255,
203,185,166,122,98,129,108,217,214,2,36,125,93,159,85,90,35,111,211,231,220,241,210,26,33,99,125,188,60,85,105,141,197,186,52,0,126,74,192,183,158,3,94,188,208,236,190,111,160,218,235,66,50,240,191,16,194,47,1,188,6,224,147,152,53,130,206,173,207,208,32,9,114,175,220,134,7,50,35,82,10,116,50,101,226,38,47,65,26,51,44,92,201,2,121,229,60,94,29,92,174,197,193,18,193,144,58,102,186,212,233,32,0,148,99,16,251,88,41,143,61,43,125,93,114,125,75,241,26,1,23,8,248,30,17,61,112,6,184,188,215,53,168,86,177,144,204,220,172,175,175,63,63,157,78,159,2,240,110,0,55,7,230,90,38,194,106,199,180,91,254,113,159,198,179,82,101,181,85,62,83,2,162,37,112,158,251,218,138,133,0,187,26,161,228,86,88,159,55,189,135,30,115,40,201,131,96,88,242,190,76,28,47,1,142,101,227,162,190,128,42,49,104,84,32,126,250,196,153,244,14,203,246,19,2,238,174,142,30,125,250,183,147,201,116,21,123,191,90,213,105,52,153,76,0,224,114,85,85,79,49,243,217,33,240,161,17,240,129,1,16,6,40,143,5,180,152,151,220,195,46,205,44,42,
197,83,22,229,108,197,55,13,242,19,202,44,183,173,196,252,120,65,178,53,217,79,130,103,91,89,31,171,87,38,71,253,246,1,206,178,0,202,157,252,216,133,11,23,50,108,226,178,7,66,114,215,8,248,125,0,238,9,33,28,127,150,249,63,255,152,78,87,181,237,251,85,34,44,115,197,24,39,0,158,25,18,157,174,153,191,88,3,247,86,192,103,3,48,178,106,233,82,80,44,85,103,74,83,17,114,249,150,62,137,68,118,220,184,190,22,202,42,225,201,5,207,185,127,243,238,81,186,110,219,2,68,81,60,184,42,19,220,107,114,98,191,181,41,44,114,196,170,188,144,245,104,242,176,203,229,209,244,164,112,253,185,90,219,251,216,6,112,154,128,239,7,162,167,127,205,188,177,155,9,125,171,60,104,118,117,125,144,104,92,49,127,58,0,223,36,224,75,0,142,1,160,212,36,39,91,178,199,176,171,178,169,16,252,195,177,2,148,217,156,86,220,19,141,96,93,235,134,89,226,129,232,65,221,6,227,107,175,140,72,131,103,75,88,159,180,153,134,98,237,82,70,95,42,36,237,133,198,94,197,166,96,135,229,180,42,41,74,224,209,149,33,158,144,35,1,76,192,197,0,
156,36,224,68,32,58,247,172,210,48,88,245,85,239,55,128,254,53,251,0,191,251,104,93,159,230,166,121,31,207,20,127,190,194,192,103,90,224,88,4,42,41,43,148,78,86,221,78,208,87,35,160,4,30,46,60,228,146,240,134,181,209,74,101,34,218,154,230,250,150,228,184,24,221,46,210,8,114,6,133,207,123,173,69,149,201,137,9,45,13,60,111,148,168,62,28,163,109,213,34,128,139,0,206,2,248,57,1,167,66,93,95,58,213,52,241,32,38,246,93,147,117,254,112,85,85,20,227,13,21,240,241,33,240,185,17,240,169,53,224,230,49,112,108,12,28,29,1,85,13,80,157,113,231,114,212,113,206,253,107,13,43,100,101,248,173,17,131,165,4,102,142,9,147,247,85,99,81,79,28,216,217,103,149,44,79,42,149,218,22,27,71,247,83,233,158,153,189,150,67,173,114,83,120,174,113,159,26,66,245,189,220,205,44,127,187,1,46,69,224,149,8,60,215,2,167,25,120,145,170,234,205,83,157,220,238,65,31,20,215,252,250,68,85,85,227,182,93,27,51,223,48,6,110,28,1,55,14,128,99,53,80,87,14,32,114,173,217,132,178,242,205,50,0,210,195,146,181,255,174,193,227,109,
96,93,55,102,1,72,118,249,202,90,195,228,190,85,6,120,60,215,247,48,109,10,206,184,224,109,38,38,237,92,180,166,5,46,70,224,245,8,188,30,137,222,140,33,108,254,234,26,0,70,95,255,7,109,251,165,112,33,68,162,122,0,0,0,0,73,69,78,68,174,66,96,130);


//additional icons (B..D) are optional -----------------------------------------
//Note: Supported image format: tea
//Note: Each icon, if specified, must be 32w x 32h.  The icons form an animation of upto 10 cells in total.
//      When triggered, the app icon - (top-left) of window - will animate and cycle through each of the cells
//      in a continuos loop for a short time period of about 2 seconds.
//How To Use:
//      Use rootwin.xhead.aniAdd(xtep,xms) to add upto 10 cells, and rootwin.xhead.aniPlay to
//      trigger animation playback for ~2 seconds.  To extend playback, call rootwin.xhead.aniPlay
//      regularly.
//
//Example:
//to setup the animation ready for playback:
//      rootwin.xhead.aniAdd(program_icon32hB,100);
//      rootwin.xhead.aniAdd(program_icon32hC,250);
//      rootwin.xhead.aniAdd(program_icon32hD,200);
//
//to play the animation call:
//      rootwin.xhead.aniPlay;
//
//to clear the animation (e.g. to create a new animation) call:
//      rootwin.xhead.aniClear;
//
//Note: the examples above must be executed within "tprogram" for access to rootwin, see
//      the unit marked "main" for tprogram.

//accessed via "tepIcon32B"
program_icon32hB
:array[0..0] of byte=(0);


//accessed via "tepIcon32C"
program_icon32hC
:array[0..0] of byte=(0);

//accessed via "tepIcon32D"
program_icon32hD
:array[0..0] of byte=(0);


//help viewer banner images - optional -----------------------------------------
//.banner 1
programbanner
:array[0..0] of byte=(0);//disabled

//.banner 2
programbanner2
:array[0..0] of byte=(0);//disabled


//app help document(s) - optional ----------------------------------------------
//note: these help documents, if specified, are processed and merged into one large
//      document at runtime by the built-in help viewer, which supports documents in
//      the following 3 formats:
//
//      1. txt - plain text
//      2. bwd - enhanced text (Blaiz Enterprises format)
//      3. bwp - advanced text (Blaiz Enterprises format)
//
//      These documents can be compressed (using zip) or left uncompressed.


//app help - app specific help document, which appears above system help
apphelp
:array[0..0] of byte=(0);


//system help - information relating to the app's system settings and general behaviour
syshelp
:array[0..12716] of byte=(
120,1,236,157,235,114,27,87,150,165,171,38,230,23,35,250,29,50,24,209,97,169,155,212,133,146,108,151,34,186,43,40,138,178,56,37,137,10,146,178,218,83,83,63,146,64,130,68,9,64,162,112,17,197,122,140,121,159,121,183,249,214,218,231,100,38,32,202,150,123,106,58,44,27,14,87,73,0,50,207,217,103,95,215,190,100,250,201,219,167,247,127,199,63,195,201,160,254,223,252,57,42,39,253,121,175,156,86,143,139,123,255,180,53,191,158,47,170,241,188,119,89,141,227,139,171,89,57,157,47,174,71,124,186,255,79,91,131,122,178,152,148,250,233,160,94,206,134,213,172,120,85,93,197,215,243,225,223,117,205,94,124,234,213,163,122,230,21,117,139,63,237,61,46,246,30,61,74,23,199,138,108,120,126,145,46,189,255,245,55,223,60,122,184,151,238,23,133,139,234,195,226,255,140,126,247,187,63,47,254,114,112,89,78,46,170,98,113,89,21,190,124,56,185,40,234,129,63,151,211,233,87,243,226,187,55,71,91,207,102,245,216,95,45,234,105,177,168,235,209,121,57,43,122,163,97,239,93,81,79,138,237,63,47,170,233,227,122,186,24,214,
147,249,222,189,191,20,199,241,215,237,162,206,87,105,121,150,43,198,213,100,153,174,135,75,127,217,46,110,105,197,217,240,226,114,161,93,175,134,147,126,125,117,187,248,247,98,59,175,113,167,40,246,217,34,127,76,151,240,199,104,84,244,135,243,233,168,188,230,146,211,106,84,245,22,38,49,168,9,190,64,203,129,216,181,93,44,202,115,8,47,230,151,245,85,81,22,163,225,220,251,249,170,34,100,50,191,179,181,117,6,153,219,190,163,56,181,160,230,219,113,237,112,94,176,213,112,81,12,39,172,178,184,156,85,85,49,45,103,139,249,227,173,251,119,138,39,203,225,104,177,123,52,9,14,230,245,138,171,203,97,239,178,40,103,85,177,156,87,253,162,156,23,195,249,214,222,157,226,96,57,95,192,207,149,205,89,179,92,20,189,114,82,156,195,169,254,95,185,68,183,76,248,31,247,79,150,227,243,106,198,23,247,181,119,189,188,184,44,238,223,219,122,112,7,214,244,139,211,242,61,191,172,172,182,147,247,30,205,235,245,69,125,74,214,212,186,223,222,251,215,226,220,196,15,215,136,111,121,90,79,208,142,186,120,63,172,
174,138,225,130,239,207,102,215,197,246,147,81,217,123,183,93,12,144,240,112,50,95,148,147,94,197,79,111,203,133,78,60,183,32,26,245,145,228,71,195,42,190,13,58,123,214,58,24,50,41,102,85,57,90,12,199,21,236,223,47,88,118,248,247,226,112,178,168,102,211,217,112,94,205,191,90,57,88,113,235,95,238,156,247,230,183,97,36,223,143,125,73,223,234,122,85,77,22,215,113,173,246,129,151,58,223,172,154,79,209,201,225,249,168,50,169,222,92,26,94,189,175,56,69,57,159,74,107,214,213,93,103,188,170,125,131,244,118,48,195,44,119,138,201,16,70,248,188,227,105,61,227,196,222,162,204,123,246,202,209,8,49,156,13,23,236,229,125,230,59,150,159,239,27,215,80,163,155,161,122,140,201,252,29,174,242,123,186,233,148,213,250,229,44,9,81,138,120,88,194,199,27,37,83,92,74,143,22,24,102,125,53,41,150,147,225,223,150,85,49,175,90,117,158,67,254,126,209,251,88,199,10,40,172,175,58,82,224,175,181,212,45,132,209,215,177,209,22,228,177,64,59,110,90,97,7,213,158,213,50,60,237,189,168,175,68,179,88,84,161,134,137,
139,54,45,219,136,200,234,201,39,20,11,49,165,143,101,153,168,109,54,122,142,54,23,215,245,242,43,22,27,96,244,168,243,77,27,22,243,81,189,208,129,204,15,105,98,178,144,56,30,10,34,170,147,241,151,190,90,159,203,107,255,181,192,52,97,197,188,194,157,194,174,65,136,5,187,29,85,139,5,223,116,125,136,252,152,78,98,87,196,109,7,201,185,89,247,123,151,117,61,23,79,38,216,128,101,203,21,199,51,228,231,171,100,165,151,245,8,86,212,69,127,86,94,232,96,72,122,57,155,215,179,187,156,238,2,103,94,98,181,92,166,45,224,97,85,77,194,182,123,127,91,14,225,68,103,229,117,147,128,65,217,88,164,209,115,27,123,185,196,129,148,139,161,244,231,26,117,217,194,145,127,55,124,111,150,206,90,42,147,39,210,242,168,240,22,210,189,104,46,10,59,12,215,151,46,64,173,138,49,34,146,78,196,62,58,208,112,254,142,211,54,17,224,64,254,52,123,199,112,142,183,146,115,198,109,251,215,219,232,73,235,143,87,54,194,222,209,215,185,181,66,108,11,6,134,203,86,108,80,244,120,169,24,241,239,133,131,138,200,208,119,242,113,
197,254,252,206,157,59,210,158,179,235,169,121,198,161,204,198,88,69,188,221,214,133,219,88,206,98,81,79,184,242,7,75,194,36,167,163,90,232,28,80,122,138,78,34,20,164,211,220,218,223,110,148,54,233,243,199,231,77,252,126,78,36,129,63,184,246,144,183,88,188,234,134,219,176,249,241,34,197,71,76,91,181,45,206,98,75,90,78,22,195,145,54,144,159,196,39,88,129,146,89,197,142,62,114,191,107,84,179,36,189,21,206,175,158,27,214,36,171,65,230,156,66,30,58,43,210,1,65,232,168,168,250,210,6,12,201,10,183,182,212,226,178,94,46,240,68,239,229,75,185,125,86,237,234,58,20,232,143,91,63,84,50,216,180,122,88,66,151,255,131,12,37,18,221,63,205,239,27,88,37,213,49,129,98,135,105,195,190,59,6,172,16,52,171,108,91,114,16,251,19,124,61,183,96,45,43,198,180,98,70,171,42,223,93,97,167,168,70,195,241,112,130,193,233,184,108,57,169,16,53,231,174,62,16,216,122,120,183,224,211,170,102,200,41,151,197,57,113,242,34,140,127,56,46,47,176,194,75,249,62,217,242,195,71,157,232,187,126,221,220,20,87,131,1,194,38,
82,204,64,105,218,221,236,35,222,98,251,150,219,132,69,209,99,14,121,6,186,185,113,199,108,140,25,67,97,91,79,26,162,164,55,141,109,103,88,212,161,37,97,163,188,6,46,88,91,193,2,161,185,229,180,207,230,235,81,252,180,198,44,111,88,2,117,1,52,205,234,105,53,91,200,33,7,60,234,87,120,200,10,255,137,53,93,86,200,105,58,173,0,151,73,154,173,175,188,139,215,185,82,92,199,169,58,216,149,62,186,194,75,98,82,220,223,98,194,226,89,7,156,224,146,68,150,67,96,142,236,67,12,101,49,171,38,23,139,75,92,185,148,20,14,115,225,84,210,181,190,52,91,212,129,104,197,242,14,62,147,106,117,84,106,86,13,140,66,87,64,77,97,231,44,103,209,175,49,171,240,229,86,34,227,10,40,32,3,96,83,48,190,247,156,3,43,223,1,33,50,101,119,227,116,127,52,54,141,91,56,178,220,26,156,154,203,6,185,175,24,213,232,198,37,46,173,6,214,216,83,72,89,36,4,145,44,149,29,41,170,223,255,195,31,238,161,76,87,86,65,179,123,48,156,225,240,203,126,61,155,112,104,45,251,172,156,47,14,8,236,161,128,175,235,171,106,22,31,111,225,182,57,
109,203,24,110,32,92,40,8,17,150,185,225,54,250,143,106,92,35,100,180,114,54,148,85,84,51,209,199,90,16,48,35,207,168,202,249,245,238,162,222,189,152,13,167,50,130,210,194,0,167,13,255,158,141,75,154,197,101,220,46,254,218,108,65,177,101,31,153,112,152,37,78,16,215,220,175,122,245,12,43,192,203,11,55,6,99,18,56,24,151,253,138,224,46,199,142,6,76,202,217,76,135,16,21,242,163,115,160,251,192,64,150,188,34,228,202,1,176,175,126,61,22,160,64,254,33,129,96,188,17,71,41,28,221,17,252,78,49,46,223,137,94,124,228,57,145,63,118,154,47,207,193,57,192,2,118,5,96,10,218,142,107,187,72,46,3,159,18,233,97,137,80,32,80,229,178,156,97,8,232,221,168,42,39,162,206,122,111,28,189,181,245,10,207,122,33,129,33,188,142,217,62,19,18,109,60,125,155,198,52,64,177,163,82,176,28,23,51,133,179,236,156,28,68,131,215,206,107,226,227,216,87,44,64,186,243,145,152,53,15,210,156,123,100,89,120,65,176,230,26,79,164,3,210,216,143,121,111,73,194,68,8,47,241,186,86,107,251,172,123,211,15,197,173,9,168,90,103,184,
173,223,191,217,211,87,211,225,135,106,100,213,17,136,217,207,192,6,132,44,2,181,15,250,37,114,150,83,67,76,50,87,127,109,103,58,223,34,88,125,133,79,64,166,82,114,229,181,197,85,53,26,253,177,40,94,213,114,54,32,255,49,2,9,115,177,44,131,194,246,114,40,113,148,74,241,2,20,98,196,121,60,11,18,16,29,59,175,70,10,12,240,225,159,76,197,183,127,42,128,243,67,204,46,204,229,236,123,161,247,79,137,143,124,125,123,107,235,57,217,46,170,49,175,71,75,227,160,188,0,78,64,202,141,154,182,139,39,111,150,236,124,128,99,116,38,17,76,43,110,217,105,193,237,62,40,249,118,129,62,129,30,101,31,147,12,50,201,132,38,82,130,247,195,122,57,47,46,42,52,205,70,99,56,156,55,198,204,14,200,146,136,151,156,125,116,173,180,197,238,253,106,24,113,62,75,160,60,31,146,2,203,117,203,208,146,163,166,26,80,204,199,138,172,236,138,12,128,67,157,117,207,240,216,168,207,121,213,43,5,148,56,155,32,116,190,192,44,132,101,197,69,137,207,84,78,86,92,56,253,152,117,248,131,191,138,120,199,30,161,80,56,2,185,190,249,37,
254,250,157,240,178,85,81,97,137,180,144,127,175,46,9,35,108,130,78,199,65,50,168,45,149,7,34,111,29,136,164,92,165,12,12,89,41,171,148,167,21,9,34,60,150,198,172,42,227,96,57,137,116,166,95,195,130,5,155,65,89,171,156,90,66,174,139,51,54,208,67,174,73,121,42,206,112,46,175,173,20,196,249,95,226,64,103,79,42,9,123,127,42,254,13,23,189,119,239,170,248,64,66,244,237,189,75,62,23,123,59,247,190,121,176,243,245,189,123,69,72,125,11,237,248,183,226,193,183,15,125,217,222,253,175,227,178,111,119,246,254,240,112,231,97,123,25,170,249,111,197,55,95,127,235,203,30,62,216,243,101,15,30,236,220,255,230,155,238,106,80,200,130,183,150,163,197,172,36,130,160,154,142,200,200,185,158,220,206,146,18,202,149,123,128,41,74,213,231,197,173,135,31,244,163,60,140,237,23,118,160,18,78,75,57,197,173,193,18,125,248,104,177,30,41,144,18,123,234,38,242,236,93,177,218,131,221,168,116,87,245,146,204,42,201,26,121,61,186,247,207,226,163,183,146,220,187,174,184,63,28,12,134,61,142,34,193,66,178,227,7,62,222,
74,143,11,137,18,28,34,87,165,192,180,88,140,197,57,122,149,4,25,230,78,84,227,114,31,205,94,128,79,59,74,42,226,242,5,233,110,228,206,35,49,36,228,46,87,24,218,134,160,225,69,235,25,138,116,132,37,88,110,22,0,206,209,33,171,122,156,13,104,184,147,243,106,180,247,111,75,184,196,229,183,246,30,253,243,237,230,192,32,20,80,32,225,170,171,241,82,58,107,244,92,42,157,141,246,28,218,132,103,164,203,16,84,32,150,172,116,80,140,224,48,20,220,113,70,178,97,18,29,29,54,131,211,103,44,88,224,151,72,158,246,245,138,168,77,103,69,11,111,206,238,212,205,204,156,50,252,231,160,236,225,29,117,130,189,123,136,238,214,222,7,210,195,229,52,243,219,30,91,110,62,252,185,61,130,118,29,203,35,68,57,144,221,251,40,29,101,28,113,182,71,220,239,115,174,240,83,253,2,141,103,85,169,227,58,223,145,254,25,43,169,54,129,105,75,43,56,114,56,46,169,43,97,196,33,87,149,36,214,38,27,164,162,83,203,7,73,141,224,16,31,243,249,90,215,160,220,230,7,80,132,60,160,46,158,17,57,67,5,214,156,5,85,148,37,92,118,4,51,44,
119,64,171,7,20,114,161,120,199,220,216,41,30,136,122,216,163,83,40,112,212,139,234,241,214,41,229,9,233,159,248,224,187,222,151,35,138,60,232,116,247,114,159,183,43,86,169,53,84,11,140,36,185,129,121,19,236,194,41,217,215,189,83,142,70,210,187,156,203,23,114,150,35,149,122,225,203,165,152,49,193,163,9,34,233,112,66,14,242,114,228,212,207,246,182,139,119,130,230,138,188,215,114,0,90,140,223,17,108,58,188,43,197,56,142,18,54,43,2,42,150,31,180,149,101,75,89,186,131,107,32,8,90,141,110,235,156,40,102,243,69,152,198,215,90,123,239,33,164,253,88,20,77,124,53,220,6,76,9,138,132,180,37,19,5,35,111,134,210,121,231,20,69,9,148,40,85,230,77,170,170,136,7,68,16,37,7,170,6,14,49,174,92,208,140,227,203,188,196,218,180,136,192,18,213,106,66,145,1,173,33,166,227,159,216,46,248,193,22,61,50,28,233,111,94,80,56,23,214,54,14,4,126,219,222,250,113,100,81,68,181,200,26,103,186,201,227,164,0,240,241,84,60,163,54,198,29,137,187,161,37,78,101,117,109,192,188,249,114,170,154,36,7,196,149,233,248,98,
170,84,21,67,125,115,142,167,91,26,219,229,131,17,225,147,94,116,152,143,22,238,207,116,124,73,63,194,181,207,131,56,194,80,0,86,67,234,125,92,215,176,160,30,12,196,191,164,67,117,79,53,175,157,98,225,28,164,191,236,101,37,110,119,1,193,151,125,3,100,138,190,132,103,123,21,231,9,33,202,73,13,51,112,146,82,18,149,55,62,86,165,230,39,161,254,124,25,204,97,53,82,116,248,177,146,36,34,228,25,14,72,224,54,29,2,47,33,126,183,137,165,4,22,247,40,19,227,56,63,166,119,208,115,108,184,175,173,231,148,104,48,168,237,83,119,118,232,49,216,248,2,252,170,132,102,84,148,252,26,92,59,31,46,198,116,124,56,125,42,36,187,160,211,80,24,213,98,175,209,199,51,59,134,133,87,198,171,75,153,164,10,66,91,194,183,144,121,6,142,81,242,55,38,201,152,1,218,116,75,214,124,184,32,23,111,3,149,210,207,179,151,18,217,220,91,188,85,8,192,41,36,144,20,167,209,222,178,62,33,53,96,17,141,42,196,67,92,44,222,83,131,16,194,229,55,234,155,106,160,180,53,85,18,153,105,56,124,18,27,154,26,45,195,87,56,45,130,13,14,99,
171,230,96,62,12,122,30,245,194,228,13,36,249,214,52,248,73,201,84,66,78,186,223,170,33,245,14,133,149,91,47,46,234,154,234,219,101,61,236,17,71,57,25,110,142,239,149,5,74,189,186,246,49,174,213,252,89,3,5,136,197,187,112,167,80,182,51,103,254,254,214,13,169,121,241,135,71,72,30,61,239,18,40,43,54,217,21,26,157,161,251,246,211,234,175,229,247,75,10,150,19,122,71,14,108,2,192,18,206,10,17,210,193,117,26,138,91,237,118,58,2,118,88,125,40,199,211,145,242,165,75,149,140,201,22,93,136,158,146,217,124,64,179,200,16,161,2,121,89,200,139,170,236,67,230,235,106,134,76,34,217,77,229,1,59,34,160,81,199,98,208,40,236,197,133,35,45,221,41,161,166,42,125,202,141,209,214,158,253,125,196,118,241,181,158,132,174,198,61,124,81,26,191,163,29,169,189,246,76,50,10,222,40,102,71,24,71,92,7,81,71,119,186,213,200,177,21,180,203,79,156,236,61,66,139,2,76,135,170,227,63,53,101,93,212,133,250,35,168,208,169,52,89,166,84,57,105,155,188,53,217,134,220,249,114,194,247,125,99,126,172,15,189,180,114,235,187,
162,28,8,91,185,172,47,31,21,229,247,150,162,93,157,82,110,48,162,162,49,21,23,157,115,167,46,146,251,100,235,106,120,49,193,141,78,46,150,96,183,244,3,27,97,28,215,99,138,2,170,72,162,122,94,53,252,4,210,189,16,116,65,49,118,67,160,243,33,210,85,189,228,163,88,11,51,127,50,192,118,113,130,183,25,0,58,228,62,101,48,35,43,159,191,110,164,79,207,10,7,63,164,182,50,185,216,122,6,173,227,18,45,90,170,21,73,142,178,24,58,191,195,149,168,38,160,175,232,147,216,122,42,32,187,116,173,123,127,2,211,17,127,42,174,173,175,195,243,37,8,2,139,18,53,157,214,71,213,135,83,57,42,183,174,2,243,169,145,8,157,50,131,30,234,114,254,40,153,205,201,102,151,11,10,0,220,55,41,94,28,60,77,158,251,167,210,108,236,128,238,207,51,41,226,179,160,67,103,118,134,24,53,31,131,193,220,115,138,244,195,225,46,81,109,82,170,222,101,244,215,4,23,177,98,186,109,192,139,185,19,81,20,141,158,166,85,138,106,10,205,58,123,119,139,155,244,200,53,161,219,22,65,118,158,18,5,191,144,255,86,179,219,24,195,145,2,145,92,20,
216,66,54,20,205,57,171,52,232,95,173,117,121,143,62,69,35,53,182,133,44,58,164,133,132,81,18,197,25,96,20,214,140,98,187,143,106,109,85,121,94,46,198,240,62,214,180,64,113,231,66,9,18,61,72,109,184,11,234,37,121,168,22,132,24,12,175,135,26,138,50,81,3,116,188,170,103,239,92,87,87,111,107,221,89,41,208,170,189,125,137,81,144,71,36,231,38,187,224,52,142,240,200,43,206,108,176,221,175,85,168,73,254,40,157,228,166,203,168,15,219,199,12,112,182,56,17,197,80,52,218,80,71,46,24,210,82,51,89,222,81,113,33,39,58,169,12,76,81,171,233,180,101,117,139,206,137,88,216,232,92,151,99,10,36,206,86,181,81,194,222,169,84,34,124,165,82,44,229,85,126,140,138,160,29,153,135,38,20,149,83,1,25,92,243,142,139,83,205,112,56,166,23,52,132,213,110,170,196,226,74,233,40,159,40,245,28,79,151,242,63,57,187,16,186,180,175,72,117,3,182,220,126,81,95,169,21,101,28,232,110,254,72,101,90,234,188,170,100,226,25,114,241,168,79,105,7,255,108,7,67,37,37,55,254,229,188,0,172,68,109,120,40,103,152,83,230,148,174,71,
174,144,155,143,202,212,59,103,231,218,237,151,208,191,28,123,130,99,91,5,171,104,245,159,131,249,181,123,66,249,204,50,216,194,78,5,156,21,94,246,187,14,198,198,54,174,168,33,41,1,243,0,64,114,210,82,11,129,137,228,235,173,169,29,229,214,252,204,76,216,195,45,60,163,203,240,175,168,143,147,214,246,90,124,149,162,215,68,144,42,119,184,146,101,34,166,51,100,18,139,107,195,0,11,86,208,52,177,176,35,45,72,181,65,175,212,53,177,14,21,184,9,76,87,54,37,149,52,36,223,179,117,158,87,8,132,230,140,129,159,92,134,203,186,173,150,217,122,2,106,162,188,103,170,98,107,178,163,161,9,159,210,158,165,213,165,8,157,136,59,18,191,243,106,113,165,142,241,246,83,10,211,140,204,184,186,146,21,207,165,212,237,23,202,73,248,137,98,50,203,55,191,177,169,171,170,170,16,44,40,40,96,230,238,55,184,153,160,80,41,61,73,46,4,121,248,26,152,9,243,169,181,81,89,25,210,0,81,77,92,69,22,34,133,203,227,24,184,8,72,167,50,14,109,78,101,72,148,116,118,114,115,13,251,52,116,122,190,13,109,77,227,233,84,196,21,167,
2,224,40,125,84,217,50,19,114,145,190,216,85,126,44,210,200,144,125,189,28,156,14,33,168,212,130,12,171,29,132,232,215,148,51,25,230,178,223,203,124,158,245,21,46,135,253,62,12,118,161,218,131,46,40,170,81,202,162,156,191,99,214,73,247,102,6,172,223,219,112,102,135,182,194,200,165,106,19,246,190,28,142,12,62,228,70,163,213,161,200,7,85,141,66,60,107,248,184,190,40,122,217,50,57,147,239,164,26,55,239,226,128,170,255,84,63,145,90,102,67,166,57,152,19,45,240,157,226,105,53,127,167,25,43,5,147,182,186,238,107,150,211,21,61,136,201,15,137,154,33,39,165,175,186,197,45,12,249,151,9,161,64,182,3,91,176,172,79,228,72,93,225,158,121,72,202,68,42,203,8,160,208,96,96,207,77,117,41,213,96,84,162,213,131,77,77,57,37,147,170,120,126,6,247,186,55,153,46,247,179,240,196,13,22,111,234,182,14,212,113,44,182,231,52,249,104,82,152,237,63,227,126,46,232,70,40,219,248,139,18,103,15,31,29,182,179,71,119,240,212,154,255,130,205,183,64,182,179,5,110,192,101,38,171,70,208,161,97,5,121,26,193,53,154,61,
225,110,124,203,146,138,122,51,148,22,152,3,53,226,148,58,68,150,138,175,76,128,36,34,172,24,197,191,231,172,69,115,168,145,118,135,120,249,153,36,253,44,242,188,92,154,8,248,204,147,209,6,140,100,184,69,68,153,137,63,231,72,196,100,184,35,60,142,39,33,209,17,40,114,121,141,252,88,109,73,142,83,209,73,32,192,124,188,143,193,10,170,150,116,56,156,176,152,162,212,141,58,6,43,102,153,177,236,173,85,201,50,117,16,194,37,251,25,4,212,57,175,168,46,26,22,240,13,139,94,127,69,26,60,46,39,132,94,112,54,185,32,136,86,109,206,133,51,226,100,143,248,16,36,243,128,105,186,117,165,75,211,80,243,187,153,8,41,142,101,150,104,158,39,175,58,170,1,86,202,117,88,210,218,145,131,124,14,200,64,1,50,130,156,223,53,190,54,157,187,109,53,136,206,159,195,252,87,174,24,42,51,37,15,74,231,9,254,89,133,220,226,203,66,181,9,64,157,191,100,112,114,18,128,123,69,117,229,79,237,6,215,78,161,219,146,148,27,197,134,86,11,46,196,138,123,80,47,57,69,228,142,53,176,36,64,79,10,112,69,0,105,120,160,208,115,225,10,
32,168,72,67,115,81,94,146,153,2,143,188,112,28,196,185,56,252,54,142,233,149,234,68,228,34,119,98,31,97,116,56,121,95,143,222,51,90,75,125,35,5,61,66,43,165,10,133,62,215,103,50,154,66,27,153,218,192,191,85,125,187,191,113,249,87,202,234,224,178,86,27,88,111,253,158,20,40,105,129,64,31,196,8,112,100,161,127,158,63,220,111,2,25,193,201,37,103,42,181,40,227,118,61,185,91,77,20,41,250,119,179,164,182,1,215,216,166,6,78,151,48,79,60,217,166,214,118,23,72,23,23,102,25,84,12,44,249,210,229,132,139,57,249,193,146,134,123,159,126,30,253,252,25,61,165,19,19,76,205,92,31,237,192,35,27,71,145,89,20,50,192,182,35,37,83,167,164,92,76,189,81,94,120,188,213,216,65,56,208,168,57,16,149,113,111,160,218,81,125,145,162,237,156,60,182,186,115,113,199,227,89,59,140,253,86,84,6,140,8,171,69,143,223,76,153,138,22,30,20,166,62,26,221,123,151,9,215,141,48,21,9,32,164,28,48,234,133,227,123,188,117,56,153,83,67,111,202,176,136,37,174,146,175,69,128,115,134,82,99,182,19,195,203,5,61,229,46,168,165,186,
84,193,56,183,161,151,231,13,201,196,176,56,4,44,192,57,228,37,164,82,173,35,114,188,70,174,251,154,28,98,88,112,60,53,96,245,116,95,198,26,6,132,40,141,70,115,240,251,180,102,200,17,196,215,214,154,139,105,61,119,39,13,78,83,223,53,176,209,170,204,32,206,231,238,167,132,106,218,95,74,39,52,86,138,163,162,135,210,179,11,212,56,116,187,136,67,88,180,155,13,67,89,234,52,42,91,150,172,210,122,236,200,87,57,187,203,9,6,43,15,134,23,203,212,250,149,176,53,110,115,138,21,204,47,31,111,61,141,41,106,83,222,128,222,247,213,93,32,25,25,4,168,94,67,214,228,225,137,195,240,218,70,21,158,15,18,222,52,129,206,211,130,82,80,182,56,201,233,232,243,106,52,5,222,196,224,232,37,31,48,159,6,223,155,88,248,14,188,198,95,9,96,103,157,236,76,230,197,36,165,146,214,52,76,25,122,164,170,58,253,160,89,157,250,20,66,57,201,134,115,53,61,169,11,119,226,24,150,80,245,92,91,61,222,122,174,54,65,52,123,218,13,229,148,197,23,211,72,194,226,76,185,75,43,234,81,34,140,41,40,250,124,121,30,199,60,147,74,63,222,
58,168,199,156,137,33,115,132,87,77,98,212,37,27,87,8,132,253,195,254,18,241,61,37,219,96,121,100,6,41,11,10,118,26,159,121,90,47,89,55,166,79,161,243,212,169,91,154,61,50,142,116,6,38,93,133,148,126,92,108,243,186,203,10,86,2,203,186,205,158,140,134,83,220,77,224,43,236,137,118,47,19,126,132,44,234,187,221,133,2,108,50,20,6,211,82,245,145,218,114,96,249,64,30,56,161,119,212,129,165,190,209,39,93,165,68,144,52,213,10,115,226,50,112,57,68,94,230,120,2,84,146,58,224,248,155,216,39,172,99,171,141,226,191,180,43,57,41,238,56,228,64,245,248,250,113,194,254,214,189,84,139,64,19,229,150,85,174,86,102,236,169,176,169,192,107,209,187,102,172,70,10,141,39,124,253,6,153,145,53,131,11,82,63,6,25,129,14,220,224,108,51,55,144,224,148,130,70,220,20,105,188,221,52,96,123,201,236,4,247,176,157,72,78,232,157,153,228,160,204,12,227,151,25,51,53,218,212,225,10,95,162,186,172,10,131,125,158,230,0,5,81,111,30,214,180,141,209,80,10,17,74,24,137,35,253,17,19,25,115,213,158,229,69,51,30,195,43,113,55,
69,149,84,252,17,67,232,191,157,215,42,97,14,39,68,71,23,78,168,222,70,113,104,12,125,238,32,49,26,7,53,17,5,187,230,130,114,97,167,146,168,91,246,93,157,102,102,77,234,23,89,196,118,247,72,244,51,234,233,176,135,8,30,236,81,220,42,190,75,12,146,36,168,192,215,120,134,74,182,110,190,212,253,242,26,247,108,70,147,184,96,196,81,227,136,80,45,190,81,212,139,117,146,83,226,208,44,138,189,129,214,40,197,34,71,208,153,202,107,200,50,58,39,112,204,126,66,245,103,234,164,212,133,29,222,181,159,245,169,179,201,108,25,88,102,239,161,40,205,162,132,116,79,76,21,221,220,77,22,158,53,47,198,165,204,246,38,139,131,81,103,72,57,126,66,12,229,136,2,29,253,209,200,16,169,121,169,16,131,39,248,56,61,59,29,85,213,59,38,163,53,33,79,206,40,243,213,102,169,197,226,92,154,2,80,120,212,230,34,104,81,91,21,145,171,92,163,94,149,80,228,128,9,113,17,210,177,81,212,46,173,27,87,211,135,81,223,202,8,30,50,91,104,221,66,3,206,159,104,138,249,174,31,35,40,77,128,125,54,53,233,122,19,254,153,164,40,174,63,
105,6,79,201,227,48,62,70,47,129,27,25,25,225,138,100,61,221,113,89,50,175,24,118,69,121,99,226,179,234,223,238,148,45,224,18,224,215,118,153,43,113,142,1,176,196,157,55,89,18,101,52,116,185,142,129,20,93,202,35,77,121,137,110,91,192,162,111,121,169,153,95,226,146,110,232,80,149,107,43,178,25,52,85,135,178,238,80,69,143,162,44,219,211,225,103,173,54,46,202,25,36,1,113,23,122,205,194,94,87,74,148,240,3,183,221,180,203,173,123,187,60,218,229,122,113,28,83,180,71,131,67,77,251,232,105,200,252,225,79,219,240,216,185,169,165,240,51,218,247,251,160,19,143,199,135,54,11,70,38,172,31,190,6,189,163,193,32,71,111,173,212,220,137,124,173,31,11,81,130,132,137,56,70,141,137,95,52,159,99,144,239,130,225,191,229,12,118,50,200,171,0,134,179,116,193,148,184,137,166,190,52,98,121,25,232,6,77,157,226,17,92,163,166,11,129,13,165,25,187,85,91,213,161,111,52,197,23,158,53,245,179,56,73,233,161,202,223,37,68,112,73,23,168,161,195,63,184,194,174,67,12,113,235,208,115,0,225,92,145,214,72,159,220,152,
100,199,149,85,184,120,127,52,218,61,154,236,30,79,52,189,209,241,48,184,68,1,85,105,159,31,153,211,95,81,167,213,219,187,154,162,92,12,31,104,176,169,43,187,81,58,47,226,162,227,77,235,108,109,189,117,187,146,16,27,219,237,22,47,170,193,226,241,150,254,31,71,166,126,85,94,196,169,21,148,127,116,71,156,20,68,19,231,255,201,235,79,84,121,68,65,244,199,39,246,56,240,252,255,129,114,13,170,147,2,75,241,55,121,117,129,38,2,54,38,170,170,86,68,95,135,56,35,112,65,134,195,9,115,36,189,170,143,159,208,67,10,17,194,202,254,123,127,41,248,78,110,223,140,128,227,59,83,213,69,118,34,132,168,37,82,82,116,74,175,130,77,242,66,32,191,72,150,136,23,253,33,199,85,245,86,211,60,181,29,68,190,249,121,57,147,250,97,98,90,242,72,51,252,184,246,253,40,228,230,169,113,133,117,253,156,186,253,44,129,109,166,219,2,173,17,195,125,49,38,163,39,64,161,234,59,106,199,45,45,23,124,218,117,25,95,84,8,49,172,209,113,72,83,17,131,57,3,61,144,25,197,135,150,129,218,92,42,44,31,166,70,7,152,129,49,122,16,12,21,
225,25,122,108,192,155,140,128,6,130,83,150,85,151,99,234,66,0,230,3,1,186,29,114,134,143,122,36,21,146,96,169,154,204,226,101,5,164,224,65,44,169,188,62,180,196,132,8,20,200,204,196,166,57,62,208,229,224,158,168,77,198,94,118,22,12,104,75,78,106,52,120,186,166,216,23,38,68,171,42,23,34,213,159,136,239,101,160,129,18,219,221,116,116,63,79,23,85,179,117,135,109,145,176,248,19,18,180,203,133,224,86,35,189,243,230,171,168,121,124,125,175,184,197,115,120,239,104,208,233,168,247,31,240,57,174,137,150,221,211,24,112,82,76,193,197,147,224,130,209,232,40,129,224,89,255,13,89,77,28,239,120,90,242,76,8,144,85,40,87,224,173,142,47,34,92,131,156,0,234,113,37,155,60,176,131,71,108,47,25,75,64,189,177,145,137,166,173,165,143,183,30,220,139,30,162,226,4,0,116,90,50,162,123,87,216,42,117,102,115,242,202,148,160,99,196,241,105,211,136,203,51,168,78,10,12,71,219,8,164,76,66,50,244,131,14,60,136,100,114,133,54,24,20,49,254,240,47,183,93,30,1,51,75,98,215,16,175,118,31,20,39,112,159,15,213,130,252,
56,147,68,180,204,188,224,92,177,75,104,156,24,183,123,95,193,241,84,157,44,194,196,173,251,233,132,120,6,62,223,141,17,37,125,127,79,249,144,21,140,253,19,141,159,96,105,135,157,143,50,59,79,171,241,112,119,133,153,143,254,191,50,19,31,3,252,198,185,197,140,131,107,104,224,131,230,217,195,102,70,63,34,232,180,38,79,81,185,192,136,43,198,97,124,79,96,95,216,236,120,5,167,246,91,107,77,80,40,89,170,102,9,79,212,155,57,54,115,119,138,215,212,77,119,138,31,168,52,170,41,245,122,57,67,77,118,138,125,250,142,59,197,19,38,8,119,192,240,244,145,252,7,221,29,63,194,187,195,40,208,112,161,145,25,215,106,237,171,178,154,75,146,7,126,100,19,73,232,137,216,254,53,1,48,30,58,12,103,134,100,240,161,156,27,91,6,194,186,107,204,4,250,104,73,118,243,184,216,222,239,151,228,174,36,119,187,133,51,238,24,136,233,126,237,240,234,71,250,60,28,132,178,121,2,175,89,52,61,31,73,40,36,127,104,156,173,25,165,240,200,117,54,148,53,207,122,230,159,232,126,142,51,236,106,82,223,204,66,83,93,108,223,97,133,
160,42,131,203,116,32,126,226,27,186,158,67,224,135,211,16,207,211,157,70,73,63,63,201,42,115,192,163,69,161,41,202,92,237,44,76,92,10,140,81,190,199,168,27,89,145,119,149,90,235,33,8,44,195,15,154,132,82,235,56,82,9,153,96,83,139,68,176,60,149,98,199,174,97,210,4,31,34,55,145,116,214,129,68,71,87,112,93,84,12,237,199,246,248,59,176,149,90,188,31,156,110,122,48,45,33,180,52,31,231,94,16,152,37,54,112,163,51,213,61,155,39,210,205,122,214,83,199,213,139,127,179,71,215,81,8,74,230,218,36,63,197,169,151,204,105,62,99,88,0,62,208,26,114,20,78,32,242,168,74,215,230,52,242,8,81,201,66,147,90,133,127,196,251,1,168,14,221,165,46,51,163,245,2,183,181,157,134,186,168,119,140,70,218,18,0,67,189,4,151,72,253,61,230,87,112,239,237,44,69,52,191,129,220,46,207,99,110,49,50,145,26,214,172,129,199,189,170,232,19,199,52,81,91,89,32,120,189,32,245,255,96,96,251,178,236,65,33,240,46,66,89,123,21,14,141,186,243,18,97,18,6,250,128,42,210,248,155,19,109,214,51,169,221,132,27,157,190,170,206,17,56,183,
47,22,211,249,227,187,119,175,174,174,238,32,213,234,242,111,119,234,217,69,193,106,79,105,89,68,86,8,215,188,4,155,158,83,6,39,18,6,156,140,135,124,200,146,185,32,234,50,219,255,243,241,255,82,131,245,36,205,232,38,139,86,189,246,137,31,141,197,79,185,40,43,173,139,167,111,161,139,82,235,165,30,182,93,191,75,79,221,166,152,173,6,184,238,73,15,16,177,161,62,29,231,10,85,192,79,120,204,82,55,15,7,183,72,239,214,90,137,93,57,206,126,119,40,244,199,103,145,209,126,128,58,45,135,49,231,36,131,84,156,93,92,83,176,176,76,53,40,208,96,74,204,134,167,199,34,178,65,27,20,203,100,35,158,201,109,41,34,183,183,38,210,211,100,28,179,131,77,111,171,49,130,247,67,230,88,57,183,246,36,69,172,70,3,104,80,119,246,53,111,69,136,22,160,126,138,14,76,192,21,100,74,118,164,201,32,97,240,233,237,14,255,117,169,202,147,153,197,126,240,153,26,205,95,58,47,199,136,32,73,201,229,29,207,131,185,134,38,167,185,198,174,132,124,59,15,140,97,127,134,213,225,226,210,239,162,60,47,146,252,99,34,255,185,106,35,
26,203,80,7,45,4,189,90,251,181,245,83,168,160,32,124,45,136,202,97,68,61,78,117,182,216,122,162,6,155,39,192,205,156,213,59,209,217,116,31,122,27,136,24,36,175,236,144,103,99,48,194,169,159,15,98,61,175,181,156,114,186,31,210,36,58,115,244,210,115,86,208,172,132,252,32,142,77,136,228,130,24,106,152,252,201,19,211,118,182,23,246,117,58,182,147,179,83,215,179,153,109,177,218,166,179,251,151,125,14,243,204,197,37,42,177,154,87,122,55,129,37,36,157,108,106,10,116,90,8,113,231,39,12,208,125,50,39,226,174,139,226,227,232,130,112,10,68,197,67,104,173,149,125,190,148,179,24,173,7,81,58,245,35,240,202,43,91,2,45,127,197,22,38,239,4,240,67,193,248,3,191,154,56,29,124,78,138,138,91,63,231,209,17,70,179,121,135,70,245,1,231,173,0,34,43,141,233,69,79,53,11,140,243,172,96,40,115,183,24,184,214,18,162,111,181,68,209,207,213,26,153,93,223,117,217,79,230,56,95,210,40,49,238,23,116,120,253,230,110,42,197,193,176,82,181,207,52,85,145,171,131,1,136,187,251,136,185,84,133,32,41,6,64,27,43,131,231,
74,98,209,78,21,77,13,106,169,197,187,166,42,183,151,95,32,226,185,78,38,53,121,227,3,21,94,93,217,94,115,255,30,53,184,127,109,12,55,206,64,205,178,91,49,78,47,182,32,6,229,162,78,252,236,241,71,246,217,27,184,148,186,207,241,98,134,244,129,23,109,159,220,185,31,23,28,9,220,41,228,167,169,58,41,96,240,192,2,50,80,234,186,28,177,115,135,43,168,83,235,82,21,173,251,244,184,180,13,255,187,129,191,20,86,193,121,72,106,159,78,144,219,62,77,133,88,161,158,185,167,40,14,139,143,97,201,82,64,180,147,73,233,81,125,14,101,169,106,188,218,155,75,240,27,131,68,135,66,16,81,1,171,144,46,101,120,141,156,139,64,193,40,252,31,180,205,24,30,156,209,138,203,146,66,66,174,106,53,49,18,26,207,112,186,50,226,21,73,211,246,196,142,5,126,60,37,241,73,27,238,122,173,180,64,54,93,169,195,199,99,201,217,232,85,95,230,2,239,204,95,217,44,155,186,186,42,4,85,134,59,53,221,238,58,95,130,7,14,104,232,32,204,50,30,250,142,148,27,168,54,3,97,56,221,165,251,235,43,9,164,154,27,123,8,170,105,31,170,196,2,250,
149,34,51,191,70,116,177,224,196,155,88,210,243,107,231,26,229,130,181,4,121,247,127,237,12,121,126,58,216,41,24,83,165,137,235,244,60,94,83,86,204,3,132,193,118,230,120,3,239,72,9,176,45,200,209,254,178,32,180,85,85,182,244,114,24,107,209,200,83,239,184,219,120,8,153,84,77,232,216,73,137,59,246,218,158,255,25,46,221,204,154,56,17,37,1,252,47,123,193,33,117,221,101,202,144,172,62,128,15,226,5,4,120,221,224,72,14,27,234,6,232,155,159,199,76,122,33,29,138,23,98,73,63,12,254,187,114,80,213,12,37,68,233,120,150,38,80,142,139,156,131,226,205,217,179,221,111,173,121,250,219,253,175,97,37,199,150,78,74,118,236,28,96,220,141,14,167,19,204,37,26,18,8,201,171,126,231,231,6,100,18,48,10,53,86,235,54,133,242,152,119,245,49,244,213,168,186,40,123,215,121,94,97,247,254,222,163,189,118,178,175,221,150,228,64,112,136,212,88,117,151,142,189,139,239,221,77,207,180,163,248,23,194,146,61,46,221,247,151,150,26,62,218,175,173,142,149,123,209,143,87,10,236,1,56,23,118,145,99,145,105,251,113,16,237,
152,186,46,205,230,156,16,39,224,217,219,156,97,177,164,222,168,0,241,222,175,153,139,141,166,185,126,98,156,233,175,16,12,120,82,21,92,156,35,38,51,128,231,202,57,246,164,66,16,141,43,189,38,132,47,61,53,103,98,51,255,212,146,34,46,198,211,55,6,43,84,140,197,162,149,211,88,235,4,185,102,67,30,211,241,56,53,182,65,28,186,44,246,79,15,142,142,82,139,81,242,0,251,10,66,133,32,26,169,35,66,191,59,9,53,58,106,113,158,38,202,53,245,154,128,169,182,196,249,67,103,227,150,36,138,107,67,191,152,207,243,88,8,64,91,9,53,215,169,88,78,36,37,98,125,164,176,169,0,5,79,39,197,203,33,5,112,13,175,103,234,56,232,25,128,145,199,104,149,248,177,163,220,108,81,242,255,0,42,105,168,158,158,19,57,180,24,201,105,236,17,46,52,76,66,238,215,22,13,165,206,86,193,186,142,49,240,1,245,118,30,32,171,144,33,79,114,171,21,18,51,181,201,5,120,229,164,197,169,35,222,212,206,7,163,242,61,183,216,6,63,34,119,103,53,201,113,79,14,161,75,171,82,227,139,9,134,152,177,128,60,102,156,40,1,169,64,205,115,13,140,55,
10,139,185,144,145,250,145,193,244,153,28,73,204,233,216,91,160,32,102,49,166,65,48,214,92,169,30,43,135,67,87,126,56,179,130,60,14,20,109,54,54,225,53,25,58,161,55,81,183,91,238,141,211,162,135,123,247,246,30,53,126,206,126,214,38,44,196,147,27,128,10,138,18,154,91,141,194,144,154,2,201,31,222,202,10,12,199,17,190,243,80,189,55,64,211,118,157,24,64,98,128,27,204,223,243,66,143,15,114,110,154,20,6,106,41,59,215,92,158,216,159,30,45,226,213,25,83,41,46,144,138,241,64,97,31,252,225,165,134,250,195,31,118,230,51,136,231,0,93,230,10,84,79,80,189,83,41,131,137,226,28,154,44,240,0,65,152,182,230,173,152,61,253,35,241,125,229,123,95,121,62,4,77,84,209,240,247,59,29,228,131,146,67,78,241,72,79,136,1,13,242,139,197,248,157,90,88,110,248,227,89,59,1,72,142,40,85,11,205,10,100,143,162,166,42,129,248,46,77,20,201,250,122,52,60,71,190,188,3,0,200,83,46,120,200,67,238,6,238,184,44,142,213,245,120,175,135,222,87,163,1,32,113,201,90,172,126,208,133,30,154,207,239,100,242,224,151,126,34,95,19,
162,135,216,83,217,107,247,168,242,146,70,46,10,75,126,243,72,170,44,177,38,92,224,169,70,193,59,72,162,229,203,134,172,70,111,128,136,202,225,8,245,246,137,104,158,205,10,249,50,159,167,167,34,133,114,205,153,62,72,85,55,235,177,177,244,118,48,189,88,70,82,121,43,112,32,210,1,23,210,131,21,170,196,15,198,230,5,126,128,72,249,57,97,23,167,89,247,88,138,203,76,146,220,134,30,22,235,158,71,129,25,104,66,19,70,110,78,234,199,124,167,94,254,193,220,176,94,39,168,147,64,117,138,95,114,62,102,112,36,31,141,21,178,72,46,176,37,132,199,121,165,105,67,214,193,139,46,205,79,156,131,245,219,167,183,248,74,32,113,79,123,232,147,36,30,121,107,168,114,67,86,83,88,163,127,93,237,50,187,15,211,232,237,49,45,145,203,201,240,39,249,55,45,100,80,209,86,226,51,67,148,26,164,153,117,74,160,89,7,59,10,104,10,164,30,29,22,225,4,209,35,129,37,249,222,85,214,25,52,203,41,25,159,134,137,59,241,202,153,4,188,119,120,229,126,66,12,227,97,82,131,134,48,78,25,0,53,221,30,16,46,133,106,237,25,143,27,129,13,
230,120,39,209,165,178,23,101,76,165,55,98,192,202,94,74,61,165,62,114,210,210,172,48,151,5,143,247,71,200,208,12,131,152,147,226,85,148,16,27,209,36,215,125,129,77,204,244,198,173,253,79,156,20,55,40,61,207,117,138,119,85,21,143,244,9,229,165,231,71,172,61,177,94,154,127,152,143,107,33,17,157,96,64,65,93,199,198,6,162,130,1,139,212,162,224,33,0,158,186,21,123,44,60,197,46,92,136,97,163,24,220,168,130,189,87,163,191,104,61,241,51,215,68,60,84,171,103,191,245,18,28,217,73,106,44,201,41,189,187,6,13,125,176,54,11,55,56,74,90,219,27,151,3,223,100,130,208,216,188,22,100,149,7,8,164,35,107,172,5,115,90,145,128,195,20,46,223,46,162,121,182,85,193,22,125,157,247,8,51,89,211,215,131,119,72,55,109,128,83,80,221,207,57,159,28,100,122,156,43,189,27,72,190,139,208,157,10,160,26,171,81,98,204,171,231,18,158,188,73,69,1,50,122,210,22,122,203,226,205,233,147,182,20,160,103,68,88,194,243,50,126,52,40,71,76,236,35,213,188,195,57,197,189,13,223,155,194,66,86,240,192,143,8,149,142,28,165,75,148,
68,128,220,118,43,20,209,232,134,126,65,7,9,90,201,161,24,62,75,53,165,28,234,6,90,6,216,8,53,24,82,172,107,63,107,98,250,228,141,168,58,7,171,4,163,228,92,80,117,128,160,6,20,253,154,75,121,16,113,198,143,45,245,129,130,10,14,176,135,234,34,71,87,215,67,176,167,77,36,46,151,51,121,235,252,34,22,237,15,169,57,143,199,109,65,64,115,104,45,14,253,204,126,167,57,24,233,138,221,2,130,211,169,46,121,29,42,171,165,2,31,7,113,152,95,19,72,222,203,103,98,185,132,176,227,197,33,185,221,140,164,252,5,239,179,100,153,163,246,253,135,249,46,161,92,237,57,14,30,171,198,227,231,60,228,24,244,53,62,92,179,154,230,106,154,241,140,203,61,169,144,189,102,240,242,138,49,99,103,27,36,94,233,205,92,90,79,130,192,165,201,224,82,196,139,17,196,228,172,147,27,249,10,13,10,167,33,120,26,69,197,117,237,198,122,129,40,60,201,130,205,231,80,160,136,182,191,46,186,182,244,128,16,41,115,177,226,149,235,93,201,235,136,23,200,218,8,31,93,107,159,222,202,62,77,74,36,73,52,52,193,96,224,141,178,169,85,6,91,219,
184,146,65,141,166,217,239,184,134,240,148,137,157,201,198,216,34,30,159,139,140,151,139,155,156,183,85,33,169,54,165,240,148,27,87,180,204,48,115,185,72,10,32,139,165,93,58,71,125,174,82,29,223,9,66,230,238,80,19,129,12,147,86,244,236,241,214,214,191,20,39,241,166,64,156,78,214,240,8,16,112,5,42,121,74,102,229,146,92,74,198,213,60,125,241,162,5,68,82,89,124,146,53,1,156,107,192,100,94,160,22,104,114,122,121,16,107,189,148,197,178,151,223,128,16,59,113,171,184,121,121,211,147,233,225,41,112,165,73,221,9,9,9,116,53,88,76,242,91,217,133,107,248,74,75,102,129,101,65,241,238,149,137,7,111,224,185,72,52,240,212,195,87,92,207,96,51,175,49,109,124,39,102,72,209,112,101,221,208,99,214,5,108,216,244,173,229,60,85,197,115,197,2,193,68,70,109,138,175,153,113,208,39,4,8,217,105,98,204,182,180,59,41,53,33,98,123,149,173,18,47,175,151,210,139,118,200,250,241,15,66,81,130,221,118,57,171,2,209,42,25,159,182,138,206,181,156,66,99,61,108,24,112,28,68,81,125,232,85,12,111,75,244,38,54,30,212,87,
110,246,126,72,75,76,21,104,109,8,194,92,240,80,38,236,80,77,53,67,121,207,229,203,249,117,174,6,102,165,215,116,105,116,120,236,34,157,251,204,0,123,1,63,206,222,62,146,9,251,171,224,68,198,120,127,31,50,153,225,116,221,54,236,71,34,112,210,174,22,226,4,3,214,194,73,138,116,29,35,22,137,26,133,16,45,17,40,178,26,88,223,4,129,217,230,24,189,83,93,14,42,44,227,102,28,83,97,196,81,30,206,12,232,246,243,172,128,175,243,245,70,171,138,88,231,51,21,18,20,143,112,30,70,251,18,138,30,85,213,226,175,73,4,53,177,69,172,215,31,81,158,20,61,13,132,40,167,67,61,15,31,239,60,151,16,58,91,181,60,117,112,140,173,98,40,0,97,174,59,50,137,1,74,50,54,233,178,183,35,9,81,213,177,221,28,72,155,219,90,251,32,148,68,147,204,163,155,141,146,96,174,173,52,239,188,58,60,147,21,252,143,253,239,247,27,43,105,29,37,247,239,182,241,143,67,72,209,105,94,232,17,150,117,242,181,76,123,224,14,197,16,252,10,144,149,194,130,116,13,132,142,232,89,3,1,79,82,159,47,236,165,165,129,141,58,191,226,220,114,88,49,143,
151,35,220,1,165,21,174,186,209,98,91,109,165,215,104,237,180,113,118,24,39,0,228,226,52,230,221,231,57,237,240,30,201,99,132,82,119,77,11,174,73,29,226,61,13,17,170,36,173,62,51,45,238,231,216,10,148,130,161,251,100,67,204,232,71,157,17,246,185,76,148,222,144,136,175,112,197,15,99,200,250,79,254,198,8,74,43,18,120,197,171,7,250,236,134,43,1,164,12,118,147,25,112,143,241,169,178,36,184,134,62,170,177,145,220,98,19,94,18,130,241,27,52,84,235,195,15,199,125,16,98,207,28,204,23,102,204,209,50,237,29,57,72,140,232,91,119,210,59,41,80,2,198,165,113,106,242,115,60,103,174,183,165,197,12,118,227,41,198,212,149,153,28,152,143,165,157,167,218,115,39,94,14,45,30,165,16,217,156,87,72,39,187,117,171,188,60,103,228,125,208,152,211,86,222,43,185,27,70,159,253,47,25,82,251,165,128,146,230,179,32,10,240,73,133,119,64,101,24,183,11,248,215,168,96,188,55,22,157,74,26,210,145,157,94,146,231,161,35,216,154,223,128,5,73,76,140,105,184,206,4,169,232,65,186,159,162,110,118,238,221,199,46,84,7,33,213,
104,50,212,159,48,234,27,237,66,188,250,222,47,185,18,48,82,73,67,129,167,211,211,143,131,115,208,28,80,52,40,205,75,46,200,98,20,77,83,109,216,98,139,217,240,198,227,117,29,137,142,222,120,49,175,223,3,29,59,235,222,181,127,30,144,62,241,242,210,201,34,106,51,169,31,186,250,44,155,217,226,23,6,169,70,147,186,195,224,38,63,253,166,74,252,26,14,245,219,2,59,249,12,170,39,85,150,147,94,73,106,36,31,128,66,126,236,81,0,161,121,28,145,74,126,196,65,92,141,205,43,26,117,138,120,41,192,27,74,217,3,52,32,59,200,182,210,249,241,71,46,17,60,55,100,194,137,136,164,230,133,121,66,78,49,138,47,116,161,87,194,146,21,168,188,7,81,26,46,78,117,212,180,109,76,160,42,107,108,135,205,209,53,98,172,252,44,175,145,160,42,236,36,239,204,229,188,200,99,127,250,121,112,30,240,61,85,39,25,119,161,215,119,227,30,38,239,58,143,5,199,231,27,31,211,244,149,122,63,79,144,171,143,200,1,106,116,200,204,135,246,241,68,105,141,188,143,31,189,233,199,171,213,164,218,181,167,4,84,167,242,164,133,170,205,150,118,
130,113,26,187,251,128,189,130,43,53,163,195,73,23,53,147,71,113,205,212,179,251,184,34,88,105,66,96,177,204,91,127,244,201,157,213,89,149,255,193,237,149,197,3,230,182,245,26,68,250,105,157,7,192,242,64,68,51,112,128,90,3,179,53,77,145,30,45,142,53,133,206,121,222,181,70,119,161,186,251,238,30,109,136,15,110,70,9,98,180,67,179,4,221,255,210,70,106,253,166,190,219,207,107,51,167,44,245,199,218,204,97,225,54,212,79,183,154,225,146,113,176,149,76,117,81,123,109,252,107,47,50,71,191,148,213,213,113,247,203,228,22,224,5,165,24,66,171,115,113,87,7,19,154,228,216,32,213,181,94,246,77,250,248,86,236,194,234,84,192,80,102,71,181,39,222,168,36,233,139,121,242,22,81,58,139,105,46,180,208,194,63,252,143,67,255,214,105,70,62,133,127,194,90,54,141,16,9,150,207,219,205,65,155,44,165,233,91,67,89,141,168,132,165,43,220,224,158,133,29,176,11,223,206,125,39,228,108,248,115,1,84,81,177,122,10,216,116,150,72,179,201,94,181,218,172,247,168,175,56,144,157,213,146,131,56,37,47,136,145,147,99,38,135,
166,199,239,232,226,234,168,242,75,202,253,70,234,49,196,153,227,53,104,161,126,71,92,240,71,105,178,8,115,51,5,134,233,45,75,222,179,187,64,28,54,150,112,222,193,21,28,5,27,204,172,86,11,22,233,113,51,107,129,12,98,189,172,238,145,139,203,232,125,61,227,130,178,90,43,70,244,28,93,25,212,119,12,163,171,149,160,183,157,105,15,21,32,62,99,208,8,39,169,23,26,185,96,130,17,210,192,79,111,158,64,237,72,9,80,5,93,145,184,44,152,24,69,134,230,181,135,18,86,140,188,248,170,237,147,52,219,148,135,170,120,2,215,157,41,100,217,213,47,31,0,50,27,93,147,212,147,119,148,218,105,150,71,187,222,176,94,242,127,182,124,195,150,76,30,90,158,71,108,50,108,96,215,51,37,193,182,72,136,215,179,46,113,150,206,197,77,227,40,213,161,236,215,87,156,237,221,167,63,246,158,8,187,94,212,217,230,145,30,80,96,147,238,216,37,190,8,88,227,199,82,9,33,100,43,110,25,170,15,144,116,239,229,209,25,205,166,158,170,77,60,1,48,189,14,149,115,79,231,163,255,234,75,113,203,3,113,105,30,238,92,255,81,24,44,39,255,55,97,
244,94,134,226,182,95,116,198,43,73,92,109,135,1,146,36,206,149,137,19,174,164,137,151,19,46,53,33,245,110,85,184,233,254,11,217,140,20,241,60,167,83,202,226,166,105,226,146,101,154,236,65,103,229,41,221,154,129,84,165,182,171,240,214,80,83,195,137,154,41,74,105,202,54,175,30,144,174,240,224,107,214,168,252,147,209,170,210,95,169,171,58,157,41,183,213,164,172,82,168,92,251,105,91,224,22,162,89,132,54,166,33,83,209,201,80,62,13,139,129,254,172,124,172,41,15,140,210,45,165,29,163,138,242,16,213,225,172,128,67,244,75,172,182,204,212,206,197,115,141,116,210,212,236,16,229,153,186,132,42,155,39,147,64,124,176,200,251,82,16,136,255,240,83,190,86,241,140,167,120,8,103,151,17,155,64,62,243,218,59,254,149,244,69,180,106,105,194,4,195,44,58,26,208,50,218,78,140,221,135,47,243,227,158,58,75,104,0,134,173,55,24,137,221,134,38,236,158,165,154,126,226,113,4,104,7,252,53,157,15,129,227,206,113,56,221,242,92,85,30,189,222,204,245,29,248,235,40,222,61,166,124,233,243,195,226,244,248,217,217,219,
253,147,195,226,232,180,120,125,114,252,253,209,211,195,167,140,42,159,242,121,155,105,232,163,179,231,199,111,206,10,174,56,217,127,117,246,67,113,252,172,216,127,245,67,241,167,163,87,79,119,138,195,255,120,125,114,120,122,90,28,159,20,71,47,95,191,56,58,228,187,163,87,7,47,222,60,61,122,245,93,241,132,251,94,29,163,229,71,232,58,139,158,29,23,218,48,45,117,116,200,125,207,138,151,135,39,7,207,89,121,255,201,209,139,163,179,31,120,94,254,232,236,149,214,124,198,162,251,197,235,253,147,179,163,131,55,47,246,79,138,215,111,78,94,31,159,30,178,253,83,150,125,117,244,234,217,9,187,28,190,60,124,117,118,135,93,249,174,56,252,158,15,197,233,243,125,10,81,218,106,255,13,212,159,152,190,131,227,215,63,156,28,125,247,252,172,120,126,252,226,233,33,95,62,57,132,178,253,39,47,14,99,43,14,117,240,98,255,232,37,175,136,217,127,185,255,157,168,59,41,142,89,229,196,151,37,234,222,62,63,244,87,236,183,207,191,7,103,71,199,175,116,140,131,227,87,103,39,124,220,225,148,39,103,205,173,111,143,
78,15,25,57,63,57,58,21,67,158,157,28,179,188,216,201,29,226,217,43,221,247,234,48,86,17,171,77,117,35,17,46,209,41,222,112,232,134,150,167,135,251,47,88,11,241,188,90,185,248,206,150,222,100,164,255,136,217,239,63,243,31,253,135,207,54,255,108,56,176,225,192,134,3,27,14,252,2,57,240,89,126,252,23,72,247,151,74,210,103,241,251,247,95,224,233,62,239,96,63,118,213,23,120,232,127,36,201,63,198,26,255,246,143,220,108,179,214,151,192,129,159,84,137,207,186,224,75,56,233,134,198,13,7,254,243,28,248,44,51,88,187,232,63,191,219,230,206,13,7,54,28,216,112,96,195,129,13,7,54,28,248,71,115,96,45,76,223,248,241,31,189,231,102,189,95,37,7,110,212,29,190,252,85,30,118,115,168,13,7,54,28,216,112,224,39,57,240,41,175,248,217,223,255,228,14,191,130,11,254,251,218,63,191,130,35,109,142,176,225,192,47,136,3,107,6,214,249,248,11,34,242,55,77,202,77,1,225,55,205,144,47,247,240,55,137,242,255,233,187,47,151,21,27,202,55,28,216,112,224,119,191,251,111,252,179,225,195,71,28,248,76,175,248,209,125,95,
250,23,155,162,216,151,46,193,13,253,27,14,108,56,176,225,192,134,3,27,14,108,56,176,225,192,134,3,27,14,108,56,176,225,192,134,3,27,14,108,56,176,225,192,134,3,27,14,108,56,176,225,192,134,3,191,52,14,124,102,207,33,93,246,75,163,254,179,223,5,242,201,99,254,226,78,244,249,4,221,112,166,207,191,249,87,121,229,58,71,126,149,135,220,28,234,183,199,129,117,197,254,244,231,223,30,111,54,39,222,112,96,195,129,13,7,54,28,248,47,226,192,167,131,79,243,203,127,17,37,155,109,54,28,216,112,96,195,129,13,7,54,28,216,112,96,195,129,13,7,54,28,216,112,96,195,129,13,7,54,28,216,112,96,195,129,223,30,7,154,10,204,207,250,203,111,143,79,155,19,111,56,176,225,192,23,192,129,159,229,199,210,197,95,192,177,54,36,126,1,28,88,213,189,47,128,224,13,137,27,14,108,56,240,75,230,192,239,255,175,0,0,0,0,255,255,3,0,37,231,137,29);


//additional program support files ---------------------------------------------
template_squircle_128px
:array[0..500] of byte=(
137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,128,0,0,0,128,8,3,0,0,0,244,224,145,249,0,0,0,23,116,69,88,116,83,111,102,116,119,97,114,101,0,68,97,110,105,32,118,49,46,48,48,46,50,54,48,20,195,73,175,0,0,0,30,116,69,88,116,98,101,46,112,110,103,46,115,101,116,116,105,110,103,115,0,53,51,54,56,55,48,57,49,49,46,45,49,46,53,41,132,140,74,0,0,0,6,80,76,84,69,0,255,255,255,0,0,15,158,76,95,0,0,0,2,116,82,78,83,0,255,91,145,34,181,0,0,1,79,73,68,65,84,120,1,237,155,59,2,132,32,12,5,209,189,255,157,23,249,8,18,252,20,132,80,140,213,110,192,204,99,80,43,117,110,197,99,83,58,122,107,221,69,209,195,69,109,80,161,215,186,13,160,71,143,139,16,253,175,1,196,240,160,165,215,109,26,70,29,160,25,170,207,26,250,251,194,169,2,168,109,189,76,95,161,74,128,170,40,207,24,93,41,176,51,64,41,141,134,117,251,157,184,28,224,44,116,167,43,20,51,48,5,200,127,21,72,119,45,19,50,6,48,224,59,23,161,121,11,238,114,170,215,67,0,19,1,73,193,18,6,140,4,68,5,43,24,48,19,16,20,172,96,64,253,78,123,4,236,
233,121,240,56,73,111,112,115,108,1,6,48,128,1,12,96,0,3,24,192,0,6,48,128,1,12,96,0,3,24,192,0,6,48,128,1,12,96,0,3,24,192,0,6,48,128,1,12,96,0,3,24,192,0,6,48,128,1,12,96,0,3,24,192,0,6,48,96,110,192,191,50,172,244,93,215,167,182,142,247,138,195,75,219,159,100,169,76,242,120,243,139,240,119,56,48,252,194,32,108,129,217,125,16,233,65,129,202,14,191,53,13,124,243,107,32,89,120,11,171,49,94,54,192,102,19,174,252,249,15,228,150,63,59,129,228,207,77,208,227,207,76,208,231,251,170,198,197,46,123,222,226,143,1,57,125,120,229,145,175,30,225,141,30,199,135,47,58,53,252,70,159,60,235,15,222,143,7,92,119,7,6,184,0,0,0,0,73,69,78,68,174,66,96,130);


template_squircle_2560px
:array[0..43086] of byte=(
137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,10,0,0,0,10,0,8,3,0,0,0,74,138,139,234,0,0,0,23,116,69,88,116,83,111,102,116,119,97,114,101,0,68,97,110,105,32,118,49,46,48,48,46,50,54,48,20,195,73,175,0,0,0,30,116,69,88,116,98,101,46,112,110,103,46,115,101,116,116,105,110,103,115,0,53,51,54,56,55,48,57,49,49,46,45,49,46,53,41,132,140,74,0,0,0,6,80,76,84,69,0,255,255,255,0,0,15,158,76,95,0,0,0,2,116,82,78,83,0,255,91,145,34,181,0,0,167,169,73,68,65,84,120,1,236,221,219,154,20,201,145,133,209,70,239,255,206,51,64,55,80,84,229,57,125,71,108,119,95,115,163,22,7,75,139,101,89,95,254,20,146,230,219,63,254,111,105,129,111,75,63,157,135,35,64,128,0,129,148,192,255,165,6,155,91,33,240,191,138,45,44,49,82,224,219,199,255,27,57,216,44,2,4,8,16,216,71,224,227,103,137,111,38,172,119,119,1,184,202,77,255,124,165,174,242,68,158,131,0,1,2,4,90,4,124,198,180,92,98,216,30,2,112,24,229,57,131,126,127,77,158,243,242,94,149,0,1,2,4,246,18,240,177,179,200,189,5,224,172,135,252,239,75,112,214,245,
237,77,128,0,1,2,115,11,248,24,154,251,126,2,112,186,251,249,146,155,238,100,22,38,64,128,192,186,2,62,148,38,189,173,0,156,232,112,255,126,149,77,180,176,85,9,16,32,64,96,19,1,159,80,179,29,90,0,78,113,49,95,88,83,156,201,146,4,8,16,216,92,192,167,213,60,111,0,1,88,126,43,95,76,229,7,178,30,1,2,4,8,124,18,240,201,245,9,164,242,223,10,192,202,179,252,92,234,231,87,80,239,122,54,35,64,128,0,1,2,215,5,124,136,93,183,105,248,25,1,216,112,133,47,59,248,178,249,66,226,7,8,16,32,64,96,62,1,31,103,181,55,19,128,117,167,249,241,213,82,183,148,133,8,16,32,64,128,192,139,2,62,215,94,132,139,254,54,1,24,229,125,114,184,63,41,61,9,230,151,19,32,64,128,192,28,2,62,224,218,238,36,0,91,46,226,15,72,45,151,176,7,1,2,4,8,100,4,124,210,101,92,95,154,42,0,95,98,27,252,155,124,73,12,6,53,142,0,1,2,4,74,5,124,226,149,28,70,0,158,125,8,95,10,103,95,192,235,19,32,64,128,192,177,2,62,249,142,245,190,248,106,2,240,34,203,65,63,232,75,224,32,104,47,67,128,0,1,2,101,2,62,1,79,62,136,0,60,235,0,222,250,
103,201,123,93,2,4,8,16,232,16,240,73,120,226,29,4,224,41,248,223,223,243,167,188,174,23,37,64,128,0,1,2,85,2,62,16,79,58,135,0,60,28,222,31,120,14,39,247,130,4,8,16,32,80,44,224,115,241,140,227,8,192,99,213,253,73,231,88,111,175,70,128,0,1,2,83,8,248,120,60,250,76,2,240,56,113,127,196,57,206,218,43,17,32,64,128,192,108,2,62,37,15,189,152,0,60,136,219,159,109,14,130,246,50,4,8,16,32,48,175,128,15,203,195,110,39,0,143,160,246,134,62,66,217,107,16,32,64,128,192,2,2,62,50,143,57,162,0,140,59,123,43,199,137,189,0,1,2,4,8,172,36,224,131,243,128,107,10,192,44,178,55,113,214,215,116,2,4,8,16,88,82,192,199,103,250,172,2,48,40,236,237,27,196,53,154,0,1,2,4,214,22,240,33,26,189,175,0,76,241,122,227,166,100,205,37,64,128,0,129,77,4,124,148,230,14,45,0,35,182,222,178,17,86,67,9,16,32,64,96,55,1,31,168,161,139,11,192,241,176,222,172,227,77,77,36,64,128,0,129,109,5,124,172,38,78,47,0,7,171,122,155,14,6,53,142,0,1,2,4,8,248,112,29,254,30,16,128,67,73,191,125,27,58,206,48,2,4,8,16,32,64,224,
167,128,79,216,177,111,4,1,56,206,211,159,79,198,89,154,68,128,0,1,2,4,62,9,248,152,253,4,242,214,191,21,128,111,241,253,249,205,222,150,127,44,252,19,1,2,4,8,16,136,8,248,176,29,198,42,0,135,80,250,198,244,16,70,67,8,16,32,64,128,192,29,1,159,184,119,128,30,252,105,1,248,32,212,141,95,230,207,35,55,112,252,20,1,2,4,8,16,24,43,224,99,119,132,167,0,124,87,209,31,69,222,21,244,251,9,16,32,64,128,192,115,2,62,123,159,243,186,240,171,5,224,5,148,199,127,200,159,66,30,183,242,43,9,16,32,64,128,192,48,1,31,192,111,82,10,192,55,0,253,1,228,13,60,191,149,0,1,2,4,8,188,37,224,83,248,29,62,1,248,170,158,63,123,188,42,231,247,17,32,64,128,0,129,33,2,62,138,95,103,20,128,175,217,249,99,199,107,110,126,23,1,2,4,8,16,24,41,224,243,248,69,77,1,248,10,156,183,219,43,106,126,15,1,2,4,8,16,24,47,224,51,249,37,83,1,248,52,155,111,56,63,77,230,55,16,32,64,128,0,129,156,128,15,230,23,108,5,224,147,104,254,160,241,36,152,95,78,128,0,1,2,4,226,2,62,157,159,37,22,128,79,137,121,131,61,197,229,23,19,
32,64,128,0,129,131,4,124,66,63,7,45,0,159,240,242,230,122,2,203,47,37,64,128,0,1,2,135,10,248,148,126,134,91,0,62,170,229,63,97,240,168,148,95,71,128,0,1,2,4,78,17,240,81,253,56,187,0,124,204,202,31,43,30,115,242,171,8,16,32,64,128,192,153,2,62,175,31,212,23,128,143,64,121,59,61,162,228,215,16,32,64,128,0,129,243,5,124,102,63,116,3,1,120,159,201,91,233,190,145,95,65,128,0,1,2,4,90,4,124,110,63,112,9,1,120,15,201,219,232,158,144,159,39,64,128,0,1,2,93,2,62,187,239,222,67,0,222,38,242,22,186,237,227,103,9,16,32,64,128,64,163,128,207,239,59,87,17,128,183,128,188,125,110,233,248,57,2,4,8,16,32,208,43,224,51,252,230,109,4,224,117,30,111,157,235,54,126,134,0,1,2,4,8,180,11,248,28,191,113,33,1,120,13,199,219,230,154,140,31,39,64,128,0,1,2,115,8,248,44,191,122,39,1,120,153,198,91,230,178,139,31,37,64,128,0,1,2,51,9,248,60,191,114,45,1,120,9,198,219,229,146,138,31,35,64,128,0,1,2,243,9,248,76,191,120,51,1,248,149,197,91,229,171,137,31,33,64,128,0,1,2,179,10,248,92,191,112,57,1,248,25,
197,219,228,179,136,127,79,128,0,1,2,4,230,22,240,217,254,229,126,2,240,111,18,111,145,191,61,252,59,2,4,8,16,32,176,130,128,207,247,79,87,20,128,31,65,188,61,62,106,248,103,2,4,8,16,32,176,142,128,207,248,191,110,41,0,63,112,124,251,240,207,254,145,0,1,2,4,8,16,88,74,192,199,252,135,115,10,192,223,24,254,104,240,155,194,63,16,32,64,128,0,129,5,5,124,210,255,57,170,0,252,207,194,155,226,207,155,194,63,17,32,64,128,0,129,53,5,124,218,255,186,171,0,252,41,225,13,241,235,13,225,95,9,16,32,64,128,192,202,2,62,241,255,189,174,0,252,238,224,205,176,242,151,186,103,35,64,128,0,1,2,31,5,124,234,255,208,16,128,223,251,239,227,219,194,63,19,32,64,128,0,1,2,107,11,248,224,23,128,190,253,183,246,215,184,167,35,64,128,0,1,2,95,5,124,19,112,251,239,0,122,11,124,253,178,240,35,4,8,16,32,64,96,117,129,237,63,255,247,254,43,224,237,207,191,250,215,183,231,35,64,128,0,1,2,87,4,54,111,128,173,3,208,127,6,224,202,23,133,31,38,64,128,0,1,2,235,11,108,157,1,27,7,224,230,233,191,254,23,182,39,36,64,
128,0,1,2,55,5,118,46,129,109,3,112,231,163,223,252,106,240,147,4,8,16,32,64,96,27,129,125,107,96,215,0,220,250,219,190,219,124,93,123,80,2,4,8,16,32,112,71,96,215,32,216,51,0,247,13,254,59,95,6,126,154,0,1,2,4,8,108,38,176,105,19,236,24,128,155,158,122,179,47,104,143,75,128,0,1,2,4,30,19,216,178,11,54,12,192,93,191,217,251,216,87,129,95,69,128,0,1,2,4,182,19,216,48,13,182,11,192,45,51,127,187,175,100,15,76,128,0,1,2,4,158,17,216,175,14,54,11,192,253,14,252,204,219,223,175,37,64,128,0,1,2,187,10,236,86,8,123,5,224,134,223,226,221,245,11,217,115,19,32,64,128,0,129,231,4,246,138,132,157,2,112,183,184,127,238,125,239,87,19,32,64,128,0,129,189,5,182,234,132,141,2,112,175,178,223,251,107,216,211,19,32,64,128,0,129,23,4,54,74,133,109,2,112,171,172,127,225,61,239,183,16,32,64,128,0,1,2,251,212,194,46,1,184,81,211,251,242,37,64,128,0,1,2,4,94,21,216,37,24,246,8,192,125,130,254,213,247,187,223,71,128,0,1,2,4,8,252,16,216,164,25,182,8,192,93,106,222,87,46,1,2,4,8,16,32,240,182,192,22,217,
176,65,0,110,146,242,111,191,223,13,32,64,128,0,1,2,4,126,8,236,80,14,235,7,224,22,29,239,11,150,0,1,2,4,8,16,24,39,176,126,60,172,30,128,59,68,252,184,247,187,73,4,8,16,32,64,128,192,15,129,229,251,97,241,0,92,191,224,125,157,18,32,64,128,0,1,2,1,129,197,19,98,233,0,92,62,223,3,239,119,35,9,16,32,64,128,0,129,31,2,107,87,196,202,1,184,120,187,251,242,36,64,128,0,1,2,4,146,2,43,135,196,186,1,184,118,184,39,223,239,102,19,32,64,128,0,1,2,63,4,22,110,137,101,3,112,229,106,247,85,73,128,0,1,2,4,8,28,34,176,108,78,172,26,128,203,30,236,144,183,187,23,33,64,128,0,1,2,4,126,10,172,26,20,107,6,224,194,223,178,245,245,72,128,0,1,2,4,8,28,40,176,104,83,44,25,128,171,214,250,129,111,119,47,69,128,0,1,2,4,8,252,43,176,100,86,44,24,128,139,166,186,47,67,2,4,8,16,32,64,224,20,129,21,203,98,189,0,92,178,211,79,121,191,123,81,2,4,8,16,32,64,224,167,192,122,113,177,92,0,174,119,34,95,123,4,8,16,32,64,128,192,201,2,203,229,197,98,1,184,226,55,105,79,126,203,123,121,2,4,8,16,32,64,96,185,255,73,
192,181,2,112,185,62,247,21,71,128,0,1,2,4,8,116,8,172,21,25,75,5,224,90,167,233,120,187,219,130,0,1,2,4,8,16,248,41,176,84,102,44,20,128,254,250,215,23,40,1,2,4,8,16,32,144,19,88,169,52,214,9,192,165,186,60,247,230,53,153,0,1,2,4,8,16,120,85,96,157,216,88,38,0,215,57,201,171,111,74,191,143,0,1,2,4,8,16,8,11,44,147,27,139,4,224,74,223,148,13,191,117,141,39,64,128,0,1,2,4,94,22,88,165,56,214,8,192,101,122,252,229,247,163,223,72,128,0,1,2,4,8,28,34,176,70,116,44,17,128,107,156,226,144,119,173,23,33,64,128,0,1,2,4,222,19,88,34,59,86,8,192,37,14,241,222,123,209,239,38,64,128,0,1,2,4,142,18,88,33,60,230,15,192,85,254,50,254,168,119,173,215,33,64,128,0,1,2,4,222,19,88,160,61,166,15,192,21,42,252,189,119,161,223,77,128,0,1,2,4,8,28,44,48,125,126,204,30,128,211,31,224,224,55,172,151,35,64,128,0,1,2,4,6,8,204,30,32,147,7,224,236,252,3,222,129,70,16,32,64,128,0,1,2,199,11,76,158,32,83,7,224,2,127,5,127,252,27,214,43,18,32,64,128,0,1,2,3,4,230,174,144,153,3,112,242,246,30,240,222,51,130,
0,1,2,4,8,16,56,77,96,230,16,153,56,0,103,102,63,237,189,234,133,9,16,32,64,128,0,129,81,2,19,167,200,188,1,56,49,250,168,247,157,57,4,8,16,32,64,128,192,153,2,243,198,200,172,1,56,247,95,188,159,249,94,245,218,4,8,16,32,64,128,192,40,129,105,123,100,210,0,156,183,184,71,189,227,204,33,64,128,0,1,2,4,10,4,38,77,146,57,3,112,82,236,130,183,169,21,8,16,32,64,128,0,129,161,2,115,70,201,148,1,56,39,245,208,119,155,97,4,8,16,32,64,128,64,135,192,148,89,50,99,0,78,9,221,241,30,181,5,1,2,4,8,16,32,48,90,96,198,48,153,48,0,103,100,30,253,86,51,143,0,1,2,4,8,16,168,17,152,48,77,166,11,192,105,255,235,54,53,239,82,139,16,32,64,128,0,1,2,99,5,230,171,147,217,2,112,194,198,30,251,22,51,141,0,1,2,4,8,16,232,19,152,45,80,38,11,192,217,120,251,222,160,54,34,64,128,0,1,2,4,2,2,147,37,202,92,1,56,25,110,224,237,101,36,1,2,4,8,16,32,80,41,48,87,164,76,21,128,115,209,86,190,59,45,69,128,0,1,2,4,8,100,4,166,202,148,153,2,112,42,216,204,123,203,84,2,4,8,16,32,64,160,85,96,166,80,153,40,0,103,98,109,
125,107,218,139,0,1,2,4,8,16,136,9,76,148,42,211,4,224,124,255,5,235,216,219,203,96,2,4,8,16,32,64,160,82,96,158,90,153,37,0,39,106,234,202,119,164,165,8,16,32,64,128,0,129,3,4,102,9,150,73,2,112,22,206,3,222,89,94,130,0,1,2,4,8,16,232,21,152,36,89,230,8,192,73,48,123,223,141,54,35,64,128,0,1,2,4,142,17,152,35,90,166,8,192,57,40,143,121,91,121,21,2,4,8,16,32,64,160,90,96,138,108,153,33,0,167,128,172,126,43,90,142,0,1,2,4,8,16,56,76,96,134,112,153,32,0,103,96,60,236,61,229,133,8,16,32,64,128,0,129,118,129,9,210,165,63,0,39,64,108,127,31,218,143,0,1,2,4,8,16,56,82,160,63,94,234,3,176,159,240,200,55,148,215,34,64,128,0,1,2,4,38,16,168,207,151,246,0,172,7,156,224,77,104,69,2,4,8,16,32,64,224,96,129,246,128,41,15,192,118,190,131,223,77,94,142,0,1,2,4,8,16,152,67,160,60,97,186,3,176,28,111,142,119,160,45,9,16,32,64,128,0,129,227,5,186,35,166,58,0,187,233,142,127,43,121,69,2,4,8,16,32,64,96,26,129,234,140,105,14,192,106,184,105,222,126,22,37,64,128,0,1,2,4,78,17,104,14,153,222,0,252,
214,204,118,202,251,200,139,18,32,64,128,0,1,2,51,9,20,183,76,109,0,202,191,153,222,225,118,37,64,128,0,1,2,4,46,8,212,230,76,107,0,214,130,93,56,174,31,34,64,128,0,1,2,4,8,92,20,104,13,154,210,0,108,229,186,120,91,63,72,128,0,1,2,4,8,16,184,44,80,154,52,157,1,88,138,117,249,178,126,148,0,1,2,4,8,16,32,112,77,160,51,106,42,3,176,147,234,218,97,253,56,1,2,4,8,16,32,64,224,170,64,101,214,52,6,96,37,212,213,179,250,9,2,4,8,16,32,64,128,192,13,129,198,176,41,12,192,70,166,27,87,245,83,4,8,16,32,64,128,0,129,91,2,133,105,211,23,128,133,72,183,110,234,231,8,16,32,64,128,0,1,2,183,5,250,226,166,47,0,111,11,250,89,2,4,8,16,32,64,128,0,129,55,5,234,2,176,175,145,223,20,246,219,9,16,32,64,128,0,129,221,5,234,242,166,45,0,235,128,118,127,199,122,126,2,4,8,16,32,64,224,125,129,182,192,41,11,192,54,158,247,15,110,2,1,2,4,8,16,32,64,224,159,178,196,233,10,192,50,28,111,87,2,4,8,16,32,64,128,192,24,129,174,200,169,10,192,46,154,49,231,54,133,0,1,2,4,8,16,32,240,93,160,42,115,154,2,176,10,198,
91,149,0,1,2,4,8,16,32,48,82,160,41,116,138,2,176,137,101,228,185,205,34,64,128,0,1,2,4,8,124,23,40,74,157,158,0,44,66,241,38,37,64,128,0,1,2,4,8,140,23,232,137,157,154,0,236,33,25,127,110,19,9,16,32,64,128,0,1,2,223,5,106,114,167,37,0,107,64,188,61,9,16,32,64,128,0,1,2,41,129,150,224,41,9,192,22,142,212,185,205,37,64,128,0,1,2,4,8,124,23,40,73,158,142,0,44,193,240,198,36,64,128,0,1,2,4,8,100,5,58,162,167,34,0,59,40,178,231,54,157,0,1,2,4,8,16,32,240,93,160,34,123,26,2,176,2,194,91,146,0,1,2,4,8,16,32,112,132,64,67,248,52,4,224,17,214,94,131,0,1,2,4,8,16,32,64,224,63,129,130,0,108,232,96,239,7,2,4,8,16,32,64,128,192,65,2,5,233,115,126,0,22,32,28,116,110,47,67,128,0,1,2,4,8,16,248,46,112,126,252,156,30,128,231,19,120,39,18,32,64,128,0,1,2,4,14,21,56,61,127,206,14,192,211,1,14,61,183,23,35,64,128,0,1,2,4,8,124,23,56,59,128,78,14,192,179,31,223,123,144,0,1,2,4,8,16,32,112,130,192,201,9,116,110,0,158,252,240,39,156,219,75,18,32,64,128,0,1,2,4,190,11,156,27,65,167,6,224,185,143,238,
221,71,128,0,1,2,4,8,16,56,77,224,212,12,58,53,0,79,35,247,194,4,8,16,32,64,128,0,129,141,5,206,12,192,83,203,119,227,155,123,116,2,4,8,16,32,64,160,64,224,204,16,58,49,0,207,124,236,130,171,91,129,0,1,2,4,8,16,216,91,224,196,20,58,47,0,79,124,232,189,223,109,158,158,0,1,2,4,8,16,232,16,56,47,134,78,11,192,243,30,185,227,228,182,32,64,128,0,1,2,4,182,23,56,45,135,206,10,192,211,30,120,251,183,26,0,2,4,8,16,32,64,160,70,224,172,32,58,41,0,207,122,220,154,123,91,132,0,1,2,4,8,16,32,112,218,255,28,224,73,1,232,226,4,8,16,32,64,128,0,1,2,103,9,156,19,128,190,1,120,214,189,189,46,1,2,4,8,16,32,80,37,112,78,20,157,18,128,231,60,106,213,181,45,67,128,0,1,2,4,8,16,248,33,112,74,22,157,17,128,167,60,168,247,24,1,2,4,8,16,32,64,160,81,224,140,48,58,33,0,207,120,204,198,115,219,137,0,1,2,4,8,16,32,240,93,224,132,52,58,33,0,157,154,0,1,2,4,8,16,32,64,224,76,129,227,3,240,132,202,61,19,216,107,19,32,64,128,0,1,2,4,110,11,28,31,71,135,7,224,241,143,120,155,220,207,18,32,64,128,0,1,2,4,78,22,
56,60,143,142,14,192,195,31,240,228,131,122,121,2,4,8,16,32,64,128,192,93,129,163,3,233,224,0,60,250,241,238,122,251,5,4,8,16,32,64,128,0,129,243,5,14,78,164,131,3,240,124,95,27,16,32,64,128,0,1,2,4,118,23,56,54,0,15,174,219,221,143,235,249,9,16,32,64,128,0,129,89,4,142,141,164,67,3,240,216,71,155,229,224,246,36,64,128,0,1,2,4,8,28,251,191,6,120,100,0,234,63,239,110,2,4,8,16,32,64,128,192,21,129,35,67,233,192,0,60,242,177,174,200,250,97,2,4,8,16,32,64,128,64,171,192,129,169,116,96,0,182,106,219,139,0,1,2,4,8,16,32,176,151,192,113,1,120,96,213,238,117,66,79,75,128,0,1,2,4,8,172,33,112,92,44,29,22,128,199,61,210,26,111,1,79,65,128,0,1,2,4,8,108,39,112,88,46,29,21,128,135,61,208,118,111,21,15,76,128,0,1,2,4,8,44,35,112,84,48,29,21,128,203,28,198,131,16,32,64,128,0,1,2,4,102,23,56,40,0,143,234,217,217,207,97,127,2,4,8,16,32,64,96,107,129,131,146,233,152,0,60,232,97,182,126,195,120,120,2,4,8,16,32,64,96,1,129,99,162,233,144,0,60,230,81,22,184,185,71,32,64,128,0,1,2,4,118,23,56,36,
155,14,9,192,221,47,233,249,9,16,32,64,128,0,1,2,77,2,71,4,224,33,37,219,132,106,23,2,4,8,16,32,64,128,192,171,2,71,132,211,1,1,120,196,99,188,74,236,247,17,32,64,128,0,1,2,4,202,4,14,72,167,124,0,30,240,16,101,103,179,14,1,2,4,8,16,32,64,224,13,129,124,60,229,3,240,141,199,247,91,9,16,32,64,128,0,1,2,4,198,11,196,3,48,223,176,227,81,76,36,64,128,0,1,2,4,8,156,40,16,207,167,116,0,198,31,224,196,227,120,105,2,4,8,16,32,64,128,64,68,32,29,80,233,0,140,160,24,74,128,0,1,2,4,8,16,32,240,186,64,56,0,211,253,250,250,131,251,157,4,8,16,32,64,128,0,129,90,129,112,66,101,3,48,188,124,237,209,44,70,128,0,1,2,4,8,16,120,75,32,27,81,209,0,204,174,254,150,170,223,76,128,0,1,2,4,8,16,104,22,136,102,84,52,0,155,85,237,70,128,0,1,2,4,8,16,216,85,32,25,128,209,114,221,245,96,158,155,0,1,2,4,8,16,216,67,32,25,82,193,0,76,174,189,199,225,61,37,1,2,4,8,16,32,176,177,64,48,165,130,1,184,241,193,60,58,1,2,4,8,16,32,64,160,88,32,23,128,193,106,45,246,180,26,1,2,4,8,16,32,64,96,144,64,46,166,98,1,152,
91,121,144,169,49,4,8,16,32,64,128,0,129,110,129,88,78,197,2,176,219,211,118,4,8,16,32,64,128,0,129,125,5,82,1,24,43,214,125,79,229,201,9,16,32,64,128,0,129,221,4,82,65,21,10,192,212,186,187,157,221,243,18,32,64,128,0,1,2,91,11,132,146,42,20,128,91,159,202,195,19,32,64,128,0,1,2,4,170,5,50,1,24,170,213,106,73,203,17,32,64,128,0,1,2,4,134,11,100,162,42,18,128,153,85,135,139,26,72,128,0,1,2,4,8,16,104,23,136,100,85,36,0,219,37,237,71,128,0,1,2,4,8,16,216,89,32,17,128,145,82,221,249,72,158,157,0,1,2,4,8,16,216,87,32,17,86,129,0,76,172,185,239,209,61,57,1,2,4,8,16,32,176,185,64,32,173,2,1,184,249,145,60,62,1,2,4,8,16,32,64,160,92,96,124,0,6,42,181,220,208,122,4,8,16,32,64,128,0,129,160,192,248,184,26,30,128,227,87,12,122,26,77,128,0,1,2,4,8,16,232,23,24,158,87,195,3,176,223,208,134,4,8,16,32,64,128,0,129,189,5,70,7,224,240,66,221,251,60,158,158,0,1,2,4,8,16,32,240,207,63,163,3,107,112,0,142,94,207,201,9,16,32,64,128,0,1,2,4,70,23,224,224,0,116,32,2,4,8,16,32,64,128,0,129,118,129,
177,1,232,27,128,237,247,182,31,1,2,4,8,16,32,48,165,192,216,200,26,27,128,83,130,90,154,0,1,2,4,8,16,32,176,151,192,208,0,28,219,166,123,29,194,211,18,32,64,128,0,1,2,4,110,8,12,205,172,145,1,56,116,177,27,0,126,138,0,1,2,4,8,16,32,176,157,192,200,208,26,25,128,219,29,194,3,19,32,64,128,0,1,2,4,102,20,24,24,128,35,187,116,70,74,59,19,32,64,128,0,1,2,4,130,2,3,83,107,96,0,6,31,216,104,2,4,8,16,32,64,128,0,129,97,2,227,2,112,96,149,14,123,58,131,8,16,32,64,128,0,1,2,203,8,140,139,173,97,1,56,110,165,101,174,228,65,8,16,32,64,128,0,1,2,35,5,134,229,214,176,0,28,249,116,102,17,32,64,128,0,1,2,4,8,228,4,70,5,224,176,34,205,61,170,201,4,8,16,32,64,128,0,129,185,5,70,5,215,168,0,156,91,211,246,4,8,16,32,64,128,0,129,141,4,6,5,224,168,30,221,72,222,163,18,32,64,128,0,1,2,4,158,21,24,148,92,99,2,112,208,50,207,26,248,245,4,8,16,32,64,128,0,129,189,4,198,68,215,152,0,220,75,222,211,18,32,64,128,0,1,2,4,166,22,24,18,128,99,90,116,106,71,203,19,32,64,128,0,1,2,4,142,16,24,146,93,67,2,240,
136,167,245,26,4,8,16,32,64,128,0,1,2,99,4,70,4,224,144,18,29,243,56,166,16,32,64,128,0,1,2,4,214,22,24,17,94,35,2,112,109,101,79,71,128,0,1,2,4,8,16,88,76,96,64,0,142,232,208,197,84,61,14,1,2,4,8,16,32,64,32,37,48,32,189,222,15,192,1,75,164,124,204,37,64,128,0,1,2,4,8,172,39,240,126,124,189,31,128,235,169,122,34,2,4,8,16,32,64,128,192,210,2,111,7,224,251,13,186,180,175,135,35,64,128,0,1,2,4,8,140,22,120,59,191,222,14,192,209,79,100,30,1,2,4,8,16,32,64,128,64,86,224,221,0,124,187,64,179,143,103,58,1,2,4,8,16,32,64,96,61,129,119,3,236,221,0,92,79,212,19,17,32,64,128,0,1,2,4,22,23,120,51,0,223,237,207,197,117,61,30,1,2,4,8,16,32,64,32,33,240,102,130,189,25,128,137,39,50,147,0,1,2,4,8,16,32,64,32,41,240,94,0,190,89,159,201,7,51,155,0,1,2,4,8,16,32,176,174,192,123,17,246,86,0,190,247,210,235,158,196,147,17,32,64,128,0,1,2,4,194,2,111,101,216,91,1,24,126,48,227,9,16,32,64,128,0,1,2,4,2,2,239,4,224,91,229,25,120,22,35,9,16,32,64,128,0,1,2,219,8,188,19,98,239,4,224,54,192,30,148,0,1,
2,4,8,16,32,176,146,192,27,1,248,78,119,174,68,232,89,8,16,32,64,128,0,1,2,39,8,188,145,98,111,4,224,9,15,234,37,9,16,32,64,128,0,1,2,4,222,22,120,61,0,223,168,206,183,183,54,128,0,1,2,4,8,16,32,176,189,192,235,49,246,122,0,110,143,14,128,0,1,2,4,8,16,32,48,167,192,203,1,248,122,115,206,9,101,107,2,4,8,16,32,64,128,64,153,192,203,57,246,114,0,150,1,88,135,0,1,2,4,8,16,32,64,224,65,129,87,3,240,229,226,124,112,47,191,140,0,1,2,4,8,16,32,64,224,142,192,171,65,246,106,0,222,89,199,79,19,32,64,128,0,1,2,4,8,180,10,188,24,128,175,246,102,43,131,189,8,16,32,64,128,0,1,2,19,10,188,152,100,47,6,224,132,64,86,38,64,128,0,1,2,4,8,16,248,41,240,90,0,190,88,155,204,9,16,32,64,128,0,1,2,4,70,10,188,22,101,175,5,224,200,189,205,34,64,128,0,1,2,4,8,16,56,84,224,165,0,124,173,53,15,125,46,47,70,128,0,1,2,4,8,16,216,65,224,165,44,123,41,0,119,208,244,140,4,8,16,32,64,128,0,129,85,5,94,9,192,151,74,115,85,64,207,69,128,0,1,2,4,8,16,56,83,224,149,48,123,37,0,207,124,70,175,77,128,0,1,2,4,8,16,32,
240,166,192,11,1,248,74,103,190,185,165,223,78,128,0,1,2,4,8,16,32,112,89,224,133,52,123,33,0,47,191,182,31,37,64,128,0,1,2,4,8,16,152,67,224,249,0,124,161,50,231,160,176,37,1,2,4,8,16,32,64,96,70,129,231,227,236,249,0,156,209,197,206,4,8,16,32,64,128,0,1,2,191,5,158,14,192,231,27,243,247,107,249,7,2,4,8,16,32,64,128,0,129,241,2,79,231,217,211,1,56,126,103,19,9,16,32,64,128,0,1,2,4,142,20,120,54,0,159,46,204,35,31,198,107,17,32,64,128,0,1,2,4,118,20,120,54,208,158,13,192,29,77,61,51,1,2,4,8,16,32,64,96,41,129,39,3,240,217,190,92,202,202,195,16,32,64,128,0,1,2,4,58,5,158,76,180,39,3,176,243,153,109,69,128,0,1,2,4,8,16,32,240,184,128,0,124,220,202,175,36,64,128,0,1,2,4,8,44,33,240,92,0,62,249,237,197,37,132,60,4,1,2,4,8,16,32,64,160,94,224,185,72,123,46,0,235,31,222,130,4,8,16,32,64,128,0,1,2,247,4,158,10,192,231,218,242,222,75,251,121,2,4,8,16,32,64,128,0,129,65,2,79,101,218,83,1,56,104,65,99,8,16,32,64,128,0,1,2,4,78,20,120,38,0,159,42,203,19,159,201,75,19,32,64,128,0,1,2,4,182,
19,120,38,212,158,9,192,237,32,61,48,1,2,4,8,16,32,64,96,69,129,39,2,240,153,174,92,145,202,51,17,32,64,128,0,1,2,4,138,5,158,72,181,39,2,176,248,129,173,70,128,0,1,2,4,8,16,32,240,176,128,0,124,152,202,47,36,64,128,0,1,2,4,8,172,33,240,120,0,62,241,109,197,53,104,60,5,1,2,4,8,16,32,64,96,38,129,199,99,237,241,0,156,233,249,237,74,128,0,1,2,4,8,16,32,112,85,224,225,0,124,188,41,175,190,150,159,32,64,128,0,1,2,4,8,16,200,9,60,156,107,15,7,96,110,87,147,9,16,32,64,128,0,1,2,4,142,20,120,52,0,31,46,202,35,151,247,90,4,8,16,32,64,128,0,1,2,127,4,30,13,182,71,3,240,207,100,255,68,128,0,1,2,4,8,16,32,48,181,128,0,156,250,124,150,39,64,128,0,1,2,4,8,60,47,240,96,0,62,250,13,197,231,23,240,59,8,16,32,64,128,0,1,2,4,6,9,60,152,108,15,6,224,160,165,140,33,64,128,0,1,2,4,8,16,56,93,224,177,0,124,176,38,79,127,26,11,16,32,64,128,0,1,2,4,182,22,120,44,218,30,11,192,173,33,61,60,1,2,4,8,16,32,64,96,45,1,1,184,214,61,61,13,1,2,4,8,16,32,64,224,174,192,67,1,248,216,55,19,239,190,150,95,64,128,
0,1,2,4,8,16,32,144,21,120,40,219,30,10,192,236,158,166,19,32,64,128,0,1,2,4,8,28,41,240,72,0,62,84,146,71,46,237,181,8,16,32,64,128,0,1,2,4,46,11,60,18,110,143,4,224,229,233,126,148,0,1,2,4,8,16,32,64,96,74,1,1,56,229,217,44,77,128,0,1,2,4,8,16,120,93,224,129,0,124,228,27,137,175,47,224,119,18,32,64,128,0,1,2,4,8,12,20,120,32,221,30,8,192,129,11,25,69,128,0,1,2,4,8,16,32,112,186,192,253,0,124,160,34,79,127,10,11,16,32,64,128,0,1,2,4,8,252,39,112,63,222,238,7,32,76,2,4,8,16,32,64,128,0,129,165,4,4,224,82,231,244,48,4,8,16,32,64,128,0,129,251,2,119,3,240,254,55,17,239,191,136,95,65,128,0,1,2,4,8,16,32,112,152,192,221,124,187,27,128,135,173,234,133,8,16,32,64,128,0,1,2,4,14,17,16,128,135,48,123,17,2,4,8,16,32,64,128,64,143,192,189,0,188,251,45,196,158,71,177,9,1,2,4,8,16,32,64,128,192,15,129,123,1,119,47,0,41,18,32,64,128,0,1,2,4,8,44,38,112,39,0,239,245,227,98,26,30,135,0,1,2,4,8,16,32,176,130,192,157,132,187,19,128,43,8,120,6,2,4,8,16,32,64,128,0,129,143,2,2,240,163,134,127,38,
64,128,0,1,2,4,8,108,32,112,59,0,239,124,251,112,3,31,143,72,128,0,1,2,4,8,16,152,80,224,118,196,221,14,192,9,31,215,202,4,8,16,32,64,128,0,1,2,183,5,4,224,109,31,63,75,128,0,1,2,4,8,16,88,78,224,102,0,222,254,230,225,114,22,30,136,0,1,2,4,8,16,32,176,138,192,205,140,187,25,128,171,8,120,14,2,4,8,16,32,64,128,0,129,63,2,2,240,143,133,127,34,64,128,0,1,2,4,8,108,33,112,43,0,111,126,235,112,11,29,15,73,128,0,1,2,4,8,16,152,84,224,86,200,221,10,192,73,31,215,218,4,8,16,32,64,128,0,1,2,183,4,4,224,45,29,63,71,128,0,1,2,4,8,16,88,80,224,70,0,222,250,198,225,130,18,30,137,0,1,2,4,8,16,32,176,146,192,141,148,187,17,128,43,9,120,22,2,4,8,16,32,64,128,0,129,95,2,2,240,151,132,127,37,64,128,0,1,2,4,8,108,34,112,61,0,111,124,219,112,19,27,143,73,128,0,1,2,4,8,16,152,88,224,122,204,93,15,192,137,31,215,234,4,8,16,32,64,128,0,1,2,215,5,4,224,117,27,63,67,128,0,1,2,4,8,16,88,82,224,106,0,94,255,166,225,146,14,30,138,0,1,2,4,8,16,32,176,154,192,213,156,187,26,128,171,9,120,30,2,4,8,16,32,64,
128,0,129,127,5,4,160,119,2,1,2,4,8,16,32,64,96,51,129,107,1,120,245,91,134,155,249,120,92,2,4,8,16,32,64,128,192,180,2,215,130,238,90,0,78,251,160,22,39,64,128,0,1,2,4,8,16,184,45,32,0,111,251,248,89,2,4,8,16,32,64,128,192,114,2,87,2,240,218,55,12,151,123,126,15,68,128,0,1,2,4,8,16,88,87,224,74,210,93,9,192,117,29,60,25,1,2,4,8,16,32,64,96,119,1,1,184,251,59,192,243,19,32,64,128,0,1,2,219,9,92,14,192,43,223,46,220,78,199,3,19,32,64,128,0,1,2,4,166,22,184,28,117,151,3,112,234,7,181,60,1,2,4,8,16,32,64,128,192,45,1,1,120,75,199,207,17,32,64,128,0,1,2,4,22,20,16,128,11,30,213,35,17,32,64,128,0,1,2,4,110,9,92,12,192,203,127,91,124,107,140,159,35,64,128,0,1,2,4,8,16,40,20,184,152,117,23,3,176,112,121,43,17,32,64,128,0,1,2,4,8,12,18,16,128,131,32,141,33,64,128,0,1,2,4,8,204,34,112,41,0,47,126,171,112,150,7,178,39,1,2,4,8,16,32,64,128,192,31,129,75,97,119,41,0,255,252,14,255,68,128,0,1,2,4,8,16,32,176,156,128,0,92,238,164,30,136,0,1,2,4,8,16,32,112,91,64,0,222,246,241,179,4,8,16,32,
64,128,0,129,229,4,46,4,224,165,191,41,94,238,185,61,16,1,2,4,8,16,32,64,96,15,129,11,105,119,33,0,247,176,240,148,4,8,16,32,64,128,0,129,93,5,4,224,174,151,247,220,4,8,16,32,64,128,192,182,2,95,3,240,194,183,9,183,213,241,224,4,8,16,32,64,128,0,129,233,5,190,198,221,215,0,156,254,33,61,0,1,2,4,8,16,32,64,128,192,45,1,1,120,75,199,207,17,32,64,128,0,1,2,4,22,20,16,128,11,30,213,35,17,32,64,128,0,1,2,4,110,9,124,9,192,175,127,75,124,235,183,251,57,2,4,8,16,32,64,128,0,129,114,129,47,121,247,37,0,203,31,192,122,4,8,16,32,64,128,0,1,2,111,10,8,192,55,1,253,118,2,4,8,16,32,64,128,192,108,2,2,112,182,139,217,151,0,1,2,4,8,16,32,240,166,192,231,0,252,242,119,196,111,206,247,219,9,16,32,64,128,0,1,2,4,78,22,248,28,120,159,3,240,228,245,188,60,1,2,4,8,16,32,64,128,64,90,64,0,166,133,205,39,64,128,0,1,2,4,8,148,9,8,192,178,131,88,135,0,1,2,4,8,16,32,144,22,248,20,128,159,255,134,56,253,242,230,19,32,64,128,0,1,2,4,8,196,5,62,37,222,167,0,140,191,188,23,32,64,128,0,1,2,4,8,16,56,89,64,
0,158,124,0,47,79,128,0,1,2,4,8,16,56,90,64,0,30,45,238,245,8,16,32,64,128,0,1,2,39,11,252,29,128,159,254,126,248,228,221,188,60,1,2,4,8,16,32,64,128,192,16,129,191,35,239,239,0,28,242,2,134,16,32,64,128,0,1,2,4,8,52,11,8,192,230,235,216,141,0,1,2,4,8,16,32,16,16,16,128,1,84,35,9,16,32,64,128,0,1,2,205,2,127,5,224,223,127,59,220,188,182,221,8,16,32,64,128,0,1,2,4,158,16,248,43,243,254,10,192,39,134,248,165,4,8,16,32,64,128,0,1,2,147,10,8,192,73,15,103,109,2,4,8,16,32,64,128,192,171,2,2,240,85,57,191,143,0,1,2,4,8,16,32,48,169,128,0,156,244,112,214,38,64,128,0,1,2,4,8,188,42,240,49,0,255,250,15,7,190,58,208,239,35,64,128,0,1,2,4,8,16,232,19,248,24,122,31,3,176,111,83,27,17,32,64,128,0,1,2,4,8,12,23,16,128,195,73,13,36,64,128,0,1,2,4,8,116,11,8,192,238,251,216,142,0,1,2,4,8,16,32,48,92,224,67,0,126,252,155,225,225,175,99,32,1,2,4,8,16,32,64,128,192,153,2,31,82,239,67,0,158,185,145,215,38,64,128,0,1,2,4,8,16,56,74,64,0,30,37,237,117,8,16,32,64,128,0,1,2,37,2,2,176,228,16,214,32,
64,128,0,1,2,4,8,28,37,32,0,143,146,246,58,4,8,16,32,64,128,0,129,18,129,63,1,248,225,63,24,88,178,155,53,8,16,32,64,128,0,1,2,4,134,9,252,137,189,63,1,56,108,184,65,4,8,16,32,64,128,0,1,2,205,2,2,176,249,58,118,35,64,128,0,1,2,4,8,4,4,4,96,0,213,72,2,4,8,16,32,64,128,64,179,128,0,108,190,142,221,8,16,32,64,128,0,1,2,1,129,223,1,248,231,63,22,24,120,21,35,9,16,32,64,128,0,1,2,4,206,22,248,157,123,191,3,240,236,141,188,62,1,2,4,8,16,32,64,128,192,49,2,2,240,24,103,175,66,128,0,1,2,4,8,16,168,17,16,128,53,167,176,8,1,2,4,8,16,32,64,224,24,1,1,120,140,179,87,33,64,128,0,1,2,4,8,212,8,8,192,154,83,88,132,0,1,2,4,8,16,32,112,140,192,175,0,252,253,223,10,57,230,101,189,10,1,2,4,8,16,32,64,128,192,209,2,191,130,239,87,0,30,253,250,94,143,0,1,2,4,8,16,32,64,224,36,1,1,120,18,188,151,37,64,128,0,1,2,4,8,156,37,32,0,207,146,247,186,4,8,16,32,64,128,0,129,147,4,4,224,73,240,94,150,0,1,2,4,8,16,32,112,150,128,0,60,75,222,235,18,32,64,128,0,1,2,4,78,18,248,47,0,127,253,119,66,78,218,194,203,
18,32,64,128,0,1,2,4,8,228,5,254,75,62,223,1,204,83,123,5,2,4,8,16,32,64,128,64,149,128,0,172,58,135,101,8,16,32,64,128,0,1,2,121,1,1,152,55,246,10,4,8,16,32,64,128,0,129,42,1,1,88,117,14,203,16,32,64,128,0,1,2,4,242,2,2,48,111,236,21,8,16,32,64,128,0,1,2,85,2,255,6,160,255,18,112,213,81,44,67,128,0,1,2,4,8,16,200,8,252,27,125,190,3,152,209,53,149,0,1,2,4,8,16,32,80,43,32,0,107,79,99,49,2,4,8,16,32,64,128,64,70,64,0,102,92,77,37,64,128,0,1,2,4,8,212,10,8,192,218,211,88,140,0,1,2,4,8,16,32,144,17,16,128,25,87,83,9,16,32,64,128,0,1,2,181,2,2,176,246,52,22,35,64,128,0,1,2,4,8,100,4,126,6,160,255,21,152,12,174,169,4,8,16,32,64,128,0,129,50,129,159,217,231,59,128,101,87,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,224,71,0,250,95,129,41,59,138,117,8,16,32,64,128,0,1,2,41,129,31,225,231,
59,128,41,93,115,9,16,32,64,128,0,1,2,165,2,2,176,244,48,214,34,64,128,0,1,2,4,8,164,4,4,96,74,214,92,2,4,8,16,32,64,128,64,169,128,0,44,61,140,181,8,16,32,64,128,0,1,2,41,1,1,152,146,53,151,0,1,2,4,8,16,32,80,42,32,0,75,15,99,45,2,4,8,16,32,64,128,64,74,64,0,166,100,205,37,64,128,0,1,2,4,8,148,10,8,192,210,195,88,139,0,1,2,4,8,16,32,144,18,248,30,128,254,119,160,83,184,230,18,32,64,128,0,1,2,4,234,4,190,167,159,239,0,214,93,197,66,4,8,16,32,64,128,0,129,172,128,0,204,250,154,78,128,0,1,2,4,8,16,168,19,16,128,117,39,177,16,1,2,4,8,16,32,64,32,43,32,0,179,190,166,19,32,64,128,0,1,2,4,234,4,4,96,221,73,44,68,128,0,1,2,4,8,16,200,10,8,192,172,175,233,4,8,16,32,64,128,0,129,58,1,1,88,119,18,11,17,32,64,128,0,1,2,4,178,2,2,48,235,107,58,1,2,4,8,16,32,64,160,78,64,0,214,157,196,66,4,8,16,32,64,128,0,129,172,192,255,252,63,2,201,2,155,78,128,0,1,2,4,8,16,232,18,248,230,255,19,72,215,65,108,67,128,0,1,2,4,8,16,200,11,248,43,224,188,177,87,32,64,128,0,1,2,4,8,84,9,8,192,170,115,88,134,
0,1,2,4,8,16,32,144,23,16,128,121,99,175,64,128,0,1,2,4,8,16,168,18,16,128,85,231,176,12,1,2,4,8,16,32,64,32,47,32,0,243,198,94,129,0,1,2,4,8,16,32,80,37,32,0,171,206,97,25,2,4,8,16,32,64,128,64,94,64,0,230,141,189,2,1,2,4,8,16,32,64,160,74,64,0,86,157,195,50,4,8,16,32,64,128,0,129,188,128,0,204,27,123,5,2,4,8,16,32,64,128,64,149,128,0,172,58,135,101,8,16,32,64,128,0,1,2,121,1,1,152,55,246,10,4,8,16,32,64,128,0,129,42,129,111,223,170,214,177,12,1,2,4,8,16,32,64,128,64,90,192,119,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,
2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,
0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,
16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,
1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,
32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,
4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,
128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,
32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,
128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,
64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,
1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,
128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,
4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,
0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,
32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,
8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,
32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,
8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,
128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,
32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,
1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,
128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,
8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,
1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,
16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,
4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,
67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,
19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,
14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,
79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,
4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,
1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,
16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,
4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,
67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,
19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,
14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,
79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,
4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,
1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,
16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,19,32,64,128,0,1,2,4,202,4,4,96,217,65,172,67,128,0,1,2,4,8,16,72,11,8,192,180,176,249,
4,8,16,32,64,128,0,129,50,1,1,88,118,16,235,16,32,64,128,0,1,2,4,210,2,2,48,45,108,62,1,2,4,8,16,32,64,160,76,64,0,150,29,196,58,4,8,16,32,64,128,0,129,180,128,0,76,11,155,79,128,0,1,2,4,8,16,40,19,16,128,101,7,177,14,1,2,4,8,16,32,64,32,45,32,0,211,194,230,255,127,123,247,162,27,57,142,44,1,180,247,254,255,63,239,117,247,142,123,252,46,73,165,144,130,228,89,44,208,126,148,82,201,147,42,32,64,218,30,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,
210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,
64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,
16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,
88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,
128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,
1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,
16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,
237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,
96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,
153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,
0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,
8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,
16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,
2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,
64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,
128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,
2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,
54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,
2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,
64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,
128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,
2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,
54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,
2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,
64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,
128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,
2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,
54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,128,0,88,54,16,237,16,32,64,128,0,1,2,4,210,2,2,96,90,88,125,2,4,8,16,32,64,128,64,153,192,127,126,253,167,172,35,237,16,32,64,128,0,1,2,4,8,36,5,254,107,7,48,201,171,54,1,2,4,8,16,32,64,160,80,64,0,44,28,138,150,8,16,32,64,128,0,1,2,73,1,1,48,169,171,54,1,2,4,8,16,32,64,160,80,64,0,44,28,138,150,8,16,32,64,128,0,1,2,73,1,1,48,169,171,54,1,2,4,8,16,32,64,160,80,64,0,44,28,138,150,8,16,32,64,128,0,1,2,73,1,1,48,169,171,54,1,2,4,8,16,32,64,160,80,64,0,44,28,138,150,8,16,32,64,128,0,1,2,73,1,1,48,169,171,54,1,2,4,8,16,32,64,160,80,64,0,44,28,138,150,8,16,32,64,128,0,1,2,73,1,1,48,169,171,54,1,2,4,8,16,32,64,160,80,64,0,44,28,138,150,8,16,32,64,128,
0,1,2,73,129,255,252,250,245,242,127,255,35,64,128,0,1,2,4,8,16,88,68,224,191,191,236,0,46,50,106,203,36,64,128,0,1,2,4,8,188,10,8,128,175,18,254,37,64,128,0,1,2,4,8,44,34,32,0,46,50,104,203,36,64,128,0,1,2,4,8,188,10,8,128,175,18,254,37,64,128,0,1,2,4,8,44,34,32,0,46,50,104,203,36,64,128,0,1,2,4,8,188,10,8,128,175,18,254,37,64,128,0,1,2,4,8,44,34,32,0,46,50,104,203,36,64,128,0,1,2,4,8,188,10,8,128,175,18,254,37,64,128,0,1,2,4,8,44,34,32,0,46,50,104,203,36,64,128,0,1,2,4,8,188,10,252,254,207,128,248,79,129,188,106,248,151,0,1,2,4,8,16,32,48,187,192,127,127,249,47,129,204,62,99,235,35,64,128,0,1,2,4,8,124,20,112,4,252,81,196,231,4,8,16,32,64,128,0,129,201,5,4,192,201,7,108,121,4,8,16,32,64,128,0,129,143,2,2,224,71,17,159,19,32,64,128,0,1,2,4,38,23,16,0,39,31,176,229,17,32,64,128,0,1,2,4,62,10,8,128,31,69,124,78,128,0,1,2,4,8,16,152,92,64,0,156,124,192,150,71,128,0,1,2,4,8,16,248,40,32,0,126,20,241,57,1,2,4,8,16,32,64,96,114,129,63,127,4,218,95,130,158,124,202,150,71,128,0,1,2,
4,8,16,248,71,224,229,239,64,251,67,208,158,6,2,4,8,16,32,64,128,192,106,2,142,128,87,155,184,245,18,32,64,128,0,1,2,203,11,8,128,203,63,2,0,8,16,32,64,128,0,129,213,4,4,192,213,38,110,189,4,8,16,32,64,128,192,242,2,2,224,242,143,0,0,2,4,8,16,32,64,96,53,1,1,112,181,137,91,47,1,2,4,8,16,32,176,188,192,255,254,2,140,191,3,179,252,131,0,128,0,1,2,4,8,16,88,66,224,247,95,129,241,103,96,150,24,181,69,18,32,64,128,0,1,2,4,222,8,56,2,126,131,225,67,2,4,8,16,32,64,128,192,10,2,2,224,10,83,182,70,2,4,8,16,32,64,128,192,27,1,1,240,13,134,15,9,16,32,64,128,0,1,2,43,8,8,128,43,76,217,26,9,16,32,64,128,0,1,2,111,4,4,192,55,24,62,36,64,128,0,1,2,4,8,172,32,240,207,31,128,241,119,96,86,24,182,53,18,32,64,128,0,1,2,171,11,252,249,43,48,254,12,204,234,143,129,245,19,32,64,128,0,1,2,235,9,56,2,94,111,230,86,76,128,0,1,2,4,8,44,46,32,0,46,254,0,88,62,1,2,4,8,16,32,176,158,128,0,184,222,204,173,152,0,1,2,4,8,16,88,92,64,0,92,252,1,176,124,2,4,8,16,32,64,96,61,129,215,95,255,125,253,119,61,1,43,
38,64,128,0,1,2,4,8,172,34,240,191,95,2,246,91,192,171,204,219,58,9,16,32,64,128,0,1,2,175,2,142,128,95,37,252,75,128,0,1,2,4,8,16,88,68,64,0,92,100,208,150,73,128,0,1,2,4,8,16,120,21,16,0,95,37,252,75,128,0,1,2,4,8,16,88,68,64,0,92,100,208,150,73,128,0,1,2,4,8,16,120,21,248,251,219,191,127,63,120,253,142,127,9,16,32,64,128,0,1,2,4,166,18,248,231,151,128,253,22,240,84,83,181,24,2,4,8,16,32,64,128,192,6,1,71,192,27,144,188,132,0,1,2,4,8,16,32,48,147,128,0,56,211,52,173,133,0,1,2,4,8,16,32,176,65,64,0,220,128,228,37,4,8,16,32,64,128,0,129,153,4,4,192,153,166,105,45,4,8,16,32,64,128,0,129,13,2,255,254,242,239,191,31,109,184,204,75,8,16,32,64,128,0,1,2,4,6,19,120,253,37,96,191,5,60,216,224,180,75,128,0,1,2,4,8,16,120,90,192,17,240,211,132,10,16,32,64,128,0,1,2,4,198,18,16,0,199,154,151,110,9,16,32,64,128,0,1,2,79,11,8,128,79,19,42,64,128,0,1,2,4,8,16,24,75,224,205,175,126,188,249,112,172,53,232,150,0,1,2,4,8,16,32,64,224,161,192,223,223,1,241,75,32,15,173,188,128,0,1,2,4,8,16,32,
48,153,128,35,224,201,6,106,57,4,8,16,32,64,128,0,129,71,2,2,224,35,33,223,39,64,128,0,1,2,4,8,76,38,32,0,78,54,80,203,33,64,128,0,1,2,4,8,60,18,120,251,155,31,111,63,126,116,157,239,19,32,64,128,0,1,2,4,8,140,36,240,239,239,128,248,37,144,145,230,166,87,2,4,8,16,32,64,128,192,25,2,142,128,207,80,84,131,0,1,2,4,8,16,32,48,144,128,0,56,208,176,180,74,128,0,1,2,4,8,16,56,67,224,221,207,253,189,251,228,140,234,106,16,32,64,128,0,1,2,4,8,84,8,188,249,17,64,63,3,88,49,17,77,16,32,64,128,0,1,2,4,46,20,112,4,124,33,182,91,17,32,64,128,0,1,2,4,26,4,4,192,134,41,232,129,0,1,2,4,8,16,32,112,161,128,0,120,33,182,91,17,32,64,128,0,1,2,4,26,4,222,255,222,199,251,207,26,250,211,3,1,2,4,8,16,32,64,128,192,243,2,111,127,7,196,47,129,60,239,169,2,1,2,4,8,16,32,64,96,44,1,71,192,99,205,75,183,4,8,16,32,64,128,0,129,167,5,4,192,167,9,21,32,64,128,0,1,2,4,8,140,37,240,225,167,254,62,124,58,214,90,116,75,128,0,1,2,4,8,16,32,240,165,192,187,31,1,244,51,128,95,26,249,34,1,2,4,8,16,32,64,96,98,1,71,
192,19,15,215,210,8,16,32,64,128,0,1,2,95,9,8,128,95,169,248,26,1,2,4,8,16,32,64,96,98,129,143,63,244,247,241,243,137,151,110,105,4,8,16,32,64,128,0,129,69,4,222,255,8,160,159,1,92,100,236,150,73,128,0,1,2,4,8,16,248,43,224,8,248,47,133,15,8,16,32,64,128,0,1,2,107,8,8,128,107,204,217,42,9,16,32,64,128,0,1,2,127,5,62,253,204,223,167,47,252,125,169,15,8,16,32,64,128,0,1,2,4,70,20,248,240,35,128,126,6,112,196,33,234,153,0,1,2,4,8,16,32,240,140,128,35,224,103,244,92,75,128,0,1,2,4,8,16,24,80,64,0,28,112,104,90,38,64,128,0,1,2,4,8,60,35,240,249,71,254,62,127,229,153,250,174,37,64,128,0,1,2,4,8,16,184,87,224,227,143,0,250,25,192,123,231,225,238,4,8,16,32,64,128,0,129,235,5,28,1,95,111,238,142,4,8,16,32,64,128,0,129,91,5,4,192,91,249,221,156,0,1,2,4,8,16,32,112,189,192,23,63,241,247,197,151,174,239,203,29,9,16,32,64,128,0,1,2,4,78,17,248,244,35,128,126,6,240,20,87,69,8,16,32,64,128,0,1,2,3,9,56,2,30,104,88,90,37,64,128,0,1,2,4,8,156,33,240,213,121,239,87,95,59,227,94,106,16,32,64,128,
0,1,2,4,8,92,45,240,249,4,216,17,240,213,51,112,63,2,4,8,16,32,64,128,192,221,2,142,128,239,158,128,251,19,32,64,128,0,1,2,4,46,22,16,0,47,6,119,59,2,4,8,16,32,64,128,192,221,2,95,254,188,223,151,95,188,187,83,247,39,64,128,0,1,2,4,8,16,216,45,240,197,143,0,250,25,192,221,138,46,32,64,128,0,1,2,4,8,12,46,224,8,120,240,1,106,159,0,1,2,4,8,16,32,176,87,224,235,211,222,175,191,186,183,182,215,19,32,64,128,0,1,2,4,8,220,43,240,213,9,176,35,224,123,103,226,238,4,8,16,32,64,128,0,129,235,5,28,1,95,111,238,142,4,8,16,32,64,128,0,129,91,5,4,192,91,249,221,156,0,1,2,4,8,16,32,112,189,192,55,63,237,247,205,151,175,239,207,29,9,16,32,64,128,0,1,2,4,14,11,124,249,35,128,126,6,240,176,167,11,9,16,32,64,128,0,1,2,131,10,56,2,30,116,112,218,38,64,128,0,1,2,4,8,28,21,248,238,172,247,187,175,31,189,143,235,8,16,32,64,128,0,1,2,4,174,22,248,250,4,216,17,240,213,115,112,63,2,4,8,16,32,64,128,192,221,2,142,128,239,158,128,251,19,32,64,128,0,1,2,4,46,22,248,246,168,247,219,111,92,220,160,219,17,32,
64,128,0,1,2,4,8,28,19,248,230,4,216,17,240,49,78,87,17,32,64,128,0,1,2,4,198,21,112,4,60,238,236,116,78,128,0,1,2,4,8,16,56,36,240,253,73,239,247,223,57,116,35,23,17,32,64,128,0,1,2,4,8,92,42,240,221,9,176,35,224,75,199,224,102,4,8,16,32,64,128,0,129,2,1,71,192,5,67,208,2,1,2,4,8,16,32,64,224,74,129,31,14,122,127,248,214,149,29,186,23,1,2,4,8,16,32,64,128,192,1,129,111,79,128,29,1,31,208,116,9,1,2,4,8,16,32,64,96,104,1,71,192,67,143,79,243,4,8,16,32,64,128,0,129,253,2,63,157,243,254,244,189,253,119,114,5,1,2,4,8,16,32,64,128,192,117,2,223,159,0,59,2,190,110,10,238,68,128,0,1,2,4,8,16,232,16,112,4,220,49,7,93,16,32,64,128,0,1,2,4,46,19,248,241,152,247,199,111,94,214,162,27,17,32,64,128,0,1,2,4,8,236,21,248,225,4,216,17,240,94,76,175,39,64,128,0,1,2,4,8,140,46,224,8,120,244,9,234,159,0,1,2,4,8,16,32,176,83,224,231,83,222,159,191,187,243,86,94,78,128,0,1,2,4,8,16,32,112,145,192,79,39,192,142,128,47,26,130,219,16,32,64,128,0,1,2,4,106,4,28,1,215,140,66,35,4,8,16,32,64,128,0,129,
107,4,30,28,242,62,248,246,53,61,186,11,1,2,4,8,16,32,64,128,192,46,129,31,79,128,29,1,239,178,244,98,2,4,8,16,32,64,128,192,4,2,142,128,39,24,162,37,16,32,64,128,0,1,2,4,246,8,60,58,227,125,244,253,61,247,242,90,2,4,8,16,32,64,128,0,129,43,4,126,62,1,118,4,124,197,12,220,131,0,1,2,4,8,16,32,208,36,240,112,135,239,225,11,154,86,163,23,2,4,8,16,32,64,128,0,129,95,15,54,0,237,0,122,70,8,16,32,64,128,0,1,2,171,9,248,37,144,213,38,110,189,4,8,16,32,64,128,192,242,2,143,79,120,31,191,98,121,68,0,4,8,16,32,64,128,0,129,34,129,71,39,192,142,128,139,134,165,21,2,4,8,16,32,64,128,192,37,2,142,128,47,97,118,19,2,4,8,16,32,64,128,64,143,192,134,3,222,13,47,233,89,143,78,8,16,32,64,128,0,1,2,139,11,60,60,1,118,4,188,248,19,98,249,4,8,16,32,64,128,192,130,2,91,182,247,182,188,102,65,58,75,38,64,128,0,1,2,4,8,20,10,60,222,0,180,3,88,56,54,45,17,32,64,128,0,1,2,4,162,2,126,9,36,202,171,56,1,2,4,8,16,32,64,160,79,96,211,241,238,166,23,245,173,77,71,4,8,16,32,64,128,0,129,229,4,54,156,0,59,2,
94,238,169,176,96,2,4,8,16,32,64,96,121,129,109,155,123,219,94,181,60,38,0,2,4,8,16,32,64,128,192,205,2,91,54,0,237,0,222,60,36,183,39,64,128,0,1,2,4,8,92,46,224,151,64,46,39,119,67,2,4,8,16,32,64,128,192,189,2,27,15,119,55,190,236,222,181,184,59,1,2,4,8,16,32,64,96,113,129,77,39,192,142,128,23,127,74,44,159,0,1,2,4,8,16,88,80,96,235,214,222,214,215,45,72,104,201,4,8,16,32,64,128,0,129,18,129,109,27,128,118,0,75,198,165,13,2,4,8,16,32,64,128,192,101,2,126,9,228,50,106,55,34,64,128,0,1,2,4,8,116,8,108,62,218,221,252,194,142,117,233,130,0,1,2,4,8,16,32,176,156,192,198,19,96,71,192,203,61,25,22,76,128,0,1,2,4,8,44,47,176,125,99,111,251,43,151,71,5,64,128,0,1,2,4,8,16,184,65,96,235,6,160,29,192,27,134,227,150,4,8,16,32,64,128,0,129,91,5,118,236,235,237,120,233,173,75,114,115,2,4,8,16,32,64,128,192,138,2,155,55,0,237,0,174,248,120,88,51,1,2,4,8,16,32,176,182,128,63,3,179,246,252,173,158,0,1,2,4,8,16,88,80,96,207,185,238,158,215,46,72,105,201,4,8,16,32,64,128,0,129,27,5,182,159,0,59,
2,190,113,76,110,77,128,0,1,2,4,8,16,184,69,96,215,174,222,174,23,223,178,28,55,37,64,128,0,1,2,4,8,172,41,176,99,3,208,14,224,154,143,136,85,19,32,64,128,0,1,2,43,11,236,219,212,219,247,234,149,93,173,157,0,1,2,4,8,16,32,112,165,192,158,13,64,59,128,87,78,198,189,8,16,32,64,128,0,1,2,13,2,59,247,244,118,190,188,97,133,122,32,64,128,0,1,2,4,8,76,47,176,107,3,208,14,224,244,207,131,5,18,32,64,128,0,1,2,4,62,8,248,67,208,31,64,124,74,128,0,1,2,4,8,16,152,93,96,239,153,238,222,215,207,238,103,125,4,8,16,32,64,128,0,129,251,5,246,157,0,59,2,190,127,98,58,32,64,128,0,1,2,4,8,92,43,176,123,71,111,247,5,215,174,199,221,8,16,32,64,128,0,1,2,203,9,236,220,0,180,3,184,220,19,98,193,4,8,16,32,64,128,192,242,2,251,55,244,246,95,177,60,50,0,2,4,8,16,32,64,128,64,80,96,239,6,160,29,192,224,48,148,38,64,128,0,1,2,4,8,84,10,28,216,207,59,112,73,229,210,53,69,128,0,1,2,4,8,16,152,65,96,247,6,160,29,192,25,198,110,13,4,8,16,32,64,128,0,129,61,2,71,182,243,142,92,179,167,39,175,37,64,128,0,1,2,
4,8,16,216,42,176,127,3,208,14,224,86,91,175,35,64,128,0,1,2,4,8,204,34,112,104,55,239,208,69,179,136,89,7,1,2,4,8,16,32,64,160,72,224,192,6,160,29,192,162,249,105,133,0,1,2,4,8,16,32,112,137,192,177,205,188,99,87,93,178,32,55,33,64,128,0,1,2,4,8,44,36,112,100,3,208,14,224,66,15,136,165,18,32,64,128,0,1,2,4,254,8,28,220,203,59,120,25,116,2,4,8,16,32,64,128,0,129,19,5,14,109,0,218,1,60,113,2,74,17,32,64,128,0,1,2,4,134,16,56,186,149,119,244,186,33,80,52,73,128,0,1,2,4,8,16,24,66,224,216,6,160,29,192,33,134,171,73,2,4,8,16,32,64,128,192,137,2,135,119,242,14,95,120,98,243,74,17,32,64,128,0,1,2,4,86,22,56,184,1,104,7,112,229,135,198,218,9,16,32,64,128,0,129,53,5,142,111,228,29,191,114,77,105,171,38,64,128,0,1,2,4,8,156,43,112,116,3,208,14,224,185,115,80,141,0,1,2,4,8,16,32,208,47,240,196,62,222,19,151,246,187,232,144,0,1,2,4,8,16,32,80,46,112,120,3,208,14,96,249,100,181,71,128,0,1,2,4,8,16,56,93,224,153,109,188,103,174,61,125,33,10,18,32,64,128,0,1,2,4,150,18,56,190,1,104,7,112,169,
7,197,98,9,16,32,64,128,0,1,2,47,2,79,237,226,61,117,49,126,2,4,8,16,32,64,128,0,129,195,2,79,108,0,218,1,60,172,238,66,2,4,8,16,32,64,128,192,160,2,207,109,226,61,119,245,160,100,218,38,64,128,0,1,2,4,8,220,46,240,204,6,224,115,71,192,79,158,32,223,46,167,1,2,4,8,16,32,64,128,192,152,2,79,229,63,71,192,99,14,93,215,4,8,16,32,64,128,0,129,227,2,207,30,226,62,123,253,241,206,93,73,128,0,1,2,4,8,16,88,85,224,185,13,64,59,128,171,62,55,214,77,128,0,1,2,4,8,172,43,240,244,14,222,211,5,214,181,183,114,2,4,8,16,32,64,128,192,33,129,39,55,0,237,0,30,82,119,17,1,2,4,8,16,32,64,96,96,129,231,55,240,158,175,48,48,159,214,9,16,32,64,128,0,1,2,151,11,60,187,1,104,7,240,242,145,185,33,1,2,4,8,16,32,64,224,102,129,19,246,239,78,40,113,51,130,219,19,32,64,128,0,1,2,4,198,17,120,122,3,240,217,63,4,253,135,74,2,28,231,137,209,41,1,2,4,8,16,32,48,186,192,243,249,207,17,240,232,207,128,254,9,16,32,64,128,0,1,2,123,5,78,217,189,59,165,200,222,206,189,158,0,1,2,4,8,16,32,176,160,192,9,27,128,118,
0,23,124,110,44,153,0,1,2,4,8,16,88,92,224,156,205,187,115,170,44,62,10,203,39,64,128,0,1,2,4,8,60,20,56,99,3,208,14,224,67,102,47,32,64,128,0,1,2,4,8,76,38,112,210,222,221,73,101,38,195,181,28,2,4,8,16,32,64,128,192,185,2,167,108,0,158,242,103,96,126,175,75,2,60,119,186,170,17,32,64,128,0,1,2,4,62,11,156,147,255,28,1,127,150,245,21,2,4,8,16,32,64,128,192,220,2,167,237,220,157,86,104,110,111,171,35,64,128,0,1,2,4,8,28,22,56,105,3,208,14,224,225,9,184,144,0,1,2,4,8,16,32,48,168,192,121,27,119,231,85,26,148,82,219,4,8,16,32,64,128,0,129,168,192,89,27,128,103,254,242,134,4,24,29,185,226,4,8,16,32,64,128,192,226,2,167,229,63,71,192,139,63,73,150,79,128,0,1,2,4,8,44,40,112,230,182,221,153,181,22,28,133,37,19,32,64,128,0,1,2,4,126,16,56,111,3,208,14,224,15,204,190,69,128,0,1,2,4,8,16,152,82,224,212,93,187,83,139,77,201,109,81,4,8,16,32,64,128,0,129,99,2,39,110,0,158,249,75,32,47,139,145,0,143,77,212,85,4,8,16,32,64,128,0,129,159,5,206,204,127,142,128,127,182,246,93,2,4,8,16,32,64,128,
192,124,2,39,239,217,157,92,110,62,111,43,34,64,128,0,1,2,4,8,28,16,56,117,3,208,14,224,129,9,184,132,0,1,2,4,8,16,32,48,180,192,217,91,118,103,215,27,26,87,243,4,8,16,32,64,128,0,129,83,4,206,221,0,60,255,215,54,36,192,83,198,172,8,1,2,4,8,16,32,64,224,175,192,201,249,207,17,240,95,89,31,16,32,64,128,0,1,2,4,22,17,56,127,195,238,252,138,139,140,194,50,9,16,32,64,128,0,1,2,95,10,156,189,1,120,254,17,176,63,6,248,229,224,124,145,0,1,2,4,8,16,32,112,80,224,244,252,231,8,248,224,36,92,70,128,0,1,2,4,8,16,24,86,32,113,96,155,168,57,44,176,198,9,16,32,64,128,0,1,2,79,9,156,191,1,152,56,2,118,8,252,212,144,93,76,128,0,1,2,4,8,16,120,35,16,200,127,142,128,223,248,250,144,0,1,2,4,8,16,32,176,132,64,230,184,54,83,117,137,129,88,36,1,2,4,8,16,32,64,224,141,64,98,3,48,115,4,236,16,248,205,216,124,72,128,0,1,2,4,8,16,56,44,16,201,127,142,128,15,207,195,133,4,8,16,32,64,128,0,129,65,5,82,135,181,169,186,131,50,107,155,0,1,2,4,8,16,32,112,64,32,179,1,152,58,2,118,8,124,96,196,46,33,64,128,
0,1,2,4,8,188,19,8,229,63,71,192,239,148,125,66,128,0,1,2,4,8,16,88,64,32,119,84,155,171,188,192,88,44,145,0,1,2,4,8,16,32,240,43,181,1,152,59,2,118,8,236,177,37,64,128,0,1,2,4,8,60,35,16,203,127,142,128,159,25,139,107,9,16,32,64,128,0,1,2,35,10,36,15,106,147,181,71,180,214,51,1,2,4,8,16,32,64,96,187,64,110,3,48,121,4,236,16,120,251,132,189,146,0,1,2,4,8,16,32,240,94,32,152,255,28,1,191,167,246,25,1,2,4,8,16,32,64,96,126,129,236,49,109,182,250,252,211,177,66,2,4,8,16,32,64,96,85,129,228,6,96,246,8,216,33,240,170,207,172,117,19,32,64,128,0,1,2,207,9,68,243,95,58,0,198,235,63,103,235,106,2,4,8,16,32,64,128,64,163,64,54,255,249,25,192,198,153,235,137,0,1,2,4,8,16,32,144,20,136,255,148,94,252,6,73,29,181,9,16,32,64,128,0,1,2,55,8,132,55,0,47,56,162,149,0,111,120,108,220,146,0,1,2,4,8,16,24,88,32,157,255,28,1,15,252,112,104,157,0,1,2,4,8,16,32,112,72,224,130,253,185,11,110,113,104,233,46,34,64,128,0,1,2,4,8,52,10,196,55,0,47,56,2,246,183,96,26,159,44,61,17,32,64,128,0,1,2,173,2,249,
252,119,73,0,188,230,38,173,67,212,23,1,2,4,8,16,32,64,96,135,192,5,249,207,207,0,238,152,135,151,18,32,64,128,0,1,2,4,166,16,184,230,7,244,174,185,203,20,3,177,8,2,4,8,16,32,64,96,105,129,43,54,0,175,58,157,149,0,151,126,148,45,158,0,1,2,4,8,16,216,40,112,73,254,187,42,0,94,118,159,141,184,94,70,128,0,1,2,4,8,16,40,20,184,38,255,249,25,192,194,209,107,137,0,1,2,4,8,16,32,16,21,184,236,108,246,178,27,69,185,20,39,64,128,0,1,2,4,8,228,4,46,218,0,188,240,104,86,2,204,61,45,42,19,32,64,128,0,1,2,51,8,92,149,255,46,12,128,87,222,106,134,71,192,26,8,16,32,64,128,0,129,197,4,46,203,127,126,6,112,177,39,203,114,9,16,32,64,128,0,1,2,151,110,203,57,4,246,192,17,32,64,128,0,1,2,4,190,19,184,110,3,240,210,0,120,237,205,190,195,245,117,2,4,8,16,32,64,128,64,161,192,133,249,239,226,76,102,15,176,240,113,211,18,1,2,4,8,16,32,80,32,112,101,254,187,56,0,94,125,187,130,105,106,129,0,1,2,4,8,16,32,240,88,224,210,252,231,151,64,30,15,196,43,8,16,32,64,128,0,1,2,115,9,92,125,40,123,245,253,230,154,
150,213,16,32,64,128,0,1,2,115,10,92,187,1,120,253,153,172,4,56,231,115,107,85,4,8,16,32,64,128,192,113,129,139,243,223,245,1,240,134,59,30,159,134,43,9,16,32,64,128,0,1,2,121,129,171,243,223,29,113,204,30,96,254,57,114,7,2,4,8,16,32,64,96,28,129,203,243,159,95,2,25,231,225,208,41,1,2,4,8,16,32,64,224,28,129,59,182,227,238,184,231,57,90,170,16,32,64,128,0,1,2,4,206,22,184,126,3,240,142,35,224,95,247,220,244,236,97,169,71,128,0,1,2,4,8,16,56,65,224,134,252,119,83,22,179,7,120,194,227,162,4,1,2,4,8,16,32,48,129,192,29,249,239,166,0,120,215,109,39,120,74,44,129,0,1,2,4,8,16,152,73,224,150,252,231,151,64,102,122,132,172,133,0,1,2,4,8,16,32,176,69,224,174,195,216,187,238,187,197,196,107,8,16,32,64,128,0,1,2,215,8,220,179,1,120,223,89,172,4,120,205,115,229,46,4,8,16,32,64,128,64,175,192,77,249,239,190,0,120,227,157,123,159,2,157,17,32,64,128,0,1,2,43,9,220,149,255,238,140,97,246,0,87,122,194,173,149,0,1,2,4,8,16,248,40,112,91,254,187,51,0,222,122,239,143,19,240,57,1,2,4,8,16,32,64,
224,90,129,251,242,223,189,33,204,30,224,181,207,153,187,17,32,64,128,0,1,2,61,2,55,230,63,127,6,166,231,49,208,9,1,2,4,8,16,32,64,224,26,129,123,55,225,238,189,251,53,194,238,66,128,0,1,2,4,8,16,248,44,112,231,6,224,189,71,192,254,171,192,159,159,6,95,33,64,128,0,1,2,4,86,16,184,53,255,221,29,0,111,191,255,10,79,152,53,18,32,64,128,0,1,2,109,2,247,230,191,251,3,152,83,224,182,39,82,63,4,8,16,32,64,128,64,90,224,230,252,119,127,0,44,232,32,61,99,245,9,16,32,64,128,0,1,2,111,5,238,206,127,13,241,203,30,224,219,39,194,199,4,8,16,32,64,128,192,236,2,183,231,191,134,0,88,209,195,236,79,154,245,17,32,64,128,0,1,2,45,2,247,231,63,127,7,176,229,89,208,7,1,2,4,8,16,32,64,224,42,129,138,243,215,138,38,174,18,119,31,2,4,8,16,32,64,96,105,129,130,13,192,146,227,87,9,112,233,55,130,197,19,32,64,128,0,129,133,4,26,242,95,73,0,108,105,99,161,167,207,82,9,16,32,64,128,0,129,59,4,42,242,95,77,242,178,7,120,199,51,232,158,4,8,16,32,64,128,192,181,2,29,249,175,38,0,246,52,114,237,99,224,110,4,
8,16,32,64,128,192,66,2,37,249,175,40,119,217,3,92,232,241,183,84,2,4,8,16,32,176,164,64,75,254,43,10,128,77,173,44,249,76,90,52,1,2,4,8,16,32,144,21,168,201,127,85,169,203,30,96,246,169,83,157,0,1,2,4,8,16,184,83,160,39,255,85,5,192,174,102,238,124,64,220,155,0,1,2,4,8,16,152,78,160,40,255,149,101,46,123,128,211,61,236,22,68,128,0,1,2,4,8,252,17,104,202,127,101,1,176,173,29,79,44,1,2,4,8,16,32,64,224,20,129,170,252,87,151,184,236,1,158,242,144,41,66,128,0,1,2,4,8,84,9,116,229,191,186,0,216,215,80,213,211,163,25,2,4,8,16,32,64,96,68,129,178,252,87,152,183,236,1,142,248,92,235,153,0,1,2,4,8,16,248,94,160,45,255,253,250,191,239,123,245,29,2,4,8,16,32,64,128,0,129,25,5,10,247,219,10,91,154,113,242,214,68,128,0,1,2,4,8,92,36,80,183,1,88,120,4,252,171,178,167,139,30,16,183,33,64,128,0,1,2,4,166,19,232,203,127,157,97,203,30,224,116,143,190,5,17,32,64,128,0,129,101,5,10,243,95,103,0,44,237,106,217,7,215,194,9,16,32,64,128,0,129,195,2,141,249,175,53,106,217,3,60,252,152,185,144,0,1,2,
4,8,16,40,18,168,204,127,173,1,176,182,175,162,7,74,43,4,8,16,32,64,128,64,189,64,103,254,235,13,90,246,0,235,31,105,13,18,32,64,128,0,1,2,15,4,74,243,95,111,0,44,238,236,193,168,125,155,0,1,2,4,8,16,32,240,71,160,53,255,85,199,44,155,128,222,61,4,8,16,32,64,128,192,184,2,181,241,175,252,111,238,73,128,227,62,243,58,39,64,128,0,1,2,171,11,20,231,191,234,29,192,242,120,186,250,99,109,253,4,8,16,32,64,128,192,15,2,205,249,175,60,0,182,183,247,195,212,125,139,0,1,2,4,8,16,88,89,160,58,255,213,39,44,167,192,43,191,119,172,157,0,1,2,4,8,140,42,208,157,255,234,3,96,127,131,163,62,152,250,38,64,128,0,1,2,4,98,2,229,249,111,128,124,101,15,48,246,112,42,76,128,0,1,2,4,8,68,4,218,243,223,0,1,112,132,22,35,207,142,162,4,8,16,32,64,128,192,152,2,245,249,111,136,116,101,15,112,204,167,95,215,4,8,16,32,64,96,77,129,254,252,55,68,0,28,163,201,53,31,113,171,38,64,128,0,1,2,4,62,8,12,144,255,6,201,86,246,0,63,60,90,62,37,64,128,0,1,2,4,74,5,70,200,127,131,4,192,81,218,44,125,18,181,69,128,0,1,
2,4,8,92,37,48,68,254,27,38,89,217,3,188,234,185,117,31,2,4,8,16,32,64,224,184,192,24,249,111,152,0,56,78,163,199,31,25,87,18,32,64,128,0,1,2,131,11,12,146,255,70,202,85,54,1,7,127,79,104,159,0,1,2,4,8,76,46,48,74,252,251,53,82,0,28,170,215,201,31,112,203,35,64,128,0,1,2,4,62,9,140,147,255,198,10,85,246,0,63,61,106,190,64,128,0,1,2,4,8,148,8,12,148,255,198,10,128,131,117,91,242,56,106,131,0,1,2,4,8,16,184,64,96,164,252,55,90,164,178,7,120,193,3,236,22,4,8,16,32,64,128,192,110,129,161,242,223,104,1,112,184,126,119,63,62,46,32,64,128,0,1,2,4,6,20,24,43,255,141,23,168,236,1,14,248,166,208,50,1,2,4,8,16,152,92,96,176,252,55,94,0,28,235,23,151,39,127,218,45,143,0,1,2,4,8,16,120,17,24,45,254,141,153,166,108,2,122,179,17,32,64,128,0,1,2,61,2,227,229,191,17,119,0,199,76,173,61,79,169,78,8,16,32,64,128,0,129,51,5,6,204,127,99,6,192,65,187,62,243,89,83,139,0,1,2,4,8,16,232,16,24,49,255,141,26,165,156,2,119,60,243,186,32,64,128,0,1,2,171,11,12,153,255,70,13,128,195,246,189,250,187,196,250,
9,16,32,64,128,192,92,2,99,230,191,129,131,148,77,192,185,222,64,86,67,128,0,1,2,4,198,19,24,52,254,13,253,251,20,18,224,120,239,19,29,19,32,64,128,0,129,153,4,134,205,127,3,239,0,14,29,94,103,122,248,173,133,0,1,2,4,8,44,42,48,110,254,27,58,0,142,221,252,162,239,21,203,38,64,128,0,1,2,179,8,12,156,255,70,207,80,142,129,103,121,19,89,7,1,2,4,8,16,24,75,96,228,248,55,254,49,170,4,56,214,187,69,183,4,8,16,32,64,96,14,129,177,243,223,232,59,128,227,39,216,57,222,5,86,65,128,0,1,2,4,214,18,24,60,255,141,31,0,39,88,193,90,239,24,171,37,64,128,0,1,2,227,11,140,158,255,166,136,79,142,129,199,127,35,89,1,1,2,4,8,16,24,71,96,248,248,55,201,9,170,4,56,206,123,70,167,4,8,16,32,64,96,116,129,9,242,223,20,59,128,147,196,216,209,223,14,250,39,64,128,0,1,2,75,8,204,144,255,38,9,128,179,44,99,137,247,141,69,18,32,64,128,0,129,145,5,166,200,127,243,36,39,199,192,35,191,153,244,78,128,0,1,2,4,198,16,152,35,254,205,116,120,42,1,142,241,206,209,37,1,2,4,8,16,24,87,96,150,252,55,207,14,224,76,89,
118,220,247,133,206,9,16,32,64,128,192,204,2,211,228,191,153,2,160,8,56,243,91,206,218,8,16,32,64,128,192,221,2,243,196,191,217,50,147,99,224,187,223,27,238,79,128,0,1,2,4,102,21,152,41,255,205,181,3,56,91,158,157,245,29,100,93,4,8,16,32,64,96,60,129,169,242,223,108,1,80,4,28,239,13,165,99,2,4,8,16,32,208,47,48,87,252,155,49,47,57,6,238,127,23,233,144,0,1,2,4,8,140,37,48,91,254,155,111,7,112,198,76,59,214,123,68,183,4,8,16,32,64,96,54,129,233,242,223,140,1,80,4,156,237,109,103,61,4,8,16,32,64,224,78,129,249,226,223,172,89,201,49,240,157,239,19,247,38,64,128,0,1,2,51,9,204,152,255,230,220,1,124,121,234,68,192,153,222,122,214,66,128,0,1,2,4,238,18,152,50,254,77,28,148,36,192,187,222,41,238,75,128,0,1,2,4,230,17,152,52,255,77,188,83,38,1,206,243,238,179,18,2,4,8,16,32,112,143,192,172,249,111,226,0,56,241,238,230,61,239,1,119,37,64,128,0,1,2,139,9,76,27,255,38,15,73,54,1,23,123,163,90,46,1,2,4,8,16,56,81,96,226,252,55,245,14,224,203,35,32,2,158,248,62,80,138,0,1,2,4,8,44,36,48,115,
252,155,63,33,73,128,11,189,85,45,149,0,1,2,4,8,156,38,48,119,254,91,96,139,76,4,60,237,189,160,16,1,2,4,8,16,88,68,96,242,248,55,255,14,224,203,115,42,1,46,242,102,181,76,2,4,8,16,32,112,146,192,244,249,111,141,116,36,2,158,244,126,80,134,0,1,2,4,8,44,32,48,127,252,91,101,123,76,2,92,224,237,106,137,4,8,16,32,64,224,20,129,21,242,223,26,59,128,47,143,131,8,120,202,123,66,17,2,4,8,16,32,48,185,192,18,241,111,161,96,36,1,78,254,134,181,60,2,4,8,16,32,112,130,192,34,249,111,165,157,49,17,240,132,247,133,18,4,8,16,32,64,96,98,129,85,226,223,66,59,128,47,79,171,4,56,241,91,214,210,8,16,32,64,128,192,211,2,235,228,191,197,66,145,8,248,244,123,67,1,2,4,8,16,32,48,169,192,66,241,111,185,93,49,9,112,210,55,173,101,17,32,64,128,0,129,39,5,150,202,127,139,237,0,190,60,26,34,224,147,239,15,151,19,32,64,128,0,129,9,5,214,138,127,75,230,33,17,112,194,247,173,37,17,32,64,128,0,129,39,4,86,139,127,75,6,64,155,128,79,188,67,92,74,128,0,1,2,4,230,19,88,47,255,45,26,134,108,2,206,247,230,181,34,
2,4,8,16,32,112,76,96,193,248,183,230,14,224,239,199,67,4,60,246,38,113,21,1,2,4,8,16,152,75,96,201,248,183,112,16,146,0,231,122,255,90,13,1,2,4,8,16,56,34,176,104,254,91,121,39,76,4,60,242,70,113,13,1,2,4,8,16,152,71,96,213,248,183,240,14,224,239,135,87,4,156,231,45,108,37,4,8,16,32,64,96,175,192,186,241,111,245,12,36,1,238,125,175,120,61,1,2,4,8,16,152,69,96,229,252,183,252,38,152,8,56,203,219,216,58,8,16,32,64,128,192,30,129,165,227,223,234,59,128,191,31,20,17,112,207,219,197,107,9,16,32,64,128,192,12,2,139,199,63,241,231,247,67,44,2,206,240,86,182,6,2,4,8,16,32,176,85,96,249,248,39,252,252,121,84,36,192,173,239,24,175,35,64,128,0,1,2,227,11,200,127,2,224,63,79,177,8,56,254,219,217,10,8,16,32,64,128,192,22,1,241,239,183,146,228,243,207,179,2,98,203,155,198,107,8,16,32,64,128,192,216,2,226,223,255,230,39,247,252,125,142,81,252,165,240,1,1,2,4,8,16,152,82,64,252,123,29,171,212,243,42,241,242,47,140,55,24,62,36,64,128,0,1,2,147,9,136,127,255,14,84,230,249,215,66,2,124,107,225,
99,2,4,8,16,32,48,151,128,252,247,102,158,2,224,27,140,151,15,121,188,247,240,25,1,2,4,8,16,152,67,64,252,123,55,71,129,231,29,199,203,39,68,62,138,248,156,0,1,2,4,8,140,46,32,254,125,152,160,184,243,1,228,229,83,38,159,77,124,133,0,1,2,4,8,140,43,32,254,125,154,157,176,243,137,228,229,11,84,190,82,241,53,2,4,8,16,32,48,162,128,248,247,197,212,68,157,47,80,94,190,196,229,107,23,95,37,64,128,0,1,2,99,9,136,127,95,206,75,208,249,146,229,229,139,100,190,147,241,117,2,4,8,16,32,48,138,128,248,247,205,164,196,156,111,96,94,190,204,230,123,27,223,33,64,128,0,1,2,253,2,226,223,183,51,18,114,190,165,121,249,6,157,159,116,124,143,0,1,2,4,8,52,11,136,127,63,76,71,196,249,1,231,229,91,124,126,246,241,93,2,4,8,16,32,208,41,32,254,253,56,23,1,231,71,158,151,111,18,122,36,228,251,4,8,16,32,64,160,77,64,252,123,48,17,241,230,1,208,203,183,25,61,54,242,10,2,4,8,16,32,208,35,32,254,61,156,133,112,243,144,232,229,5,148,182,40,121,13,1,2,4,8,16,104,16,16,255,54,76,65,180,217,128,244,242,18,78,219,
156,188,138,0,1,2,4,8,220,43,32,254,109,242,23,108,54,49,189,188,136,212,86,41,175,35,64,128,0,1,2,119,9,136,127,27,229,197,154,141,80,191,95,6,107,7,150,151,18,32,64,128,0,129,139,5,164,191,237,224,50,205,118,171,151,87,226,218,197,229,197,4,8,16,32,64,224,50,1,241,111,15,181,68,179,71,235,229,181,192,118,130,121,57,1,2,4,8,16,184,64,64,252,219,135,44,207,236,243,122,121,53,178,221,100,46,32,64,128,0,1,2,81,1,241,111,47,175,52,179,87,236,247,235,169,29,81,115,13,1,2,4,8,16,72,8,72,127,7,84,69,153,3,104,47,151,112,59,230,230,42,2,4,8,16,32,112,174,128,248,119,200,83,144,57,196,246,114,17,185,163,114,174,35,64,128,0,1,2,103,9,136,127,7,37,197,152,131,112,191,47,131,247,4,158,75,9,16,32,64,128,192,147,2,210,223,113,64,25,230,184,221,203,149,248,158,226,115,49,1,2,4,8,16,56,44,32,254,29,166,123,185,80,130,121,70,239,247,181,4,159,21,116,61,1,2,4,8,16,216,43,32,253,237,21,251,240,122,241,229,3,200,129,79,25,30,64,115,9,1,2,4,8,16,56,44,32,254,29,166,123,189,80,120,121,149,120,234,
95,140,79,241,185,152,0,1,2,4,8,108,22,144,254,54,83,253,240,66,201,229,7,156,61,223,2,185,71,203,107,9,16,32,64,128,192,49,1,241,239,152,219,199,171,228,150,143,34,199,63,103,121,220,206,149,4,8,16,32,64,224,177,128,244,247,216,104,227,43,132,150,141,80,219,94,134,115,155,147,87,17,32,64,128,0,129,189,2,210,223,94,177,159,94,47,177,252,164,115,224,123,64,15,160,185,132,0,1,2,4,8,60,16,16,255,30,0,237,252,182,188,178,19,108,195,203,153,110,64,242,18,2,4,8,16,32,176,89,64,250,219,76,181,245,133,194,202,86,169,93,175,195,186,139,203,139,9,16,32,64,128,192,183,2,210,223,183,52,79,124,67,82,121,2,239,199,75,201,254,200,227,155,4,8,16,32,64,96,131,128,244,183,1,233,200,75,196,148,35,106,27,175,129,187,17,202,203,8,16,32,64,128,192,23,2,210,223,23,40,39,125,73,70,57,9,242,155,50,124,191,129,241,101,2,4,8,16,32,240,163,128,244,247,35,207,179,223,20,80,158,21,124,120,61,226,135,68,94,64,128,0,1,2,4,222,9,72,127,239,56,2,159,72,39,1,212,79,37,41,127,34,241,5,2,4,8,16,32,240,141,128,244,
247,13,204,153,95,22,77,206,212,252,161,22,232,31,112,124,139,0,1,2,4,8,252,35,32,253,93,243,40,200,37,215,56,255,190,11,235,235,172,221,137,0,1,2,4,70,20,144,254,46,155,154,80,114,25,245,159,27,241,190,214,219,221,8,16,32,64,96,20,1,225,239,210,73,9,36,151,114,255,190,25,242,203,201,221,144,0,1,2,4,202,5,164,191,171,7,36,141,92,45,254,231,126,216,111,97,119,83,2,4,8,16,40,20,16,254,238,24,138,36,114,135,250,239,123,146,191,75,222,125,9,16,32,64,160,71,64,250,187,105,22,98,200,77,240,127,110,75,255,78,125,247,38,64,128,0,129,123,5,132,191,27,253,69,144,27,241,255,220,218,4,238,158,128,251,19,32,64,128,192,245,2,194,223,245,230,239,238,40,126,188,227,184,233,19,83,184,9,222,109,9,16,32,64,224,6,1,225,239,6,244,143,183,20,61,62,138,220,245,185,73,220,37,239,190,4,8,16,32,112,157,128,240,119,157,245,143,119,18,59,126,228,185,248,155,166,113,49,184,219,17,32,64,128,192,133,2,194,223,133,216,143,110,37,114,60,18,186,252,251,70,114,57,185,27,18,32,64,128,64,88,64,246,11,3,239,46,47,
109,236,38,187,226,2,99,185,66,217,61,8,16,32,64,224,10,1,225,239,10,229,189,247,144,52,246,138,93,247,122,179,185,206,218,157,8,16,32,64,32,33,32,251,37,84,79,169,41,100,156,194,152,43,98,64,57,91,149,9,16,32,64,32,39,32,251,229,108,207,168,44,95,156,161,24,175,97,76,113,98,55,32,64,128,0,129,147,4,68,191,147,32,163,101,36,139,40,239,185,197,13,235,92,79,213,8,16,32,64,224,92,1,209,239,92,207,100,53,153,34,169,27,169,109,100,17,86,69,9,16,32,64,224,9,1,209,239,9,188,91,46,149,38,110,97,63,225,166,38,119,2,162,18,4,8,16,32,240,164,128,228,247,36,224,93,151,139,17,119,201,159,116,95,3,60,9,82,25,2,4,8,16,216,37,32,249,237,226,170,123,177,252,80,55,146,131,13,153,228,65,56,151,17,32,64,128,192,14,1,185,111,7,86,243,75,197,134,230,233,28,235,205,76,143,185,185,138,0,1,2,4,190,19,16,251,190,147,25,246,235,194,194,176,163,219,214,184,1,111,115,242,42,2,4,8,16,120,47,32,243,189,247,152,237,179,255,7,14,215,65,163,253,149,199,243,0,0,0,0,73,69,78,68,174,66,96,130);


template_circle_2560px
:array[0..50938] of byte=(
137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,10,0,0,0,10,0,8,3,0,0,0,74,138,139,234,0,0,0,23,116,69,88,116,83,111,102,116,119,97,114,101,0,68,97,110,105,32,118,49,46,48,48,46,50,54,48,20,195,73,175,0,0,0,30,116,69,88,116,98,101,46,112,110,103,46,115,101,116,116,105,110,103,115,0,53,51,54,56,55,48,57,49,49,46,45,49,46,53,41,132,140,74,0,0,0,6,80,76,84,69,255,0,0,0,0,0,65,163,18,3,0,0,0,2,116,82,78,83,0,255,91,145,34,181,0,0,198,85,73,68,65,84,120,1,236,221,233,98,219,184,146,6,208,244,188,255,59,223,113,146,110,91,78,180,144,34,150,90,206,252,25,217,150,0,212,41,144,248,36,59,125,127,252,240,127,4,8,16,32,64,128,0,1,2,4,8,16,32,64,128,0,1,2,4,8,16,32,64,128,0,1,2,4,8,16,32,64,128,0,1,2,4,8,16,32,64,128,0,1,2,4,8,16,32,64,128,0,1,2,4,8,16,32,64,128,0,1,2,4,8,16,32,64,128,0,1,2,4,8,16,32,64,128,0,1,2,4,8,16,32,64,128,0,1,2,4,8,16,32,64,128,0,1,2,4,8,16,32,64,224,130,192,63,47,255,239,194,224,94,74,128,0,129,76,2,255,151,105,177,214,74,128,0,129,7,2,47,179,221,199,19,30,188,
244,230,219,99,70,185,25,208,67,2,4,8,196,20,16,0,99,246,197,170,8,16,120,44,112,47,166,61,126,246,232,159,236,157,125,116,53,198,35,64,160,169,128,0,216,180,241,202,38,144,72,224,207,200,21,111,233,241,87,24,207,204,138,8,16,216,42,32,0,110,229,55,57,1,2,247,5,190,37,170,251,79,137,252,221,228,203,143,76,107,109,4,8,140,17,16,0,199,56,26,133,0,129,235,2,55,177,233,250,96,113,70,40,90,86,28,96,43,33,64,224,13,1,1,240,13,52,47,33,64,96,156,64,171,120,212,170,216,113,123,196,72,4,8,140,23,16,0,199,155,26,145,0,129,151,2,95,73,232,229,83,139,62,129,64,209,198,42,139,64,18,1,1,48,73,163,44,147,64,9,129,207,212,83,162,154,65,69,64,25,4,105,24,2,4,78,8,8,128,39,176,60,149,0,129,55,5,254,203,56,111,190,188,201,203,40,53,105,180,50,9,68,16,16,0,35,116,193,26,8,20,21,248,47,209,28,248,79,48,23,21,120,171,44,108,111,177,121,17,1,2,103,4,4,192,51,90,158,75,128,192,49,129,127,35,204,177,39,123,214,35,1,140,143,100,124,159,0,129,171,2,2,224,85,65,175,39,64,224,75,224,191,207,174,190,190,227,
209,117,1,170,215,13,141,64,128,192,31,2,2,224,31,32,190,36,64,224,29,1,159,85,189,163,118,242,53,144,79,130,121,58,1,2,143,5,4,192,199,54,126,66,128,192,1,129,223,169,228,192,19,61,101,144,0,241,65,144,134,33,208,90,64,0,108,221,126,197,19,184,32,224,243,168,11,120,215,95,138,255,186,161,17,8,116,22,16,0,59,119,95,237,4,222,19,240,17,212,123,110,51,94,165,23,51,84,141,73,160,129,128,0,216,160,201,74,36,48,76,64,220,24,70,57,116,32,125,25,202,105,48,2,29,4,4,192,14,93,86,35,129,235,2,34,198,117,195,217,35,232,209,108,97,227,19,40,36,32,0,22,106,166,82,8,76,17,16,43,166,176,78,27,84,191,166,209,26,152,64,37,1,1,176,82,55,213,66,96,172,128,40,49,214,115,229,104,122,183,82,219,92,4,18,10,8,128,9,155,102,201,4,230,11,252,202,15,243,167,49,195,92,1,109,156,235,107,116,2,137,5,4,192,196,205,179,116,2,51,4,124,116,52,67,117,231,152,58,186,83,223,220,4,162,10,8,128,81,59,99,93,4,214,11,248,188,104,189,249,178,25,53,119,25,181,137,8,164,16,16,0,83,180,201,34,9,204,22,16,15,102,11,135,
24,95,155,67,180,193,34,8,68,16,16,0,35,116,193,26,8,236,20,16,10,118,234,111,152,91,195,55,160,155,146,64,56,1,1,48,92,75,44,136,192,58,1,81,96,157,117,176,153,180,62,88,67,44,135,192,106,1,1,112,181,184,249,8,196,16,16,0,98,244,97,235,42,108,130,173,252,38,39,176,85,64,0,220,202,111,114,2,91,4,126,158,251,91,38,54,105,64,1,187,33,96,83,44,137,192,124,1,1,112,190,177,25,8,4,18,240,153,79,160,102,196,89,138,109,17,167,23,86,66,96,145,128,0,184,8,218,52,4,246,11,248,168,103,127,15,34,175,192,254,136,220,29,107,35,48,90,64,0,28,45,106,60,2,17,5,124,194,19,177,43,1,215,100,163,4,108,138,37,17,152,34,32,0,78,97,53,40,129,64,2,62,216,9,212,140,20,75,177,99,82,180,201,34,9,92,19,16,0,175,249,121,53,129,208,2,62,207,9,221,158,200,139,179,117,34,119,199,218,8,12,16,16,0,7,32,26,130,64,68,1,31,227,68,236,74,174,53,217,67,185,250,101,181,4,206,8,8,128,103,180,60,151,64,18,1,7,119,146,70,37,88,166,189,148,160,73,150,72,224,13,1,1,240,13,52,47,33,16,89,192,129,29,185,59,57,215,102,79,229,
236,155,85,19,120,38,32,0,62,211,241,51,2,201,4,28,212,201,26,150,104,185,246,86,162,102,89,42,129,3,2,2,224,1,36,79,33,144,65,192,1,157,161,75,185,215,104,143,229,238,159,213,19,184,21,16,0,111,53,60,38,144,84,192,193,156,180,113,9,151,109,175,37,108,154,37,19,184,35,32,0,222,65,241,45,2,153,4,28,200,153,186,85,99,173,246,92,141,62,170,162,183,128,0,216,187,255,170,79,46,224,32,78,222,192,196,203,183,247,18,55,207,210,9,124,8,8,128,182,1,129,164,2,14,224,164,141,43,180,108,123,176,80,51,149,210,78,64,0,108,215,114,5,87,16,112,240,86,232,98,141,26,236,197,26,125,84,69,63,1,1,176,95,207,85,156,92,192,129,155,188,129,5,151,111,79,22,108,170,146,202,11,8,128,229,91,172,192,82,2,31,39,109,169,122,20,83,70,192,214,44,211,74,133,52,17,16,0,155,52,90,153,5,4,124,204,82,160,137,165,75,176,67,75,183,87,113,213,4,4,192,106,29,85,79,81,1,159,175,20,109,108,181,178,108,212,106,29,85,79,89,1,1,176,108,107,21,86,71,192,7,43,117,122,217,161,18,251,181,67,151,213,152,95,64,0,204,223,67,21,148,
22,112,152,150,110,111,217,226,236,219,178,173,85,88,25,1,1,176,76,43,21,82,80,224,227,20,45,88,149,146,154,8,216,190,77,26,173,204,164,2,2,96,210,198,89,118,125,1,199,103,253,30,151,175,208,38,46,223,98,5,230,21,16,0,243,246,206,202,11,11,124,156,155,62,251,43,220,223,78,165,217,203,157,186,173,214,76,2,2,96,166,110,89,107,15,1,225,175,71,159,27,85,105,75,55,106,182,82,211,8,8,128,105,90,101,161,45,4,124,92,210,162,205,13,139,180,179,27,54,93,201,177,5,4,192,216,253,177,186,86,2,62,39,105,213,238,126,197,218,224,253,122,174,226,192,2,2,96,224,230,88,90,43,1,135,99,171,118,119,45,214,54,239,218,121,117,199,19,16,0,227,245,196,138,26,10,56,22,27,54,189,107,201,54,123,215,206,171,59,152,128,0,24,172,33,150,211,79,224,227,60,244,47,126,251,181,189,117,197,246,124,235,246,43,62,136,128,0,24,164,17,150,209,84,64,248,107,218,120,101,219,250,246,0,129,189,2,2,224,94,127,179,183,22,112,4,182,110,191,226,93,0,246,0,129,141,2,2,224,70,124,83,183,22,112,248,181,110,191,226,127,11,184,12,
236,4,2,187,4,4,192,93,242,230,109,45,224,216,107,221,126,197,223,10,184,24,110,53,60,38,176,76,64,0,92,70,109,34,2,255,10,56,240,108,5,2,223,4,92,18,223,56,124,65,96,137,128,0,184,132,217,36,4,254,19,112,212,253,39,225,255,19,184,17,112,97,220,96,120,72,96,133,128,0,184,66,217,28,4,126,11,56,228,236,4,2,15,5,92,30,15,105,252,128,192,4,1,1,112,2,170,33,9,220,19,112,188,221,83,241,61,2,55,2,46,146,27,12,15,9,204,21,16,0,231,250,26,157,192,111,1,7,155,157,64,224,144,128,75,229,16,147,39,17,184,44,32,0,94,38,52,0,129,87,2,142,180,87,66,126,78,224,70,192,5,115,131,225,33,129,89,2,2,224,44,89,227,18,248,45,224,48,179,19,8,156,22,112,217,156,38,243,2,2,39,5,4,192,147,96,158,78,224,140,128,99,236,140,150,231,18,184,17,112,241,220,96,120,72,96,188,128,0,56,222,212,136,4,126,11,56,192,236,4,2,151,4,92,66,151,248,188,152,192,83,1,1,240,41,143,31,18,120,87,192,209,245,174,156,215,17,184,17,112,33,221,96,120,72,96,164,128,0,56,82,211,88,4,126,11,56,180,236,4,2,195,4,92,78,195,40,13,68,224,
70,64,0,188,193,240,144,192,16,129,127,254,25,50,140,65,8,16,248,87,192,53,101,43,16,24,46,32,0,14,39,53,96,111,1,159,86,244,238,191,234,39,9,184,176,38,193,26,182,175,128,0,216,183,247,42,31,47,224,144,26,111,106,68,2,255,10,184,188,108,5,2,35,5,4,192,145,154,198,106,45,224,120,106,221,126,197,175,16,112,145,173,80,54,71,19,1,1,176,73,163,149,57,89,192,193,52,25,216,240,4,126,11,184,212,236,4,2,99,4,4,192,49,142,70,233,45,224,79,212,123,247,95,245,107,5,92,111,107,189,205,86,84,64,0,44,218,88,101,173,19,240,137,196,58,107,51,17,248,37,224,162,179,17,8,92,22,16,0,47,19,26,160,181,128,131,168,117,251,21,191,79,192,165,183,207,222,204,53,4,4,192,26,125,84,197,30,1,191,138,218,227,110,86,2,63,5,92,127,246,1,129,11,2,2,224,5,60,47,237,45,224,19,136,222,253,87,125,0,1,23,97,128,38,88,66,86,1,1,48,107,231,172,123,175,128,131,103,175,191,217,9,252,43,224,82,180,21,8,188,39,32,0,190,231,230,85,189,5,252,234,169,119,255,85,31,75,192,245,24,171,31,86,147,68,64,0,76,210,40,203,140,35,224,
19,135,56,189,176,18,2,191,4,92,148,54,2,129,211,2,2,224,105,50,47,232,45,224,195,134,222,253,87,125,84,1,87,102,212,206,88,87,84,1,1,48,106,103,172,43,162,128,207,25,34,118,197,154,8,252,18,112,121,218,8,4,206,8,8,128,103,180,60,183,183,128,143,24,122,247,95,245,241,5,92,163,241,123,100,133,97,4,4,192,48,173,176,144,216,2,62,93,136,221,31,171,35,240,75,192,133,106,35,16,56,40,32,0,30,132,242,180,222,2,62,88,232,221,127,213,103,18,112,181,102,234,150,181,238,19,16,0,247,217,155,57,139,128,207,20,178,116,202,58,9,252,18,112,201,218,8,4,94,11,8,128,175,141,60,163,183,128,143,19,122,247,95,245,57,5,92,183,57,251,102,213,11,5,4,192,133,216,166,202,39,224,147,132,124,61,179,98,2,191,4,92,188,54,2,129,167,2,2,224,83,30,63,236,45,224,67,132,222,253,87,125,118,1,87,112,246,14,90,255,76,1,1,112,166,174,177,51,11,248,252,32,115,247,172,157,192,47,1,151,177,141,64,224,145,128,0,248,72,198,247,123,11,248,232,160,119,255,85,95,71,192,181,92,167,151,42,25,42,32,0,14,229,52,88,13,1,159,26,212,
232,163,42,8,252,18,112,65,219,8,4,238,8,8,128,119,80,124,171,183,128,15,12,122,247,95,245,21,5,92,213,21,187,170,166,107,2,2,224,53,63,175,46,39,224,160,40,215,82,5,17,248,16,112,101,219,6,4,190,11,8,128,223,61,124,213,91,192,175,138,122,247,95,245,165,5,92,222,165,219,171,184,211,2,2,224,105,50,47,40,43,224,35,130,178,173,85,24,129,95,2,174,113,27,129,192,167,128,0,248,73,225,65,115,1,71,67,243,13,160,252,22,2,174,243,22,109,86,228,17,1,1,240,136,146,231,148,23,240,203,161,242,45,86,32,129,223,2,46,118,59,129,192,47,1,1,208,70,32,224,207,195,237,1,2,173,4,124,12,216,170,221,138,125,32,32,0,62,128,241,237,62,2,14,131,62,189,86,41,129,223,2,174,122,59,129,128,0,104,15,244,22,240,235,160,222,253,87,125,91,1,151,126,219,214,43,252,95,1,1,208,86,232,44,224,99,128,206,221,87,123,119,1,215,127,247,29,208,188,126,1,176,249,6,104,93,190,219,127,235,246,43,158,128,63,255,181,7,58,11,8,128,157,187,223,187,118,241,175,119,255,85,79,224,167,128,251,128,125,208,86,64,0,108,219,250,222,133,
251,251,159,222,253,87,61,129,79,1,55,131,79,10,15,122,9,8,128,189,250,173,218,95,2,222,244,219,8,4,8,124,9,184,35,124,89,120,212,71,64,0,236,211,107,149,254,43,224,102,111,43,16,32,240,93,192,93,225,187,135,175,58,8,8,128,29,186,172,198,27,1,55,250,27,12,15,9,16,248,87,192,157,193,86,232,38,32,0,118,235,120,243,122,221,228,155,111,0,229,19,120,40,224,238,240,144,198,15,74,10,8,128,37,219,170,168,187,2,254,216,251,46,139,111,18,32,240,91,192,45,194,78,232,36,32,0,118,234,118,239,90,189,189,239,221,127,213,19,56,34,224,62,113,68,201,115,74,8,8,128,37,218,168,136,151,2,110,235,47,137,60,129,0,129,15,1,247,10,219,160,137,128,0,216,164,209,205,203,116,75,111,190,1,148,79,224,132,128,251,197,9,44,79,205,43,32,0,230,237,157,149,31,21,112,59,63,42,229,121,4,8,252,20,112,207,176,15,26,8,8,128,13,154,220,188,68,183,242,230,27,64,249,4,222,16,112,223,120,3,205,75,114,9,8,128,185,250,101,181,39,5,252,171,190,147,96,158,78,128,192,111,1,55,15,59,161,184,128,0,88,188,193,189,203,243,38,190,
119,255,85,79,224,154,128,59,200,53,63,175,142,45,32,0,198,238,143,213,93,16,112,243,190,128,231,165,4,8,124,8,184,139,216,6,117,5,4,192,186,189,109,94,153,27,119,243,13,160,124,2,67,4,220,73,134,48,26,36,160,128,0,24,176,41,150,116,93,192,77,251,186,161,17,8,16,248,41,224,110,98,31,212,20,16,0,107,246,181,121,85,110,216,205,55,128,242,9,12,21,112,71,25,202,105,176,32,2,2,96,144,70,88,198,56,1,55,235,113,150,70,34,64,224,167,128,187,138,125,80,79,64,0,172,215,211,230,21,185,81,55,223,0,202,39,48,69,192,157,101,10,171,65,55,10,8,128,27,241,77,61,94,192,77,122,188,169,17,9,16,248,41,224,238,98,31,212,18,16,0,107,245,179,121,53,110,208,205,55,128,242,9,76,21,112,135,153,202,107,240,197,2,2,224,98,112,211,205,19,112,115,158,103,107,100,2,4,126,10,184,203,216,7,117,4,4,192,58,189,108,94,137,27,115,243,13,160,124,2,75,4,220,105,150,48,155,100,129,128,0,184,0,217,20,243,5,220,148,231,27,155,129,0,129,159,2,238,54,246,65,13,1,1,176,70,31,155,87,225,134,220,124,3,40,159,192,82,1,119,156,
165,220,38,155,36,32,0,78,130,53,236,58,1,55,227,117,214,102,34,64,224,167,128,187,142,125,144,95,64,0,204,223,195,230,21,184,17,55,223,0,202,39,176,69,192,157,103,11,187,73,7,10,8,128,3,49,13,181,94,192,77,120,189,185,25,9,16,248,41,224,238,99,31,228,22,16,0,115,247,175,249,234,221,128,155,111,0,229,19,216,42,224,14,180,149,223,228,23,5,4,192,139,128,94,190,79,192,205,119,159,189,153,9,16,248,41,224,46,100,31,228,21,16,0,243,246,174,249,202,221,120,155,111,0,229,19,8,33,224,78,20,162,13,22,241,134,128,0,248,6,154,151,236,23,112,211,221,223,3,43,32,64,224,167,128,187,145,125,144,83,64,0,204,217,183,230,171,118,195,109,190,1,148,79,32,148,128,59,82,168,118,88,204,65,1,1,240,32,148,167,197,17,112,179,141,211,11,43,33,64,224,167,128,187,146,125,144,79,64,0,204,215,179,230,43,118,163,109,190,1,148,79,32,164,128,59,83,200,182,88,212,19,1,1,240,9,142,31,197,19,112,147,141,215,19,43,34,64,224,167,128,187,147,125,144,75,64,0,204,213,175,238,171,253,167,59,128,250,9,16,136,43,224,6,21,
183,55,86,246,183,128,0,248,183,137,239,68,21,240,6,59,106,103,172,139,0,129,159,2,238,81,246,65,34,1,1,48,81,179,154,47,213,173,181,249,6,80,62,129,4,2,238,83,9,154,100,137,191,5,4,64,59,33,135,128,219,106,142,62,89,37,129,238,2,238,85,221,119,64,154,250,5,192,52,173,106,189,80,183,212,214,237,87,60,129,84,2,238,87,169,218,213,119,177,2,96,223,222,231,169,220,237,52,79,175,172,148,0,1,127,11,104,15,164,16,16,0,83,180,169,245,34,197,191,214,237,87,60,129,148,2,238,91,41,219,214,107,209,2,96,175,126,39,172,214,127,88,33,97,211,44,153,0,1,183,46,123,32,184,128,0,24,188,65,221,151,231,109,116,247,29,160,126,2,89,5,220,189,178,118,174,203,186,5,192,46,157,78,89,167,27,104,202,182,89,52,1,2,191,4,220,193,108,132,200,2,2,96,228,238,52,95,155,155,103,243,13,160,124,2,233,5,220,197,210,183,176,112,1,2,96,225,230,230,46,205,141,51,119,255,172,158,0,129,159,2,238,100,246,65,84,1,1,48,106,103,154,175,203,77,179,249,6,80,62,129,50,2,238,102,101,90,89,172,16,1,176,88,67,139,148,227,223,207,
21,105,164,50,8,16,248,248,20,16,2,129,128,2,2,96,192,166,180,95,146,55,204,237,183,0,0,2,165,4,220,211,74,181,179,74,49,2,96,149,78,214,169,195,173,178,78,47,85,66,128,192,111,1,247,53,59,33,156,128,0,24,174,37,205,23,228,54,217,124,3,40,159,64,81,1,247,182,162,141,205,91,150,0,152,183,119,37,87,238,111,101,74,182,85,81,4,8,248,83,64,123,32,152,128,0,24,172,33,189,151,227,45,114,239,254,171,158,64,109,1,119,184,218,253,205,86,157,0,152,173,99,133,215,235,230,88,184,185,74,35,64,224,67,192,93,206,54,136,35,32,0,198,233,69,243,149,184,49,54,223,0,202,39,208,66,192,157,174,69,155,83,20,41,0,166,104,83,253,69,186,41,214,239,177,10,9,16,248,41,224,110,103,31,196,16,16,0,99,244,161,251,42,252,219,143,238,59,64,253,4,26,9,184,225,53,106,118,224,82,5,192,192,205,105,179,52,111,136,219,180,90,161,4,8,124,8,184,231,217,6,1,4,4,192,0,77,104,190,4,183,194,230,27,64,249,4,26,10,184,239,53,108,122,180,146,5,192,104,29,105,183,30,191,12,105,215,114,5,19,32,224,191,10,104,15,108,23,16,0,183,
183,160,247,2,188,13,238,221,127,213,19,232,43,224,238,215,183,247,49,42,23,0,99,244,161,233,42,220,0,155,54,94,217,4,8,124,8,184,3,218,6,59,5,4,192,157,250,205,231,118,243,107,190,1,148,79,160,189,128,187,96,251,45,176,17,64,0,220,136,223,124,106,127,252,215,124,3,40,159,0,1,127,10,104,15,236,19,16,0,247,217,247,158,217,27,223,222,253,87,61,1,2,191,5,220,11,237,132,77,2,2,224,38,248,230,211,186,229,53,223,0,202,39,64,224,83,192,253,240,147,194,131,149,2,2,224,74,109,115,253,43,224,183,191,182,2,1,2,4,62,5,220,18,63,41,60,88,39,32,0,174,179,54,211,191,2,222,238,218,10,4,8,16,184,21,112,87,188,213,240,120,141,128,0,184,198,217,44,159,2,110,116,159,20,30,16,32,64,224,95,1,119,70,91,97,181,128,0,184,90,188,251,124,126,213,209,125,7,168,159,0,129,187,2,110,142,119,89,124,115,154,128,0,56,141,214,192,119,4,188,201,189,131,226,91,4,8,16,248,16,112,127,180,13,150,10,8,128,75,185,155,79,230,246,214,124,3,40,159,0,129,167,2,238,145,79,121,252,112,172,128,0,56,214,211,104,79,4,252,130,
227,9,142,31,17,32,64,192,127,23,218,30,88,40,32,0,46,196,238,61,149,183,182,189,251,175,122,2,4,142,8,184,83,30,81,242,156,17,2,2,224,8,69,99,188,20,112,83,123,73,228,9,4,8,16,248,16,112,183,180,13,214,8,8,128,107,156,187,207,226,183,191,221,119,128,250,9,16,56,44,224,134,121,152,202,19,47,8,8,128,23,240,188,244,160,128,55,180,7,161,60,141,0,1,2,31,2,238,153,182,193,2,1,1,112,1,114,247,41,188,155,237,190,3,212,79,128,192,73,1,183,205,147,96,158,126,94,64,0,60,111,230,21,167,4,188,149,61,197,229,201,4,8,16,248,16,112,231,180,13,102,11,8,128,179,133,155,143,239,38,214,124,3,40,159,0,129,55,5,220,61,223,132,243,178,131,2,2,224,65,40,79,123,75,192,175,49,222,98,243,34,2,4,8,248,143,2,218,3,115,5,4,192,185,190,189,71,247,6,182,119,255,85,79,128,192,53,1,247,208,107,126,94,253,84,64,0,124,202,227,135,87,4,124,252,119,69,207,107,9,16,32,240,195,109,212,38,152,38,32,0,78,163,237,62,176,183,174,221,119,128,250,9,16,184,46,224,78,122,221,208,8,247,5,4,192,251,46,190,123,81,192,77,235,
34,160,151,19,32,64,224,151,128,187,169,141,48,71,64,0,156,227,218,125,84,191,182,232,190,3,212,79,128,192,48,1,55,212,97,148,6,186,17,16,0,111,48,60,28,36,224,13,235,32,72,195,16,32,64,224,67,192,61,213,54,152,32,32,0,78,64,237,62,164,119,171,221,119,128,250,9,16,24,44,224,182,58,24,212,112,63,126,8,128,118,193,96,1,111,85,7,131,26,142,0,1,2,62,4,180,7,134,11,8,128,195,73,155,15,232,125,106,243,13,160,124,2,4,230,8,184,185,206,113,237,59,170,0,216,183,247,51,42,247,241,223,12,85,99,18,32,64,192,95,2,218,3,131,5,4,192,193,160,173,135,19,255,90,183,95,241,4,8,76,22,112,143,157,12,220,107,120,1,176,87,191,167,86,235,23,20,83,121,13,78,128,0,1,183,89,123,96,152,128,0,56,140,178,251,64,222,154,118,223,1,234,39,64,96,190,128,59,237,124,227,46,51,8,128,93,58,61,187,78,239,75,103,11,27,159,0,1,2,31,2,110,182,182,193,24,1,1,112,140,99,247,81,188,41,237,190,3,212,79,128,192,42,1,247,219,85,210,197,231,17,0,139,55,120,77,121,222,145,174,113,54,11,1,2,4,62,4,220,114,109,131,1,2,2,224,0,
196,238,67,120,59,218,125,7,168,159,0,129,181,2,238,186,107,189,107,206,38,0,214,236,235,202,170,188,23,93,169,109,46,2,4,8,124,8,184,241,218,6,87,5,4,192,171,130,221,95,239,141,104,247,29,160,126,2,4,118,8,184,247,238,80,47,53,167,0,88,170,157,235,139,241,46,116,189,185,25,9,16,32,240,33,224,246,107,27,92,18,16,0,47,241,117,127,177,183,160,221,119,128,250,9,16,216,39,224,14,188,207,190,194,204,2,96,133,46,238,170,193,251,207,93,242,230,37,64,128,192,135,128,155,176,109,240,190,128,0,248,190,93,247,87,122,243,217,125,7,168,159,0,129,221,2,238,195,187,59,144,120,126,1,48,113,243,246,46,221,59,207,189,254,102,39,64,128,192,135,128,91,177,109,240,166,128,0,248,38,92,247,151,121,219,217,125,7,168,159,0,129,24,2,238,198,49,250,144,111,21,2,96,190,158,69,88,177,247,156,17,186,96,13,4,8,16,248,16,112,67,182,13,222,17,16,0,223,81,235,254,26,111,56,187,239,0,245,19,32,16,73,192,61,57,82,55,210,172,69,0,76,211,170,56,11,245,110,51,78,47,172,132,0,1,2,31,2,110,203,182,193,105,1,1,240,52,
89,247,23,120,171,217,125,7,168,159,0,129,120,2,238,204,241,122,18,125,69,2,96,244,14,69,91,159,247,153,209,58,98,61,4,8,16,248,16,112,115,182,13,206,9,8,128,231,188,218,63,219,45,166,253,22,0,64,128,64,76,1,183,231,152,125,9,187,42,1,48,108,107,34,46,204,47,25,34,118,197,154,8,16,32,240,83,192,29,218,62,56,35,32,0,158,209,234,254,92,239,47,187,239,0,245,19,32,16,90,192,77,58,116,123,130,45,78,0,12,214,144,192,203,241,230,50,112,115,44,141,0,1,2,31,2,238,211,182,193,97,1,1,240,48,85,247,39,122,103,217,125,7,168,159,0,129,4,2,110,213,9,154,20,99,137,2,96,140,62,132,95,133,183,149,225,91,100,129,4,8,16,248,16,112,183,182,13,142,9,8,128,199,156,186,63,203,123,202,238,59,64,253,4,8,164,17,112,195,78,211,170,173,11,21,0,183,242,103,153,220,237,36,75,167,172,147,0,1,2,254,147,128,246,192,17,1,1,240,136,82,243,231,248,133,66,243,13,160,124,2,4,146,9,184,107,39,107,216,150,229,10,128,91,216,83,77,234,227,191,84,237,178,88,2,4,8,248,223,5,177,7,94,11,8,128,175,141,122,63,195,27,201,222,
253,87,61,1,2,57,5,220,187,115,246,109,225,170,5,192,133,216,25,167,242,241,95,198,174,89,51,1,2,4,252,37,160,61,240,92,64,0,124,238,211,252,167,222,66,54,223,0,202,39,64,32,177,128,59,120,226,230,45,88,186,0,184,0,57,237,20,62,254,75,219,58,11,39,64,128,128,191,4,180,7,158,9,8,128,207,116,154,255,76,254,107,190,1,148,79,128,64,118,1,183,241,236,29,156,184,126,1,112,34,110,238,161,253,242,32,119,255,172,158,0,1,2,254,119,65,236,129,199,2,2,224,99,155,222,63,241,190,177,119,255,85,79,128,64,17,1,55,243,34,141,28,94,134,0,56,156,180,196,128,62,254,43,209,70,69,16,32,64,192,255,56,176,61,112,95,64,0,188,239,210,252,187,222,49,54,223,0,202,39,64,160,146,128,91,122,165,110,14,171,69,0,28,70,89,104,32,55,139,66,205,84,10,1,2,4,220,212,237,129,191,5,4,192,191,77,186,127,199,175,127,187,239,0,245,19,32,80,77,192,125,189,90,71,7,212,35,0,14,64,172,53,132,119,138,181,250,169,26,2,4,8,124,8,184,181,219,6,127,8,8,128,127,128,180,255,210,77,162,253,22,0,64,128,64,69,1,55,247,138,93,189,82,
147,0,120,69,175,222,107,253,154,160,94,79,85,68,128,0,129,159,2,238,239,246,193,55,1,1,240,27,71,247,47,188,67,236,190,3,212,79,128,64,97,1,183,248,194,205,61,95,154,0,120,222,172,238,43,220,28,234,246,86,101,4,8,16,240,135,128,246,192,141,128,0,120,131,209,252,161,95,15,52,223,0,202,39,64,160,188,128,251,124,249,22,31,47,80,0,60,110,85,252,153,62,254,43,222,96,229,17,32,64,192,191,6,182,7,62,5,4,192,79,138,222,15,188,45,236,221,127,213,19,32,208,69,192,221,190,75,167,95,213,41,0,190,18,234,241,115,31,255,245,232,179,42,9,16,32,224,47,1,237,129,95,2,2,160,141,240,33,32,255,217,6,4,8,16,104,35,224,150,223,166,213,207,10,21,0,159,233,52,249,153,95,8,52,105,180,50,9,16,32,240,75,192,93,223,70,248,241,67,0,180,11,188,23,180,7,8,16,32,208,76,192,141,191,89,195,239,148,43,0,222,65,233,245,45,183,129,94,253,86,45,1,2,4,62,4,220,250,219,111,3,1,176,249,22,240,139,128,230,27,64,249,4,8,52,21,112,247,111,218,248,207,178,5,192,79,138,150,15,188,7,108,217,118,69,19,32,64,192,135,128,221,
247,128,0,216,122,7,200,127,173,219,175,120,2,4,122,11,56,2,90,247,95,0,236,220,126,23,127,231,238,171,157,0,129,246,2,14,129,206,91,64,0,236,219,125,127,0,210,183,247,42,39,64,128,192,79,1,231,64,227,125,32,0,182,109,190,119,126,109,91,175,112,2,4,8,252,39,224,40,248,79,162,221,255,23,0,219,181,252,223,130,93,244,93,59,175,110,2,4,8,220,8,56,12,110,48,90,61,20,0,91,181,251,179,88,31,251,127,82,120,64,128,0,129,214,2,206,131,166,237,23,0,91,54,222,59,190,150,109,87,52,1,2,4,238,9,56,18,238,169,148,255,158,0,88,190,197,119,10,116,177,223,65,241,45,2,4,8,116,21,112,40,116,236,188,0,216,175,235,62,238,239,215,115,21,19,32,64,224,153,128,115,225,153,78,209,159,9,128,69,27,251,184,44,239,244,30,219,248,9,1,2,4,154,10,56,26,218,53,94,0,236,214,114,23,121,183,142,171,151,0,1,2,7,4,28,14,7,144,74,61,69,0,44,213,206,215,197,184,196,95,27,121,6,1,2,4,26,10,56,30,154,53,93,0,108,213,112,127,230,209,170,221,138,37,64,128,192,9,1,39,196,9,172,2,79,21,0,11,52,241,112,9,222,223,29,166,242,68,
2,4,8,244,19,112,72,116,234,185,0,216,168,219,46,237,70,205,86,42,1,2,4,206,11,56,38,206,155,165,125,133,0,152,182,117,167,23,238,194,62,77,230,5,4,8,16,232,37,224,160,232,211,111,1,176,75,175,253,113,71,151,78,171,147,0,1,2,239,11,56,43,222,183,75,246,74,1,48,89,195,222,93,174,119,117,239,202,121,29,1,2,4,90,9,56,46,154,180,91,0,236,209,104,23,116,143,62,171,146,0,1,2,151,5,28,24,151,9,83,12,32,0,166,104,211,213,69,186,156,175,10,122,61,1,2,4,218,8,56,50,90,180,90,0,108,208,102,127,210,209,160,201,74,36,64,128,192,48,1,167,198,48,202,192,3,9,128,129,155,51,104,105,222,203,13,130,52,12,1,2,4,186,8,56,56,234,119,90,0,44,223,99,151,113,249,22,43,144,0,1,2,163,5,28,29,163,69,195,141,39,0,134,107,201,224,5,185,136,7,131,26,142,0,1,2,29,4,28,30,213,187,44,0,214,238,176,63,228,168,221,95,213,17,32,64,96,150,128,243,99,150,108,144,113,5,192,32,141,152,179,12,239,224,230,184,26,149,0,1,2,13,4,28,33,165,155,44,0,86,110,175,139,183,114,119,213,70,128,0,129,201,2,14,145,201,192,91,135,23,
0,183,242,207,157,220,165,59,215,215,232,4,8,16,40,46,224,24,41,220,96,1,176,108,115,253,249,70,217,214,42,140,0,1,2,139,4,156,36,139,160,55,76,35,0,110,64,95,50,165,247,109,75,152,77,66,128,0,129,218,2,14,147,170,253,21,0,139,118,214,37,91,180,177,202,34,64,128,192,90,1,199,201,90,239,101,179,9,128,203,168,151,78,228,130,93,202,109,50,2,4,8,212,21,112,160,212,236,173,0,88,178,175,46,215,146,109,85,20,1,2,4,118,8,56,82,118,168,79,159,83,0,156,78,188,126,2,127,180,187,222,220,140,4,8,16,168,43,224,84,169,216,91,1,176,94,87,189,87,171,215,83,21,17,32,64,96,171,128,131,101,43,255,148,201,5,192,41,172,59,7,117,153,238,212,55,55,1,2,4,74,10,56,90,202,181,85,0,172,214,82,23,105,181,142,170,135,0,1,2,1,4,28,46,1,154,48,116,9,2,224,80,206,253,131,185,68,247,247,192,10,8,16,32,80,80,192,241,82,172,169,2,96,169,134,250,67,221,82,237,84,12,1,2,4,2,9,56,97,2,53,99,192,82,4,192,1,136,97,134,240,254,44,76,43,44,132,0,1,2,245,4,28,50,149,122,42,0,22,234,166,75,179,80,51,149,66,128,0,129,120,
2,142,153,120,61,121,123,69,2,224,219,116,225,94,232,194,12,215,18,11,34,64,128,64,45,1,7,77,157,126,10,128,101,122,233,178,44,211,74,133,16,32,64,32,170,128,163,38,106,103,78,175,75,0,60,77,22,243,5,254,56,55,102,95,172,138,0,1,2,181,4,156,54,85,250,41,0,214,232,164,247,100,53,250,168,10,2,4,8,132,23,112,224,132,111,209,161,5,10,128,135,152,162,63,201,229,24,189,67,214,71,128,0,129,50,2,142,156,18,173,20,0,43,180,209,197,88,161,139,106,32,64,128,64,18,1,135,78,146,70,61,93,166,0,248,148,39,199,15,93,138,57,250,100,149,4,8,16,40,34,224,216,41,208,72,1,48,127,19,93,136,249,123,168,2,2,4,8,164,18,112,240,164,106,215,221,197,10,128,119,89,18,125,211,63,200,74,212,44,75,37,64,128,64,17,1,103,79,250,70,10,128,201,91,232,93,88,242,6,90,62,1,2,4,114,10,56,126,114,246,237,115,213,2,224,39,69,202,7,46,192,148,109,179,104,2,4,8,228,23,112,0,229,238,161,0,152,186,127,46,191,212,237,179,120,2,4,8,100,22,112,4,101,238,222,15,1,48,115,251,92,124,153,187,103,237,4,8,16,72,46,224,16,202,220,
64,1,48,113,247,92,122,137,155,103,233,4,8,16,200,47,224,24,74,220,67,1,48,111,243,92,120,121,123,103,229,4,8,16,40,33,224,32,202,219,70,1,48,107,239,252,19,252,172,157,179,110,2,4,8,212,17,112,22,165,237,165,0,152,180,117,222,117,37,109,156,101,19,32,64,160,150,128,227,40,105,63,5,192,156,141,115,193,229,236,155,85,19,32,64,160,156,128,3,41,103,75,5,192,148,125,115,185,165,108,155,69,19,32,64,160,162,128,35,41,101,87,5,192,140,109,115,177,101,236,154,53,19,32,64,160,168,128,67,41,99,99,5,192,132,93,115,169,37,108,154,37,19,32,64,160,174,128,99,41,97,111,5,192,124,77,115,161,229,235,153,21,19,32,64,160,180,128,131,41,95,123,5,192,108,61,243,79,238,179,117,204,122,9,16,32,80,95,192,217,148,174,199,2,96,178,150,121,151,149,172,97,150,75,128,0,129,30,2,142,167,100,125,22,0,115,53,204,5,150,171,95,86,75,128,0,129,54,2,14,168,92,173,22,0,83,245,203,229,149,170,93,22,75,128,0,129,78,2,142,168,84,221,22,0,51,181,203,197,149,169,91,214,74,128,0,129,102,2,14,169,76,13,23,0,19,117,203,
165,149,168,89,150,74,128,0,129,126,2,142,169,68,61,23,0,243,52,203,133,149,167,87,86,74,128,0,129,150,2,14,170,60,109,23,0,211,244,202,101,149,166,85,22,74,128,0,129,174,2,142,170,52,157,23,0,179,180,202,69,149,165,83,214,73,128,0,129,198,2,14,171,44,205,23,0,115,116,202,127,98,51,71,159,172,146,0,1,2,221,5,156,87,73,118,128,0,152,162,81,222,81,165,104,147,69,18,32,64,128,192,143,31,142,172,20,187,64,0,204,208,38,23,83,134,46,89,35,1,2,4,8,252,18,112,104,101,216,8,2,96,130,46,185,148,18,52,201,18,9,16,32,64,224,63,1,199,214,127,18,129,255,191,0,24,184,57,255,46,205,133,20,191,71,86,72,128,0,1,2,55,2,14,174,27,140,160,15,5,192,160,141,249,90,150,203,232,203,194,35,2,4,8,16,72,33,224,232,10,223,38,1,48,122,139,92,68,209,59,100,125,4,8,16,32,240,151,128,195,235,47,146,96,223,16,0,131,53,228,207,229,184,132,254,20,241,53,1,2,4,8,36,16,112,124,5,111,146,0,24,187,65,46,160,216,253,177,58,2,4,8,16,120,32,224,0,123,0,19,228,219,2,96,144,70,220,95,134,203,231,190,139,239,18,32,64,128,
64,120,1,71,88,232,22,9,128,145,219,227,226,137,220,29,107,35,64,128,0,129,167,2,14,177,167,60,155,127,40,0,110,110,192,179,233,93,58,207,116,252,140,0,1,2,4,130,11,56,198,2,55,72,0,140,219,28,23,78,220,222,88,25,1,2,4,8,28,16,112,144,29,64,218,244,20,1,112,19,252,203,105,253,207,105,191,36,242,4,2,4,8,16,8,46,224,44,11,219,32,1,48,104,107,188,107,10,218,24,203,34,64,128,0,129,51,2,142,179,51,90,11,159,43,0,46,196,62,49,149,11,230,4,150,167,18,32,64,128,64,92,1,7,90,204,222,8,128,33,251,226,114,9,217,22,139,34,64,128,0,129,243,2,142,180,243,102,11,94,33,0,46,64,62,61,133,139,229,52,153,23,16,32,64,128,64,84,1,135,90,196,206,8,128,1,187,226,82,9,216,20,75,34,64,128,0,129,119,5,28,107,239,202,77,124,157,0,56,17,247,205,161,93,40,111,194,121,25,1,2,4,8,196,20,112,176,197,235,139,0,24,174,39,46,147,112,45,177,32,2,4,8,16,184,38,224,104,187,230,55,225,213,2,224,4,212,75,67,186,72,46,241,121,49,1,2,4,8,68,20,112,184,69,235,138,0,24,172,35,46,145,96,13,177,28,2,4,8,16,24,33,224,120,
27,161,56,112,12,1,112,32,230,128,161,92,32,3,16,13,65,128,0,1,2,241,4,28,112,177,122,34,0,134,234,135,203,35,84,59,44,134,0,1,2,4,198,9,56,226,198,89,14,24,73,0,28,128,56,108,8,23,199,48,74,3,17,32,64,128,64,52,1,135,92,164,142,8,128,129,186,225,210,8,212,12,75,33,64,128,0,129,209,2,142,185,209,162,23,198,19,0,47,224,13,126,169,11,99,48,168,225,8,16,32,64,32,150,128,131,46,78,63,4,192,48,189,112,89,132,105,133,133,16,32,64,128,192,28,1,71,221,28,215,55,70,21,0,223,64,155,242,18,23,197,20,86,131,18,32,64,128,64,36,1,135,93,148,110,8,128,65,58,225,146,8,210,8,203,32,64,128,0,129,153,2,142,187,153,186,39,198,22,0,79,96,77,124,170,11,98,34,174,161,9,16,32,64,32,142,128,3,47,70,47,4,192,16,125,112,57,132,104,131,69,16,32,64,128,192,124,1,71,222,124,227,3,51,8,128,7,144,166,63,197,197,48,157,216,4,4,8,16,32,16,69,192,161,23,161,19,2,96,128,46,184,20,2,52,193,18,8,16,32,64,96,149,128,99,111,149,244,147,121,4,192,39,56,139,126,228,66,88,4,109,26,2,4,8,16,136,33,224,224,219,223,7,1,112,
123,15,92,6,219,91,96,1,4,8,16,32,176,86,192,209,183,214,251,206,108,2,224,29,148,165,223,114,17,44,229,54,25,1,2,4,8,68,16,112,248,237,238,130,0,184,185,3,46,129,205,13,48,61,1,2,4,8,236,16,112,252,237,80,191,153,83,0,188,193,216,240,208,5,176,1,221,148,4,8,16,32,176,95,192,1,184,183,7,2,224,86,127,219,127,43,191,201,9,16,32,64,96,159,128,35,112,159,253,199,204,2,224,78,126,155,127,167,190,185,9,16,32,64,96,171,128,67,112,39,191,0,184,81,223,214,223,136,111,106,2,4,8,16,216,45,224,24,220,216,1,1,112,31,190,141,191,207,222,204,4,8,16,32,16,64,192,65,184,175,9,2,224,54,123,219,126,27,189,137,9,16,32,64,32,134,128,163,112,91,31,4,192,93,244,54,253,46,121,243,18,32,64,128,64,24,1,135,225,174,86,8,128,155,228,109,249,77,240,166,37,64,128,0,129,72,2,142,195,77,221,16,0,247,192,219,240,123,220,205,74,128,0,1,2,193,4,28,136,123,26,34,0,110,113,183,221,183,176,155,148,0,1,2,4,226,9,56,18,183,244,68,0,220,193,110,179,239,80,55,39,1,2,4,8,132,20,112,40,238,104,139,0,184,65,221,86,223,128,
110,74,2,4,8,16,136,42,224,88,220,208,25,1,112,61,186,141,190,222,220,140,4,8,16,32,16,88,192,193,184,190,57,2,224,114,115,219,124,57,185,9,9,16,32,64,32,182,128,163,113,121,127,4,192,213,228,54,249,106,113,243,17,32,64,128,64,120,1,135,227,234,22,9,128,139,197,109,241,197,224,166,35,64,128,0,129,12,2,142,199,197,93,18,0,215,130,219,224,107,189,205,70,128,0,1,2,73,4,28,144,107,27,37,0,46,245,182,189,151,114,155,140,0,1,2,4,242,8,56,34,151,246,74,0,92,201,109,115,175,212,54,23,1,2,4,8,164,18,112,72,174,108,151,0,184,80,219,214,94,136,109,42,2,4,8,16,200,38,224,152,92,216,49,1,112,29,182,141,189,206,218,76,4,8,16,32,144,80,192,65,185,174,105,2,224,50,107,219,122,25,181,137,8,16,32,64,32,167,128,163,114,89,223,4,192,85,212,54,245,42,105,243,16,32,64,128,64,90,1,135,229,170,214,9,128,139,164,109,233,69,208,166,33,64,128,0,129,204,2,142,203,69,221,19,0,215,64,219,208,107,156,205,66,128,0,1,2,201,5,28,152,107,26,40,0,46,113,182,157,151,48,155,132,0,1,2,4,242,11,56,50,151,244,80,0,92,
193,108,51,175,80,54,7,1,2,4,8,148,16,112,104,174,104,163,0,184,64,217,86,94,128,108,10,2,4,8,16,168,34,224,216,92,208,73,1,112,62,178,141,60,223,216,12,4,8,16,32,80,72,192,193,57,191,153,2,224,116,99,219,120,58,177,9,8,16,32,64,160,150,128,163,115,122,63,5,192,233,196,38,32,64,128,0,1,2,4,8,196,18,16,0,103,247,195,187,152,217,194,198,39,64,128,0,129,114,2,14,207,217,45,21,0,39,11,219,194,147,129,13,79,128,0,1,2,21,5,28,159,147,187,42,0,206,5,182,129,231,250,26,157,0,1,2,4,138,10,56,64,231,54,86,0,156,234,107,251,78,229,53,56,1,2,4,8,212,21,112,132,78,237,173,0,56,147,215,230,157,169,107,108,2,4,8,16,40,45,224,16,157,217,94,1,112,162,174,173,59,17,215,208,4,8,16,32,80,93,192,49,58,177,195,2,224,60,92,27,119,158,173,145,9,16,32,64,160,129,128,131,116,94,147,5,192,105,182,182,237,52,90,3,19,32,64,128,64,15,1,71,233,180,62,11,128,179,104,109,218,89,178,198,37,64,128,0,129,54,2,14,211,89,173,22,0,39,201,218,178,147,96,13,75,128,0,1,2,157,4,28,167,147,186,45,0,206,129,181,97,231,184,
26,149,0,1,2,4,154,9,56,80,231,52,92,0,156,226,106,187,78,97,53,40,1,2,4,8,244,19,112,164,78,233,185,0,56,131,213,102,157,161,106,76,2,4,8,16,104,41,224,80,157,209,118,1,112,134,170,49,9,16,32,64,128,0,1,2,129,5,4,192,9,205,241,94,101,2,170,33,9,16,32,64,160,171,128,99,117,66,231,5,192,241,168,54,234,120,83,35,18,32,64,128,64,99,1,7,235,248,230,11,128,195,77,109,211,225,164,6,36,64,128,0,129,222,2,142,214,225,253,23,0,71,147,218,164,163,69,141,71,128,0,1,2,237,5,28,174,163,183,128,0,56,88,212,22,29,12,106,56,2,4,8,16,32,240,227,135,227,117,240,46,16,0,199,130,218,160,99,61,141,70,128,0,1,2,4,126,9,56,96,199,110,4,1,112,168,167,237,57,148,211,96,4,8,16,32,64,224,63,1,71,236,127,18,67,254,191,0,56,132,241,223,65,108,206,145,154,198,34,64,128,0,1,2,55,2,14,217,27,140,203,15,5,192,203,132,95,3,216,154,95,22,30,17,32,64,128,0,129,193,2,142,217,129,160,2,224,64,76,67,17,32,64,128,0,1,2,4,50,8,8,128,227,186,228,157,201,56,75,35,17,32,64,128,0,129,191,4,28,180,127,145,188,253,13,1,240,
109,186,63,95,104,91,254,41,226,107,2,4,8,16,32,48,84,192,81,59,140,83,0,28,69,105,83,142,146,52,14,1,2,4,8,16,120,32,224,176,125,0,115,250,219,2,224,105,178,251,47,176,37,239,187,248,46,1,2,4,8,16,24,40,224,184,29,132,41,0,142,129,180,33,199,56,26,133,0,1,2,4,8,60,21,112,224,62,229,57,252,67,1,240,48,213,179,39,218,142,207,116,252,140,0,1,2,4,8,12,19,112,228,14,161,20,0,71,48,218,140,35,20,141,65,128,0,1,2,4,14,8,56,116,15,32,189,124,138,0,248,146,200,19,8,16,32,64,128,0,1,2,181,4,4,192,1,253,244,94,100,0,162,33,8,16,32,64,128,192,49,1,199,238,49,167,167,207,18,0,159,242,28,250,161,141,120,136,201,147,8,16,32,64,128,192,24,1,7,239,117,71,1,240,178,161,109,120,153,208,0,4,8,16,32,64,224,140,128,163,247,140,214,221,231,10,128,119,89,78,124,211,38,60,129,229,169,4,8,16,32,64,96,132,128,195,247,170,162,0,120,81,208,22,188,8,232,229,4,8,16,32,64,224,188,128,227,247,188,217,183,87,8,128,223,56,78,127,97,3,158,38,243,2,2,4,8,16,32,112,93,192,1,124,205,80,0,188,228,103,251,93,226,243,
98,2,4,8,16,32,240,174,128,35,248,93,185,95,175,19,0,47,241,121,49,1,2,4,8,16,32,64,32,159,128,0,120,165,103,222,125,92,209,243,90,2,4,8,16,32,112,65,192,33,124,1,239,135,0,120,65,207,214,187,128,231,165,4,8,16,32,64,224,154,128,99,248,130,159,0,248,62,158,141,247,190,157,87,18,32,64,128,0,129,203,2,14,226,247,9,5,192,183,237,108,187,183,233,188,144,0,1,2,4,8,140,16,112,20,191,173,40,0,190,75,103,211,189,43,231,117,4,8,16,32,64,96,144,128,195,248,93,72,1,240,77,57,91,238,77,56,47,35,64,128,0,1,2,227,4,28,199,111,90,10,128,111,194,121,25,1,2,4,8,16,32,64,32,171,128,0,248,94,231,188,227,120,207,205,171,8,16,32,64,128,192,80,1,7,242,123,156,2,224,91,110,182,219,91,108,94,68,128,0,1,2,4,70,11,56,146,223,18,21,0,223,97,179,217,222,81,243,26,2,4,8,16,32,48,65,192,161,252,14,170,0,248,134,154,173,246,6,154,151,16,32,64,128,0,129,57,2,142,229,55,92,5,192,243,104,54,218,121,51,175,32,64,128,0,1,2,211,4,28,204,231,105,5,192,243,102,94,65,128,0,1,2,4,8,16,72,45,32,0,158,110,159,247,25,167,
201,188,128,0,1,2,4,8,204,20,112,52,159,214,21,0,207,146,217,100,103,197,60,159,0,1,2,4,8,76,22,112,56,159,5,22,0,79,138,217,98,39,193,60,157,0,1,2,4,8,204,23,112,60,159,52,22,0,207,129,217,96,231,188,60,155,0,1,2,4,8,44,17,112,64,159,99,22,0,79,121,217,94,167,184,60,153,0,1,2,4,8,172,18,112,68,159,146,22,0,79,113,121,50,1,2,4,8,16,32,64,32,191,128,0,120,166,135,222,93,156,209,242,92,2,4,8,16,32,176,80,192,33,125,6,91,0,60,161,101,107,157,192,242,84,2,4,8,16,32,176,86,192,49,125,194,91,0,60,142,101,99,29,183,242,76,2,4,8,16,32,176,92,192,65,125,156,92,0,60,108,101,91,29,166,242,68,2,4,8,16,32,176,67,192,81,125,88,93,0,60,74,101,83,29,149,242,60,2,4,8,16,32,176,73,192,97,125,20,94,0,60,42,229,121,4,8,16,32,64,128,0,129,34,2,2,224,193,70,122,79,113,16,202,211,8,16,32,64,128,192,62,1,199,245,65,123,1,240,24,148,13,117,204,201,179,8,16,32,64,128,192,86,1,7,246,49,126,1,240,144,147,237,116,136,201,147,8,16,32,64,128,192,110,1,71,246,161,14,8,128,71,152,108,166,35,74,158,67,128,0,1,
2,4,2,8,56,180,143,52,65,0,60,162,228,57,4,8,16,32,64,128,0,129,66,2,2,224,129,102,122,47,113,0,201,83,8,16,32,64,128,64,12,1,199,246,129,62,8,128,175,145,108,164,215,70,158,65,128,0,1,2,4,194,8,56,184,95,183,66,0,124,105,100,27,189,36,242,4,2,4,8,16,32,16,73,192,209,253,178,27,2,224,43,34,155,232,149,144,159,19,32,64,128,0,129,96,2,14,239,87,13,17,0,95,9,249,57,1,2,4,8,16,32,64,160,152,128,0,248,162,161,222,67,188,0,242,99,2,4,8,16,32,16,79,192,241,253,162,39,2,224,115,32,27,232,185,143,159,18,32,64,128,0,129,144,2,14,240,231,109,17,0,159,250,216,62,79,121,252,144,0,1,2,4,8,68,21,112,132,63,237,140,0,248,140,199,230,121,166,227,103,4,8,16,32,64,32,176,128,67,252,89,115,4,192,103,58,126,70,128,0,1,2,4,8,16,40,40,32,0,62,105,170,247,14,79,112,252,136,0,1,2,4,8,196,22,112,140,63,233,143,0,248,24,199,198,121,108,227,39,4,8,16,32,64,32,188,128,131,252,113,139,4,192,135,54,182,205,67,26,63,32,64,128,0,1,2,25,4,28,229,15,187,36,0,62,162,177,105,30,201,248,62,1,2,4,8,16,72,34,224,48,
127,212,40,1,240,145,140,239,19,32,64,128,0,1,2,4,138,10,8,128,15,26,235,61,195,3,24,223,38,64,128,0,1,2,121,4,28,231,15,122,37,0,222,135,177,97,238,187,248,46,1,2,4,8,16,72,37,224,64,191,223,46,1,240,174,139,237,114,151,197,55,9,16,32,64,128,64,54,1,71,250,221,142,9,128,247,88,108,150,123,42,190,71,128,0,1,2,4,18,10,56,212,239,53,77,0,188,167,226,123,4,8,16,32,64,128,0,129,194,2,2,224,157,230,122,175,112,7,197,183,8,16,32,64,128,64,78,1,199,250,157,190,9,128,127,163,216,40,127,155,248,14,1,2,4,8,16,72,43,224,96,255,187,117,2,224,95,38,182,201,95,36,190,65,128,0,1,2,4,50,11,56,218,255,234,158,0,248,23,137,111,16,32,64,128,0,1,2,4,106,11,8,128,127,246,215,187,132,63,69,124,77,128,0,1,2,4,146,11,56,220,255,108,160,0,248,135,136,45,242,7,136,47,9,16,32,64,128,64,126,1,199,251,31,61,20,0,191,131,216,32,223,61,124,69,128,0,1,2,4,74,8,56,224,191,183,81,0,252,238,225,43,2,4,8,16,32,64,128,64,121,1,1,240,91,139,189,63,248,198,225,11,2,4,8,16,32,80,69,192,17,255,173,147,2,224,45,135,205,
113,171,225,49,1,2,4,8,16,40,36,224,144,191,109,166,0,120,163,97,107,220,96,120,72,128,0,1,2,4,106,9,56,230,111,250,41,0,126,97,216,24,95,22,30,17,32,64,128,0,129,114,2,14,250,175,150,10,128,95,22,30,17,32,64,128,0,1,2,4,90,8,8,128,159,109,246,190,224,147,194,3,2,4,8,16,32,80,81,192,81,255,217,85,1,240,63,10,155,226,63,9,255,159,0,1,2,4,8,20,21,112,216,255,215,88,1,240,95,9,91,226,191,45,225,255,19,32,64,128,0,129,178,2,142,251,127,91,43,0,150,221,227,10,35,64,128,0,1,2,4,8,220,23,16,0,127,187,120,71,112,127,127,248,46,1,2,4,8,16,40,37,224,192,255,221,78,1,240,151,131,237,80,234,234,86,12,1,2,4,8,16,120,36,224,200,255,37,35,0,254,100,176,25,30,93,38,190,79,128,0,1,2,4,138,9,56,244,127,54,84,0,44,182,173,149,67,128,0,1,2,4,8,16,120,37,32,0,126,8,121,47,240,106,155,248,57,1,2,4,8,16,40,35,224,216,255,104,165,0,40,255,149,185,162,21,66,128,0,1,2,4,142,8,72,128,2,224,145,125,226,57,4,8,16,32,64,128,0,129,82,2,62,1,244,11,224,82,27,90,49,4,8,16,32,64,224,165,128,143,0,253,10,88,254,
123,121,153,120,2,1,2,4,8,16,40,38,32,1,182,255,4,208,22,40,118,77,43,135,0,1,2,4,8,188,22,104,127,252,183,15,128,175,247,136,103,16,32,64,128,0,1,2,4,106,9,116,15,128,237,223,1,212,218,206,170,33,64,128,0,1,2,199,4,186,7,128,230,1,176,123,251,143,93,36,158,69,128,0,1,2,4,202,9,52,143,0,189,3,96,243,230,151,187,152,21,68,128,0,1,2,4,14,11,244,14,1,189,3,224,225,77,226,137,4,8,16,32,64,128,0,129,58,2,173,3,96,239,236,95,103,19,171,132,0,1,2,4,8,188,33,208,58,6,116,14,128,173,27,255,198,133,226,37,4,8,16,32,64,160,148,64,231,32,208,56,0,118,110,123,169,235,87,49,4,8,16,32,64,224,61,129,198,81,160,113,0,124,111,175,120,21,1,2,4,8,16,32,64,32,187,64,223,0,216,56,245,103,223,180,214,79,128,0,1,2,4,198,8,244,13,3,109,3,96,223,150,143,185,100,140,66,128,0,1,2,4,10,8,180,141,3,109,3,96,129,77,171,4,2,4,8,16,32,64,128,192,91,2,93,3,96,219,196,255,214,46,241,34,2,4,8,16,32,80,84,160,107,32,104,26,0,187,182,187,232,213,171,44,2,4,8,16,32,240,174,64,211,72,208,51,0,54,109,246,187,215,134,
215,17,32,64,128,0,129,186,2,61,67,65,207,0,88,119,23,171,140,0,1,2,4,8,16,32,240,82,160,101,0,236,153,245,95,238,5,79,32,64,128,0,1,2,29,5,90,198,130,142,1,176,101,163,59,94,209,106,38,64,128,0,1,2,71,4,58,6,131,142,1,240,200,94,240,28,2,4,8,16,32,64,128,64,89,129,134,1,176,99,206,47,187,127,21,70,128,0,1,2,4,174,11,52,140,6,253,2,96,195,38,95,191,50,140,64,128,0,1,2,4,42,11,244,11,7,253,2,96,229,253,171,54,2,4,8,16,32,64,128,192,1,129,118,1,176,95,198,63,176,11,60,133,0,1,2,4,8,244,22,104,23,15,186,5,192,118,13,238,125,61,171,158,0,1,2,4,8,28,19,232,22,16,154,5,192,110,237,61,182,233,61,139,0,1,2,4,8,180,23,104,22,17,154,5,192,246,219,27,0,1,2,4,8,16,32,64,224,71,175,0,216,44,221,219,223,4,8,16,32,64,128,192,81,129,94,33,161,85,0,236,213,218,163,27,222,243,8,16,32,64,128,0,129,15,129,86,49,161,85,0,180,189,9,16,32,64,128,0,1,2,4,126,180,250,21,112,171,100,111,115,19,32,64,128,0,1,2,231,4,58,5,133,70,159,0,118,106,235,185,13,239,217,4,8,16,32,64,128,192,135,64,163,168,208,40,
0,218,218,4,8,16,32,64,128,0,1,2,63,5,250,4,192,70,169,222,214,38,64,128,0,1,2,4,222,17,232,19,22,218,4,192,62,45,125,103,195,123,13,1,2,4,8,16,32,240,33,208,38,46,116,9,128,109,26,234,242,37,64,128,0,1,2,4,222,23,232,18,24,186,4,192,247,119,130,87,18,32,64,128,0,1,2,4,138,9,52,9,128,93,242,124,177,221,169,28,2,4,8,16,32,176,90,160,73,100,232,17,0,155,52,115,245,53,98,62,2,4,8,16,32,80,79,160,71,104,232,17,0,235,237,78,21,17,32,64,128,0,1,2,4,222,22,104,17,0,123,100,249,183,247,128,23,18,32,64,128,0,1,2,95,2,45,98,67,135,0,216,162,145,95,251,214,35,2,4,8,16,32,64,224,138,64,135,224,208,33,0,94,217,3,94,75,128,0,1,2,4,8,16,40,39,208,32,0,118,200,241,229,246,165,130,8,16,32,64,128,192,62,129,6,209,161,126,0,108,208,196,125,87,136,153,9,16,32,64,128,64,69,129,250,225,161,126,0,172,184,47,213,68,128,0,1,2,4,8,16,184,32,80,62,0,214,207,240,23,186,239,165,4,8,16,32,64,128,192,61,129,242,241,161,122,0,44,223,192,123,187,214,247,8,16,32,64,128,0,129,107,2,213,3,68,245,0,120,173,251,94,
77,128,0,1,2,4,8,16,40,40,80,60,0,86,207,239,5,119,164,146,8,16,32,64,128,64,4,129,226,17,162,118,0,44,222,188,8,215,135,53,16,32,64,128,0,129,154,2,181,67,68,237,0,88,115,71,170,138,0,1,2,4,8,16,32,112,73,160,116,0,172,157,221,47,245,221,139,9,16,32,64,128,0,129,231,2,165,99,68,229,0,88,186,113,207,247,172,159,18,32,64,128,0,1,2,87,5,42,7,137,202,1,240,106,223,189,158,0,1,2,4,8,16,32,80,82,160,112,0,172,156,219,75,238,69,69,17,32,64,128,0,129,88,2,133,163,68,221,0,88,184,105,177,174,14,171,33,64,128,0,1,2,85,5,234,134,137,186,1,176,234,94,84,23,1,2,4,8,16,32,64,224,162,64,217,0,88,55,179,95,236,184,151,19,32,64,128,0,1,2,71,5,202,198,137,170,1,176,108,195,142,238,88,207,35,64,128,0,1,2,4,174,11,84,13,20,85,3,224,245,142,27,129,0,1,2,4,8,16,32,80,84,160,104,0,172,154,215,139,238,66,101,17,32,64,128,0,129,168,2,69,35,69,205,0,88,180,89,81,175,13,235,34,64,128,0,1,2,117,5,106,134,138,154,1,176,238,46,84,25,1,2,4,8,16,32,64,224,178,64,201,0,88,51,171,95,238,181,1,8,16,32,64,128,
0,129,243,2,37,99,69,201,0,120,190,183,94,65,128,0,1,2,4,8,16,232,35,80,49,0,150,76,234,125,182,164,74,9,16,32,64,128,64,44,129,138,193,162,96,0,172,216,166,88,23,130,213,16,32,64,128,0,129,86,2,5,163,69,193,0,216,106,75,42,150,0,1,2,4,8,16,32,112,90,160,94,0,44,152,210,79,119,213,11,8,16,32,64,128,0,129,129,2,245,194,69,185,0,88,175,69,3,247,175,161,8,16,32,64,128,0,129,119,4,202,197,139,114,1,240,157,174,122,13,1,2,4,8,16,32,64,160,147,64,181,0,88,46,161,119,218,140,106,37,64,128,0,1,2,81,5,170,5,140,98,1,176,90,123,162,94,6,214,69,128,0,1,2,4,154,9,20,139,24,197,2,96,179,205,168,92,2,4,8,16,32,64,128,192,27,2,181,2,96,177,116,254,70,63,189,132,0,1,2,4,8,16,152,34,80,43,100,212,10,128,83,26,110,80,2,4,8,16,32,64,128,64,45,129,82,1,176,86,54,175,181,209,84,67,128,0,1,2,4,146,11,148,138,25,149,2,96,169,198,36,191,72,44,159,0,1,2,4,8,148,19,168,20,52,42,5,192,114,27,77,65,4,8,16,32,64,128,0,129,25,2,133,2,96,165,92,62,163,213,198,36,64,128,0,1,2,4,46,9,20,138,26,117,2,96,161,166,
92,218,156,94,76,128,0,1,2,4,8,76,18,168,19,54,234,4,192,73,173,54,44,1,2,4,8,16,32,64,160,154,64,153,0,88,39,147,87,219,98,234,33,64,128,0,1,2,101,4,202,196,141,50,1,176,204,214,82,8,1,2,4,8,16,32,64,96,178,64,149,0,88,38,145,79,238,183,225,9,16,32,64,128,0,129,11,2,85,2,71,145,0,88,165,29,23,118,164,151,18,32,64,128,0,1,2,243,5,138,68,142,34,1,112,126,191,205,64,128,0,1,2,4,8,16,168,34,80,35,0,22,73,227,85,54,149,58,8,16,32,64,128,64,93,129,26,161,163,68,0,172,209,138,186,151,138,202,8,16,32,64,128,64,33,129,18,177,163,68,0,44,180,169,148,66,128,0,1,2,4,8,16,152,46,80,33,0,150,72,226,211,59,109,2,2,4,8,16,32,64,96,136,64,133,224,81,33,0,14,105,166,65,8,16,32,64,128,0,1,2,93,4,10,4,192,10,57,188,203,118,83,39,1,2,4,8,16,40,32,80,32,122,228,15,128,5,154,80,224,82,80,2,1,2,4,8,16,104,36,144,63,124,228,15,128,141,182,155,82,9,16,32,64,128,0,1,2,35,4,210,7,192,252,25,124,68,27,141,65,128,0,1,2,4,8,44,20,72,31,63,210,7,192,133,205,54,21,1,2,4,8,16,32,64,160,132,64,246,0,152,62,129,
151,216,69,138,32,64,128,0,1,2,205,4,178,7,144,228,1,48,59,127,179,171,69,185,4,8,16,32,64,160,138,64,242,8,146,60,0,86,217,69,234,32,64,128,0,1,2,4,8,172,19,200,29,0,147,167,239,117,109,54,19,1,2,4,8,16,32,48,86,32,119,8,201,29,0,199,118,210,104,4,8,16,32,64,128,0,129,22,2,169,3,96,238,236,221,98,127,41,146,0,1,2,4,8,84,21,72,29,67,50,7,192,212,240,85,175,6,117,17,32,64,128,0,129,46,2,153,131,72,230,0,216,101,127,169,147,0,1,2,4,8,16,32,48,84,32,113,0,204,156,187,135,246,208,96,4,8,16,32,64,128,192,14,129,196,81,36,113,0,220,209,105,115,18,32,64,128,0,1,2,4,242,11,228,13,128,137,83,119,254,109,163,2,2,4,8,16,32,64,224,199,143,188,97,36,109,0,204,75,238,130,33,64,128,0,1,2,4,138,8,164,141,35,105,3,96,145,141,163,12,2,4,8,16,32,64,128,192,114,129,172,1,48,109,226,94,222,97,19,18,32,64,128,0,1,2,211,4,178,6,146,172,1,112,90,35,13,76,128,0,1,2,4,8,16,168,46,144,52,0,102,205,219,213,183,147,250,8,16,32,64,128,64,51,129,164,145,36,105,0,108,182,185,148,75,128,0,1,2,4,8,16,24,40,144,
51,0,38,77,219,3,251,102,40,2,4,8,16,32,64,32,132,64,206,80,146,50,0,230,164,14,177,75,45,130,0,1,2,4,8,16,24,43,144,50,150,164,12,128,99,251,102,52,2,4,8,16,32,64,128,64,47,129,140,1,48,101,210,238,181,173,84,75,128,0,1,2,4,250,8,100,12,38,25,3,96,159,29,165,82,2,4,8,16,32,64,128,192,4,129,132,1,48,99,206,158,208,57,67,18,32,64,128,0,1,2,49,4,18,70,147,124,1,48,33,114,140,221,105,21,4,8,16,32,64,128,192,28,129,124,225,36,95,0,156,211,57,163,18,32,64,128,0,1,2,4,218,8,164,11,128,249,50,118,155,189,164,80,2,4,8,16,32,208,85,32,93,60,73,23,0,187,238,44,117,19,32,64,128,0,1,2,4,70,9,100,11,128,233,18,246,168,70,25,135,0,1,2,4,8,16,136,43,144,45,160,100,11,128,113,59,111,101,4,8,16,32,64,128,0,129,36,2,201,2,96,182,124,157,100,23,88,38,1,2,4,8,16,32,112,77,32,89,68,73,22,0,175,245,198,171,9,16,32,64,128,0,1,2,4,126,252,200,21,0,147,165,107,27,140,0,1,2,4,8,16,232,34,144,43,164,164,10,128,185,104,187,108,120,117,18,32,64,128,0,1,2,31,2,169,98,74,170,0,104,123,17,32,64,128,0,1,2,4,
8,92,23,200,20,0,83,37,235,235,173,49,2,1,2,4,8,16,32,144,73,32,83,80,201,20,0,51,237,1,107,37,64,128,0,1,2,4,8,132,21,72,20,0,51,229,234,176,253,182,48,2,4,8,16,32,64,96,150,64,162,168,146,40,0,206,234,150,113,9,16,32,64,128,0,1,2,189,4,242,4,192,68,169,186,215,22,82,45,1,2,4,8,16,32,240,91,32,79,88,73,19,0,243,144,186,8,8,16,32,64,128,0,129,166,2,105,226,74,154,0,216,116,35,41,155,0,1,2,4,8,16,32,48,92,32,75,0,76,147,168,135,119,200,128,4,8,16,32,64,128,64,26,129,44,129,37,75,0,76,211,120,11,37,64,128,0,1,2,4,8,68,23,72,18,0,179,228,233,232,237,182,62,2,4,8,16,32,64,96,170,64,146,200,146,36,0,78,109,149,193,9,16,32,64,128,0,1,2,173,4,114,4,192,36,105,186,213,206,81,44,1,2,4,8,16,32,112,71,32,71,104,201,17,0,239,240,250,22,1,2,4,8,16,32,64,128,192,123,2,41,2,96,142,44,253,94,3,188,138,0,1,2,4,8,16,40,37,144,34,182,164,8,128,165,182,133,98,8,16,32,64,128,0,1,2,155,5,50,4,192,20,73,122,115,31,77,79,128,0,1,2,4,8,4,17,200,16,92,18,4,192,12,140,65,118,156,101,16,32,64,128,0,1,2,
251,5,18,68,151,4,1,112,127,31,173,128,0,1,2,4,8,16,32,80,73,32,126,0,76,144,162,43,109,8,181,16,32,64,128,0,1,2,87,5,226,135,151,248,1,240,106,15,188,158,0,1,2,4,8,16,32,64,224,155,64,248,0,24,63,67,127,243,244,5,1,2,4,8,16,32,64,32,124,124,9,31,0,237,33,2,4,8,16,32,64,128,0,129,177,2,209,3,96,248,4,61,182,29,70,35,64,128,0,1,2,4,42,8,68,15,48,209,3,96,133,61,160,6,2,4,8,16,32,64,128,64,40,129,224,1,48,122,126,14,213,75,139,33,64,128,0,1,2,4,162,8,4,143,48,193,3,96,148,46,90,7,1,2,4,8,16,32,64,160,142,64,236,0,24,60,61,215,217,6,42,33,64,128,0,1,2,4,198,10,196,14,49,177,3,224,216,78,24,141,0,1,2,4,8,16,32,64,224,67,32,116,0,140,157,157,237,31,2,4,8,16,32,64,128,192,67,129,208,49,38,114,0,12,13,247,176,221,126,64,128,0,1,2,4,8,16,248,16,136,28,100,34,7,64,155,135,0,1,2,4,8,16,32,64,96,130,64,224,0,24,57,55,79,232,132,33,9,16,32,64,128,0,129,90,2,129,163,76,224,0,88,107,15,168,134,0,1,2,4,8,16,32,16,69,32,110,0,12,156,154,163,52,207,58,8,16,32,64,128,0,129,200,2,113,195,76,220,
0,24,185,159,214,70,128,0,1,2,4,8,16,72,44,16,54,0,198,205,204,137,187,109,233,4,8,16,32,64,128,192,74,129,176,113,38,108,0,92,217,29,115,17,32,64,128,0,1,2,4,58,9,68,13,128,97,19,115,167,205,161,86,2,4,8,16,32,64,224,154,64,212,64,19,53,0,94,211,246,106,2,4,8,16,32,64,128,0,129,135,2,65,3,96,212,188,252,208,209,15,8,16,32,64,128,0,1,2,127,11,4,141,52,65,3,224,223,126,190,67,128,0,1,2,4,8,16,32,48,70,32,102,0,12,154,150,199,144,27,133,0,1,2,4,8,16,232,35,16,51,212,196,12,128,125,118,133,74,9,16,32,64,128,0,1,2,203,5,66,6,192,152,89,121,121,111,76,72,128,0,1,2,4,8,228,23,8,25,107,66,6,192,252,189,86,1,1,2,4,8,16,32,64,32,174,64,196,0,24,50,41,199,109,161,149,17,32,64,128,0,1,2,145,5,34,6,155,136,1,48,114,15,173,141,0,1,2,4,8,16,32,144,94,32,96,0,140,152,147,211,247,89,1,4,8,16,32,64,128,192,46,129,128,209,38,96,0,220,213,29,243,18,32,64,128,0,1,2,4,122,8,196,11,128,1,83,114,143,173,160,74,2,4,8,16,32,64,96,142,64,188,112,19,47,0,206,145,55,42,1,2,4,8,16,32,64,128,192,191,2,225,
2,96,188,140,108,175,16,32,64,128,0,1,2,4,46,9,132,139,55,225,2,224,37,94,47,38,64,128,0,1,2,4,8,16,120,41,16,45,0,134,75,200,47,5,61,129,0,1,2,4,8,16,32,240,66,32,90,192,137,22,0,95,240,249,49,1,2,4,8,16,32,64,128,192,85,129,96,1,48,90,62,190,202,235,245,4,8,16,32,64,128,0,129,15,129,96,17,39,88,0,180,69,8,16,32,64,128,0,1,2,4,102,11,196,10,128,193,210,241,108,124,227,19,32,64,128,0,1,2,93,4,98,133,156,88,1,176,203,30,80,39,1,2,4,8,16,32,64,96,163,64,168,0,24,43,27,111,236,138,169,9,16,32,64,128,0,129,106,2,161,98,78,168,0,88,173,211,234,33,64,128,0,1,2,4,8,68,20,136,20,0,67,37,227,136,205,178,38,2,4,8,16,32,64,32,175,64,164,160,19,41,0,230,237,168,149,19,32,64,128,0,1,2,4,18,9,4,10,128,145,114,113,162,14,90,42,1,2,4,8,16,32,144,67,32,80,212,9,20,0,115,244,206,42,9,16,32,64,128,0,1,2,217,5,226,4,192,64,169,56,123,83,173,159,0,1,2,4,8,16,136,40,16,39,236,196,9,128,17,251,100,77,4,8,16,32,64,128,0,129,130,2,97,2,96,156,76,92,176,203,74,34,64,128,0,1,2,4,34,8,132,137,59,97,2,96,
132,174,88,3,1,2,4,8,16,32,64,160,131,64,148,0,24,38,17,119,104,186,26,9,16,32,64,128,0,129,61,2,81,2,79,148,0,184,167,11,102,37,64,128,0,1,2,4,8,52,20,8,18,0,163,228,225,134,59,64,201,4,8,16,32,64,128,192,58,129,32,145,39,72,0,92,231,110,38,2,4,8,16,32,64,128,64,119,129,24,1,48,72,26,238,190,25,212,79,128,0,1,2,4,8,204,22,136,17,122,98,4,192,217,214,198,39,64,128,0,1,2,4,8,16,248,20,8,17,0,99,100,225,79,19,15,8,16,32,64,128,0,1,2,179,4,66,196,158,16,1,112,150,176,113,9,16,32,64,128,0,1,2,4,254,22,136,16,0,67,36,225,191,105,124,135,0,1,2,4,8,16,32,48,94,32,66,240,137,16,0,199,203,26,145,0,1,2,4,8,16,32,64,224,161,128,0,248,144,198,15,8,16,32,64,128,0,1,2,53,5,2,4,192,8,31,132,214,108,174,170,8,16,32,64,128,0,129,128,2,1,162,79,128,0,24,176,49,150,68,128,0,1,2,4,8,16,40,44,176,63,0,6,72,193,133,251,171,52,2,4,8,16,32,64,32,156,192,254,240,179,63,0,134,107,138,5,17,32,64,128,0,1,2,4,106,11,108,15,128,251,51,112,237,6,171,142,0,1,2,4,8,16,8,39,176,61,254,108,15,128,225,90,98,65,
4,8,16,32,64,128,0,129,226,2,187,3,224,246,4,92,188,191,202,35,64,128,0,1,2,4,2,10,236,14,64,187,3,96,192,150,88,18,1,2,4,8,16,32,64,160,182,192,230,0,184,59,255,214,110,174,234,8,16,32,64,128,0,129,160,2,155,35,208,230,0,24,180,41,150,69,128,0,1,2,4,8,16,40,44,176,55,0,110,78,191,133,251,170,52,2,4,8,16,32,64,32,180,192,222,16,180,55,0,134,110,140,197,17,32,64,128,0,1,2,4,106,10,8,128,53,251,170,42,2,4,8,16,32,64,128,192,67,129,173,1,112,239,135,159,15,77,252,128,0,1,2,4,8,16,32,48,91,96,107,12,218,26,0,103,203,26,159,0,1,2,4,8,16,32,64,224,111,129,157,1,112,107,242,253,155,194,119,8,16,32,64,128,0,1,2,235,4,118,6,161,157,1,112,157,176,153,8,16,32,64,128,0,1,2,4,62,5,54,6,192,157,185,247,179,126,15,8,16,32,64,128,0,1,2,123,4,54,70,161,141,1,112,143,181,89,9,16,32,64,128,0,1,2,221,5,246,5,192,141,169,183,123,211,213,79,128,0,1,2,4,8,68,16,216,23,134,246,5,192,8,238,214,64,128,0,1,2,4,8,16,104,40,176,45,0,238,203,188,13,187,172,100,2,4,8,16,32,64,32,162,192,182,56,180,45,0,70,
236,130,53,17,32,64,128,0,1,2,4,58,8,8,128,29,186,172,70,2,4,8,16,32,64,128,192,141,192,174,0,184,237,35,207,155,218,61,36,64,128,0,1,2,4,8,108,21,216,21,136,118,5,192,173,216,38,39,64,128,0,1,2,4,8,116,22,216,20,0,119,229,221,206,173,86,59,1,2,4,8,16,32,16,78,96,83,36,218,20,0,195,241,91,16,1,2,4,8,16,32,64,160,141,192,158,0,184,41,237,182,233,170,66,9,16,32,64,128,0,129,36,2,123,66,209,158,0,152,164,37,150,73,128,0,1,2,4,8,16,168,40,176,37,0,238,201,186,21,219,167,38,2,4,8,16,32,64,32,185,192,150,88,180,37,0,38,111,148,229,19,32,64,128,0,1,2,4,82,11,8,128,169,219,103,241,4,8,16,32,64,128,0,129,243,2,59,2,224,150,143,58,207,211,120,5,1,2,4,8,16,32,64,96,190,192,142,96,180,35,0,206,151,52,3,1,2,4,8,16,32,64,128,192,67,129,13,1,112,71,206,125,88,191,31,16,32,64,128,0,1,2,4,246,10,108,136,70,27,2,224,94,99,179,19,32,64,128,0,1,2,4,186,11,172,15,128,27,82,110,247,38,171,159,0,1,2,4,8,16,136,44,176,62,28,173,15,128,145,253,173,141,0,1,2,4,8,16,32,208,64,64,0,108,208,100,37,18,32,
64,128,0,1,2,4,110,5,150,7,192,245,31,114,222,150,235,49,1,2,4,8,16,32,64,32,156,192,242,120,180,60,0,134,35,183,32,2,4,8,16,32,64,128,64,51,129,213,1,112,121,194,109,214,79,229,18,32,64,128,0,1,2,9,5,86,7,164,213,1,48,97,75,44,153,0,1,2,4,8,16,32,80,75,96,113,0,92,157,111,107,53,75,53,4,8,16,32,64,128,64,81,129,197,17,105,113,0,44,218,52,101,17,32,64,128,0,1,2,4,18,9,8,128,137,154,101,169,4,8,16,32,64,128,0,129,17,2,107,3,224,226,143,55,71,0,25,131,0,1,2,4,8,16,32,48,95,96,109,72,90,27,0,231,235,153,129,0,1,2,4,8,16,32,64,224,133,192,210,0,184,54,219,190,168,220,143,9,16,32,64,128,0,1,2,113,4,150,198,164,165,1,48,142,177,149,16,32,64,128,0,1,2,4,250,10,172,12,128,75,147,109,223,150,170,156,0,1,2,4,8,16,200,40,176,50,40,173,12,128,25,123,97,205,4,8,16,32,64,128,0,129,114,2,2,96,185,150,42,136,0,1,2,4,8,16,32,240,92,96,97,0,92,249,193,230,243,162,253,148,0,1,2,4,8,16,32,16,78,96,97,84,90,24,0,195,49,91,16,1,2,4,8,16,32,64,160,165,192,186,0,184,48,213,182,236,164,162,9,16,32,64,
128,0,129,228,2,235,194,210,186,0,152,188,37,150,79,128,0,1,2,4,8,16,168,34,32,0,86,233,164,58,8,16,32,64,128,0,1,2,7,5,150,5,192,117,31,106,30,172,220,211,8,16,32,64,128,0,1,2,177,4,150,197,165,101,1,48,150,175,213,16,32,64,128,0,1,2,4,250,10,172,10,128,203,18,109,223,86,170,156,0,1,2,4,8,16,200,46,176,42,48,173,10,128,217,251,97,253,4,8,16,32,64,128,0,129,50,2,2,96,153,86,42,132,0,1,2,4,8,16,32,112,76,96,81,0,92,245,129,230,177,162,61,139,0,1,2,4,8,16,32,16,82,96,81,100,90,20,0,67,18,91,20,1,2,4,8,16,32,64,160,165,192,154,0,184,40,205,182,236,160,162,9,16,32,64,128,0,129,66,2,107,66,211,154,0,88,168,45,74,33,64,128,0,1,2,4,8,100,23,16,0,179,119,208,250,9,16,32,64,128,0,1,2,39,5,150,4,192,53,31,102,158,172,220,211,9,16,32,64,128,0,1,2,241,4,150,196,166,37,1,48,158,173,21,17,32,64,128,0,1,2,4,250,10,172,8,128,75,146,108,223,22,170,156,0,1,2,4,8,16,168,36,176,34,56,173,8,128,149,122,162,22,2,4,8,16,32,64,128,64,122,1,1,48,125,11,21,64,128,0,1,2,4,8,16,56,39,176,32,0,174,248,32,
243,92,209,158,77,128,0,1,2,4,8,16,8,43,176,32,58,45,8,128,97,121,45,140,0,1,2,4,8,16,32,208,82,96,126,0,92,144,98,91,118,78,209,4,8,16,32,64,128,64,81,129,249,225,105,126,0,44,218,26,101,17,32,64,128,0,1,2,4,178,10,8,128,89,59,103,221,4,8,16,32,64,128,0,129,55,5,166,7,192,249,31,98,190,89,185,151,17,32,64,128,0,1,2,4,98,10,76,143,79,211,3,96,76,87,171,34,64,128,0,1,2,4,8,244,21,152,29,0,167,39,216,190,173,83,57,1,2,4,8,16,32,80,85,96,118,128,154,29,0,171,246,69,93,4,8,16,32,64,128,0,129,180,2,2,96,218,214,89,56,1,2,4,8,16,32,64,224,61,129,201,1,112,246,7,152,239,21,237,85,4,8,16,32,64,128,0,129,208,2,147,35,212,228,0,24,154,214,226,8,16,32,64,128,0,1,2,45,5,230,6,192,201,233,181,101,199,20,77,128,0,1,2,4,8,52,16,152,27,162,230,6,192,6,237,81,34,1,2,4,8,16,32,64,32,155,128,0,152,173,99,214,75,128,0,1,2,4,8,16,184,40,48,53,0,206,253,240,242,98,229,94,78,128,0,1,2,4,8,16,136,43,48,53,70,77,13,128,113,77,173,140,0,1,2,4,8,16,32,208,87,64,0,236,219,123,149,19,32,64,128,0,1,2,77,5,
102,6,192,169,31,93,54,237,151,178,9,16,32,64,128,0,129,38,2,51,131,212,204,0,216,164,61,202,36,64,128,0,1,2,4,8,228,18,152,24,0,103,230,214,92,200,86,75,128,0,1,2,4,8,16,56,45,48,49,74,77,12,128,167,203,244,2,2,4,8,16,32,64,128,0,129,5,2,2,224,2,100,83,16,32,64,128,0,1,2,4,34,9,204,11,128,19,63,182,140,4,104,45,4,8,16,32,64,128,0,129,57,2,243,194,212,188,0,56,71,194,168,4,8,16,32,64,128,0,1,2,23,5,4,192,139,128,94,78,128,0,1,2,4,8,16,200,38,48,45,0,206,251,208,50,27,177,245,18,32,64,128,0,1,2,4,222,18,152,22,167,166,5,192,183,202,244,34,2,4,8,16,32,64,128,0,129,233,2,179,2,224,180,196,58,93,196,4,4,8,16,32,64,128,0,129,32,2,179,2,213,172,0,24,132,205,50,8,16,32,64,128,0,1,2,4,254,20,16,0,255,20,241,53,1,2,4,8,16,32,64,160,184,192,164,0,56,235,3,203,226,221,80,30,1,2,4,8,16,32,64,224,86,96,82,164,154,20,0,111,87,238,49,1,2,4,8,16,32,64,128,64,36,1,1,48,82,55,172,133,0,1,2,4,8,16,32,176,64,96,78,0,156,244,113,229,2,15,83,16,32,64,128,0,1,2,4,2,9,204,9,85,115,2,96,32,54,75,33,64,
128,0,1,2,4,8,16,248,46,32,0,126,247,240,21,1,2,4,8,16,32,64,160,188,192,148,0,56,231,195,202,242,189,80,32,1,2,4,8,16,32,64,224,79,129,41,177,106,74,0,252,115,229,190,38,64,128,0,1,2,4,8,16,136,35,32,0,198,233,133,149,16,32,64,128,0,1,2,4,150,8,204,8,128,83,62,170,92,162,97,18,2,4,8,16,32,64,128,64,48,129,25,193,106,70,0,12,198,102,57,4,8,16,32,64,128,0,1,2,183,2,19,2,224,140,156,122,187,100,143,9,16,32,64,128,0,1,2,141,4,38,68,171,9,1,176,81,67,148,74,128,0,1,2,4,8,16,72,40,32,0,38,108,154,37,19,32,64,128,0,1,2,4,174,8,140,15,128,19,62,166,188,82,160,215,18,32,64,128,0,1,2,4,114,11,140,15,87,227,3,96,110,97,171,39,64,128,0,1,2,4,8,148,23,16,0,203,183,88,129,4,8,16,32,64,128,0,129,239,2,195,3,224,248,15,41,191,47,216,87,4,8,16,32,64,128,0,129,102,2,195,227,213,240,0,216,172,33,202,37,64,128,0,1,2,4,8,164,19,16,0,211,181,204,130,9,16,32,64,128,0,1,2,215,4,70,7,192,225,31,81,94,43,207,171,9,16,32,64,128,0,1,2,249,5,70,7,172,209,1,48,191,176,10,8,16,32,64,128,0,1,2,197,5,4,192,
226,13,86,30,1,2,4,8,16,32,64,224,79,129,193,1,112,244,7,148,127,46,215,215,4,8,16,32,64,128,0,129,134,2,131,35,214,224,0,216,176,33,74,38,64,128,0,1,2,4,8,36,19,16,0,147,53,204,114,9,16,32,64,128,0,1,2,87,5,198,6,192,193,31,79,94,45,206,235,9,16,32,64,128,0,1,2,53,4,198,134,172,177,1,176,134,176,42,8,16,32,64,128,0,1,2,165,5,4,192,210,237,85,28,1,2,4,8,16,32,64,224,111,129,161,1,112,236,135,147,127,47,214,119,8,16,32,64,128,0,1,2,77,5,134,198,172,161,1,176,105,67,148,77,128,0,1,2,4,8,16,72,37,32,0,166,106,151,197,18,32,64,128,0,1,2,4,174,11,140,12,128,67,63,154,188,94,154,17,8,16,32,64,128,0,1,2,117,4,70,6,173,145,1,176,142,176,74,8,16,32,64,128,0,1,2,133,5,4,192,194,205,85,26,1,2,4,8,16,32,64,224,158,192,192,0,56,242,131,201,123,75,245,61,2,4,8,16,32,64,128,64,99,129,129,81,107,96,0,108,220,16,165,19,32,64,128,0,1,2,4,18,9,8,128,137,154,101,169,4,8,16,32,64,128,0,129,17,2,227,2,224,192,143,37,71,20,102,12,2,4,8,16,32,64,128,64,45,129,113,97,107,92,0,172,37,172,26,2,4,8,16,32,
64,128,64,89,1,1,176,108,107,21,70,128,0,1,2,4,8,16,184,47,48,44,0,142,251,80,242,254,66,125,151,0,1,2,4,8,16,32,208,92,96,88,220,26,22,0,155,55,68,249,4,8,16,32,64,128,0,129,52,2,2,96,154,86,89,40,1,2,4,8,16,32,64,96,140,192,168,0,56,236,35,201,49,101,25,133,0,1,2,4,8,16,32,80,79,96,84,224,26,21,0,235,9,171,136,0,1,2,4,8,16,32,80,84,64,0,44,218,88,101,17,32,64,128,0,1,2,4,30,9,12,10,128,163,62,144,124,180,76,223,39,64,128,0,1,2,4,8,16,248,49,40,114,13,10,128,26,66,128,0,1,2,4,8,16,32,144,69,64,0,204,210,41,235,36,64,128,0,1,2,4,8,12,18,24,19,0,7,125,28,57,168,38,195,16,32,64,128,0,1,2,4,138,10,140,9,93,99,2,96,81,98,101,17,32,64,128,0,1,2,4,42,10,8,128,21,187,170,38,2,4,8,16,32,64,128,192,19,129,33,1,112,204,135,145,79,86,233,71,4,8,16,32,64,128,0,1,2,63,5,134,196,174,33,1,80,63,8,16,32,64,128,0,1,2,4,242,8,8,128,121,122,101,165,4,8,16,32,64,128,0,129,33,2,35,2,224,144,143,34,135,84,99,16,2,4,8,16,32,64,128,64,113,129,17,193,107,68,0,44,206,172,60,2,4,8,16,32,64,128,64,45,1,
1,176,86,63,85,67,128,0,1,2,4,8,16,120,41,32,0,190,36,242,4,2,4,8,16,32,64,128,64,45,129,1,1,112,196,111,162,107,161,170,134,0,1,2,4,8,16,32,48,77,96,64,244,26,16,0,167,149,103,96,2,4,8,16,32,64,128,0,129,9,2,2,224,4,84,67,18,32,64,128,0,1,2,4,34,11,92,15,128,3,62,134,140,12,100,109,4,8,16,32,64,128,0,129,88,2,215,195,215,245,0,24,75,196,106,8,16,32,64,128,0,1,2,4,94,8,8,128,47,128,252,152,0,1,2,4,8,16,32,80,77,224,114,0,188,254,33,100,53,82,245,16,32,64,128,0,1,2,4,166,10,92,142,95,151,3,224,212,242,12,78,128,0,1,2,4,8,16,32,48,92,64,0,28,78,106,64,2,4,8,16,32,64,128,64,108,129,171,1,240,242,71,144,177,121,172,142,0,1,2,4,8,16,32,16,79,224,106,0,187,26,0,227,137,88,17,1,2,4,8,16,32,64,128,192,83,1,1,240,41,143,31,18,32,64,128,0,1,2,4,234,9,8,128,245,122,170,34,2,4,8,16,32,64,128,192,83,129,139,1,240,234,111,160,159,174,205,15,9,16,32,64,128,0,1,2,4,238,9,92,140,96,23,3,224,189,21,249,30,1,2,4,8,16,32,64,128,64,100,1,1,48,114,119,172,141,0,1,2,4,8,16,32,48,65,224,90,0,188,248,
241,227,132,122,12,73,128,0,1,2,4,8,16,104,32,112,45,132,93,11,128,13,120,149,72,128,0,1,2,4,8,16,168,38,32,0,86,235,168,122,8,16,32,64,128,0,1,2,47,4,4,192,23,64,126,76,128,0,1,2,4,8,16,168,38,112,41,0,94,251,237,115,53,74,245,16,32,64,128,0,1,2,4,150,9,92,138,97,151,2,224,178,18,77,68,128,0,1,2,4,8,16,32,48,76,64,0,28,70,105,32,2,4,8,16,32,64,128,64,14,129,43,1,240,210,71,143,57,120,172,146,0,1,2,4,8,16,32,16,83,224,74,16,187,18,0,99,106,88,21,1,2,4,8,16,32,64,128,192,83,1,1,240,41,143,31,18,32,64,128,0,1,2,4,234,9,92,8,128,87,62,120,172,7,169,34,2,4,8,16,32,64,128,192,82,129,11,81,236,66,0,92,90,162,201,8,16,32,64,128,0,1,2,4,6,9,8,128,131,32,13,67,128,0,1,2,4,8,16,200,34,32,0,102,233,148,117,18,32,64,128,0,1,2,4,6,9,188,31,0,47,252,222,121,208,218,13,67,128,0,1,2,4,8,16,104,44,240,126,24,123,63,0,54,230,86,58,1,2,4,8,16,32,64,32,179,128,0,152,185,123,214,78,128,0,1,2,4,8,16,120,67,64,0,124,3,205,75,8,16,32,64,128,0,1,2,153,5,222,14,128,239,255,214,57,51,151,181,19,32,64,128,
0,1,2,4,194,8,188,29,199,222,14,128,97,74,183,16,2,4,8,16,32,64,128,0,129,83,2,2,224,41,46,79,38,64,128,0,1,2,4,8,228,23,120,55,0,190,253,145,99,126,50,21,16,32,64,128,0,1,2,4,98,8,188,27,200,222,13,128,49,170,182,10,2,4,8,16,32,64,128,0,129,211,2,2,224,105,50,47,32,64,128,0,1,2,4,8,228,22,16,0,115,247,207,234,9,16,32,64,128,0,1,2,167,5,222,12,128,239,254,198,249,244,250,188,128,0,1,2,4,8,16,32,64,224,145,192,155,145,236,205,0,248,104,21,190,79,128,0,1,2,4,8,16,32,16,93,64,0,140,222,33,235,35,64,128,0,1,2,4,8,12,22,120,47,0,190,249,113,227,224,181,27,142,0,1,2,4,8,16,32,208,92,224,189,80,246,94,0,108,78,173,124,2,4,8,16,32,64,128,64,102,1,1,48,115,247,172,157,0,1,2,4,8,16,32,240,134,128,0,248,6,154,151,16,32,64,128,0,1,2,4,50,11,188,21,0,223,251,109,115,102,38,107,39,64,128,0,1,2,4,8,132,20,120,43,150,189,21,0,67,150,111,81,4,8,16,32,64,128,0,1,2,135,4,4,192,67,76,158,68,128,0,1,2,4,8,16,168,35,32,0,214,233,165,74,8,16,32,64,128,0,1,2,135,4,222,9,128,111,253,174,249,208,106,60,
137,0,1,2,4,8,16,32,64,224,148,192,59,193,236,157,0,120,106,81,158,76,128,0,1,2,4,8,16,32,16,75,64,0,140,213,15,171,33,64,128,0,1,2,4,8,76,23,16,0,167,19,155,128,0,1,2,4,8,16,32,16,75,224,141,0,248,206,111,154,99,21,109,53,4,8,16,32,64,128,0,129,50,2,111,68,179,55,2,96,25,46,133,16,32,64,128,0,1,2,4,90,10,8,128,45,219,174,104,2,4,8,16,32,64,160,179,128,0,216,185,251,106,39,64,128,0,1,2,4,90,10,156,15,128,111,252,158,185,165,172,162,9,16,32,64,128,0,1,2,75,4,206,135,179,243,1,112,73,33,38,33,64,128,0,1,2,4,8,16,152,37,32,0,206,146,53,46,1,2,4,8,16,32,64,32,168,128,0,24,180,49,150,69,128,0,1,2,4,8,16,152,37,112,58,0,158,255,45,243,172,165,27,151,0,1,2,4,8,16,32,64,224,67,224,116,60,59,29,0,49,19,32,64,128,0,1,2,4,8,228,22,16,0,115,247,207,234,9,16,32,64,128,0,1,2,167,5,4,192,211,100,94,64,128,0,1,2,4,8,16,200,45,112,54,0,158,254,29,115,110,30,171,39,64,128,0,1,2,4,8,196,23,56,27,208,206,6,192,248,2,86,72,128,0,1,2,4,8,16,32,240,84,64,0,124,202,227,135,4,8,16,32,64,128,0,129,122,
2,2,96,189,158,170,136,0,1,2,4,8,16,32,240,84,224,100,0,60,251,27,230,167,115,251,33,1,2,4,8,16,32,64,128,192,8,129,147,17,237,100,0,28,177,66,99,16,32,64,128,0,1,2,4,8,236,20,16,0,119,234,155,155,0,1,2,4,8,16,32,176,65,64,0,220,128,110,74,2,4,8,16,32,64,128,192,78,129,115,1,240,228,239,151,119,22,102,110,2,4,8,16,32,64,128,64,31,129,115,33,237,92,0,236,163,168,82,2,4,8,16,32,64,128,64,89,1,1,176,108,107,21,70,128,0,1,2,4,8,16,184,47,32,0,222,119,241,93,2,4,8,16,32,64,128,64,89,129,83,1,240,220,111,151,203,154,41,140,0,1,2,4,8,16,32,16,77,224,84,76,59,21,0,163,85,106,61,4,8,16,32,64,128,0,1,2,231,5,4,192,243,102,94,65,128,0,1,2,4,8,16,72,45,32,0,166,110,159,197,19,32,64,128,0,1,2,4,206,11,156,9,128,167,126,183,124,126,41,94,65,128,0,1,2,4,8,16,32,240,174,192,153,160,118,38,0,190,187,30,175,35,64,128,0,1,2,4,8,16,8,36,32,0,6,106,134,165,16,32,64,128,0,1,2,4,86,8,8,128,43,148,205,65,128,0,1,2,4,8,16,8,36,112,34,0,158,249,205,114,160,10,45,133,0,1,2,4,8,16,32,208,65,224,68,84,59,
17,0,59,200,169,145,0,1,2,4,8,16,32,80,95,64,0,172,223,99,21,18,32,64,128,0,1,2,4,190,9,8,128,223,56,124,65,128,0,1,2,4,8,16,168,47,32,0,214,239,177,10,9,16,32,64,128,0,1,2,223,4,142,7,192,19,127,88,248,109,6,95,16,32,64,128,0,1,2,4,8,44,16,56,30,214,142,7,192,5,203,54,5,1,2,4,8,16,32,64,128,192,124,1,1,112,190,177,25,8,16,32,64,128,0,1,2,161,4,4,192,80,237,176,24,2,4,8,16,32,64,128,192,124,129,195,1,240,248,111,149,231,47,218,12,4,8,16,32,64,128,0,1,2,127,9,28,142,107,135,3,224,95,83,248,6,1,2,4,8,16,32,64,128,64,74,1,1,48,101,219,44,154,0,1,2,4,8,16,32,240,190,128,0,248,190,157,87,18,32,64,128,0,1,2,4,82,10,8,128,41,219,102,209,4,8,16,32,64,128,0,129,247,5,142,6,192,195,127,84,248,254,82,188,146,0,1,2,4,8,16,32,64,224,138,192,209,192,118,52,0,94,89,139,215,18,32,64,128,0,1,2,4,8,4,18,16,0,3,53,195,82,8,16,32,64,128,0,1,2,43,4,4,192,21,202,230,32,64,128,0,1,2,4,8,4,18,56,24,0,143,254,70,57,80,101,150,66,128,0,1,2,4,8,16,232,38,112,48,178,29,12,128,221,244,212,75,128,0,1,2,4,
8,16,168,43,32,0,214,237,173,202,8,16,32,64,128,0,1,2,119,5,4,192,187,44,190,73,128,0,1,2,4,8,16,168,43,32,0,214,237,173,202,8,16,32,64,128,0,1,2,119,5,142,5,192,131,127,80,120,119,6,223,36,64,128,0,1,2,4,8,16,88,36,112,44,180,29,11,128,139,150,108,26,2,4,8,16,32,64,128,0,129,249,2,2,224,124,99,51,16,32,64,128,0,1,2,4,66,9,8,128,161,218,97,49,4,8,16,32,64,128,0,129,249,2,135,2,224,177,223,38,207,95,172,25,8,16,32,64,128,0,1,2,4,158,10,28,138,109,135,2,224,211,105,252,144,0,1,2,4,8,16,32,64,32,149,128,0,152,170,93,22,75,128,0,1,2,4,8,16,184,46,32,0,94,55,52,2,1,2,4,8,16,32,64,32,149,128,0,152,170,93,22,75,128,0,1,2,4,8,16,184,46,112,36,0,30,250,99,194,235,75,49,2,1,2,4,8,16,32,64,128,192,85,129,35,193,237,72,0,188,186,14,175,39,64,128,0,1,2,4,8,16,8,36,32,0,6,106,134,165,16,32,64,128,0,1,2,4,86,8,8,128,43,148,205,65,128,0,1,2,4,8,16,8,36,32,0,6,106,134,165,16,32,64,128,0,1,2,4,86,8,28,8,128,71,254,148,112,197,82,205,65,128,0,1,2,4,8,16,32,240,82,224,64,116,59,16,0,95,78,227,9,
4,8,16,32,64,128,0,1,2,137,4,4,192,68,205,178,84,2,4,8,16,32,64,128,192,8,1,1,112,132,162,49,8,16,32,64,128,0,1,2,137,4,4,192,68,205,178,84,2,4,8,16,32,64,128,192,8,129,215,1,240,192,31,18,142,88,136,49,8,16,32,64,128,0,1,2,4,70,8,188,14,111,175,3,224,136,117,24,131,0,1,2,4,8,16,32,64,32,140,128,0,24,166,21,22,66,128,0,1,2,4,8,16,88,35,32,0,174,113,54,11,1,2,4,8,16,32,64,32,140,128,0,24,166,21,22,66,128,0,1,2,4,8,16,88,35,240,50,0,190,254,51,194,53,11,53,11,1,2,4,8,16,32,64,128,192,33,129,151,241,237,101,0,60,52,141,39,17,32,64,128,0,1,2,4,8,164,17,16,0,211,180,202,66,9,16,32,64,128,0,1,2,99,4,4,192,49,142,70,33,64,128,0,1,2,4,8,164,17,16,0,211,180,202,66,9,16,32,64,128,0,1,2,99,4,94,5,192,151,127,68,56,102,25,70,33,64,128,0,1,2,4,8,16,24,37,240,42,192,189,10,128,163,214,97,28,2,4,8,16,32,64,128,0,129,32,2,2,96,144,70,88,6,1,2,4,8,16,32,64,96,149,128,0,184,74,218,60,4,8,16,32,64,128,0,129,32,2,2,96,144,70,88,6,1,2,4,8,16,32,64,96,149,128,0,184,74,218,60,4,8,16,32,64,128,0,129,
32,2,47,2,224,171,127,67,18,164,10,203,32,64,128,0,1,2,4,8,16,248,18,120,17,225,94,4,192,175,113,60,34,64,128,0,1,2,4,8,16,168,33,32,0,214,232,163,42,8,16,32,64,128,0,1,2,135,5,4,192,195,84,158,72,128,0,1,2,4,8,16,168,33,32,0,214,232,163,42,8,16,32,64,128,0,1,2,135,5,158,7,192,23,127,64,120,120,22,79,36,64,128,0,1,2,4,8,16,88,40,240,60,196,61,15,128,11,151,105,42,2,4,8,16,32,64,128,0,129,53,2,2,224,26,103,179,16,32,64,128,0,1,2,4,194,8,8,128,97,90,97,33,4,8,16,32,64,128,0,129,53,2,2,224,26,103,179,16,32,64,128,0,1,2,4,194,8,8,128,97,90,97,33,4,8,16,32,64,128,0,129,53,2,79,3,224,243,127,63,178,102,129,102,33,64,128,0,1,2,4,8,16,56,45,240,52,198,61,13,128,167,167,242,2,2,4,8,16,32,64,128,0,129,240,2,2,96,248,22,89,32,1,2,4,8,16,32,64,96,172,128,0,56,214,211,104,4,8,16,32,64,128,0,129,240,2,2,96,248,22,89,32,1,2,4,8,16,32,64,96,172,128,0,56,214,211,104,4,8,16,32,64,128,0,129,240,2,207,2,224,211,127,61,18,190,50,11,36,64,128,0,1,2,4,8,52,22,120,22,228,158,5,192,198,100,74,39,64,128,
0,1,2,4,8,212,21,16,0,235,246,86,101,4,8,16,32,64,128,0,129,187,2,2,224,93,22,223,36,64,128,0,1,2,4,8,212,21,16,0,235,246,86,101,4,8,16,32,64,128,0,129,187,2,2,224,93,22,223,36,64,128,0,1,2,4,8,212,21,120,18,0,159,253,219,145,186,32,42,35,64,128,0,1,2,4,8,148,16,120,18,229,158,4,192,18,165,43,130,0,1,2,4,8,16,32,64,224,15,1,1,240,15,16,95,18,32,64,128,0,1,2,4,170,11,8,128,213,59,172,62,2,4,8,16,32,64,128,192,31,2,2,224,31,32,190,36,64,128,0,1,2,4,8,84,23,16,0,171,119,88,125,4,8,16,32,64,128,0,129,63,4,30,7,192,39,255,114,228,143,49,124,73,128,0,1,2,4,8,16,32,16,78,224,113,152,123,28,0,195,21,97,65,4,8,16,32,64,128,0,1,2,35,4,4,192,17,138,198,32,64,128,0,1,2,4,8,36,18,16,0,19,53,203,82,9,16,32,64,128,0,1,2,35,4,4,192,17,138,198,32,64,128,0,1,2,4,8,36,18,16,0,19,53,203,82,9,16,32,64,128,0,1,2,35,4,30,6,192,199,255,110,100,196,180,198,32,64,128,0,1,2,4,8,16,152,44,240,48,206,61,12,128,147,23,100,120,2,4,8,16,32,64,128,0,129,77,2,2,224,38,120,211,18,32,64,128,0,1,2,4,118,9,8,128,
187,228,205,75,128,0,1,2,4,8,16,216,36,32,0,110,130,55,45,1,2,4,8,16,32,64,96,151,128,0,184,75,222,188,4,8,16,32,64,128,0,129,77,2,2,224,38,120,211,18,32,64,128,0,1,2,4,118,9,60,10,128,15,255,217,240,174,133,154,151,0,1,2,4,8,16,32,64,224,156,192,163,64,247,40,0,158,27,221,179,9,16,32,64,128,0,1,2,4,210,8,8,128,105,90,101,161,4,8,16,32,64,128,0,129,49,2,2,224,24,71,163,16,32,64,128,0,1,2,4,210,8,8,128,105,90,101,161,4,8,16,32,64,128,0,129,49,2,2,224,24,71,163,16,32,64,128,0,1,2,4,210,8,60,8,128,143,254,205,72,154,186,44,148,0,1,2,4,8,16,32,64,224,65,164,123,16,0,121,17,32,64,128,0,1,2,4,8,84,21,16,0,171,118,86,93,4,8,16,32,64,128,0,129,7,2,2,224,3,24,223,38,64,128,0,1,2,4,8,84,21,16,0,171,118,86,93,4,8,16,32,64,128,0,129,7,2,2,224,3,24,223,38,64,128,0,1,2,4,8,84,21,16,0,171,118,86,93,4,8,16,32,64,128,0,129,7,2,247,3,224,131,127,50,252,96,12,223,38,64,128,0,1,2,4,8,16,8,41,112,63,212,221,15,128,33,11,176,40,2,4,8,16,32,64,128,0,129,17,2,2,224,8,69,99,16,32,64,128,0,1,2,4,18,9,
8,128,137,154,101,169,4,8,16,32,64,128,0,129,17,2,2,224,8,69,99,16,32,64,128,0,1,2,4,18,9,8,128,137,154,101,169,4,8,16,32,64,128,0,129,17,2,2,224,8,69,99,16,32,64,128,0,1,2,4,18,9,8,128,137,154,101,169,4,8,16,32,64,128,0,129,17,2,119,3,224,253,255,98,204,136,233,140,65,128,0,1,2,4,8,16,32,176,80,224,110,172,187,27,0,23,46,202,84,4,8,16,32,64,128,0,1,2,139,5,4,192,197,224,166,35,64,128,0,1,2,4,8,236,22,16,0,119,119,192,252,4,8,16,32,64,128,0,129,197,2,2,224,98,112,211,17,32,64,128,0,1,2,4,118,11,8,128,187,59,96,126,2,4,8,16,32,64,128,192,98,1,1,112,49,184,233,8,16,32,64,128,0,1,2,187,5,238,5,192,187,255,92,120,247,66,205,79,128,0,1,2,4,8,16,32,112,94,224,94,176,187,23,0,207,143,236,21,4,8,16,32,64,128,0,1,2,105,4,4,192,52,173,178,80,2,4,8,16,32,64,128,192,24,1,1,112,140,163,81,8,16,32,64,128,0,1,2,105,4,4,192,52,173,178,80,2,4,8,16,32,64,128,192,24,1,1,112,140,163,81,8,16,32,64,128,0,1,2,105,4,4,192,52,173,178,80,2,4,8,16,32,64,128,192,24,1,1,112,140,163,81,8,16,32,64,128,0,1,
2,105,4,4,192,52,173,178,80,2,4,8,16,32,64,128,192,24,129,59,1,240,222,127,46,112,204,100,70,33,64,128,0,1,2,4,8,16,88,44,112,39,218,221,9,128,139,23,101,58,2,4,8,16,32,64,128,0,129,165,2,2,224,82,110,147,17,32,64,128,0,1,2,4,246,11,8,128,251,123,96,5,4,8,16,32,64,128,0,129,165,2,2,224,82,110,147,17,32,64,128,0,1,2,4,246,11,8,128,251,123,96,5,4,8,16,32,64,128,0,129,165,2,2,224,82,110,147,17,32,64,128,0,1,2,4,246,11,8,128,251,123,96,5,4,8,16,32,64,128,0,129,165,2,127,7,192,59,255,173,152,165,43,50,25,1,2,4,8,16,32,64,128,192,64,129,191,195,221,223,1,112,224,116,134,34,64,128,0,1,2,4,8,16,136,39,32,0,198,235,137,21,17,32,64,128,0,1,2,4,166,10,8,128,83,121,13,78,128,0,1,2,4,8,16,136,39,32,0,198,235,137,21,17,32,64,128,0,1,2,4,166,10,8,128,83,121,13,78,128,0,1,2,4,8,16,136,39,32,0,198,235,137,21,17,32,64,128,0,1,2,4,166,10,8,128,83,121,13,78,128,0,1,2,4,8,16,136,39,32,0,198,235,137,21,17,32,64,128,0,1,2,4,166,10,252,21,0,255,254,79,5,78,157,223,224,4,8,16,32,64,128,0,1,2,115,5,254,
138,119,127,5,192,185,243,27,157,0,1,2,4,8,16,32,64,96,183,128,0,184,187,3,230,39,64,128,0,1,2,4,8,44,22,16,0,23,131,155,142,0,1,2,4,8,16,32,176,91,64,0,220,221,1,243,19,32,64,128,0,1,2,4,22,11,8,128,139,193,77,71,128,0,1,2,4,8,16,216,45,32,0,238,238,128,249,9,16,32,64,128,0,1,2,139,5,4,192,197,224,166,35,64,128,0,1,2,4,8,236,22,16,0,119,119,192,252,4,8,16,32,64,128,0,129,197,2,2,224,98,112,211,17,32,64,128,0,1,2,4,118,11,252,25,0,255,250,47,69,239,94,160,249,9,16,32,64,128,0,1,2,4,174,9,252,25,240,254,12,128,215,70,247,106,2,4,8,16,32,64,128,0,129,240,2,2,96,248,22,89,32,1,2,4,8,16,32,64,96,172,128,0,56,214,211,104,4,8,16,32,64,128,0,129,240,2,2,96,248,22,89,32,1,2,4,8,16,32,64,96,172,128,0,56,214,211,104,4,8,16,32,64,128,0,129,240,2,2,96,248,22,89,32,1,2,4,8,16,32,64,96,172,128,0,56,214,211,104,4,8,16,32,64,128,0,129,240,2,2,96,248,22,89,32,1,2,4,8,16,32,64,96,172,128,0,56,214,211,104,4,8,16,32,64,128,0,129,240,2,2,96,248,22,89,32,1,2,4,8,16,32,64,96,172,192,31,1,240,207,255,
161,144,177,147,25,141,0,1,2,4,8,16,32,64,96,131,192,31,17,239,143,0,184,97,65,166,36,64,128,0,1,2,4,8,16,88,42,32,0,46,229,54,25,1,2,4,8,16,32,64,96,191,128,0,184,191,7,86,64,128,0,1,2,4,8,16,88,42,32,0,46,229,54,25,1,2,4,8,16,32,64,96,191,128,0,184,191,7,86,64,128,0,1,2,4,8,16,88,42,32,0,46,229,54,25,1,2,4,8,16,32,64,96,191,128,0,184,191,7,86,64,128,0,1,2,4,8,16,88,42,32,0,46,229,54,25,1,2,4,8,16,32,64,96,191,128,0,184,191,7,86,64,128,0,1,2,4,8,16,88,42,32,0,46,229,54,25,1,2,4,8,16,32,64,96,191,192,247,0,248,199,255,76,200,254,229,89,1,1,2,4,8,16,32,64,128,192,117,129,239,33,239,123,0,188,62,186,17,8,16,32,64,128,0,1,2,4,130,11,8,128,193,27,100,121,4,8,16,32,64,128,0,129,209,2,2,224,104,81,227,17,32,64,128,0,1,2,4,130,11,8,128,193,27,100,121,4,8,16,32,64,128,0,129,209,2,2,224,104,81,227,17,32,64,128,0,1,2,4,130,11,8,128,193,27,100,121,4,8,16,32,64,128,0,129,209,2,2,224,104,81,227,17,32,64,128,0,1,2,4,130,11,8,128,193,27,100,121,4,8,16,32,64,128,0,129,209,2,2,224,104,81,227,
17,32,64,128,0,1,2,4,130,11,8,128,193,27,100,121,4,8,16,32,64,128,0,129,209,2,2,224,104,81,227,17,32,64,128,0,1,2,4,130,11,8,128,193,27,100,121,4,8,16,32,64,128,0,129,209,2,223,2,224,247,255,153,224,209,83,25,143,0,1,2,4,8,16,32,64,96,147,192,183,152,247,45,0,110,90,144,105,9,16,32,64,128,0,1,2,4,22,10,8,128,11,177,77,69,128,0,1,2,4,8,16,136,32,32,0,70,232,130,53,16,32,64,128,0,1,2,4,22,10,8,128,11,177,77,69,128,0,1,2,4,8,16,136,32,32,0,70,232,130,53,16,32,64,128,0,1,2,4,22,10,8,128,11,177,77,69,128,0,1,2,4,8,16,136,32,32,0,70,232,130,53,16,32,64,128,0,1,2,4,22,10,8,128,11,177,77,69,128,0,1,2,4,8,16,136,32,32,0,70,232,130,53,16,32,64,128,0,1,2,4,22,10,8,128,11,177,77,69,128,0,1,2,4,8,16,136,32,32,0,70,232,130,53,16,32,64,128,0,1,2,4,22,10,8,128,11,177,77,69,128,0,1,2,4,8,16,136,32,32,0,70,232,130,53,16,32,64,128,0,1,2,4,22,10,8,128,11,177,77,69,128,0,1,2,4,8,16,136,32,32,0,70,232,130,53,16,32,64,128,0,1,2,4,22,10,220,6,192,127,22,206,107,42,2,4,8,16,32,64,128,0,129,133,2,183,
65,239,54,0,46,92,130,169,8,16,32,64,128,0,1,2,4,118,9,8,128,187,228,205,75,128,0,1,2,4,8,16,216,36,32,0,110,130,55,45,1,2,4,8,16,32,64,96,151,128,0,184,75,222,188,4,8,16,32,64,128,0,129,77,2,2,224,38,120,211,18,32,64,128,0,1,2,4,118,9,8,128,187,228,205,75,128,0,1,2,4,8,16,216,36,32,0,110,130,55,45,1,2,4,8,16,32,64,96,151,128,0,184,75,222,188,4,8,16,32,64,128,0,129,77,2,2,224,38,120,211,18,32,64,128,0,1,2,4,118,9,8,128,187,228,205,75,128,0,1,2,4,8,16,216,36,32,0,110,130,55,45,1,2,4,8,16,32,64,96,151,128,0,184,75,222,188,4,8,16,32,64,128,0,129,77,2,2,224,38,120,211,18,32,64,128,0,1,2,4,118,9,8,128,187,228,205,75,128,0,1,2,4,8,16,216,36,32,0,110,130,55,45,1,2,4,8,16,32,64,96,151,128,0,184,75,222,188,4,8,16,32,64,128,0,129,77,2,2,224,38,120,211,18,32,64,128,0,1,2,4,118,9,8,128,187,228,205,75,128,0,1,2,4,8,16,216,36,32,0,110,130,55,45,1,2,4,8,16,32,64,96,151,192,77,0,252,103,215,26,204,75,128,0,1,2,4,8,16,32,48,91,224,38,234,221,4,192,217,179,26,159,0,1,2,4,8,16,32,64,32,130,128,
0,24,161,11,214,64,128,0,1,2,4,8,16,88,40,32,0,46,196,54,21,1,2,4,8,16,32,64,32,130,128,0,24,161,11,214,64,128,0,1,2,4,8,16,88,40,32,0,46,196,54,21,1,2,4,8,16,32,64,32,130,128,0,24,161,11,214,64,128,0,1,2,4,8,16,88,40,32,0,46,196,54,21,1,2,4,8,16,32,64,32,130,128,0,24,161,11,214,64,128,0,1,2,4,8,16,88,40,32,0,46,196,54,21,1,2,4,8,16,32,64,32,130,128,0,24,161,11,214,64,128,0,1,2,4,8,16,88,40,32,0,46,196,54,21,1,2,4,8,16,32,64,32,130,128,0,24,161,11,214,64,128,0,1,2,4,8,16,88,40,32,0,46,196,54,21,1,2,4,8,16,32,64,32,130,128,0,24,161,11,214,64,128,0,1,2,4,8,16,88,40,32,0,46,196,54,21,1,2,4,8,16,32,64,32,130,128,0,24,161,11,214,64,128,0,1,2,4,8,16,88,40,32,0,46,196,54,21,1,2,4,8,16,32,64,32,130,128,0,24,161,11,214,64,128,0,1,2,4,8,16,88,40,32,0,46,196,54,21,1,2,4,8,16,32,64,32,130,128,0,24,161,11,214,64,128,0,1,2,4,8,16,88,40,32,0,46,196,54,21,1,2,4,8,16,32,64,32,130,128,0,24,161,11,214,64,128,0,1,2,4,8,16,88,40,32,0,46,196,54,21,1,2,4,8,16,32,64,32,130,128,0,24,161,11,214,64,128,
0,1,2,4,8,16,88,40,32,0,46,196,54,21,1,2,4,8,16,32,64,32,130,128,0,24,161,11,214,64,128,0,1,2,4,8,16,88,40,32,0,46,196,54,21,1,2,4,8,16,32,64,32,130,128,0,24,161,11,214,64,128,0,1,2,4,8,16,88,40,32,0,46,196,54,21,1,2,4,8,16,32,64,32,130,128,0,24,161,11,214,64,128,0,1,2,4,8,16,88,40,32,0,46,196,54,21,1,2,4,8,16,32,64,32,130,128,0,24,161,11,214,64,128,0,1,2,4,8,16,88,40,32,0,46,196,54,21,1,2,4,8,16,32,64,32,130,128,0,24,161,11,214,64,128,0,1,2,4,8,16,88,40,32,0,46,196,54,21,1,2,4,8,16,32,64,32,130,192,87,0,252,39,194,114,172,129,0,1,2,4,8,16,32,64,96,142,192,87,216,251,10,128,115,102,50,42,1,2,4,8,16,32,64,128,64,48,1,1,48,88,67,44,135,0,1,2,4,8,16,32,48,91,64,0,156,45,108,124,2,4,8,16,32,64,128,64,48,1,1,48,88,67,44,135,0,1,2,4,8,16,32,48,91,64,0,156,45,108,124,2,4,8,16,32,64,128,64,48,1,1,48,88,67,44,135,0,1,2,4,8,16,32,48,91,64,0,156,45,108,124,2,4,8,16,32,64,128,64,48,1,1,48,88,67,44,135,0,1,2,4,8,16,32,48,91,64,0,156,45,108,124,2,4,8,16,32,64,128,64,48,1,1,48,88,67,44,135,
0,1,2,4,8,16,32,48,91,64,0,156,45,108,124,2,4,8,16,32,64,128,64,48,1,1,48,88,67,44,135,0,1,2,4,8,16,32,48,91,64,0,156,45,108,124,2,4,8,16,32,64,128,64,48,1,1,48,88,67,44,135,0,1,2,4,8,16,32,48,91,64,0,156,45,108,124,2,4,8,16,32,64,128,64,48,1,1,48,88,67,44,135,0,1,2,4,8,16,32,48,91,64,0,156,45,108,124,2,4,8,16,32,64,128,64,48,1,1,48,88,67,44,135,0,1,2,4,8,16,32,48,91,64,0,156,45,108,124,2,4,8,16,32,64,128,64,48,1,1,48,88,67,44,135,0,1,2,4,8,16,32,48,91,64,0,156,45,108,124,2,4,8,16,32,64,128,64,48,1,1,48,88,67,44,135,0,1,2,4,8,16,32,48,91,64,0,156,45,108,124,2,4,8,16,32,64,128,64,48,1,1,48,88,67,44,135,0,1,2,4,8,16,32,48,91,64,0,156,45,108,124,2,4,8,16,32,64,128,64,48,1,1,48,88,67,44,135,0,1,2,4,8,16,32,48,91,64,0,156,45,108,124,2,4,8,16,32,64,128,64,48,1,1,48,88,67,44,135,0,1,2,4,8,16,32,48,91,64,0,156,45,108,124,2,4,8,16,32,64,128,64,48,1,1,48,88,67,44,135,0,1,2,4,8,16,32,48,91,64,0,156,45,108,124,2,4,8,16,32,64,128,64,48,1,1,48,88,67,44,135,0,1,2,4,8,16,32,48,91,64,0,156,45,
108,124,2,4,8,16,32,64,128,64,48,129,127,190,214,115,243,240,235,155,30,17,32,64,128,0,1,2,4,8,148,16,248,223,103,21,62,1,252,164,240,128,0,1,2,4,8,16,32,208,67,64,0,236,209,103,85,18,32,64,128,0,1,2,4,62,5,4,192,79,10,15,8,16,32,64,128,0,1,2,61,4,4,192,30,125,86,37,1,2,4,8,16,32,64,224,83,64,0,252,164,240,128,0,1,2,4,8,16,32,208,67,64,0,236,209,103,85,18,32,64,128,0,1,2,4,62,5,4,192,79,10,15,8,16,32,64,128,0,1,2,61,4,4,192,30,125,86,37,1,2,4,8,16,32,64,224,83,64,0,252,164,240,128,0,1,2,4,8,16,32,208,67,64,0,236,209,103,85,18,32,64,128,0,1,2,4,62,5,4,192,79,10,15,8,16,32,64,128,0,1,2,61,4,4,192,30,125,86,37,1,2,4,8,16,32,64,224,83,64,0,252,164,240,128,0,1,2,4,8,16,32,208,67,64,0,236,209,103,85,18,32,64,128,0,1,2,4,62,5,4,192,79,10,15,8,16,32,64,128,0,1,2,61,4,4,192,30,125,86,37,1,2,4,8,16,32,64,224,83,64,0,252,164,240,128,0,1,2,4,8,16,32,208,67,64,0,236,209,103,85,18,32,64,128,0,1,2,4,62,5,4,192,79,10,15,8,16,32,64,128,0,1,2,61,4,4,192,30,125,86,37,1,2,4,8,16,32,64,224,83,64,
0,252,164,240,128,0,1,2,4,8,16,32,208,67,64,0,236,209,103,85,18,32,64,128,0,1,2,4,62,5,4,192,79,10,15,8,16,32,64,128,0,1,2,61,4,4,192,30,125,86,37,1,2,4,8,16,32,64,224,83,64,0,252,164,240,128,0,1,2,4,8,16,32,208,67,64,0,236,209,103,85,18,32,64,128,0,1,2,4,62,5,4,192,79,10,15,8,16,32,64,128,0,1,2,61,4,4,192,30,125,86,37,1,2,4,8,16,32,64,224,83,64,0,252,164,240,128,0,1,2,4,8,16,32,208,67,64,0,236,209,103,85,18,32,64,128,0,1,2,4,62,5,4,192,79,10,15,8,16,32,64,128,0,1,2,61,4,4,192,30,125,86,37,1,2,4,8,16,32,64,224,83,64,0,252,164,240,128,0,1,2,4,8,16,32,208,67,64,0,236,209,103,85,18,32,64,128,0,1,2,4,62,5,4,192,79,10,15,8,16,32,64,128,0,1,2,61,4,254,185,41,243,246,241,205,183,61,36,64,128,0,1,2,4,8,16,72,47,240,191,175,10,124,2,248,101,225,17,1,2,4,8,16,32,64,160,133,128,0,216,162,205,138,36,64,128,0,1,2,4,8,124,9,8,128,95,22,30,17,32,64,128,0,1,2,4,90,8,8,128,45,218,172,72,2,4,8,16,32,64,128,192,151,128,0,248,101,225,17,1,2,4,8,16,32,64,160,133,128,0,216,162,205,138,36,64,128,0,
1,2,4,8,124,9,8,128,95,22,30,17,32,64,128,0,1,2,4,90,8,8,128,45,218,172,72,2,4,8,16,32,64,128,192,151,128,0,248,101,225,17,1,2,4,8,16,32,64,160,133,128,0,216,162,205,138,36,64,128,0,1,2,4,8,124,9,8,128,95,22,30,17,32,64,128,0,1,2,4,90,8,8,128,45,218,172,72,2,4,8,16,32,64,128,192,151,128,0,248,101,225,17,1,2,4,8,16,32,64,160,133,128,0,216,162,205,138,36,64,128,0,1,2,4,8,124,9,8,128,95,22,30,17,32,64,128,0,1,2,4,90,8,8,128,45,218,172,72,2,4,8,16,32,64,128,192,151,128,0,248,101,225,17,1,2,4,8,16,32,64,160,133,128,0,216,162,205,138,36,64,128,0,1,2,4,8,124,9,8,128,95,22,30,17,32,64,128,0,1,2,4,90,8,252,115,91,229,183,47,110,127,224,49,1,2,4,8,16,32,64,128,64,106,129,255,221,172,222,39,128,55,24,30,18,32,64,128,0,1,2,4,58,8,8,128,29,186,172,70,2,4,8,16,32,64,128,192,141,128,0,120,131,225,33,1,2,4,8,16,32,64,160,131,128,0,216,161,203,106,36,64,128,0,1,2,4,8,220,8,8,128,55,24,30,18,32,64,128,0,1,2,4,58,8,8,128,29,186,172,70,2,4,8,16,32,64,128,192,141,128,0,120,131,225,33,1,2,4,8,16,32,
64,160,131,128,0,216,161,203,106,36,64,128,0,1,2,4,8,220,8,8,128,55,24,30,18,32,64,128,0,1,2,4,58,8,8,128,29,186,172,70,2,4,8,16,32,64,128,192,141,128,0,120,131,225,33,1,2,4,8,16,32,64,160,131,128,0,216,161,203,106,36,64,128,0,1,2,4,8,220,8,8,128,55,24,30,18,32,64,128,0,1,2,4,58,8,8,128,29,186,172,70,2,4,8,16,32,64,128,192,141,128,0,120,131,225,33,1,2,4,8,16,32,64,160,131,192,63,223,138,252,254,213,183,31,249,130,0,1,2,4,8,16,32,64,32,173,192,255,110,87,238,19,192,91,13,143,9,16,32,64,128,0,1,2,13,4,4,192,6,77,86,34,1,2,4,8,16,32,64,224,86,64,0,188,213,240,152,0,1,2,4,8,16,32,208,64,64,0,108,208,100,37,18,32,64,128,0,1,2,4,110,5,4,192,91,13,143,9,16,32,64,128,0,1,2,13,4,4,192,6,77,86,34,1,2,4,8,16,32,64,224,86,64,0,188,213,240,152,0,1,2,4,8,16,32,208,64,64,0,108,208,100,37,18,32,64,128,0,1,2,4,110,5,4,192,91,13,143,9,16,32,64,128,0,1,2,13,4,4,192,6,77,86,34,1,2,4,8,16,32,64,224,86,64,0,188,213,240,152,0,1,2,4,8,16,32,208,64,64,0,108,208,100,37,18,32,64,128,0,1,2,4,110,5,254,185,
253,226,199,143,63,190,252,254,67,95,17,32,64,128,192,255,183,119,95,219,145,220,200,162,64,117,231,255,255,249,92,73,173,102,211,148,73,3,19,102,207,203,144,85,153,64,96,7,146,136,5,84,181,8,16,32,64,32,165,192,255,125,137,218,14,224,23,14,191,16,32,64,128,0,1,2,4,234,11,40,0,235,231,216,8,9,16,32,64,128,0,1,2,95,4,20,128,95,56,252,66,128,0,1,2,4,8,16,168,47,160,0,172,159,99,35,36,64,128,0,1,2,4,8,124,17,80,0,126,225,240,11,1,2,4,8,16,32,64,160,190,128,2,176,126,142,141,144,0,1,2,4,8,16,32,240,69,64,1,248,133,195,47,4,8,16,32,64,128,0,129,250,2,10,192,250,57,54,66,2,4,8,16,32,64,128,192,23,1,5,224,23,14,191,16,32,64,128,0,1,2,4,234,11,40,0,235,231,216,8,9,16,32,64,128,0,1,2,95,4,190,255,183,223,190,255,254,229,98,191,16,32,64,128,0,1,2,4,8,36,20,248,250,95,130,251,203,14,96,194,28,10,153,0,1,2,4,8,16,32,112,71,64,1,120,71,207,189,4,8,16,32,64,128,0,129,132,2,10,192,132,73,19,50,1,2,4,8,16,32,64,224,142,128,2,240,142,158,123,9,16,32,64,128,0,1,2,9,5,20,128,9,147,38,100,2,4,8,
16,32,64,128,192,29,1,5,224,29,61,247,18,32,64,128,0,1,2,4,18,10,40,0,19,38,77,200,4,8,16,32,64,128,0,129,59,2,10,192,59,122,238,37,64,128,0,1,2,4,8,36,20,80,0,38,76,154,144,9,16,32,64,128,0,1,2,119,4,20,128,119,244,220,75,128,0,1,2,4,8,16,72,40,240,227,63,253,246,227,133,132,131,18,50,1,2,4,8,16,32,64,128,192,31,129,111,255,37,56,255,41,184,63,52,126,34,64,128,0,1,2,4,8,244,16,112,4,220,35,207,70,73,128,0,1,2,4,8,16,248,16,80,0,126,80,248,129,0,1,2,4,8,16,32,208,67,64,1,216,35,207,70,73,128,0,1,2,4,8,16,248,16,80,0,126,80,248,129,0,1,2,4,8,16,32,208,67,64,1,216,35,207,70,73,128,0,1,2,4,8,16,248,16,80,0,126,80,248,129,0,1,2,4,8,16,32,208,67,64,1,216,35,207,70,73,128,0,1,2,4,8,16,248,16,80,0,126,80,248,129,0,1,2,4,8,16,32,208,67,224,231,127,248,227,231,43,61,36,140,146,0,1,2,4,8,16,32,80,83,224,251,127,8,196,127,9,164,102,158,141,138,0,1,2,4,8,16,32,240,92,192,17,240,115,27,239,16,32,64,128,0,1,2,4,74,10,40,0,75,166,213,160,8,16,32,64,128,0,1,2,207,5,20,128,207,109,188,67,128,
0,1,2,4,8,16,40,41,160,0,44,153,86,131,34,64,128,0,1,2,4,8,60,23,80,0,62,183,241,14,1,2,4,8,16,32,64,160,164,128,2,176,100,90,13,138,0,1,2,4,8,16,32,240,92,64,1,248,220,198,59,4,8,16,32,64,128,0,129,146,2,15,254,217,231,7,47,149,28,186,65,17,32,64,128,0,1,2,4,58,8,252,248,119,160,253,67,208,29,210,110,140,4,8,16,32,64,128,0,129,207,2,142,128,63,107,248,153,0,1,2,4,8,16,32,208,64,64,1,216,32,201,134,72,128,0,1,2,4,8,16,248,44,160,0,252,172,225,103,2,4,8,16,32,64,128,64,3,1,5,96,131,36,27,34,1,2,4,8,16,32,64,224,179,128,2,240,179,134,159,9,16,32,64,128,0,1,2,13,4,20,128,13,146,108,136,4,8,16,32,64,128,0,129,207,2,143,254,209,191,71,175,125,190,199,207,4,8,16,32,64,128,0,1,2,89,4,126,254,51,128,254,29,192,44,185,19,39,1,2,4,8,16,32,64,96,148,128,35,224,81,146,218,33,64,128,0,1,2,4,8,36,17,80,0,38,73,148,48,9,16,32,64,128,0,1,2,163,4,20,128,163,36,181,67,128,0,1,2,4,8,16,72,34,160,0,76,146,40,97,18,32,64,128,0,1,2,4,70,9,40,0,71,73,106,135,0,1,2,4,8,16,32,144,68,64,1,152,36,81,194,
36,64,128,0,1,2,4,8,140,18,80,0,142,146,212,14,1,2,4,8,16,32,64,32,137,192,195,127,244,249,225,139,73,6,36,76,2,4,8,16,32,64,128,0,129,63,2,15,254,29,104,255,16,244,31,30,63,17,32,64,128,0,1,2,4,122,8,56,2,238,145,103,163,36,64,128,0,1,2,4,8,124,8,40,0,63,40,252,64,128,0,1,2,4,8,16,232,33,160,0,236,145,103,163,36,64,128,0,1,2,4,8,124,8,40,0,63,40,252,64,128,0,1,2,4,8,16,232,33,160,0,236,145,103,163,36,64,128,0,1,2,4,8,124,8,60,254,23,95,30,191,250,113,147,31,8,16,32,64,128,0,1,2,4,82,8,60,250,87,96,252,51,48,41,82,39,72,2,4,8,16,32,64,128,192,64,1,71,192,3,49,53,69,128,0,1,2,4,8,16,200,32,160,0,204,144,37,49,18,32,64,128,0,1,2,4,6,10,40,0,7,98,106,138,0,1,2,4,8,16,32,144,65,64,1,152,33,75,98,36,64,128,0,1,2,4,8,12,20,80,0,14,196,212,20,1,2,4,8,16,32,64,32,131,128,2,48,67,150,196,72,128,0,1,2,4,8,16,24,40,240,228,95,252,123,242,242,192,142,53,69,128,0,1,2,4,8,16,32,48,91,224,225,63,3,232,223,1,156,205,174,125,2,4,8,16,32,64,128,64,52,1,71,192,209,50,34,30,2,4,8,16,32,64,128,192,
100,1,5,224,100,96,205,19,32,64,128,0,1,2,4,162,9,40,0,163,101,68,60,4,8,16,32,64,128,0,129,201,2,10,192,201,192,154,39,64,128,0,1,2,4,8,68,19,80,0,70,203,136,120,8,16,32,64,128,0,1,2,147,5,158,253,123,47,207,94,159,28,142,230,9,16,32,64,128,0,1,2,4,134,9,60,254,87,96,252,51,48,195,128,53,68,128,0,1,2,4,8,16,72,34,224,8,56,73,162,132,73,128,0,1,2,4,8,16,24,37,160,0,28,37,169,29,2,4,8,16,32,64,128,64,18,1,5,96,146,68,9,147,0,1,2,4,8,16,32,48,74,64,1,56,74,82,59,4,8,16,32,64,128,0,129,36,2,79,191,237,251,244,141,36,3,19,38,1,2,4,8,16,32,64,160,187,192,147,47,1,251,22,112,247,137,97,252,4,8,16,32,64,128,64,63,1,71,192,253,114,110,196,4,8,16,32,64,128,64,115,1,5,96,243,9,96,248,4,8,16,32,64,128,64,63,1,5,96,191,156,27,49,1,2,4,8,16,32,208,92,64,1,216,124,2,24,62,1,2,4,8,16,32,208,79,64,1,216,47,231,70,76,128,0,1,2,4,8,52,23,120,254,175,189,60,127,167,57,153,225,19,32,64,128,0,1,2,4,82,8,60,251,87,96,252,51,48,41,210,39,72,2,4,8,16,32,64,128,192,64,1,71,192,3,49,53,69,128,0,1,2,4,
8,16,200,32,160,0,204,144,37,49,18,32,64,128,0,1,2,4,6,10,40,0,7,98,106,138,0,1,2,4,8,16,32,144,65,64,1,152,33,75,98,36,64,128,0,1,2,4,8,12,20,120,241,93,223,23,111,13,12,64,83,4,8,16,32,64,128,0,1,2,51,4,158,126,9,216,183,128,103,112,107,147,0,1,2,4,8,16,32,16,89,192,17,112,228,236,136,141,0,1,2,4,8,16,32,48,65,64,1,56,1,85,147,4,8,16,32,64,128,0,129,200,2,10,192,200,217,17,27,1,2,4,8,16,32,64,96,130,128,2,112,2,170,38,9,16,32,64,128,0,1,2,145,5,94,125,213,247,213,123,145,199,36,54,2,4,8,16,32,64,128,0,129,231,95,2,246,45,96,179,131,0,1,2,4,8,16,32,208,77,192,17,112,183,140,27,47,1,2,4,8,16,32,208,94,64,1,216,126,10,0,32,64,128,0,1,2,4,186,9,40,0,187,101,220,120,9,16,32,64,128,0,129,246,2,10,192,246,83,0,0,1,2,4,8,16,32,208,77,224,229,55,125,95,190,217,77,202,120,9,16,32,64,128,0,1,2,137,4,94,124,9,216,183,128,19,229,81,168,4,8,16,32,64,128,0,129,33,2,142,128,135,48,106,132,0,1,2,4,8,16,32,144,71,64,1,152,39,87,34,37,64,128,0,1,2,4,8,12,17,80,0,14,97,212,8,1,2,4,8,16,32,64,32,
143,128,2,48,79,174,68,74,128,0,1,2,4,8,16,24,34,240,250,139,190,175,223,29,18,128,70,8,16,32,64,128,0,1,2,4,134,11,188,250,18,176,111,1,15,231,214,32,1,2,4,8,16,32,64,32,184,128,35,224,224,9,18,30,1,2,4,8,16,32,64,96,180,128,2,112,180,168,246,8,16,32,64,128,0,1,2,193,5,20,128,193,19,36,60,2,4,8,16,32,64,128,192,104,1,5,224,104,81,237,17,32,64,128,0,1,2,4,130,11,188,249,158,239,155,183,131,15,78,120,4,8,16,32,64,128,0,129,158,2,47,191,4,236,91,192,61,39,133,81,19,32,64,128,0,1,2,157,5,28,1,119,206,190,177,19,32,64,128,0,1,2,45,5,20,128,45,211,110,208,4,8,16,32,64,128,64,103,1,5,96,231,236,27,59,1,2,4,8,16,32,208,82,224,221,183,60,222,189,223,18,205,160,9,16,32,64,128,0,1,2,161,5,94,127,7,196,151,64,66,39,79,112,4,8,16,32,64,128,0,129,9,2,142,128,39,160,106,146,0,1,2,4,8,16,32,16,89,64,1,24,57,59,98,35,64,128,0,1,2,4,8,76,16,80,0,78,64,213,36,1,2,4,8,16,32,64,32,178,128,2,48,114,118,196,70,128,0,1,2,4,8,16,152,32,240,246,91,190,111,47,152,16,148,38,9,16,32,64,128,0,1,2,4,174,11,
188,249,18,176,111,1,95,167,117,39,1,2,4,8,16,32,64,32,167,128,35,224,156,121,19,53,1,2,4,8,16,32,64,224,178,128,2,240,50,157,27,9,16,32,64,128,0,1,2,57,5,20,128,57,243,38,106,2,4,8,16,32,64,128,192,101,129,247,223,241,120,127,197,229,206,221,72,128,0,1,2,4,8,16,32,48,92,224,221,119,64,124,9,100,56,185,6,9,16,32,64,128,0,1,2,193,5,28,1,7,79,144,240,8,16,32,64,128,0,1,2,163,5,20,128,163,69,181,71,128,0,1,2,4,8,16,8,46,160,0,12,158,32,225,17,32,64,128,0,1,2,4,70,11,28,248,138,199,129,75,70,71,165,61,2,4,8,16,32,64,128,0,129,139,2,111,191,3,226,75,32,23,101,221,70,128,0,1,2,4,8,16,72,43,224,8,56,109,234,4,78,128,0,1,2,4,8,16,184,38,160,0,188,230,230,46,2,4,8,16,32,64,128,64,90,1,5,96,218,212,9,156,0,1,2,4,8,16,32,112,77,224,200,55,60,142,92,115,173,119,119,17,32,64,128,0,1,2,4,8,140,21,120,255,29,16,95,2,25,43,174,53,2,4,8,16,32,64,128,64,124,1,71,192,241,115,36,66,2,4,8,16,32,64,128,192,80,1,5,224,80,78,141,17,32,64,128,0,1,2,4,226,11,40,0,227,231,72,132,4,8,16,32,64,128,0,129,
161,2,135,190,224,113,232,162,161,97,105,140,0,1,2,4,8,16,32,64,224,138,192,129,239,128,248,18,200,21,88,247,16,32,64,128,0,1,2,4,50,11,56,2,206,156,61,177,19,32,64,128,0,1,2,4,46,8,40,0,47,160,185,133,0,1,2,4,8,16,32,144,89,64,1,152,57,123,98,39,64,128,0,1,2,4,8,92,16,56,246,253,142,99,87,93,232,222,45,4,8,16,32,64,128,0,1,2,3,5,142,124,7,196,151,64,6,130,107,138,0,1,2,4,8,16,32,144,66,192,17,112,138,52,9,146,0,1,2,4,8,16,32,48,78,64,1,56,206,82,75,4,8,16,32,64,128,0,129,20,2,7,63,221,119,240,178,20,67,22,36,1,2,4,8,16,32,64,160,170,192,161,143,0,250,12,96,213,244,27,23,1,2,4,8,16,32,64,224,153,128,35,224,103,50,94,39,64,128,0,1,2,4,8,20,21,80,0,22,77,172,97,17,32,64,128,0,1,2,4,158,9,40,0,159,201,120,157,0,1,2,4,8,16,32,80,84,224,232,183,59,142,94,87,148,201,176,8,16,32,64,128,0,1,2,9,4,142,125,7,196,151,64,18,164,82,136,4,8,16,32,64,128,0,129,161,2,142,128,135,114,106,140,0,1,2,4,8,16,32,16,95,64,1,24,63,71,34,36,64,128,0,1,2,4,8,12,21,56,252,217,190,195,23,14,13,79,99,4,8,
16,32,64,128,0,1,2,71,5,14,126,4,208,103,0,143,130,186,142,0,1,2,4,8,16,32,80,69,192,17,112,149,76,26,7,1,2,4,8,16,32,64,224,160,128,2,240,32,148,203,8,16,32,64,128,0,1,2,85,4,20,128,85,50,105,28,4,8,16,32,64,128,0,129,131,2,199,191,219,113,252,202,131,93,187,140,0,1,2,4,8,16,32,64,96,160,192,209,239,128,248,18,200,64,116,77,17,32,64,128,0,1,2,4,82,8,56,2,78,145,38,65,18,32,64,128,0,1,2,4,198,9,40,0,199,89,106,137,0,1,2,4,8,16,32,144,66,224,196,39,251,78,92,154,98,232,130,36,64,128,0,1,2,4,8,84,18,56,252,17,64,159,1,172,148,118,99,33,64,128,0,1,2,4,8,28,17,112,4,124,68,201,53,4,8,16,32,64,128,0,129,66,2,10,192,66,201,52,20,2,4,8,16,32,64,128,192,17,1,5,224,17,37,215,16,32,64,128,0,1,2,4,10,9,156,249,102,199,153,107,11,17,25,10,1,2,4,8,16,32,64,32,129,192,241,239,128,248,18,72,130,116,10,145,0,1,2,4,8,16,32,48,84,192,17,240,80,78,141,17,32,64,128,0,1,2,4,226,11,40,0,227,231,72,132,4,8,16,32,64,128,0,129,161,2,167,62,215,119,234,226,161,97,106,140,0,1,2,4,8,16,32,64,224,149,192,
137,143,0,250,12,224,43,72,239,17,32,64,128,0,1,2,4,42,10,56,2,174,152,85,99,34,64,128,0,1,2,4,8,188,16,80,0,190,192,241,22,1,2,4,8,16,32,64,160,162,192,185,143,245,157,187,186,162,151,49,17,32,64,128,0,1,2,4,34,10,156,249,8,160,207,0,70,204,160,152,8,16,32,64,128,0,1,2,51,5,28,1,207,212,213,54,1,2,4,8,16,32,64,32,160,128,2,48,96,82,132,68,128,0,1,2,4,8,16,152,41,112,242,83,125,39,47,159,25,185,182,9,16,32,64,128,0,1,2,4,254,19,56,245,17,64,159,1,52,111,8,16,32,64,128,0,1,2,221,4,28,1,119,203,184,241,18,32,64,128,0,1,2,237,5,20,128,237,167,0,0,2,4,8,16,32,64,160,155,192,217,15,245,157,189,190,155,167,241,18,32,64,128,0,1,2,4,214,11,156,251,8,160,207,0,174,207,144,30,9,16,32,64,128,0,1,2,123,5,28,1,239,245,215,59,1,2,4,8,16,32,64,96,185,128,2,112,57,185,14,9,16,32,64,128,0,1,2,123,5,78,127,166,239,244,13,123,199,167,119,2,4,8,16,32,64,128,64,121,129,147,31,1,244,25,192,242,51,194,0,9,16,32,64,128,0,1,2,223,4,28,1,127,3,241,43,1,2,4,8,16,32,64,160,186,128,2,176,122,134,141,143,
0,1,2,4,8,16,32,240,77,224,252,71,250,206,223,241,173,75,191,18,32,64,128,0,1,2,4,8,12,20,56,251,17,64,159,1,28,136,175,41,2,4,8,16,32,64,128,64,10,1,71,192,41,210,36,72,2,4,8,16,32,64,128,192,56,1,5,224,56,75,45,17,32,64,128,0,1,2,4,82,8,92,248,68,223,133,91,82,80,8,146,0,1,2,4,8,16,32,144,81,224,244,71,0,125,6,48,99,154,197,76,128,0,1,2,4,8,16,184,35,224,8,248,142,158,123,9,16,32,64,128,0,1,2,9,5,20,128,9,147,38,100,2,4,8,16,32,64,128,192,29,129,43,31,232,187,114,207,157,24,221,75,128,0,1,2,4,8,16,32,240,76,224,252,71,0,125,6,240,153,165,215,9,16,32,64,128,0,1,2,85,5,28,1,87,205,172,113,17,32,64,128,0,1,2,4,158,8,40,0,159,192,120,153,0,1,2,4,8,16,32,80,85,224,210,231,249,46,221,84,85,208,184,8,16,32,64,128,0,1,2,27,5,46,124,4,208,103,0,55,230,75,215,4,8,16,32,64,128,0,129,45,2,142,128,183,176,235,148,0,1,2,4,8,16,32,176,79,64,1,184,207,94,207,4,8,16,32,64,128,0,129,45,2,215,62,206,119,237,174,45,3,212,41,1,2,4,8,16,32,64,160,176,192,149,143,0,250,12,96,225,9,97,104,4,8,16,32,
64,128,0,129,135,2,142,128,31,178,120,145,0,1,2,4,8,16,32,80,87,64,1,88,55,183,70,70,128,0,1,2,4,8,16,120,40,112,241,211,124,23,111,123,24,130,23,9,16,32,64,128,0,1,2,4,174,9,92,250,8,160,207,0,94,195,118,23,1,2,4,8,16,32,64,32,175,128,35,224,188,185,19,57,1,2,4,8,16,32,64,224,146,192,213,179,220,171,247,93,10,210,77,4,8,16,32,64,128,0,1,2,15,4,174,157,0,59,2,126,64,233,37,2,4,8,16,32,64,128,64,105,1,71,192,165,211,107,112,4,8,16,32,64,128,0,129,159,2,10,192,159,38,94,33,64,128,0,1,2,4,8,148,22,184,252,89,190,203,55,150,230,52,56,2,4,8,16,32,64,128,192,58,129,139,31,1,244,25,192,117,41,210,19,1,2,4,8,16,32,64,32,134,128,35,224,24,121,16,5,1,2,4,8,16,32,64,96,153,192,245,147,220,235,119,46,27,156,142,8,16,32,64,128,0,1,2,133,5,174,158,0,59,2,46,60,41,12,141,0,1,2,4,8,16,32,240,80,192,17,240,67,22,47,18,32,64,128,0,1,2,4,234,10,40,0,235,230,214,200,8,16,32,64,128,0,1,2,15,5,110,124,146,239,198,173,15,67,241,34,1,2,4,8,16,32,64,128,192,113,129,203,31,1,244,25,192,227,200,174,36,
64,128,0,1,2,4,8,212,16,112,4,92,35,143,70,65,128,0,1,2,4,8,16,56,44,160,0,60,76,229,66,2,4,8,16,32,64,128,64,13,129,59,31,228,187,115,111,13,61,163,32,64,128,0,1,2,4,8,236,18,184,254,17,64,159,1,220,149,51,253,18,32,64,128,0,1,2,4,118,9,56,2,222,37,175,95,2,4,8,16,32,64,128,192,38,129,91,199,184,183,110,222,52,96,221,18,32,64,128,0,1,2,4,42,8,220,56,1,118,4,92,97,2,24,3,1,2,4,8,16,32,64,224,140,128,35,224,51,90,174,37,64,128,0,1,2,4,8,20,16,184,119,138,123,239,238,2,124,134,64,128,0,1,2,4,8,16,216,34,112,231,4,216,17,240,150,148,233,148,0,1,2,4,8,16,32,176,81,192,17,240,70,124,93,19,32,64,128,0,1,2,4,118,8,40,0,119,168,235,147,0,1,2,4,8,16,32,176,81,224,230,167,248,110,222,190,113,224,186,38,64,128,0,1,2,4,8,228,21,184,245,17,64,159,1,204,155,120,145,19,32,64,128,0,1,2,4,174,9,56,2,190,230,230,46,2,4,8,16,32,64,128,64,90,129,187,103,184,119,239,79,11,39,112,2,4,8,16,32,64,128,192,54,129,123,39,192,142,128,183,37,78,199,4,8,16,32,64,128,0,129,77,2,142,128,55,193,235,150,0,1,2,
4,8,16,32,176,75,64,1,184,75,94,191,4,8,16,32,64,128,0,129,77,2,183,63,195,119,187,129,77,3,215,45,1,2,4,8,16,32,64,32,171,192,205,143,0,250,12,96,214,196,139,155,0,1,2,4,8,16,32,112,85,192,17,240,85,57,247,17,32,64,128,0,1,2,4,146,10,220,63,193,189,223,66,82,58,97,19,32,64,128,0,1,2,4,182,8,220,61,1,118,4,188,37,109,58,37,64,128,0,1,2,4,8,108,20,112,4,188,17,95,215,4,8,16,32,64,128,0,129,29,2,3,14,112,7,52,177,99,228,250,36,64,128,0,1,2,4,8,164,20,184,125,2,236,8,56,101,222,5,77,128,0,1,2,4,8,16,184,33,224,8,248,6,158,91,9,16,32,64,128,0,1,2,25,5,70,156,223,142,104,35,163,157,152,9,16,32,64,128,0,1,2,235,5,238,159,0,59,2,94,159,53,61,18,32,64,128,0,1,2,4,246,10,56,2,222,235,175,119,2,4,8,16,32,64,128,192,114,1,5,224,114,114,29,18,32,64,128,0,1,2,4,246,10,12,249,252,222,144,70,246,58,232,157,0,1,2,4,8,16,32,144,66,96,192,71,0,125,6,48,69,166,5,73,128,0,1,2,4,8,16,24,40,224,8,120,32,166,166,8,16,32,64,128,0,1,2,25,4,198,156,222,142,105,37,131,151,24,9,16,32,64,128,0,1,2,59,5,70,
156,0,59,2,222,153,65,125,19,32,64,128,0,1,2,4,118,8,56,2,222,161,174,79,2,4,8,16,32,64,128,192,70,129,65,135,183,131,154,217,8,161,107,2,4,8,16,32,64,128,64,124,129,33,39,192,142,128,227,39,90,132,4,8,16,32,64,128,0,129,177,2,142,128,199,122,106,141,0,1,2,4,8,16,32,16,94,96,212,217,237,168,118,194,131,9,144,0,1,2,4,8,16,32,176,77,96,204,9,176,35,224,109,9,212,49,1,2,4,8,16,32,64,96,147,128,35,224,77,240,186,37,64,128,0,1,2,4,8,236,18,24,118,116,59,172,161,93,18,250,37,64,128,0,1,2,4,8,4,23,24,116,2,236,8,56,120,158,133,71,128,0,1,2,4,8,16,24,46,224,8,120,56,169,6,9,16,32,64,128,0,1,2,177,5,198,157,220,142,107,41,182,152,232,8,16,32,64,128,0,1,2,123,4,70,157,0,59,2,222,147,63,189,18,32,64,128,0,1,2,4,246,9,56,2,222,103,175,103,2,4,8,16,32,64,128,192,22,129,129,7,183,3,155,218,66,161,83,2,4,8,16,32,64,128,64,100,129,97,39,192,142,128,35,167,89,108,4,8,16,32,64,128,0,129,25,2,142,128,103,168,106,147,0,1,2,4,8,16,32,16,88,96,228,185,237,200,182,2,147,9,141,0,1,2,4,8,16,32,176,65,
96,220,9,176,35,224,13,233,211,37,1,2,4,8,16,32,64,96,171,128,35,224,173,252,58,39,64,128,0,1,2,4,8,172,23,24,122,108,59,180,177,245,22,122,36,64,128,0,1,2,4,8,132,21,24,120,2,236,8,56,108,150,5,70,128,0,1,2,4,8,16,152,36,224,8,120,18,172,102,9,16,32,64,128,0,1,2,81,5,198,158,218,142,109,45,170,153,184,8,16,32,64,128,0,1,2,171,5,70,158,0,59,2,94,157,61,253,17,32,64,128,0,1,2,4,118,11,56,2,222,157,1,253,19,32,64,128,0,1,2,4,22,11,12,62,180,29,220,220,98,12,221,17,32,64,128,0,1,2,4,98,10,12,61,1,118,4,28,51,201,162,34,64,128,0,1,2,4,8,204,19,112,4,60,207,86,203,4,8,16,32,64,128,0,129,144,2,163,207,108,71,183,23,18,77,80,4,8,16,32,64,128,0,129,165,2,99,79,128,29,1,47,77,158,206,8,16,32,64,128,0,1,2,1,4,28,1,7,72,130,16,8,16,32,64,128,0,1,2,43,5,134,31,217,14,111,112,165,134,190,8,16,32,64,128,0,1,2,1,5,6,159,0,59,2,14,152,99,33,17,32,64,128,0,1,2,4,166,10,56,2,158,202,171,113,2,4,8,16,32,64,128,64,60,129,241,39,182,227,91,140,167,38,34,2,4,8,16,32,64,128,192,58,129,209,39,192,142,
128,215,229,78,79,4,8,16,32,64,128,0,129,24,2,142,128,99,228,65,20,4,8,16,32,64,128,0,129,101,2,19,14,108,39,52,185,140,67,71,4,8,16,32,64,128,0,129,104,2,195,79,128,29,1,71,75,177,120,8,16,32,64,128,0,1,2,179,5,28,1,207,22,214,62,1,2,4,8,16,32,64,32,152,192,140,243,218,25,109,6,99,19,14,1,2,4,8,16,32,64,96,145,192,248,19,96,71,192,139,82,167,27,2,4,8,16,32,64,128,64,24,129,41,187,117,83,26,13,67,38,16,2,4,8,16,32,64,128,192,58,129,9,27,128,118,0,215,165,79,79,4,8,16,32,64,128,0,129,24,2,190,4,18,35,15,162,32,64,128,0,1,2,4,8,44,19,152,115,90,59,167,213,101,40,58,34,64,128,0,1,2,4,8,4,17,152,113,2,236,8,56,72,114,133,65,128,0,1,2,4,8,16,88,38,224,8,120,25,181,142,8,16,32,64,128,0,1,2,49,4,38,29,214,78,106,54,134,153,40,8,16,32,64,128,0,1,2,139,4,166,156,0,59,2,94,148,61,221,16,32,64,128,0,1,2,4,194,8,56,2,14,147,10,129,16,32,64,128,0,1,2,4,214,8,204,58,171,157,213,238,26,21,189,16,32,64,128,0,1,2,4,34,8,204,57,1,118,4,28,33,183,98,32,64,128,0,1,2,4,8,172,20,112,4,188,82,91,95,
4,8,16,32,64,128,0,129,0,2,211,142,106,167,53,28,0,77,8,4,8,16,32,64,128,0,129,21,2,147,78,128,29,1,175,72,158,62,8,16,32,64,128,0,1,2,145,4,230,109,212,205,107,57,146,159,88,8,16,32,64,128,0,1,2,179,4,102,109,0,218,1,156,149,49,237,18,32,64,128,0,1,2,4,162,10,248,18,72,212,204,136,139,0,1,2,4,8,16,32,48,73,96,226,65,237,196,166,39,97,104,150,0,1,2,4,8,16,32,16,71,96,218,9,176,35,224,56,73,22,9,1,2,4,8,16,32,64,96,141,128,35,224,53,206,122,33,64,128,0,1,2,4,8,132,17,152,121,78,59,179,237,48,128,2,33,64,128,0,1,2,4,8,76,17,152,119,2,236,8,120,74,194,52,74,128,0,1,2,4,8,16,8,44,48,117,151,110,106,227,129,81,133,70,128,0,1,2,4,8,16,184,43,48,113,3,208,14,224,221,228,184,159,0,1,2,4,8,16,32,144,77,192,151,64,178,101,76,188,4,8,16,32,64,128,0,129,155,2,115,79,105,231,182,126,115,232,110,39,64,128,0,1,2,4,8,132,21,152,121,2,236,8,56,108,218,5,70,128,0,1,2,4,8,16,152,36,224,8,120,18,172,102,9,16,32,64,128,0,1,2,81,5,38,31,210,78,110,62,170,170,184,8,16,32,64,128,0,1,2,183,4,166,158,
0,59,2,190,149,27,55,19,32,64,128,0,1,2,4,18,10,204,222,162,155,221,126,66,114,33,19,32,64,128,0,1,2,4,222,8,204,221,0,180,3,248,134,223,219,4,8,16,32,64,128,0,129,114,2,190,4,82,46,165,6,68,128,0,1,2,4,8,16,120,45,48,253,136,118,122,7,175,199,231,93,2,4,8,16,32,64,128,64,58,129,201,39,192,142,128,211,205,8,1,19,32,64,128,0,1,2,4,110,10,204,223,160,155,223,195,77,2,183,19,32,64,128,0,1,2,4,66,9,204,222,0,180,3,24,42,221,130,33,64,128,0,1,2,4,8,44,16,240,37,144,5,200,186,32,64,128,0,1,2,4,8,68,18,88,112,64,187,160,139,72,162,98,33,64,128,0,1,2,4,8,220,18,152,126,2,236,8,248,86,126,220,76,128,0,1,2,4,8,16,72,40,176,98,123,110,69,31,9,233,133,76,128,0,1,2,4,8,16,120,32,48,127,3,208,14,224,3,118,47,17,32,64,128,0,1,2,4,74,11,248,18,72,233,244,26,28,1,2,4,8,16,32,64,224,167,192,146,227,217,37,157,252,28,155,87,8,16,32,64,128,0,1,2,233,4,22,156,0,59,2,78,55,43,4,76,128,0,1,2,4,8,16,184,41,176,102,115,110,77,47,55,41,220,78,128,0,1,2,4,8,16,216,46,176,98,3,208,14,224,246,52,11,128,0,
1,2,4,8,16,32,176,88,192,151,64,22,131,235,142,0,1,2,4,8,16,32,176,91,96,209,225,236,162,110,118,107,234,159,0,1,2,4,8,16,32,112,75,96,201,9,176,35,224,91,57,114,51,1,2,4,8,16,32,64,32,161,192,170,173,185,85,253,36,76,129,144,9,16,32,64,128,0,1,2,255,9,172,217,0,180,3,104,194,17,32,64,128,0,1,2,4,186,9,248,18,72,183,140,27,47,1,2,4,8,16,32,208,94,96,217,209,236,178,142,218,167,20,0,1,2,4,8,16,32,144,85,96,209,9,176,35,224,172,19,68,220,4,8,16,32,64,128,0,129,171,2,235,54,230,214,245,116,213,194,125,4,8,16,32,64,128,0,129,157,2,171,54,0,237,0,238,204,178,190,9,16,32,64,128,0,1,2,59,4,124,9,100,135,186,62,9,16,32,64,128,0,1,2,27,5,22,30,204,46,236,106,35,168,174,9,16,32,64,128,0,1,2,215,4,150,157,0,59,2,190,150,32,119,17,32,64,128,0,1,2,4,242,10,172,220,150,91,217,87,222,140,136,156,0,1,2,4,8,16,232,41,176,110,3,208,14,96,207,25,102,212,4,8,16,32,64,128,64,103,1,95,2,233,156,125,99,39,64,128,0,1,2,4,90,10,44,61,150,93,218,89,203,116,26,52,1,2,4,8,16,32,144,85,96,225,9,176,35,224,
172,147,68,220,4,8,16,32,64,128,0,129,171,2,107,55,229,214,246,118,213,196,125,4,8,16,32,64,128,0,129,213,2,43,55,0,237,0,174,206,174,254,8,16,32,64,128,0,1,2,187,5,22,239,201,45,238,110,183,174,254,9,16,32,64,128,0,1,2,135,4,150,110,0,218,1,60,148,19,23,17,32,64,128,0,1,2,4,10,9,248,103,96,10,37,211,80,8,16,32,64,128,0,1,2,71,4,86,159,201,174,238,239,136,129,107,8,16,32,64,128,0,1,2,123,5,214,158,0,59,2,222,155,109,189,19,32,64,128,0,1,2,4,214,11,44,223,145,91,222,225,122,83,61,18,32,64,128,0,1,2,4,78,9,44,222,0,180,3,120,42,59,46,38,64,128,0,1,2,4,8,20,16,88,191,33,183,190,199,2,105,50,4,2,4,8,16,32,64,160,176,192,234,13,64,59,128,133,39,147,161,17,32,64,128,0,1,2,4,30,10,248,103,96,30,178,120,145,0,1,2,4,8,16,32,80,87,96,195,129,236,134,46,235,230,207,200,8,16,32,64,128,0,129,244,2,203,79,128,29,1,167,159,51,6,64,128,0,1,2,4,8,16,56,41,176,99,59,110,71,159,39,89,92,78,128,0,1,2,4,8,16,88,36,176,126,3,208,14,224,162,212,234,134,0,1,2,4,8,16,32,16,70,96,203,110,220,150,78,195,
144,11,132,0,1,2,4,8,16,32,240,71,96,195,6,160,29,192,63,252,126,34,64,128,0,1,2,4,8,244,16,240,207,192,244,200,179,81,18,32,64,128,0,1,2,4,62,4,246,156,198,238,233,245,99,208,126,32,64,128,0,1,2,4,8,4,17,216,113,2,236,8,56,72,242,133,65,128,0,1,2,4,8,16,88,38,176,105,47,110,83,183,203,88,117,68,128,0,1,2,4,8,16,56,34,176,101,3,208,14,224,145,212,184,134,0,1,2,4,8,16,32,80,73,96,215,86,220,174,126,43,229,206,88,8,16,32,64,128,0,129,236,2,123,54,0,237,0,102,159,55,226,39,64,128,0,1,2,4,8,156,21,216,182,19,183,173,227,179,66,174,39,64,128,0,1,2,4,8,76,18,216,180,1,104,7,112,82,62,53,75,128,0,1,2,4,8,16,8,43,224,31,130,14,155,26,129,17,32,64,128,0,1,2,4,230,8,236,59,137,221,215,243,28,73,173,18,32,64,128,0,1,2,4,206,9,236,58,1,118,4,124,46,79,174,38,64,128,0,1,2,4,8,228,23,216,184,15,183,177,235,252,121,51,2,2,4,8,16,32,64,32,189,192,182,13,64,59,128,233,231,142,1,16,32,64,128,0,1,2,4,78,10,236,220,134,219,217,247,73,38,151,19,32,64,128,0,1,2,4,6,11,236,219,0,180,3,56,56,149,154,
35,64,128,0,1,2,4,8,132,23,216,186,11,183,181,243,240,169,17,32,1,2,4,8,16,32,80,89,96,227,6,160,29,192,202,19,203,216,8,16,32,64,128,0,1,2,143,4,246,110,194,237,237,253,145,135,215,8,16,32,64,128,0,1,2,43,4,118,110,0,218,1,92,145,97,125,16,32,64,128,0,1,2,4,34,9,248,79,193,69,202,134,88,8,16,32,64,128,0,1,2,11,4,54,31,194,110,238,126,1,176,46,8,16,32,64,128,0,1,2,63,5,182,158,0,59,2,254,153,16,175,16,32,64,128,0,1,2,4,106,11,236,222,130,219,221,127,237,236,26,29,1,2,4,8,16,32,16,83,96,239,6,160,29,192,152,179,66,84,4,8,16,32,64,128,0,129,121,2,219,119,224,182,7,48,207,86,203,4,8,16,32,64,128,0,129,135,2,155,55,0,237,0,62,204,138,23,9,16,32,64,128,0,1,2,133,5,246,111,192,237,143,160,112,122,13,141,0,1,2,4,8,16,8,40,176,123,3,208,14,96,192,73,33,36,2,4,8,16,32,64,128,192,84,129,0,251,111,1,66,152,74,172,113,2,4,8,16,32,64,128,192,103,129,237,27,128,118,0,63,167,195,207,4,8,16,32,64,128,0,129,14,2,17,182,223,34,196,208,33,215,198,72,128,0,1,2,4,8,68,16,216,191,1,104,7,48,194,60,
16,3,1,2,4,8,16,32,64,96,165,192,255,86,118,166,47,2,4,8,16,32,64,128,0,129,253,2,33,142,95,67,4,177,63,23,34,32,64,128,0,1,2,4,26,8,4,56,1,118,4,220,96,158,25,34,1,2,4,8,16,32,64,224,139,64,140,205,183,24,81,124,129,241,11,1,2,4,8,16,32,64,96,130,64,132,13,64,59,128,19,18,171,73,2,4,8,16,32,64,128,64,104,129,32,123,111,65,194,8,157,42,193,17,32,64,128,0,1,2,249,5,66,108,0,218,1,204,63,145,140,128,0,1,2,4,8,16,32,112,78,32,202,214,91,148,56,206,233,185,154,0,1,2,4,8,16,32,112,70,32,198,6,160,29,192,51,57,115,45,1,2,4,8,16,32,64,160,130,64,152,157,183,48,129,84,200,170,49,16,32,64,128,0,1,2,33,5,130,108,0,218,1,12,57,59,4,69,128,0,1,2,4,8,16,152,40,16,103,227,45,78,36,19,185,53,77,128,0,1,2,4,8,52,22,136,178,1,104,7,176,241,36,52,116,2,4,8,16,32,64,160,169,64,160,125,183,64,161,52,157,12,134,77,128,0,1,2,4,8,204,20,8,179,1,104,7,112,102,154,181,77,128,0,1,2,4,8,16,136,40,16,105,219,45,82,44,17,115,37,38,2,4,8,16,32,64,32,179,64,156,13,64,59,128,153,231,145,216,9,16,32,64,128,0,
1,2,87,4,66,237,186,133,10,230,138,166,123,8,16,32,64,128,0,1,2,79,4,2,109,0,218,1,124,146,35,47,19,32,64,128,0,1,2,4,202,10,196,218,116,139,21,77,217,164,27,24,1,2,4,8,16,32,176,92,32,210,6,160,29,192,229,233,215,33,1,2,4,8,16,32,64,96,179,64,176,61,183,96,225,108,78,142,238,9,16,32,64,128,0,129,42,2,161,54,0,237,0,86,153,86,198,65,128,0,1,2,4,8,16,56,42,16,109,203,45,90,60,71,29,93,71,128,0,1,2,4,8,16,120,46,16,107,3,208,14,224,243,76,121,135,0,1,2,4,8,16,32,80,83,32,220,142,91,184,128,106,230,221,168,8,16,32,64,128,0,129,133,2,193,54,0,237,0,46,204,189,174,8,16,32,64,128,0,1,2,33,4,226,109,184,197,139,40,68,162,4,65,128,0,1,2,4,8,164,21,136,182,1,104,7,48,237,84,18,56,1,2,4,8,16,32,64,224,162,64,192,253,182,128,33,93,196,117,27,1,2,4,8,16,32,64,224,175,191,194,109,0,218,1,52,45,9,16,32,64,128,0,1,2,221,4,34,110,183,69,140,169,219,188,48,94,2,4,8,16,32,64,96,148,64,188,13,64,59,128,163,114,171,29,2,4,8,16,32,64,128,64,22,129,144,187,109,33,131,202,146,81,113,18,32,64,128,0,
1,2,161,4,2,110,0,218,1,12,53,67,4,67,128,0,1,2,4,8,16,88,32,16,115,179,45,102,84,11,210,161,11,2,4,8,16,32,64,160,152,64,196,13,64,59,128,197,38,153,225,16,32,64,128,0,1,2,4,222,10,4,221,107,11,26,214,91,78,23,16,32,64,128,0,1,2,4,62,11,132,220,0,180,3,248,57,69,126,38,64,128,0,1,2,4,8,116,16,136,186,213,22,53,174,14,115,194,24,9,16,32,64,128,0,129,81,2,49,55,0,237,0,142,202,175,118,8,16,32,64,128,0,1,2,89,4,194,238,180,133,13,44,75,102,197,73,128,0,1,2,4,8,108,23,8,186,1,104,7,112,251,204,16,0,1,2,4,8,16,32,64,96,177,64,220,141,182,184,145,45,78,145,238,8,16,32,64,128,0,129,164,2,81,55,0,237,0,38,157,80,194,38,64,128,0,1,2,4,8,92,22,8,188,207,22,56,180,203,220,110,36,64,128,0,1,2,4,250,8,132,221,0,180,3,216,103,18,26,41,1,2,4,8,16,32,64,224,151,64,228,109,182,200,177,153,63,4,8,16,32,64,128,0,129,215,2,113,55,0,237,0,190,206,156,119,9,16,32,64,128,0,1,2,245,4,66,239,178,133,14,174,222,92,48,34,2,4,8,16,32,64,96,160,64,224,13,192,191,98,215,88,177,163,27,56,69,52,69,128,0,1,
2,4,8,20,19,136,92,255,57,2,46,54,217,12,135,0,1,2,4,8,16,32,240,86,32,248,30,91,240,240,222,242,186,128,0,1,2,4,8,16,232,41,16,122,3,208,14,96,207,73,105,212,4,8,16,32,64,128,64,103,129,232,91,108,209,227,235,60,119,140,157,0,1,2,4,8,16,120,38,16,123,3,208,14,224,179,188,121,157,0,1,2,4,8,16,32,80,85,32,252,14,91,248,0,171,206,12,227,34,64,128,0,1,2,4,46,11,4,223,0,180,3,120,57,179,110,36,64,128,0,1,2,4,8,36,21,136,191,193,22,63,194,164,169,23,54,1,2,4,8,16,32,48,73,32,250,6,160,29,192,73,137,215,44,1,2,4,8,16,32,64,32,172,64,130,253,181,4,33,134,77,175,192,8,16,32,64,128,0,129,245,2,225,55,0,237,0,174,159,20,122,36,64,128,0,1,2,4,8,236,21,200,176,189,150,33,198,189,89,212,59,1,2,4,8,16,32,16,71,32,254,6,224,95,41,138,171,20,65,198,153,118,34,33,64,128,0,1,2,4,54,10,36,168,255,28,1,111,156,31,186,38,64,128,0,1,2,4,8,108,17,200,177,185,150,35,202,45,9,212,41,1,2,4,8,16,32,16,74,32,195,6,160,29,192,80,83,70,48,4,8,16,32,64,128,0,129,5,2,73,246,214,146,132,185,32,97,186,32,64,128,
0,1,2,4,34,11,164,216,0,180,3,24,121,10,137,141,0,1,2,4,8,16,32,48,67,32,203,214,90,150,56,103,228,72,155,4,8,16,32,64,128,64,22,129,28,27,128,118,0,179,204,39,113,18,32,64,128,0,1,2,4,70,9,164,217,89,75,19,232,168,204,104,135,0,1,2,4,8,16,72,39,144,100,3,208,14,96,186,153,37,96,2,4,8,16,32,64,128,192,77,129,60,27,107,121,34,189,153,18,183,19,32,64,128,0,1,2,73,5,178,108,0,230,248,79,193,253,154,4,42,192,164,15,131,176,9,16,32,64,128,64,19,129,52,245,159,35,224,38,51,210,48,9,16,32,64,128,0,1,2,31,2,153,182,213,50,197,250,1,236,7,2,4,8,16,32,64,160,137,64,158,13,64,59,128,77,166,164,97,18,32,64,128,0,1,2,4,62,4,82,237,170,165,10,246,131,216,15,4,8,16,32,64,128,64,7,129,68,27,128,118,0,59,76,72,99,36,64,128,0,1,2,4,8,124,22,200,181,169,150,43,218,207,206,126,38,64,128,0,1,2,4,106,11,100,218,0,204,244,207,192,252,51,107,84,128,181,159,29,163,35,64,128,0,1,2,89,5,82,213,127,142,128,179,78,51,113,19,32,64,128,0,1,2,4,174,10,100,219,82,203,22,239,213,188,184,143,0,1,2,4,8,16,200,36,
144,107,3,208,14,96,166,185,37,86,2,4,8,16,32,64,128,192,8,129,116,59,106,233,2,30,145,37,109,16,32,64,128,0,1,2,161,5,146,109,0,218,1,12,61,155,4,71,128,0,1,2,4,8,16,152,32,144,111,67,45,95,196,19,210,166,73,2,4,8,16,32,64,32,144,64,182,13,64,59,128,129,38,143,80,8,16,32,64,128,0,1,2,75,4,18,238,167,37,12,121,73,42,117,66,128,0,1,2,4,8,236,17,72,183,1,152,242,95,86,86,1,238,153,221,122,37,64,128,0,1,2,4,30,9,228,171,255,28,1,63,202,163,215,8,16,32,64,128,0,1,2,149,5,82,238,166,165,12,186,242,44,50,54,2,4,8,16,32,208,88,32,225,6,160,29,192,198,243,213,208,9,16,32,64,128,0,129,166,2,57,55,211,114,70,221,116,138,25,54,1,2,4,8,16,40,45,144,113,3,48,229,151,64,254,158,69,42,192,210,143,146,193,17,32,64,128,0,129,52,2,41,235,63,71,192,105,230,151,64,9,16,32,64,128,0,1,2,131,4,178,110,165,101,141,123,80,218,52,67,128,0,1,2,4,8,132,16,200,185,1,104,7,48,196,228,17,4,1,2,4,8,16,32,64,96,161,64,218,157,180,180,129,47,76,174,174,8,16,32,64,128,0,129,185,2,73,55,0,237,0,206,157,22,90,39,
64,128,0,1,2,4,8,196,19,200,187,145,150,55,242,120,179,64,68,4,8,16,32,64,128,192,21,129,172,27,128,153,255,61,21,21,224,149,153,234,30,2,4,8,16,32,64,96,148,64,218,250,207,17,240,168,41,160,29,2,4,8,16,32,64,128,64,22,129,204,219,104,153,99,207,50,63,196,73,128,0,1,2,4,8,60,19,200,187,1,104,7,240,89,78,189,78,128,0,1,2,4,8,16,168,42,144,122,23,45,117,240,85,103,148,113,17,32,64,128,0,129,38,2,137,55,0,51,127,9,228,239,217,165,2,108,242,136,25,38,1,2,4,8,16,8,39,144,185,254,115,4,28,110,58,9,136,0,1,2,4,8,16,32,48,89,32,249,30,90,242,240,39,39,87,243,4,8,16,32,64,128,192,44,129,212,27,128,118,0,103,77,11,237,18,32,64,128,0,1,2,4,162,10,100,223,66,203,30,127,212,121,33,46,2,4,8,16,32,64,224,149,64,238,13,192,252,95,163,80,1,190,154,157,222,35,64,128,0,1,2,4,102,8,36,175,255,28,1,207,152,20,218,36,64,128,0,1,2,4,8,68,22,200,191,129,150,127,4,145,231,135,216,8,16,32,64,128,0,129,159,2,217,55,0,237,0,254,204,169,87,8,16,32,64,128,0,1,2,181,5,10,236,159,21,24,66,237,57,102,116,4,8,
16,32,64,160,152,64,250,13,192,252,95,2,249,123,70,169,0,139,61,86,134,67,128,0,1,2,4,66,11,228,175,255,28,1,135,158,96,130,35,64,128,0,1,2,4,8,76,16,40,177,123,86,98,16,19,146,171,73,2,4,8,16,32,64,96,188,64,129,13,64,59,128,227,167,133,22,9,16,32,64,128,0,1,2,177,5,106,108,158,213,24,69,236,153,34,58,2,4,8,16,32,64,224,31,129,10,27,128,85,190,64,161,2,244,76,18,32,64,128,0,1,2,43,4,74,212,127,142,128,87,76,21,125,16,32,64,128,0,1,2,4,34,9,84,217,58,171,50,142,72,115,67,44,4,8,16,32,64,128,192,119,129,26,27,128,85,142,128,253,99,128,223,231,167,223,9,16,32,64,128,0,129,241,2,69,234,63,71,192,227,167,134,22,9,16,32,64,128,0,1,2,177,5,234,28,157,214,25,73,236,25,35,58,2,4,8,16,32,208,87,160,202,6,160,29,192,190,115,216,200,9,16,32,64,128,0,129,174,2,133,246,205,10,13,165,235,108,52,110,2,4,8,16,32,16,90,160,204,6,96,157,47,129,252,61,95,84,128,161,31,26,193,17,32,64,128,0,129,228,2,117,234,63,71,192,201,167,162,240,9,16,32,64,128,0,1,2,167,5,74,237,154,149,26,204,233,84,186,129,
0,1,2,4,8,16,152,41,80,104,3,176,216,177,169,10,112,230,188,215,54,1,2,4,8,16,232,44,80,169,254,115,4,220,121,38,27,59,1,2,4,8,16,32,208,83,160,216,158,89,177,225,244,156,147,70,77,128,0,1,2,4,2,10,148,218,0,180,3,24,112,134,9,137,0,1,2,4,8,16,32,48,85,160,218,150,89,181,241,76,77,190,198,9,16,32,64,128,0,129,131,2,181,54,0,139,125,9,228,239,28,170,0,15,78,100,151,17,32,64,128,0,1,2,135,5,138,213,127,142,128,15,103,222,133,4,8,16,32,64,128,0,129,34,2,245,54,204,234,141,168,200,84,51,12,2,4,8,16,32,144,86,160,218,6,96,197,19,83,21,96,218,199,75,224,4,8,16,32,64,32,164,64,185,250,207,17,112,200,121,38,40,2,4,8,16,32,64,128,192,68,129,138,219,101,21,199,52,113,10,104,154,0,1,2,4,8,16,120,41,80,111,3,176,226,17,176,111,2,191,156,196,222,36,64,128,0,1,2,4,78,9,20,172,255,28,1,159,154,1,46,38,64,128,0,1,2,4,8,20,16,168,121,92,90,115,84,5,166,155,33,16,32,64,128,0,129,116,2,21,55,0,237,0,166,155,134,2,38,64,128,0,1,2,4,8,220,20,40,186,87,86,116,88,55,147,237,118,2,4,8,16,32,64,224,172,
64,201,13,192,154,95,2,249,59,181,42,192,179,243,219,245,4,8,16,32,64,128,192,79,129,154,245,159,35,224,159,153,246,10,1,2,4,8,16,32,64,160,182,64,217,157,178,178,3,171,61,31,141,142,0,1,2,4,8,132,18,40,186,1,88,248,168,84,5,24,234,1,18,12,1,2,4,8,16,72,40,80,181,254,115,4,156,112,50,10,153,0,1,2,4,8,16,32,112,75,160,240,62,89,225,161,221,74,185,155,9,16,32,64,128,0,129,99,2,101,55,0,11,31,1,251,38,240,177,185,237,42,2,4,8,16,32,64,224,177,64,221,250,207,17,240,227,140,123,149,0,1,2,4,8,16,32,80,87,160,244,57,105,233,193,213,157,147,70,70,128,0,1,2,4,66,8,20,222,0,44,125,4,236,16,56,196,227,35,8,2,4,8,16,32,144,82,160,114,253,231,8,56,229,148,20,52,1,2,4,8,16,32,64,224,134,64,241,83,210,226,195,187,145,120,183,18,32,64,128,0,1,2,175,4,74,111,0,22,63,2,118,8,252,106,102,123,143,0,1,2,4,8,16,120,38,80,187,254,115,4,252,44,239,94,39,64,128,0,1,2,4,8,84,21,40,127,70,90,126,128,85,103,166,113,17,32,64,128,0,129,141,2,197,55,0,203,31,1,59,4,222,248,240,232,154,0,1,2,4,8,36,21,168,94,
255,57,2,78,58,49,133,77,128,0,1,2,4,8,16,184,44,208,224,132,180,193,16,47,167,223,141,4,8,16,32,64,128,192,79,129,242,27,128,13,142,128,29,2,255,156,215,94,33,64,128,0,1,2,4,158,11,212,175,255,28,1,63,207,190,119,8,16,32,64,128,0,1,2,53,5,90,156,143,182,24,100,205,249,105,84,4,8,16,32,64,96,185,64,131,13,192,22,71,192,14,129,151,63,58,58,36,64,128,0,1,2,105,5,58,212,127,142,128,211,78,79,129,19,32,64,128,0,1,2,4,46,10,52,57,29,109,50,204,139,147,192,109,4,8,16,32,64,128,192,111,129,22,27,128,77,142,128,29,2,255,158,213,254,159,0,1,2,4,8,16,120,37,208,163,254,115,4,252,106,14,120,143,0,1,2,4,8,16,32,80,81,160,205,217,104,155,129,86,156,165,198,68,128,0,1,2,4,22,9,52,217,0,108,115,4,236,16,120,209,131,163,27,2,4,8,16,32,144,88,160,75,253,215,168,0,236,52,212,196,79,158,208,9,16,32,64,128,192,62,129,54,245,159,207,0,238,155,100,122,38,64,128,0,1,2,4,8,236,17,232,244,209,184,78,99,221,51,155,244,74,128,0,1,2,4,50,11,244,217,0,236,117,46,170,2,204,252,84,138,157,0,1,2,4,8,204,21,
104,84,255,57,2,158,59,149,180,78,128,0,1,2,4,8,16,136,39,208,107,83,172,215,104,227,205,54,17,17,32,64,128,0,129,184,2,157,54,0,123,29,1,251,183,96,226,62,117,34,35,64,128,0,1,2,123,5,90,213,127,142,128,247,78,54,189,19,32,64,128,0,1,2,4,214,11,116,59,20,237,54,222,245,51,74,143,4,8,16,32,64,32,163,64,175,13,192,110,71,192,14,129,51,62,147,98,38,64,128,0,1,2,179,5,154,213,127,253,10,192,134,35,158,253,204,104,159,0,1,2,4,8,100,23,232,86,255,249,12,96,246,25,43,126,2,4,8,16,32,64,128,192,89,129,134,159,137,107,56,228,179,179,194,245,4,8,16,32,64,160,149,64,187,13,192,150,7,162,42,192,86,15,181,193,18,32,64,128,0,129,55,2,253,234,63,71,192,111,166,132,183,9,16,32,64,128,0,1,2,229,4,90,238,134,181,28,116,185,169,107,64,4,8,16,32,64,96,140,64,195,13,192,150,71,192,254,45,152,49,207,139,86,8,16,32,64,128,64,5,129,142,245,159,35,224,10,51,215,24,8,16,32,64,128,0,1,2,103,4,154,158,134,54,29,246,153,153,225,90,2,4,8,16,32,208,66,160,229,6,96,211,35,96,135,192,45,30,105,131,36,64,128,
0,1,2,111,5,122,214,127,109,11,192,190,3,127,251,36,184,128,0,1,2,4,8,244,17,104,90,255,249,12,96,159,41,110,164,4,8,16,32,64,128,0,129,95,2,125,63,12,215,119,228,230,62,1,2,4,8,16,32,240,75,160,235,6,96,231,147,80,21,160,167,159,0,1,2,4,8,244,22,104,91,255,57,2,238,61,241,141,158,0,1,2,4,8,16,232,40,208,121,27,172,243,216,59,206,117,99,38,64,128,0,1,2,95,5,250,110,0,118,62,2,246,111,193,124,125,10,252,70,128,0,1,2,4,122,9,52,174,255,122,23,128,205,71,223,235,41,55,90,2,4,8,16,32,240,85,160,115,253,231,51,128,95,231,130,223,8,16,32,64,128,0,1,2,245,5,154,127,14,174,249,240,235,207,111,35,36,64,128,0,1,2,79,4,90,111,0,182,63,4,85,1,62,121,44,188,76,128,0,1,2,4,74,11,244,174,255,218,23,128,0,74,63,221,6,71,128,0,1,2,4,30,11,52,175,255,124,6,240,241,180,240,42,1,2,4,8,16,32,64,160,174,128,35,80,2,117,103,183,145,17,32,64,128,0,129,199,2,221,55,0,157,128,250,215,0,31,63,25,94,37,64,128,0,1,2,117,5,218,215,127,10,192,191,39,183,61,192,186,79,184,145,17,32,64,128,0,129,159,2,234,63,
159,1,252,57,43,188,66,128,0,1,2,4,8,16,168,45,96,247,235,239,252,66,168,61,201,141,142,0,1,2,4,8,124,22,176,1,168,246,249,53,31,84,128,159,159,11,63,19,32,64,128,0,129,202,2,234,191,191,179,251,191,202,25,54,54,2,4,8,16,32,64,128,0,129,159,2,246,190,254,53,193,240,115,106,120,133,0,1,2,4,8,84,20,176,1,248,79,86,85,62,191,230,54,135,138,207,184,49,17,32,64,128,0,129,239,2,234,191,127,69,20,62,255,77,12,16,223,159,16,191,19,32,64,128,0,129,122,2,234,191,95,57,245,25,192,122,115,219,136,8,16,32,64,128,0,1,2,47,5,108,124,253,230,33,241,91,194,255,19,32,64,128,0,129,170,2,54,0,255,203,172,178,231,99,138,163,248,160,240,3,1,2,4,8,16,40,41,160,254,251,157,86,85,207,111,9,95,136,249,35,225,39,2,4,8,16,32,80,81,64,253,247,145,85,159,1,252,160,240,3,1,2,4,8,16,32,64,160,135,128,29,192,79,121,134,241,9,195,143,4,8,16,32,64,160,152,128,13,192,63,9,85,243,252,177,112,8,252,217,194,207,4,8,16,32,64,160,150,128,250,239,83,62,21,128,159,48,84,128,95,48,252,66,128,0,1,2,4,10,9,168,255,62,39,
83,1,248,89,67,5,248,85,195,111,4,8,16,32,64,160,138,128,250,239,75,38,125,9,228,11,135,95,8,16,32,64,128,0,1,2,245,5,236,0,126,203,49,144,111,32,126,37,64,128,0,1,2,5,4,108,0,126,77,162,122,231,171,135,67,224,239,30,126,39,64,128,0,1,2,249,5,212,127,223,114,168,0,252,6,162,2,252,1,226,5,2,4,8,16,32,144,92,64,253,247,61,129,62,3,248,93,196,239,4,8,16,32,64,128,0,129,226,2,118,0,127,38,152,201,79,19,175,16,32,64,128,0,129,188,2,54,0,127,228,78,177,243,131,196,33,240,3,18,47,17,32,64,128,0,129,180,2,234,191,159,169,83,0,254,52,81,1,62,50,241,26,1,2,4,8,16,200,41,160,254,123,144,55,159,1,124,128,226,37,2,4,8,16,32,64,128,64,101,1,59,128,15,179,139,229,33,139,23,9,16,32,64,128,64,58,1,27,128,143,82,166,210,121,164,226,16,248,177,138,87,9,16,32,64,128,64,54,1,245,223,195,140,41,0,31,178,168,0,159,176,120,153,0,1,2,4,8,164,18,80,255,61,78,151,2,240,177,139,10,240,153,139,215,9,16,32,64,128,64,30,1,245,223,147,92,249,18,200,19,24,47,19,32,64,128,0,1,2,4,170,10,216,1,124,154,89,52,79,
105,188,65,128,0,1,2,4,82,8,216,0,124,150,38,85,206,51,25,135,192,207,101,188,67,128,0,1,2,4,50,8,168,255,158,102,73,1,248,148,70,5,248,130,198,91,4,8,16,32,64,32,188,128,250,239,121,138,20,128,207,109,84,128,175,108,188,71,128,0,1,2,4,98,11,168,255,94,228,199,151,64,94,224,120,139,0,1,2,4,8,16,32,80,81,192,14,224,203,172,226,121,201,227,77,2,4,8,16,32,16,86,192,6,224,171,212,168,112,94,233,56,4,126,173,227,93,2,4,8,16,32,16,85,64,253,247,50,51,10,192,151,60,42,192,55,60,222,38,64,128,0,1,2,33,5,212,127,175,211,162,0,124,237,163,2,124,231,227,125,2,4,8,16,32,16,79,64,253,247,38,39,190,4,242,6,200,219,4,8,16,32,64,128,0,129,106,2,118,0,223,102,20,209,91,34,23,16,32,64,128,0,129,80,2,54,0,223,165,67,117,243,78,200,33,240,123,33,87,16,32,64,128,0,129,72,2,234,191,183,217,80,0,190,37,82,1,30,32,114,9,1,2,4,8,16,8,35,160,254,123,159,10,5,224,123,35,21,224,17,35,215,16,32,64,128,0,129,24,2,234,191,3,121,240,37,144,3,72,46,33,64,128,0,1,2,4,8,84,18,176,3,120,40,155,152,14,49,185,136,
0,1,2,4,8,108,23,176,1,120,36,5,42,155,35,74,14,129,143,41,185,138,0,1,2,4,8,236,22,80,255,29,202,128,2,240,16,147,10,240,32,147,203,8,16,32,64,128,192,86,1,245,223,49,126,5,224,49,39,21,224,81,39,215,17,32,64,128,0,129,125,2,234,191,131,246,190,4,114,16,202,101,4,8,16,32,64,128,0,129,42,2,118,0,15,103,18,213,97,42,23,18,32,64,128,0,129,45,2,54,0,143,178,171,106,142,74,57,4,62,46,229,74,2,4,8,16,32,176,67,64,253,119,88,93,1,120,152,74,5,120,130,202,165,4,8,16,32,64,96,185,128,250,239,56,185,2,240,184,149,10,240,140,149,107,9,16,32,64,128,192,90,1,245,223,9,111,5,224,9,44,21,224,41,44,23,19,32,64,128,0,129,133,2,234,191,51,216,190,5,124,70,203,181,4,8,16,32,64,128,0,129,2,2,118,0,207,37,145,215,57,47,87,19,32,64,128,0,129,53,2,54,0,79,57,43,104,78,113,57,4,62,201,229,114,2,4,8,16,32,176,68,64,253,119,142,89,1,120,206,75,5,120,214,203,245,4,8,16,32,64,96,190,128,250,239,164,177,2,240,36,152,10,240,52,152,27,8,16,32,64,128,192,100,1,245,223,89,96,5,224,89,49,21,224,121,49,119,16,
32,64,128,0,129,153,2,234,191,211,186,190,5,124,154,204,13,4,8,16,32,64,128,0,129,220,2,118,0,47,228,15,218,5,52,183,16,32,64,128,0,129,73,2,54,0,207,195,170,101,206,155,57,4,190,98,230,30,2,4,8,16,32,48,71,64,253,119,193,85,1,120,1,77,5,120,9,205,77,4,8,16,32,64,96,130,128,250,239,10,170,2,240,138,154,10,240,154,154,187,8,16,32,64,128,192,104,1,245,223,37,81,5,224,37,54,21,224,69,54,183,17,32,64,128,0,129,161,2,234,191,107,156,190,5,124,205,205,93,4,8,16,32,64,128,0,129,180,2,118,0,175,166,142,220,85,57,247,17,32,64,128,0,129,81,2,54,0,47,74,42,99,46,194,57,4,190,12,231,70,2,4,8,16,32,48,72,64,253,119,21,82,1,120,85,78,5,120,93,206,157,4,8,16,32,64,96,132,128,250,239,178,162,2,240,50,157,10,240,6,157,91,9,16,32,64,128,192,109,1,245,223,117,66,5,224,117,59,21,224,29,59,247,18,32,64,128,0,129,123,2,234,191,27,126,10,192,27,120,42,192,91,120,110,38,64,128,0,1,2,55,4,212,127,55,240,254,242,207,192,220,209,115,47,1,2,4,8,16,32,64,32,161,128,29,192,123,73,227,119,207,207,221,4,8,16,
32,64,224,154,128,13,192,107,110,255,221,165,128,185,197,231,16,248,38,159,219,9,16,32,64,128,192,37,1,245,223,37,182,143,155,20,128,31,20,23,127,32,120,17,206,109,4,8,16,32,64,224,178,128,250,239,50,221,175,27,149,47,55,1,237,1,222,6,212,0,1,2,4,8,16,56,41,160,254,59,9,246,227,114,5,224,15,146,211,47,48,60,77,230,6,2,4,8,16,32,112,67,64,253,119,3,239,215,173,138,151,219,132,246,0,7,16,106,130,0,1,2,4,8,28,22,80,255,29,166,122,122,161,2,240,41,205,137,55,40,158,192,114,41,1,2,4,8,16,184,37,160,254,187,197,247,235,102,255,14,224,0,68,77,16,32,64,128,0,1,2,4,50,9,216,187,26,146,45,140,67,24,53,66,128,0,1,2,4,222,10,216,0,124,75,116,224,2,149,203,1,164,3,151,112,60,128,228,18,2,4,8,16,32,112,91,64,253,119,155,240,159,6,20,46,67,24,65,14,98,212,12,1,2,4,8,16,120,41,160,254,123,201,115,248,77,5,224,97,170,55,23,146,124,3,228,109,2,4,8,16,32,112,91,64,253,119,155,240,87,3,202,150,65,144,246,0,135,65,106,136,0,1,2,4,8,60,17,80,255,61,129,57,253,178,2,240,52,217,211,27,88,62,165,241,
6,1,2,4,8,16,24,32,160,254,27,128,248,171,9,69,203,48,74,123,128,3,41,53,69,128,0,1,2,4,126,8,168,255,126,144,92,126,193,191,3,120,153,206,141,4,8,16,32,64,128,0,129,156,2,118,0,71,230,141,230,72,77,109,17,32,64,128,0,129,207,2,54,0,63,107,220,252,89,201,114,19,240,235,237,56,191,122,248,141,0,1,2,4,8,140,18,80,255,141,146,252,167,29,21,203,72,77,158,99,53,181,70,128,0,1,2,4,126,11,168,255,126,75,12,249,127,5,224,16,198,63,141,0,253,99,225,39,2,4,8,16,32,48,74,64,253,55,74,242,87,59,234,149,177,158,246,0,71,123,106,143,0,1,2,4,8,252,245,151,250,111,240,44,80,0,14,6,85,1,14,7,213,32,1,2,4,8,180,23,80,255,141,158,2,10,192,209,162,42,192,241,162,90,36,64,128,0,129,222,2,234,191,225,249,87,0,14,39,85,1,78,32,213,36,1,2,4,8,52,22,80,255,141,79,190,2,112,188,169,10,112,134,169,54,9,16,32,64,160,171,128,250,111,66,230,253,151,64,38,160,106,146,0,1,2,4,8,16,32,16,89,192,14,224,148,236,96,157,194,170,81,2,4,8,16,104,40,96,3,112,70,210,85,42,51,84,29,2,207,81,213,42,1,2,4,8,244,19,80,255,
77,201,185,2,112,10,171,10,112,18,171,102,9,16,32,64,160,153,128,250,111,78,194,21,128,115,92,85,128,179,92,181,75,128,0,1,2,157,4,212,127,147,178,173,0,156,4,171,2,156,6,171,97,2,4,8,16,104,35,160,254,155,149,106,5,224,44,89,21,224,60,89,45,19,32,64,128,64,15,1,245,223,180,60,43,0,167,209,170,0,39,210,106,154,0,1,2,4,26,8,168,255,230,37,89,1,56,207,86,5,56,211,86,219,4,8,16,32,80,93,64,253,55,49,195,10,192,137,184,42,192,169,184,26,39,64,128,0,129,210,2,234,191,153,233,85,0,206,212,85,1,206,213,213,58,1,2,4,8,212,21,80,255,77,205,173,2,112,42,175,10,112,50,175,230,9,16,32,64,160,168,128,250,111,110,98,21,128,115,125,85,128,179,125,181,79,128,0,1,2,21,5,212,127,147,179,170,0,156,12,172,2,156,14,172,3,2,4,8,16,40,39,160,254,155,157,210,255,205,238,64,251,4,8,16,32,64,128,0,1,2,177,4,236,0,206,207,7,227,249,198,122,32,64,128,0,129,74,2,54,0,167,103,83,113,50,157,216,33,240,2,98,93,16,32,64,128,64,33,1,245,223,252,100,42,0,231,27,171,0,87,24,235,131,0,1,2,4,170,8,168,255,22,100,
82,1,184,0,89,5,184,4,89,39,4,8,16,32,80,66,64,253,183,34,141,10,192,21,202,42,192,53,202,122,33,64,128,0,129,252,2,234,191,37,57,84,0,46,97,86,1,46,98,214,13,1,2,4,8,36,23,80,255,173,73,160,2,112,141,179,10,112,149,179,126,8,16,32,64,32,179,128,250,111,81,246,20,128,139,160,85,128,203,160,117,68,128,0,1,2,105,5,212,127,171,82,167,0,92,37,173,2,92,39,173,39,2,4,8,16,200,41,160,254,91,150,55,5,224,50,106,21,224,66,106,93,17,32,64,128,64,66,1,245,223,186,164,41,0,215,89,171,0,87,90,235,139,0,1,2,4,178,9,168,255,22,102,76,1,184,16,91,5,184,20,91,103,4,8,16,32,144,74,64,253,183,50,93,10,192,149,218,42,192,181,218,122,35,64,128,0,129,60,2,234,191,165,185,82,0,46,229,86,1,46,230,214,29,1,2,4,8,36,17,80,255,173,77,148,2,112,173,183,10,112,181,183,254,8,16,32,64,32,131,128,250,111,113,150,20,128,139,193,85,128,203,193,117,72,128,0,1,2,225,5,212,127,171,83,164,0,92,45,174,2,92,47,174,71,2,4,8,16,136,45,160,254,91,158,31,5,224,114,114,21,224,6,114,93,18,32,64,128,64,96,1,245,223,250,228,
40,0,215,155,171,0,119,152,235,147,0,1,2,4,162,10,168,255,54,100,70,1,184,1,93,5,184,5,93,167,4,8,16,32,16,82,64,253,183,35,45,10,192,29,234,42,192,61,234,122,37,64,128,0,129,120,2,234,191,45,57,81,0,110,97,87,1,110,98,215,45,1,2,4,8,4,19,80,255,237,73,136,2,112,143,187,10,112,151,187,126,9,16,32,64,32,146,128,250,111,83,54,20,128,155,224,85,128,219,224,117,76,128,0,1,2,97,4,212,127,187,82,161,0,220,37,175,2,220,39,175,103,2,4,8,16,136,33,160,254,219,150,7,5,224,54,122,21,224,70,122,93,19,32,64,128,64,0,1,245,223,190,36,40,0,247,217,171,0,119,218,235,155,0,1,2,4,118,11,168,255,54,102,64,1,184,17,95,5,184,21,95,231,4,8,16,32,176,85,64,253,183,147,95,1,184,83,95,5,184,87,95,239,4,8,16,32,176,79,64,253,183,207,254,239,158,21,128,91,249,37,96,51,191,238,9,16,32,64,96,147,128,250,111,19,252,127,221,42,0,247,250,171,0,119,251,235,159,0,1,2,4,118,8,168,255,118,168,127,234,83,1,248,9,99,207,143,82,176,199,93,175,4,8,16,32,176,79,64,253,183,207,254,87,207,170,143,221,25,176,7,184,63,3,
34,32,64,128,0,129,181,2,234,191,181,222,15,122,83,0,62,64,89,253,146,36,172,22,215,31,1,2,4,8,236,20,80,255,237,212,255,213,183,218,99,127,14,236,1,70,200,129,24,8,16,32,64,96,149,128,250,111,149,244,139,126,20,128,47,112,214,189,37,13,235,172,245,68,128,0,1,2,123,5,212,127,123,253,127,245,174,242,136,144,5,123,128,49,178,32,10,2,4,8,16,152,47,160,254,155,111,124,160,7,5,224,1,164,21,151,72,196,10,101,125,16,32,64,128,192,110,1,245,223,238,12,252,234,95,221,17,35,15,246,0,163,228,65,28,4,8,16,32,48,83,64,253,55,83,247,68,219,10,192,19,88,115,47,149,138,185,190,90,39,64,128,0,129,253,2,234,191,253,57,248,21,129,170,35,74,38,236,1,198,201,132,72,8,16,32,64,96,142,128,250,111,142,235,133,86,21,128,23,208,102,221,34,25,179,100,181,75,128,0,1,2,17,4,212,127,17,178,240,43,6,53,71,156,92,216,3,140,148,11,177,16,32,64,128,192,104,1,245,223,104,209,27,237,41,0,111,224,141,191,85,58,198,155,106,145,0,1,2,4,98,8,168,255,98,228,225,87,20,42,142,72,217,176,7,24,43,27,162,33,64,128,0,129,
113,2,234,191,113,150,3,90,82,0,14,64,28,217,132,132,140,212,212,22,1,2,4,8,68,17,80,255,69,201,196,175,56,212,27,177,242,97,15,48,90,62,196,67,128,0,1,2,35,4,212,127,35,20,7,182,161,0,28,136,57,166,41,41,25,227,168,21,2,4,8,16,136,35,160,254,139,147,139,95,145,168,54,162,101,196,30,96,188,140,136,136,0,1,2,4,238,9,168,255,238,249,77,184,91,1,56,1,245,110,147,146,114,87,208,253,4,8,16,32,16,73,64,253,23,41,27,191,98,81,107,196,203,137,61,192,136,57,17,19,1,2,4,8,92,21,80,255,93,149,155,120,159,2,112,34,238,245,166,165,229,186,157,59,9,16,32,64,32,150,128,250,47,86,62,126,69,163,210,136,152,21,123,128,49,179,34,42,2,4,8,16,56,47,160,254,59,111,182,224,14,5,224,2,228,43,93,72,204,21,53,247,16,32,64,128,64,52,1,245,95,180,140,252,138,71,157,17,51,47,246,0,163,230,69,92,4,8,16,32,112,70,64,253,119,70,107,225,181,10,192,133,216,39,187,146,155,147,96,46,39,64,128,0,129,96,2,202,191,96,9,249,19,142,34,227,143,69,184,159,36,39,92,74,4,68,128,0,1,2,39,4,212,127,39,176,22,95,170,198,88,
12,126,170,59,217,57,197,229,98,2,4,8,16,8,37,160,254,11,149,142,175,193,40,49,190,122,4,251,77,122,130,37,68,56,4,8,16,32,112,88,64,253,119,152,106,195,133,42,140,13,232,39,186,148,159,19,88,46,37,64,128,0,129,64,2,234,191,64,201,248,25,138,2,227,167,73,168,87,36,40,84,58,4,67,128,0,1,2,7,5,212,127,7,161,54,93,166,190,216,4,127,184,91,25,58,76,229,66,2,4,8,16,8,35,160,254,11,147,138,199,129,40,47,30,187,4,122,85,138,2,37,67,40,4,8,16,32,112,72,64,253,119,136,105,227,69,170,139,141,248,7,187,150,163,131,80,46,35,64,128,0,129,32,2,234,191,32,137,120,30,134,226,226,185,77,152,119,36,41,76,42,4,66,128,0,1,2,7,4,212,127,7,144,54,95,162,182,216,156,128,67,221,203,210,33,38,23,17,32,64,128,64,8,1,245,95,136,52,188,14,66,105,241,218,39,200,187,210,20,36,17,194,32,64,128,0,129,183,2,234,191,183,68,1,46,80,89,4,72,194,129,16,228,233,0,146,75,8,16,32,64,32,128,128,250,47,64,18,222,135,160,176,120,111,20,227,10,153,138,145,7,81,16,32,64,128,192,43,1,229,223,43,157,64,239,41,43,2,37,227,
117,40,82,245,218,199,187,4,8,16,32,176,95,64,253,183,63,7,199,34,80,85,28,115,138,112,149,92,69,200,130,24,8,16,32,64,224,185,128,250,239,185,77,176,119,20,21,193,18,242,42,28,201,122,165,227,61,2,4,8,16,216,45,160,254,219,157,129,227,253,171,41,142,91,237,191,82,182,246,231,64,4,4,8,16,32,240,76,64,253,247,76,38,224,235,74,138,128,73,121,30,146,116,61,183,241,14,1,2,4,8,236,21,80,255,237,245,63,215,187,138,226,156,215,238,171,229,107,119,6,244,79,128,0,1,2,143,5,212,127,143,93,130,190,170,160,8,154,152,103,97,73,216,51,25,175,19,32,64,128,192,78,1,245,223,78,253,243,125,171,39,206,155,237,189,67,198,246,250,235,157,0,1,2,4,30,9,168,255,30,169,4,126,77,57,17,56,57,79,66,147,179,39,48,94,38,64,128,0,129,77,2,202,191,77,240,215,187,85,76,92,183,219,118,167,164,109,163,215,49,1,2,4,8,60,16,80,255,61,64,9,254,146,90,34,120,130,30,134,39,107,15,89,188,72,128,0,1,2,91,4,212,127,91,216,239,117,170,148,184,231,183,233,110,105,219,4,175,91,2,4,8,16,248,33,160,254,251,65,146,224,5,149,
68,130,36,61,8,81,222,30,160,120,137,0,1,2,4,54,8,168,255,54,160,223,239,82,33,113,223,112,75,11,18,183,133,93,167,4,8,16,32,240,77,64,253,247,13,36,201,175,234,136,36,137,250,17,166,204,253,32,241,2,1,2,4,8,44,23,80,255,45,39,31,211,161,50,98,140,227,142,86,228,110,135,186,62,9,16,32,64,224,143,128,242,239,143,69,178,159,20,17,201,18,246,57,92,201,251,172,225,103,2,4,8,16,88,45,160,254,91,45,62,174,63,53,196,56,203,245,45,201,222,122,115,61,18,32,64,128,192,111,1,245,223,111,137,132,255,175,132,72,152,180,63,33,75,223,31,11,63,17,32,64,128,192,90,1,245,223,90,239,177,189,169,32,198,122,174,110,77,254,86,139,235,143,0,1,2,4,126,9,168,255,82,207,4,5,68,234,244,253,245,151,4,38,79,160,240,9,16,32,144,84,64,253,151,52,113,255,133,173,126,200,157,63,21,96,246,252,137,159,0,1,2,57,5,212,127,57,243,246,17,181,2,240,131,34,237,15,114,152,54,117,2,39,64,128,64,82,1,229,95,210,196,253,9,91,241,240,199,34,237,79,146,152,54,117,2,39,64,128,64,74,1,245,95,202,180,125,9,90,237,240,133,35,
233,47,178,152,52,113,194,38,64,128,64,74,1,245,95,202,180,125,13,90,233,240,213,35,233,111,210,152,52,113,194,38,64,128,64,66,1,245,95,194,164,253,8,89,229,240,131,36,229,11,242,152,50,109,130,38,64,128,64,66,1,245,95,194,164,253,12,89,225,240,211,36,229,43,18,153,50,109,130,38,64,128,64,58,1,245,95,186,148,61,12,88,221,240,144,37,227,139,82,153,49,107,98,38,64,128,64,46,1,229,95,174,124,61,143,86,213,240,220,38,219,59,114,153,45,99,226,37,64,128,64,54,1,245,95,182,140,61,141,87,209,240,148,38,223,27,146,153,47,103,34,38,64,128,64,38,1,245,95,166,108,189,142,85,205,240,218,39,215,187,178,153,43,95,162,37,64,128,64,46,1,245,95,174,124,189,140,86,201,240,146,39,219,155,210,153,45,99,226,37,64,128,64,30,1,245,95,158,92,189,143,84,197,240,222,40,213,21,18,154,42,93,130,37,64,128,64,26,1,229,95,154,84,29,10,84,189,112,136,41,209,69,50,154,40,89,66,37,64,128,64,26,1,245,95,154,84,29,11,84,185,112,204,41,209,85,82,154,40,89,66,37,64,128,64,18,1,245,95,146,68,29,14,83,181,112,152,42,
205,133,114,154,38,85,2,37,64,128,64,18,1,245,95,146,68,29,15,83,177,112,220,42,205,149,146,154,38,85,2,37,64,128,64,10,1,245,95,138,52,157,10,82,173,112,138,43,203,197,210,154,37,83,226,36,64,128,64,124,1,229,95,252,28,157,143,80,165,112,222,44,195,29,242,154,33,75,98,36,64,128,64,6,1,245,95,134,44,157,142,81,161,112,154,44,199,13,18,155,35,79,162,36,64,128,64,116,1,245,95,244,12,93,139,79,157,112,205,45,254,93,50,27,63,71,34,36,64,128,64,124,1,245,95,252,28,93,138,80,153,112,137,45,195,77,82,155,33,75,98,36,64,128,64,108,1,245,95,236,252,92,143,78,149,112,221,46,252,157,146,27,62,69,2,36,64,128,64,104,1,229,95,232,244,220,10,78,141,112,139,47,248,205,178,27,60,65,194,35,64,128,64,104,1,245,95,232,244,220,11,78,137,112,207,47,248,221,210,27,60,65,194,35,64,128,64,96,1,245,95,224,228,220,14,77,133,112,155,48,116,3,242,27,58,61,130,35,64,128,64,96,1,245,95,224,228,220,15,77,129,112,223,48,118,11,50,28,59,63,162,35,64,128,64,76,1,229,95,204,188,12,139,74,121,48,140,50,106,67,82,
28,53,51,226,34,64,128,64,92,1,245,95,220,220,140,137,76,117,48,198,49,114,43,114,28,57,59,98,35,64,128,64,68,1,245,95,196,172,12,141,73,113,48,148,51,102,99,146,28,51,47,162,34,64,128,64,84,1,245,95,212,204,140,139,75,109,48,206,50,112,75,210,28,56,57,66,35,64,128,64,48,1,229,95,176,132,76,9,71,101,48,133,53,92,163,242,28,46,37,2,34,64,128,64,80,1,245,95,208,196,140,13,75,97,48,214,51,108,107,18,29,54,53,2,35,64,128,64,40,1,245,95,168,116,76,11,70,93,48,141,54,88,195,50,29,44,33,194,33,64,128,64,72,1,245,95,200,180,140,15,74,89,48,222,52,106,139,114,29,53,51,226,34,64,128,64,20,1,229,95,148,76,76,143,67,81,48,157,56,78,7,146,29,39,23,34,33,64,128,64,68,1,245,95,196,172,204,137,73,77,48,199,53,102,171,178,29,51,47,162,34,64,128,64,12,1,245,95,140,60,44,137,66,73,176,132,57,74,39,210,29,37,19,226,32,64,128,64,60,1,245,95,188,156,204,139,72,69,48,207,54,100,203,18,30,50,45,130,34,64,128,192,118,1,229,223,246,20,44,13,64,61,176,148,59,64,103,50,30,32,9,66,32,64,128,64,56,1,245,95,
184,148,204,13,72,57,48,215,55,96,235,82,30,48,41,66,34,64,128,192,102,1,245,223,230,4,44,239,94,53,176,156,124,123,135,114,190,61,5,2,32,64,128,64,48,1,245,95,176,132,204,15,71,49,48,223,56,94,15,178,30,47,39,34,34,64,128,192,62,1,229,223,62,251,109,61,43,5,182,209,239,236,88,218,119,234,235,155,0,1,2,177,4,212,127,177,242,177,38,26,149,192,26,231,104,189,200,123,180,140,136,135,0,1,2,187,4,212,127,187,228,183,246,171,16,216,202,191,177,115,153,223,136,175,107,2,4,8,132,17,80,254,133,73,197,218,64,148,1,107,189,3,245,38,245,129,146,33,20,2,4,8,108,18,80,255,109,130,223,222,173,42,96,123,10,182,5,32,247,219,232,117,76,128,0,129,32,2,234,191,32,137,88,31,134,34,96,189,121,156,30,101,63,78,46,68,66,128,0,129,245,2,202,191,245,230,97,122,84,2,132,73,197,142,64,164,127,135,186,62,9,16,32,16,67,64,253,23,35,15,123,162,80,1,236,113,143,210,171,252,71,201,132,56,8,16,32,176,90,64,253,183,90,60,84,127,10,128,80,233,88,31,140,9,176,222,92,143,4,8,16,136,32,160,254,139,144,133,125,49,88,
255,247,217,7,233,217,20,8,146,8,97,16,32,64,96,161,128,242,111,33,118,200,174,172,254,33,211,178,52,40,115,96,41,183,206,8,16,32,16,64,64,253,23,32,9,123,67,176,248,239,245,15,209,187,73,16,34,13,130,32,64,128,192,50,1,245,223,50,234,176,29,89,251,195,166,102,101,96,166,193,74,109,125,17,32,64,96,175,128,242,111,175,127,140,222,173,252,49,242,176,59,10,243,96,119,6,244,79,128,0,129,85,2,234,191,85,210,161,251,177,240,135,78,207,186,224,76,132,117,214,122,34,64,128,192,78,1,245,223,78,253,56,125,91,247,227,228,98,115,36,166,194,230,4,232,158,0,1,2,11,4,148,127,11,144,83,116,97,213,79,145,166,37,65,154,11,75,152,117,66,128,0,129,141,2,234,191,141,248,177,186,182,232,199,202,199,222,104,204,134,189,254,122,39,64,128,192,92,1,229,223,92,223,84,173,91,242,83,165,107,118,176,166,195,108,97,237,19,32,64,96,159,128,250,111,159,125,188,158,173,248,241,114,178,51,34,243,97,167,190,190,9,16,32,48,83,64,253,55,83,55,93,219,22,252,116,41,155,28,176,25,49,25,88,243,4,8,16,216,34,160,252,219,
194,30,183,83,203,125,220,220,108,138,204,148,216,4,175,91,2,4,8,76,20,80,255,77,196,77,217,180,213,62,101,218,166,6,109,78,76,229,213,56,1,2,4,54,8,168,255,54,160,199,238,210,98,31,59,63,123,162,51,43,246,184,235,149,0,1,2,115,4,148,127,115,92,83,183,106,169,79,157,190,89,193,155,22,179,100,181,75,128,0,129,245,2,234,191,245,230,241,123,180,210,199,207,209,142,8,205,139,29,234,250,36,64,128,192,12,1,245,223,12,213,244,109,90,232,211,167,112,210,0,204,140,73,176,154,37,64,128,192,82,1,229,223,82,238,60,157,89,230,243,228,106,113,164,166,198,98,112,221,17,32,64,96,130,128,250,111,2,106,137,38,173,242,37,210,56,103,16,38,199,28,87,173,18,32,64,96,149,128,242,111,149,116,190,126,172,241,249,114,182,46,98,179,99,157,181,158,8,16,32,48,94,64,253,55,222,180,76,139,150,248,50,169,156,49,16,211,99,134,170,54,9,16,32,176,70,64,253,183,198,57,103,47,86,248,156,121,91,22,181,9,178,140,90,71,4,8,16,24,42,160,252,27,202,89,174,49,235,123,185,148,14,30,144,25,50,24,84,115,4,8,16,88,34,160,
254,91,194,156,183,19,203,123,222,220,173,138,220,28,89,37,173,31,2,4,8,140,18,80,254,141,146,44,219,142,197,189,108,106,199,13,204,36,25,103,169,37,2,4,8,172,16,80,255,173,80,206,221,135,181,61,119,254,22,69,111,154,44,130,214,13,1,2,4,6,8,40,255,6,32,150,111,194,202,94,62,197,67,6,104,158,12,97,212,8,1,2,4,22,8,168,255,22,32,231,239,194,194,158,63,135,75,70,96,162,44,97,214,9,1,2,4,110,11,168,255,110,19,182,104,192,186,222,34,205,35,6,105,170,140,80,212,6,1,2,4,230,10,40,255,230,250,214,105,221,170,94,39,151,179,71,98,174,204,22,214,62,1,2,4,238,10,168,255,238,10,182,185,223,162,222,38,213,3,6,106,182,12,64,212,4,1,2,4,166,9,40,255,166,209,214,107,216,146,94,47,167,19,71,100,186,76,196,213,52,1,2,4,110,10,168,255,110,2,182,186,221,138,222,42,221,247,7,107,194,220,55,212,2,1,2,4,102,8,40,255,102,168,214,109,211,122,94,55,183,115,70,102,198,204,113,213,42,1,2,4,238,9,168,255,238,249,181,187,219,114,222,46,229,119,7,108,202,220,21,116,63,1,2,4,198,11,168,255,198,155,214,110,209,
106,94,59,191,83,70,103,210,76,97,213,40,1,2,4,46,11,40,255,46,211,181,189,209,90,222,54,245,55,6,110,214,220,192,115,43,1,2,4,134,11,168,255,134,147,214,111,208,82,94,63,199,51,70,104,222,204,80,213,38,1,2,4,174,8,40,255,174,168,181,191,199,66,222,126,10,92,3,48,113,174,185,185,139,0,1,2,163,5,212,127,163,69,123,180,103,29,239,145,231,9,163,52,117,38,160,106,146,0,1,2,39,5,148,127,39,193,92,254,159,128,85,220,84,184,42,96,238,92,149,115,31,1,2,4,70,9,168,255,70,73,182,107,199,34,222,46,229,3,7,108,246,12,196,212,20,1,2,4,78,11,40,255,78,147,185,225,183,128,37,252,183,132,255,191,32,96,250,92,64,115,11,1,2,4,6,9,168,255,6,65,182,108,198,10,222,50,237,227,6,109,2,141,179,212,18,1,2,4,206,8,40,255,206,104,185,246,187,128,245,251,187,136,223,207,9,152,65,231,188,92,77,128,0,129,49,2,234,191,49,142,109,91,177,124,183,77,253,176,129,155,67,195,40,53,68,128,0,129,131,2,202,191,131,80,46,123,38,96,241,126,38,227,245,195,2,38,209,97,42,23,18,32,64,96,136,128,250,111,8,99,235,70,172,221,
173,211,63,106,240,166,209,40,73,237,16,32,64,224,189,128,242,239,189,145,43,222,9,88,185,223,9,121,255,136,128,121,116,68,201,53,4,8,16,24,33,160,254,27,161,216,190,13,11,119,251,41,48,8,192,76,26,4,169,25,2,4,8,188,20,80,254,189,228,241,230,81,1,203,246,81,41,215,189,17,48,149,222,0,121,155,0,1,2,3,4,212,127,3,16,53,241,183,128,85,219,52,24,38,96,50,13,163,212,16,1,2,4,30,10,40,255,30,178,120,241,130,128,53,251,2,154,91,158,8,152,77,79,96,188,76,128,0,129,33,2,234,191,33,140,26,249,71,192,146,109,30,140,20,48,159,70,106,106,139,0,1,2,159,5,148,127,159,53,252,124,83,192,130,125,19,208,237,223,4,204,168,111,32,126,37,64,128,192,16,1,229,223,16,70,141,252,22,176,92,255,150,240,255,131,4,76,169,65,144,154,33,64,128,192,39,1,245,223,39,12,63,14,16,176,90,15,64,212,196,87,1,147,234,171,135,223,8,16,32,112,87,64,249,119,87,208,253,223,5,172,213,223,69,252,126,95,192,172,186,111,168,5,2,4,8,252,17,80,255,253,177,240,211,32,1,75,245,32,72,205,124,17,48,175,190,112,248,133,0,1,2,55,4,
148,127,55,240,220,250,76,192,66,253,76,198,235,183,4,76,172,91,124,110,38,64,128,192,135,128,250,239,131,194,15,3,5,172,211,3,49,53,245,89,192,212,250,172,225,103,2,4,8,92,19,80,254,93,115,115,215,59,1,171,244,59,33,239,95,22,48,185,46,211,185,145,0,1,2,255,10,40,255,76,132,89,2,214,232,89,178,218,245,207,140,155,3,4,8,16,184,39,160,254,187,231,231,238,23,2,10,192,23,56,222,186,45,96,126,221,38,212,0,1,2,109,5,148,127,109,83,191,98,224,22,232,21,202,141,251,48,193,26,39,223,208,9,16,184,37,160,254,187,197,231,230,55,2,214,231,55,64,222,190,43,96,138,221,21,116,63,1,2,29,5,148,127,29,179,190,114,204,86,231,149,218,77,251,50,201,154,38,222,176,9,16,184,44,160,252,187,76,231,198,131,2,214,230,131,80,46,187,33,96,150,221,192,115,43,1,2,13,5,212,127,13,147,190,122,200,150,230,213,226,61,251,51,207,122,230,221,168,9,16,184,34,160,252,187,162,230,158,147,2,22,230,147,96,46,191,38,96,162,93,115,115,23,1,2,253,4,212,127,253,114,190,99,196,214,229,29,234,45,251,52,213,90,166,221,160,9,
16,56,41,160,252,59,9,230,242,139,2,86,229,139,112,110,59,47,96,178,157,55,115,7,1,2,189,4,148,127,189,242,189,115,180,214,228,157,250,221,250,54,219,186,101,220,120,9,16,56,39,160,254,59,231,229,234,27,2,150,228,27,120,110,61,45,96,190,157,38,115,3,1,2,109,4,148,127,109,82,29,97,160,22,228,8,89,232,20,131,25,215,41,219,198,74,128,192,113,1,229,223,113,43,87,14,16,176,28,15,64,212,196,25,1,83,238,140,150,107,9,16,232,34,160,254,235,146,233,40,227,180,26,71,201,68,163,56,76,186,70,201,54,84,2,4,14,9,40,255,14,49,185,104,160,128,181,120,32,166,166,142,10,152,118,71,165,92,71,128,64,7,1,229,95,135,44,71,27,163,149,56,90,70,122,196,99,222,245,200,179,81,18,32,112,68,64,253,119,68,201,53,131,5,44,196,131,65,53,119,80,192,204,59,8,229,50,2,4,138,11,40,255,138,39,56,234,240,44,195,81,51,83,63,46,115,175,126,142,141,144,0,129,119,2,202,191,119,66,222,159,36,96,17,158,4,171,217,247,2,38,223,123,35,87,16,32,80,91,64,253,87,59,191,145,71,103,13,142,156,157,242,177,153,126,229,83,108,128,
4,8,188,16,80,254,189,192,241,214,100,1,43,240,100,96,205,191,22,48,1,95,251,120,151,0,129,186,2,202,191,186,185,205,48,50,235,111,134,44,149,142,209,20,44,157,94,131,35,64,224,137,128,242,239,9,140,151,23,9,88,125,23,65,235,230,169,128,57,248,148,198,27,4,8,148,21,80,255,149,77,109,150,129,89,124,179,100,170,114,156,102,97,229,236,26,27,1,2,63,5,148,127,63,77,188,178,88,192,210,187,24,92,119,15,5,204,195,135,44,94,36,64,160,164,128,242,175,100,90,179,13,202,194,155,45,99,69,227,53,17,139,38,214,176,8,16,248,33,160,254,251,65,226,133,13,2,214,221,13,232,186,124,36,96,42,62,82,241,26,1,2,213,4,148,127,213,50,154,117,60,86,221,172,153,43,24,183,201,88,48,169,134,68,128,192,23,1,229,223,23,14,191,108,20,176,230,110,196,215,245,119,1,211,241,187,136,223,9,16,168,36,160,252,171,148,205,236,99,177,226,102,207,96,177,248,77,200,98,9,53,28,2,4,62,4,148,127,31,20,126,8,32,96,189,13,144,4,33,124,18,48,35,63,97,248,145,0,129,66,2,234,191,66,201,172,48,20,203,109,133,44,214,26,131,57,89,
43,159,70,67,128,192,63,2,202,63,243,32,152,128,197,54,88,66,132,243,183,128,89,105,26,16,32,80,75,64,249,87,43,159,37,70,99,169,45,145,198,114,131,48,47,203,165,212,128,8,52,22,80,254,53,78,126,220,161,91,104,227,230,166,117,100,38,102,235,244,27,60,129,82,2,234,191,82,233,44,51,24,235,108,153,84,86,27,136,169,89,45,163,198,67,160,167,128,242,175,103,222,227,143,218,42,27,63,71,109,35,52,57,219,166,222,192,9,148,17,80,254,149,73,101,185,129,88,99,203,165,180,210,128,76,207,74,217,52,22,2,253,4,148,127,253,114,158,103,196,86,216,60,185,106,25,169,9,218,50,237,6,77,160,132,128,242,175,68,26,203,14,194,250,90,54,181,85,6,102,138,86,201,164,113,16,232,37,160,252,235,149,239,124,163,181,186,230,203,89,183,136,205,209,110,25,55,94,2,21,4,212,127,21,178,88,122,12,22,215,210,233,45,50,56,179,180,72,34,13,131,64,27,1,229,95,155,84,231,29,168,165,53,111,238,58,69,110,158,118,202,182,177,18,200,46,160,252,203,158,193,22,241,91,88,91,164,185,192,32,205,212,2,73,52,4,2,45,4,148,127,45,210,
156,127,144,150,213,252,57,236,50,2,115,181,75,166,141,147,64,102,1,229,95,230,236,181,138,221,162,218,42,221,201,7,107,182,38,79,160,240,9,148,23,80,254,149,79,113,157,1,90,82,235,228,178,195,72,204,215,14,89,54,70,2,89,5,148,127,89,51,215,50,110,11,106,203,180,231,29,180,9,155,55,119,34,39,80,93,64,253,87,61,195,181,198,103,61,173,149,207,6,163,49,101,27,36,217,16,9,36,20,80,254,37,76,90,235,144,173,166,173,211,159,115,240,38,109,206,188,137,154,64,101,1,229,95,229,236,214,28,155,181,180,102,94,139,143,202,180,45,158,96,195,35,144,76,64,249,151,44,97,194,253,91,192,74,106,26,164,20,48,113,83,166,77,208,4,74,10,40,255,74,166,181,252,160,172,163,229,83,92,117,128,166,110,213,204,26,23,129,92,2,202,191,92,249,18,237,111,1,171,232,111,9,255,159,78,192,228,77,151,50,1,19,40,39,160,252,43,151,210,54,3,178,134,182,73,117,197,129,154,190,21,179,106,76,4,242,8,40,255,242,228,74,164,223,5,172,160,223,69,252,158,74,192,4,78,149,46,193,18,40,37,160,252,43,149,206,118,131,177,126,182,75,
121,181,1,155,194,213,50,106,60,4,114,8,40,255,114,228,73,148,207,4,172,158,207,100,188,158,70,192,36,78,147,42,129,18,40,35,160,252,43,147,202,182,3,177,118,182,77,125,165,129,155,198,149,178,105,44,4,226,11,40,255,226,231,72,132,239,4,172,156,239,132,188,159,66,192,68,78,145,38,65,18,40,33,160,252,43,145,198,246,131,176,110,182,159,2,85,0,76,229,42,153,52,14,2,177,5,148,127,177,243,35,186,163,2,86,205,163,82,174,11,47,96,50,135,79,145,0,9,164,23,80,254,165,79,161,1,252,39,96,205,52,21,10,9,152,206,133,146,105,40,4,2,10,40,255,2,38,69,72,23,5,172,152,23,225,220,22,83,192,132,142,153,23,81,17,168,32,160,252,171,144,69,99,248,45,96,189,252,45,225,255,139,8,152,210,69,18,105,24,4,130,9,40,255,130,37,68,56,55,5,172,150,55,1,221,30,79,192,164,142,151,19,17,17,200,46,160,252,203,158,65,241,127,23,176,86,126,23,241,123,1,1,211,186,64,18,13,129,64,32,1,229,95,160,100,8,101,144,128,149,114,16,164,102,98,9,152,216,177,242,33,26,2,153,5,148,127,153,179,39,246,103,2,214,201,103,50,94,79,
46,96,106,39,79,160,240,9,4,17,80,254,5,73,132,48,6,11,88,37,7,131,106,46,142,128,201,29,39,23,34,33,144,85,64,249,151,53,115,226,126,39,96,141,124,39,228,253,196,2,166,119,226,228,9,157,64,0,1,229,95,128,36,8,97,146,128,21,114,18,172,102,99,8,152,224,49,242,32,10,2,25,5,148,127,25,179,38,230,163,2,214,199,163,82,174,203,42,96,142,103,205,156,184,9,236,20,80,253,237,212,215,247,2,1,139,227,2,100,93,108,22,48,203,55,39,64,247,4,210,9,40,255,210,165,76,192,103,5,44,141,103,197,92,159,81,192,60,207,152,53,49,19,216,37,160,252,219,37,175,223,133,2,22,198,133,216,186,218,40,96,166,111,196,215,53,129,84,2,202,191,84,233,18,236,85,1,203,226,85,57,247,101,19,48,215,179,101,76,188,4,118,8,40,255,118,168,235,115,131,128,69,113,3,186,46,55,9,152,237,155,224,117,75,32,141,128,242,47,77,170,4,122,87,192,146,120,87,208,253,169,4,76,248,84,233,18,44,129,165,2,170,191,165,220,58,219,44,96,61,220,156,0,221,175,22,48,229,87,139,235,143,64,14,1,229,95,142,60,137,114,148,128,213,112,148,164,118,
210,8,152,244,105,82,37,80,2,203,4,148,127,203,168,117,20,68,192,90,24,36,17,194,88,41,96,218,175,212,214,23,129,248,2,202,191,248,57,18,225,104,1,43,225,104,81,237,165,16,48,241,83,164,73,144,4,150,8,40,255,150,48,235,36,152,128,117,48,88,66,132,179,76,192,220,95,70,173,35,2,129,5,84,127,129,147,35,180,153,2,22,193,153,186,218,142,45,96,246,199,206,143,232,8,204,23,80,254,205,55,214,67,80,1,75,96,208,196,8,107,137,128,249,191,132,89,39,4,130,10,40,255,130,38,70,88,43,4,44,128,43,148,245,17,87,192,19,16,55,55,34,35,48,87,64,249,55,215,87,235,193,5,44,127,193,19,36,188,249,2,30,130,249,198,122,32,16,77,64,245,23,45,35,226,89,45,96,237,91,45,174,191,128,2,30,131,128,73,17,18,129,137,2,202,191,137,184,154,78,34,96,229,75,146,40,97,206,21,240,32,204,245,213,58,129,72,2,202,191,72,217,16,203,46,1,235,222,46,121,253,70,19,240,44,68,203,136,120,8,204,16,80,253,205,80,213,102,66,1,139,94,194,164,9,121,146,128,167,97,18,172,102,9,132,17,80,254,133,73,133,64,118,11,88,242,118,103,64,255,
145,4,60,15,145,178,33,22,2,163,5,148,127,163,69,181,151,88,192,130,151,56,121,66,159,33,224,145,152,161,170,77,2,251,5,84,127,251,115,32,130,72,2,86,187,72,217,16,75,8,1,15,69,136,52,8,130,192,80,1,229,223,80,78,141,21,16,176,214,21,72,162,33,140,22,240,88,140,22,213,30,129,189,2,202,191,189,254,122,143,40,96,165,139,152,21,49,237,23,240,100,236,207,129,8,8,140,17,80,253,141,113,212,74,49,1,203,92,177,132,26,206,48,1,207,198,48,74,13,17,216,40,160,252,219,136,175,235,200,2,22,185,200,217,17,219,102,1,143,199,230,4,232,158,192,77,1,213,223,77,64,183,23,22,176,194,21,78,174,161,221,23,240,128,220,55,212,2,129,93,2,202,191,93,242,250,205,32,96,125,203,144,37,49,238,20,240,140,236,212,215,55,129,171,2,170,191,171,114,238,107,34,96,113,107,146,104,195,188,33,224,41,185,129,231,86,2,91,4,148,127,91,216,117,154,73,192,210,150,41,91,98,221,38,224,65,217,70,175,99,2,167,5,84,127,167,201,220,208,80,192,186,214,48,233,134,124,69,192,163,114,69,205,61,4,214,11,40,255,214,155,235,49,163,
128,85,45,99,214,196,188,71,192,211,178,199,93,175,4,142,11,168,254,142,91,185,178,185,128,37,173,249,4,48,252,83,2,158,151,83,92,46,38,176,88,64,249,183,24,92,119,153,5,44,104,153,179,39,246,13,2,30,153,13,232,186,36,112,64,64,245,119,0,201,37,4,62,4,172,102,31,20,126,32,112,76,192,67,115,204,201,85,4,86,10,40,255,86,106,235,171,130,128,181,172,66,22,141,97,181,128,231,102,181,184,254,8,188,18,80,253,189,210,241,30,129,135,2,22,178,135,44,94,36,240,70,192,147,243,6,200,219,4,150,9,40,255,150,81,235,168,146,128,101,172,82,54,141,101,169,128,135,103,41,183,206,8,60,20,80,253,61,100,241,34,129,183,2,214,176,183,68,46,32,240,84,192,243,243,148,198,27,4,22,8,168,254,22,32,235,162,170,128,5,172,106,102,141,107,141,128,39,104,141,179,94,8,252,20,80,254,253,52,241,10,129,195,2,150,175,195,84,46,36,240,88,192,67,244,216,197,171,4,102,10,168,254,102,234,106,187,131,128,181,171,67,150,141,113,182,128,231,104,182,176,246,9,124,22,80,253,125,214,240,51,129,75,2,22,174,75,108,110,34,240,77,
192,147,244,13,196,175,4,166,9,40,255,166,209,106,184,147,128,101,171,83,182,141,117,170,128,135,105,42,175,198,9,252,43,160,250,51,17,8,140,17,176,102,141,113,212,10,129,127,4,60,79,230,1,129,153,2,170,191,153,186,218,110,38,96,193,106,150,112,195,157,45,224,145,154,45,172,253,174,2,170,191,174,153,55,238,57,2,86,171,57,174,90,237,44,224,169,234,156,125,99,159,35,160,250,155,227,170,213,198,2,150,170,198,201,55,244,105,2,158,171,105,180,26,110,41,160,252,107,153,118,131,158,43,96,161,154,235,171,245,182,2,30,173,182,169,55,240,193,2,170,191,193,160,154,35,240,175,128,85,202,68,32,48,75,192,211,53,75,86,187,125,4,84,127,125,114,109,164,139,5,44,81,139,193,117,215,75,192,3,214,43,223,70,59,86,64,245,55,214,83,107,4,62,11,88,159,62,107,248,153,192,120,1,207,216,120,83,45,118,16,80,253,117,200,178,49,110,20,176,56,109,196,215,117,23,1,143,89,151,76,27,231,40,1,213,223,40,73,237,16,120,38,96,101,122,38,227,117,2,35,5,60,105,35,53,181,85,91,64,245,87,59,191,70,23,68,192,178,20,36,
17,194,168,47,224,97,171,159,99,35,188,47,160,250,187,111,168,5,2,71,4,172,73,71,148,92,67,96,140,128,231,109,140,163,86,170,10,168,254,170,102,214,184,2,10,88,144,2,38,69,72,149,5,60,114,149,179,107,108,119,4,84,127,119,244,220,75,224,172,128,213,232,172,152,235,9,220,21,240,212,221,21,116,127,61,1,213,95,189,156,26,81,112,1,75,81,240,4,9,175,166,128,7,175,102,94,141,234,154,128,234,239,154,155,187,8,220,17,176,14,221,209,115,47,129,235,2,158,189,235,118,238,172,36,160,250,171,148,77,99,73,36,96,17,74,148,44,161,86,19,240,248,85,203,168,241,156,21,80,253,157,21,115,61,129,81,2,86,160,81,146,218,33,112,69,192,19,120,69,205,61,53,4,84,127,53,242,104,20,73,5,44,63,73,19,39,236,58,2,30,194,58,185,52,146,227,2,170,191,227,86,174,36,48,67,192,218,51,67,85,155,4,78,10,120,16,79,130,185,60,181,128,226,47,117,250,4,95,68,192,186,83,36,145,134,145,94,192,179,152,62,133,6,112,72,64,245,119,136,201,69,4,102,11,88,116,102,11,107,159,192,97,1,143,227,97,42,23,38,21,80,253,37,77,156,176,11,
10,88,113,10,38,213,144,18,11,120,34,19,39,79,232,111,4,84,127,111,128,188,77,96,165,128,229,102,165,182,190,8,28,17,240,84,30,81,114,77,46,1,197,95,174,124,137,182,129,128,165,166,65,146,13,49,159,128,7,51,95,206,68,252,92,64,245,247,220,198,59,4,118,9,88,103,118,201,235,151,192,27,1,15,231,27,32,111,167,16,80,252,165,72,147,32,27,10,88,99,26,38,221,144,211,8,120,62,211,164,74,160,15,5,84,127,15,89,188,72,32,130,128,5,38,66,22,196,64,224,169,128,71,244,41,141,55,130,11,168,254,130,39,72,120,205,5,172,46,205,39,128,225,103,16,240,152,102,200,146,24,63,11,40,254,62,107,248,153,64,68,1,43,75,196,172,136,137,192,15,1,143,234,15,18,47,4,21,80,252,5,77,140,176,8,124,17,176,170,124,225,240,11,129,192,2,158,214,192,201,17,218,127,2,170,63,83,129,64,18,1,75,74,146,68,9,147,192,191,2,158,88,19,33,174,128,226,47,110,110,68,70,224,135,128,229,228,7,137,23,8,196,22,240,208,198,206,79,215,232,84,127,93,51,111,220,89,5,172,37,89,51,39,238,214,2,30,220,214,233,15,55,120,197,95,184,148,8,136,
192,91,1,235,200,91,34,23,16,136,41,224,225,141,153,151,110,81,41,254,186,101,220,120,171,8,88,67,170,100,210,56,90,10,120,128,91,166,61,204,160,21,127,97,82,33,16,2,167,5,172,31,167,201,220,64,32,150,128,135,56,86,62,186,68,163,248,235,146,105,227,172,42,96,237,168,154,89,227,106,37,224,65,110,149,238,237,131,85,252,109,79,129,0,8,220,22,176,110,220,38,212,0,129,24,2,30,230,24,121,168,30,133,226,175,122,134,141,175,139,128,53,163,75,166,141,179,133,128,7,186,69,154,183,13,82,241,183,141,94,199,4,134,11,88,47,134,147,106,144,192,94,1,15,245,94,255,170,189,43,254,170,102,214,184,186,10,88,43,186,102,222,184,75,11,120,176,75,167,119,249,224,20,127,203,201,117,72,96,186,128,117,98,58,177,14,8,236,17,240,112,239,113,175,214,171,226,175,90,70,141,135,192,47,1,107,132,153,64,160,176,128,7,188,112,114,23,12,77,241,183,0,89,23,4,54,9,88,31,54,193,235,150,192,50,1,79,249,50,234,66,29,169,253,10,37,211,80,8,60,18,176,52,60,82,241,26,129,106,2,158,244,106,25,157,57,30,197,223,76,93,109,
19,8,34,96,89,8,146,8,97,16,152,47,224,113,159,111,156,189,7,181,95,246,12,138,159,192,81,1,43,194,81,41,215,17,40,33,224,145,47,145,198,41,131,80,252,77,97,213,40,129,160,2,86,131,160,137,17,22,129,137,2,158,251,137,184,41,155,86,251,165,76,155,160,9,220,17,176,16,220,209,115,47,129,188,2,158,253,188,185,27,27,185,226,111,172,167,214,8,36,17,176,8,36,73,148,48,9,204,16,240,7,96,134,106,158,54,213,126,121,114,37,82,2,163,5,252,253,31,45,170,61,2,217,4,252,21,200,150,177,17,241,170,253,70,40,106,131,64,98,1,127,250,19,39,79,232,4,198,9,248,83,48,206,50,122,75,106,191,232,25,18,31,129,21,2,254,234,175,80,214,7,129,28,2,254,30,228,200,211,245,40,213,126,215,237,220,73,160,152,128,63,248,197,18,106,56,4,238,10,248,163,112,87,48,230,253,106,191,152,121,17,21,129,93,2,254,214,239,146,215,47,129,208,2,254,52,132,78,207,169,224,148,126,167,184,92,76,160,137,128,191,242,77,18,109,152,4,46,8,248,251,112,1,45,212,45,106,191,80,233,16,12,129,72,2,254,192,71,202,134,88,8,68,20,240,87,34,98,
86,222,197,164,244,123,39,228,125,2,205,5,252,105,111,62,1,12,159,192,65,1,127,43,14,66,109,191,76,233,183,61,5,2,32,144,65,192,31,245,12,89,18,35,129,40,2,254,98,68,201,196,163,56,148,126,143,84,188,70,128,192,67,1,127,206,31,178,120,145,0,129,23,2,254,110,188,192,217,242,150,210,111,11,187,78,9,100,22,240,135,60,115,246,196,78,96,167,128,191,30,59,245,127,247,173,244,251,45,225,255,9,16,56,37,224,79,248,41,46,23,19,32,240,93,192,31,145,239,34,107,126,87,249,173,113,214,11,129,170,2,254,118,87,205,172,113,17,88,42,224,79,201,58,110,165,223,58,107,61,17,168,43,224,175,118,221,220,26,25,129,245,2,254,162,204,52,87,249,205,212,213,54,129,102,2,254,92,55,75,184,225,18,88,34,224,47,203,72,102,133,223,72,77,109,17,32,240,175,128,63,211,38,2,1,2,211,4,252,129,185,71,171,242,187,231,231,110,2,4,158,11,248,251,252,220,198,59,4,8,140,18,240,151,230,140,164,186,239,140,150,107,9,16,184,36,224,207,242,37,54,55,17,32,112,73,192,95,156,87,108,10,191,87,58,222,35,64,96,168,128,63,199,67,57,
53,70,128,192,49,1,127,122,254,56,169,251,254,88,248,137,0,1,2,4,8,16,104,33,240,255,154,254,175,69,114,13,146,0,1,2,4,8,16,32,240,70,160,126,41,248,6,192,219,4,8,16,32,64,128,0,129,198,2,149,106,193,198,105,52,116,2,4,8,16,32,64,128,192,69,129,124,213,224,197,129,186,141,0,1,2,4,8,16,32,64,224,177,64,188,130,240,113,156,94,37,64,128,0,1,2,4,8,16,152,40,176,174,40,156,56,8,77,19,32,64,128,0,1,2,4,8,140,21,120,95,36,142,237,79,107,4,8,16,32,64,128,0,1,2,4,8,16,32,64,128,0,1,2,4,8,16,32,64,128,0,1,2,4,8,16,32,64,128,0,1,2,4,8,16,32,64,128,0,1,2,4,8,16,32,64,128,0,1,2,4,8,16,32,64,128,0,1,2,4,8,16,32,64,128,0,1,2,4,8,16,32,64,128,0,1,2,4,8,16,32,64,128,0,1,2,4,8,16,32,64,128,0,1,2,4,8,16,32,208,77,224,255,3,122,153,1,124,253,16,176,73,0,0,0,0,73,69,78,68,174,66,96,130);

implementation

end.
